/*
    Copyright (c) 2008-2009 NetAllied Systems GmbH

    This file is part of COLLADASaxFrameworkLoader.

    Licensed under the MIT Open Source License,
    for details please see LICENSE file or the website
    http://www.opensource.org/licenses/mit-license.php
*/

#include "COLLADASaxFWLStableHeaders.h"
#include "COLLADASaxFWLColladaParserAutoGen15Private.h"


// disable warning "unused variable"
#if defined(__INTEL_COMPILER) && !defined(WIN32)
template <typename T>
inline void disable_warning_unused(T &x) { (void)x; }
#    define DISABLE_WARNING_UNUSED(x) disable_warning_unused(x);
#else
#    define DISABLE_WARNING_UNUSED(x) (void)x;
#endif



namespace COLLADASaxFWL15
{


#ifdef GENERATEDSAXPARSER_VALIDATION


//---------------------------------------------------------------------
const int PCRE_OVECCOUNT = 30;    /* should be a multiple of 3 */

//---------------------------------------------------------------------
ParserError::ErrorType validate__version_enum( ParserString value )
{
    return validate__version_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__version_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__version_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__version_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__COLLADA( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* validationData = (COLLADA__ValidationData*)mValidationDataStack.newObject(sizeof(COLLADA__ValidationData));
        memset(validationData, 0, sizeof(COLLADA__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__COLLADA()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* validationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ASSET) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (validationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) || (validationData->validation_current_state == HASH_ELEMENT_SCENE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_COLLADA, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__asset( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 481:
        {
        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 334:
        {
        evaluate_scene__ValidationData* parentValidationData = (evaluate_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->render != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: render" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 614:
        {
        profile_cg_type____technique__ValidationData* parentValidationData = (profile_cg_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->annotate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: annotate" ) )
                return false;
        }
        if ( parentValidationData->pass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 615:
        {
        profile_common_type____technique__ValidationData* parentValidationData = (profile_common_type____technique__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 616:
        {
        profile_gles2_type____technique__ValidationData* parentValidationData = (profile_gles2_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->annotate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: annotate" ) )
                return false;
        }
        if ( parentValidationData->pass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 617:
        {
        profile_gles_type____technique__ValidationData* parentValidationData = (profile_gles_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->annotate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: annotate" ) )
                return false;
        }
        if ( parentValidationData->pass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 618:
        {
        profile_glsl_type____technique__ValidationData* parentValidationData = (profile_glsl_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->annotate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: annotate" ) )
                return false;
        }
        if ( parentValidationData->pass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 320:
        {
        animation_clip__ValidationData* parentValidationData = (animation_clip__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_animation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: instance_animation" ) )
                return false;
        }
        if ( parentValidationData->instance_formula != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: instance_formula" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 12:
        {
        animation__ValidationData* parentValidationData = (animation__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 536:
        {
        articulated_system__ValidationData* parentValidationData = (articulated_system__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 46:
        {
        camera__ValidationData* parentValidationData = (camera__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->optics != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: optics" ) )
                return false;
        }
        if ( parentValidationData->imager != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: imager" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 84:
        {
        controller__ValidationData* parentValidationData = (controller__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 69:
        {
        effect__ValidationData* parentValidationData = (effect__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 8:
        {
        extra__ValidationData* parentValidationData = (extra__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 48:
        {
        force_field__ValidationData* parentValidationData = (force_field__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 613:
        {
        geometry__ValidationData* parentValidationData = (geometry__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 262:
        {
        image____image_type__ValidationData* parentValidationData = (image____image_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 17:
        {
        instance_kinematics_scene__ValidationData* parentValidationData = (instance_kinematics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->newparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: newparam" ) )
                return false;
        }
        if ( parentValidationData->setparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: setparam" ) )
                return false;
        }
        if ( parentValidationData->bind_kinematics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: bind_kinematics_model" ) )
                return false;
        }
        if ( parentValidationData->bind_joint_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: bind_joint_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 216:
        {
        kinematics_model__ValidationData* parentValidationData = (kinematics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 300:
        {
        kinematics_scene__ValidationData* parentValidationData = (kinematics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_kinematics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: instance_kinematics_model" ) )
                return false;
        }
        if ( parentValidationData->instance_articulated_system != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: instance_articulated_system" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 257:
        {
        library_animation_clips__ValidationData* parentValidationData = (library_animation_clips__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->animation_clip != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: animation_clip" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 239:
        {
        library_animations__ValidationData* parentValidationData = (library_animations__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->animation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: animation" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 584:
        {
        library_articulated_systems__ValidationData* parentValidationData = (library_articulated_systems__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->articulated_system != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: articulated_system" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 565:
        {
        library_cameras__ValidationData* parentValidationData = (library_cameras__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->camera != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: camera" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 275:
        {
        library_controllers__ValidationData* parentValidationData = (library_controllers__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->controller != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: controller" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 206:
        {
        library_effects__ValidationData* parentValidationData = (library_effects__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->effect != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: effect" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 96:
        {
        library_force_fields__ValidationData* parentValidationData = (library_force_fields__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->force_field != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: force_field" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 101:
        {
        library_formulas__ValidationData* parentValidationData = (library_formulas__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->formula != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: formula" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 635:
        {
        library_geometries__ValidationData* parentValidationData = (library_geometries__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->geometry != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: geometry" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 369:
        {
        library_images__ValidationData* parentValidationData = (library_images__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->image != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: image" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 285:
        {
        library_joints__ValidationData* parentValidationData = (library_joints__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->joint != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: joint" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 227:
        {
        library_kinematics_models__ValidationData* parentValidationData = (library_kinematics_models__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->kinematics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: kinematics_model" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 172:
        {
        library_kinematics_scenes__ValidationData* parentValidationData = (library_kinematics_scenes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->kinematics_scene != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: kinematics_scene" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 145:
        {
        library_lights__ValidationData* parentValidationData = (library_lights__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->light != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: light" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 45:
        {
        library_materials__ValidationData* parentValidationData = (library_materials__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->material != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: material" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 620:
        {
        library_nodes__ValidationData* parentValidationData = (library_nodes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->node != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: node" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 462:
        {
        library_physics_materials__ValidationData* parentValidationData = (library_physics_materials__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->physics_material != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: physics_material" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 7:
        {
        library_physics_models__ValidationData* parentValidationData = (library_physics_models__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->physics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: physics_model" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 468:
        {
        library_physics_scenes__ValidationData* parentValidationData = (library_physics_scenes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->physics_scene != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: physics_scene" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 607:
        {
        library_visual_scenes__ValidationData* parentValidationData = (library_visual_scenes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->visual_scene != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: visual_scene" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 392:
        {
        light__ValidationData* parentValidationData = (light__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 326:
        {
        material__ValidationData* parentValidationData = (material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_effect != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: instance_effect" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 36:
        {
        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 288:
        {
        physics_material__ValidationData* parentValidationData = (physics_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 241:
        {
        physics_model__ValidationData* parentValidationData = (physics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->rigid_body != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: rigid_body" ) )
                return false;
        }
        if ( parentValidationData->rigid_constraint != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: rigid_constraint" ) )
                return false;
        }
        if ( parentValidationData->instance_physics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: instance_physics_model" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 586:
        {
        physics_scene__ValidationData* parentValidationData = (physics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_force_field != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: instance_force_field" ) )
                return false;
        }
        if ( parentValidationData->instance_physics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: instance_physics_model" ) )
                return false;
        }
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 42:
        {
        profile_BRIDGE__ValidationData* parentValidationData = (profile_BRIDGE__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 143:
        {
        profile_CG__ValidationData* parentValidationData = (profile_CG__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 480:
        {
        profile_COMMON__ValidationData* parentValidationData = (profile_COMMON__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->newparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: newparam" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 452:
        {
        profile_GLES2__ValidationData* parentValidationData = (profile_GLES2__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 196:
        {
        profile_GLES__ValidationData* parentValidationData = (profile_GLES__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->newparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: newparam" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        case 148:
        {
        profile_GLSL__ValidationData* parentValidationData = (profile_GLSL__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 710:
        {
        source__ValidationData* parentValidationData = (source__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASSET;
        }
//        parentValidationData->asset++;

        break;
        }

        case 395:
        {
        visual_scene__ValidationData* parentValidationData = (visual_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->node != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: node" ) )
                return false;
        }
        if ( parentValidationData->evaluate_scene != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: evaluate_scene" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ASSET, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->asset >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ASSET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->asset++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        asset__ValidationData* validationData = (asset__ValidationData*)mValidationDataStack.newObject(sizeof(asset__ValidationData));
        memset(validationData, 0, sizeof(asset__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__asset()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* validationData = (asset__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->created < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ASSET, (const ParserChar*)0, "child: created" ) )
                return false;
        }
        if ( validationData->modified < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ASSET, (const ParserChar*)0, "child: modified" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__contributor( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->coverage != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTRIBUTOR, (const ParserChar*)0, "sibling: coverage" ) )
                return false;
        }
        if ( parentValidationData->created != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTRIBUTOR, (const ParserChar*)0, "sibling: created" ) )
                return false;
        }
        if ( parentValidationData->keywords != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTRIBUTOR, (const ParserChar*)0, "sibling: keywords" ) )
                return false;
        }
        if ( parentValidationData->modified != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTRIBUTOR, (const ParserChar*)0, "sibling: modified" ) )
                return false;
        }
        if ( parentValidationData->revision != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTRIBUTOR, (const ParserChar*)0, "sibling: revision" ) )
                return false;
        }
        if ( parentValidationData->subject != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTRIBUTOR, (const ParserChar*)0, "sibling: subject" ) )
                return false;
        }
        if ( parentValidationData->title != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTRIBUTOR, (const ParserChar*)0, "sibling: title" ) )
                return false;
        }
        if ( parentValidationData->unit != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTRIBUTOR, (const ParserChar*)0, "sibling: unit" ) )
                return false;
        }
        if ( parentValidationData->up_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTRIBUTOR, (const ParserChar*)0, "sibling: up_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTRIBUTOR, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->contributor++;

        contributor__ValidationData* validationData = (contributor__ValidationData*)mValidationDataStack.newObject(sizeof(contributor__ValidationData));
        memset(validationData, 0, sizeof(contributor__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__contributor()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        contributor__ValidationData* validationData = (contributor__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__author( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        contributor__ValidationData* parentValidationData = (contributor__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->author_email != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR, (const ParserChar*)0, "sibling: author_email" ) )
                return false;
        }
        if ( parentValidationData->author_website != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR, (const ParserChar*)0, "sibling: author_website" ) )
                return false;
        }
        if ( parentValidationData->authoring_tool != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR, (const ParserChar*)0, "sibling: authoring_tool" ) )
                return false;
        }
        if ( parentValidationData->comments != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR, (const ParserChar*)0, "sibling: comments" ) )
                return false;
        }
        if ( parentValidationData->copyright != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR, (const ParserChar*)0, "sibling: copyright" ) )
                return false;
        }
        if ( parentValidationData->source_data != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR, (const ParserChar*)0, "sibling: source_data" ) )
                return false;
        }
        if ( parentValidationData->author >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_AUTHOR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->author++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__author()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__author_email( ParserString value )
{
    return validate__author_email( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__author_email( const ParserChar* value, size_t length )
{
// regular expression: "^[A-Za-z0-9!#-'\*\+\-/=\?\^_`\{-~]+(\.[A-Za-z0-9!#-'\*\+\-/=\?\^_`\{-~]+)*@[A-Za-z0-9!#-'\*\+\-/=\?\^_`\{-~]+(\.[A-Za-z0-9!#-'\*\+\-/=\?\^_`\{-~]+)*$"
static const COLLADABU::PcreCompiledPattern compiledPattern("^[A-Za-z0-9!#-'\\*\\+\\-/=\\?\\^_`\\{-~]+(\\.[A-Za-z0-9!#-'\\*\\+\\-/=\\?\\^_`\\{-~]+)*@[A-Za-z0-9!#-'\\*\\+\\-/=\\?\\^_`\\{-~]+(\\.[A-Za-z0-9!#-'\\*\\+\\-/=\\?\\^_`\\{-~]+)*$");
pcre* pattern = compiledPattern.getCompiledPattern();
int ovector[ PCRE_OVECCOUNT ];
int pcre_result = pcre_exec(
    pattern,
    0,
    value,
    (int)length,
    0,
    0,
    ovector,
    PCRE_OVECCOUNT);
if (pcre_result < 0)
    return ParserError::ERROR_VALIDATION_PATTERN;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__author_email__stream( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__author_email__streamEnd( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__author_email( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        contributor__ValidationData* parentValidationData = (contributor__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->author_website != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR_EMAIL, (const ParserChar*)0, "sibling: author_website" ) )
                return false;
        }
        if ( parentValidationData->authoring_tool != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR_EMAIL, (const ParserChar*)0, "sibling: authoring_tool" ) )
                return false;
        }
        if ( parentValidationData->comments != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR_EMAIL, (const ParserChar*)0, "sibling: comments" ) )
                return false;
        }
        if ( parentValidationData->copyright != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR_EMAIL, (const ParserChar*)0, "sibling: copyright" ) )
                return false;
        }
        if ( parentValidationData->source_data != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR_EMAIL, (const ParserChar*)0, "sibling: source_data" ) )
                return false;
        }
        if ( parentValidationData->author_email >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_AUTHOR_EMAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->author_email++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__author_email()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__author_website( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        contributor__ValidationData* parentValidationData = (contributor__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->authoring_tool != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR_WEBSITE, (const ParserChar*)0, "sibling: authoring_tool" ) )
                return false;
        }
        if ( parentValidationData->comments != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR_WEBSITE, (const ParserChar*)0, "sibling: comments" ) )
                return false;
        }
        if ( parentValidationData->copyright != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR_WEBSITE, (const ParserChar*)0, "sibling: copyright" ) )
                return false;
        }
        if ( parentValidationData->source_data != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHOR_WEBSITE, (const ParserChar*)0, "sibling: source_data" ) )
                return false;
        }
        if ( parentValidationData->author_website >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_AUTHOR_WEBSITE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->author_website++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__author_website()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__authoring_tool( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        contributor__ValidationData* parentValidationData = (contributor__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->comments != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHORING_TOOL, (const ParserChar*)0, "sibling: comments" ) )
                return false;
        }
        if ( parentValidationData->copyright != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHORING_TOOL, (const ParserChar*)0, "sibling: copyright" ) )
                return false;
        }
        if ( parentValidationData->source_data != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AUTHORING_TOOL, (const ParserChar*)0, "sibling: source_data" ) )
                return false;
        }
        if ( parentValidationData->authoring_tool >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_AUTHORING_TOOL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->authoring_tool++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__authoring_tool()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__comments( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        contributor__ValidationData* parentValidationData = (contributor__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->copyright != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COMMENTS, (const ParserChar*)0, "sibling: copyright" ) )
                return false;
        }
        if ( parentValidationData->source_data != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COMMENTS, (const ParserChar*)0, "sibling: source_data" ) )
                return false;
        }
        if ( parentValidationData->comments >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_COMMENTS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->comments++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__comments()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__copyright( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        contributor__ValidationData* parentValidationData = (contributor__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->source_data != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COPYRIGHT, (const ParserChar*)0, "sibling: source_data" ) )
                return false;
        }
        if ( parentValidationData->copyright >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_COPYRIGHT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->copyright++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__copyright()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__source_data( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        contributor__ValidationData* parentValidationData = (contributor__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->source_data >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SOURCE_DATA, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->source_data++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__source_data()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__coverage( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->created != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COVERAGE, (const ParserChar*)0, "sibling: created" ) )
                return false;
        }
        if ( parentValidationData->keywords != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COVERAGE, (const ParserChar*)0, "sibling: keywords" ) )
                return false;
        }
        if ( parentValidationData->modified != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COVERAGE, (const ParserChar*)0, "sibling: modified" ) )
                return false;
        }
        if ( parentValidationData->revision != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COVERAGE, (const ParserChar*)0, "sibling: revision" ) )
                return false;
        }
        if ( parentValidationData->subject != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COVERAGE, (const ParserChar*)0, "sibling: subject" ) )
                return false;
        }
        if ( parentValidationData->title != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COVERAGE, (const ParserChar*)0, "sibling: title" ) )
                return false;
        }
        if ( parentValidationData->unit != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COVERAGE, (const ParserChar*)0, "sibling: unit" ) )
                return false;
        }
        if ( parentValidationData->up_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COVERAGE, (const ParserChar*)0, "sibling: up_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COVERAGE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->coverage >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_COVERAGE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->coverage++;

        coverage__ValidationData* validationData = (coverage__ValidationData*)mValidationDataStack.newObject(sizeof(coverage__ValidationData));
        memset(validationData, 0, sizeof(coverage__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__coverage()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        coverage__ValidationData* validationData = (coverage__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__geographic_location( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        coverage__ValidationData* parentValidationData = (coverage__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->geographic_location >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_GEOGRAPHIC_LOCATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->geographic_location++;

        geographic_location__ValidationData* validationData = (geographic_location__ValidationData*)mValidationDataStack.newObject(sizeof(geographic_location__ValidationData));
        memset(validationData, 0, sizeof(geographic_location__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__geographic_location()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        geographic_location__ValidationData* validationData = (geographic_location__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->longitude < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_GEOGRAPHIC_LOCATION, (const ParserChar*)0, "child: longitude" ) )
                return false;
        }
        if ( validationData->latitude < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_GEOGRAPHIC_LOCATION, (const ParserChar*)0, "child: latitude" ) )
                return false;
        }
        if ( validationData->altitude < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_GEOGRAPHIC_LOCATION, (const ParserChar*)0, "child: altitude" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__longitude( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        geographic_location__ValidationData* parentValidationData = (geographic_location__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->latitude != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LONGITUDE, (const ParserChar*)0, "sibling: latitude" ) )
                return false;
        }
        if ( parentValidationData->altitude != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LONGITUDE, (const ParserChar*)0, "sibling: altitude" ) )
                return false;
        }
        if ( parentValidationData->longitude >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_LONGITUDE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->longitude++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__longitude()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__latitude( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        geographic_location__ValidationData* parentValidationData = (geographic_location__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->longitude < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_LATITUDE, (const ParserChar*)0, "sibling: longitude" ) )
                return false;
        }
        if ( parentValidationData->altitude != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LATITUDE, (const ParserChar*)0, "sibling: altitude" ) )
                return false;
        }
        if ( parentValidationData->latitude >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_LATITUDE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->latitude++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__latitude()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__altitude_mode_enum( ParserString value )
{
    return validate__altitude_mode_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__altitude_mode_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__altitude_mode_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__altitude_mode_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__altitude( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        geographic_location__ValidationData* parentValidationData = (geographic_location__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->latitude < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ALTITUDE, (const ParserChar*)0, "sibling: latitude" ) )
                return false;
        }
        if ( parentValidationData->altitude >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ALTITUDE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->altitude++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__altitude()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__created( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->keywords != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CREATED, (const ParserChar*)0, "sibling: keywords" ) )
                return false;
        }
        if ( parentValidationData->modified != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CREATED, (const ParserChar*)0, "sibling: modified" ) )
                return false;
        }
        if ( parentValidationData->revision != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CREATED, (const ParserChar*)0, "sibling: revision" ) )
                return false;
        }
        if ( parentValidationData->subject != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CREATED, (const ParserChar*)0, "sibling: subject" ) )
                return false;
        }
        if ( parentValidationData->title != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CREATED, (const ParserChar*)0, "sibling: title" ) )
                return false;
        }
        if ( parentValidationData->unit != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CREATED, (const ParserChar*)0, "sibling: unit" ) )
                return false;
        }
        if ( parentValidationData->up_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CREATED, (const ParserChar*)0, "sibling: up_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CREATED, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->created >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CREATED, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->created++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__created()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__keywords( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->created < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_KEYWORDS, (const ParserChar*)0, "sibling: created" ) )
                return false;
        }
        if ( parentValidationData->modified != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_KEYWORDS, (const ParserChar*)0, "sibling: modified" ) )
                return false;
        }
        if ( parentValidationData->revision != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_KEYWORDS, (const ParserChar*)0, "sibling: revision" ) )
                return false;
        }
        if ( parentValidationData->subject != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_KEYWORDS, (const ParserChar*)0, "sibling: subject" ) )
                return false;
        }
        if ( parentValidationData->title != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_KEYWORDS, (const ParserChar*)0, "sibling: title" ) )
                return false;
        }
        if ( parentValidationData->unit != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_KEYWORDS, (const ParserChar*)0, "sibling: unit" ) )
                return false;
        }
        if ( parentValidationData->up_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_KEYWORDS, (const ParserChar*)0, "sibling: up_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_KEYWORDS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->keywords >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_KEYWORDS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->keywords++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__keywords()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__modified( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->created < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MODIFIED, (const ParserChar*)0, "sibling: created" ) )
                return false;
        }
        if ( parentValidationData->revision != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MODIFIED, (const ParserChar*)0, "sibling: revision" ) )
                return false;
        }
        if ( parentValidationData->subject != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MODIFIED, (const ParserChar*)0, "sibling: subject" ) )
                return false;
        }
        if ( parentValidationData->title != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MODIFIED, (const ParserChar*)0, "sibling: title" ) )
                return false;
        }
        if ( parentValidationData->unit != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MODIFIED, (const ParserChar*)0, "sibling: unit" ) )
                return false;
        }
        if ( parentValidationData->up_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MODIFIED, (const ParserChar*)0, "sibling: up_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MODIFIED, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->modified >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MODIFIED, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->modified++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__modified()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__revision( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->modified < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_REVISION, (const ParserChar*)0, "sibling: modified" ) )
                return false;
        }
        if ( parentValidationData->subject != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REVISION, (const ParserChar*)0, "sibling: subject" ) )
                return false;
        }
        if ( parentValidationData->title != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REVISION, (const ParserChar*)0, "sibling: title" ) )
                return false;
        }
        if ( parentValidationData->unit != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REVISION, (const ParserChar*)0, "sibling: unit" ) )
                return false;
        }
        if ( parentValidationData->up_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REVISION, (const ParserChar*)0, "sibling: up_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REVISION, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->revision >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REVISION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->revision++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__revision()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__subject( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->modified < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_SUBJECT, (const ParserChar*)0, "sibling: modified" ) )
                return false;
        }
        if ( parentValidationData->title != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SUBJECT, (const ParserChar*)0, "sibling: title" ) )
                return false;
        }
        if ( parentValidationData->unit != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SUBJECT, (const ParserChar*)0, "sibling: unit" ) )
                return false;
        }
        if ( parentValidationData->up_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SUBJECT, (const ParserChar*)0, "sibling: up_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SUBJECT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->subject >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SUBJECT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->subject++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__subject()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__title( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->modified < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TITLE, (const ParserChar*)0, "sibling: modified" ) )
                return false;
        }
        if ( parentValidationData->unit != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TITLE, (const ParserChar*)0, "sibling: unit" ) )
                return false;
        }
        if ( parentValidationData->up_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TITLE, (const ParserChar*)0, "sibling: up_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TITLE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->title >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TITLE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->title++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__title()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NMTOKEN( ParserString value )
{
    return validate__NMTOKEN( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NMTOKEN( const ParserChar* value, size_t length )
{
// regular expression: "^([A-Z]|[_]|[a-z]|[:]|[-]|[.]|[0-9])+$"
static const COLLADABU::PcreCompiledPattern compiledPattern("^([A-Z]|[_]|[a-z]|[:]|[-]|[.]|[0-9])+$");
pcre* pattern = compiledPattern.getCompiledPattern();
int ovector[ PCRE_OVECCOUNT ];
int pcre_result = pcre_exec(
    pattern,
    0,
    value,
    (int)length,
    0,
    0,
    ovector,
    PCRE_OVECCOUNT);
if (pcre_result < 0)
    return ParserError::ERROR_VALIDATION_PATTERN;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NMTOKEN__stream( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NMTOKEN__streamEnd( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__unit( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->modified < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_UNIT, (const ParserChar*)0, "sibling: modified" ) )
                return false;
        }
        if ( parentValidationData->up_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_UNIT, (const ParserChar*)0, "sibling: up_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_UNIT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->unit >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_UNIT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->unit++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__unit()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__up_axis_enum( ParserString value )
{
    return validate__up_axis_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__up_axis_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__up_axis_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__up_axis_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__up_axis( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->modified < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_UP_AXIS, (const ParserChar*)0, "sibling: modified" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_UP_AXIS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->up_axis >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_UP_AXIS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->up_axis++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__up_axis()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__extra( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 481:
        {
        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCENE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 194:
        {
        attachment__ValidationData* parentValidationData = (attachment__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 207:
        {
        nurbs_surface_type____control_vertices__ValidationData* parentValidationData = (nurbs_surface_type____control_vertices__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 208:
        {
        nurbs_type____control_vertices__ValidationData* parentValidationData = (nurbs_type____control_vertices__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 209:
        {
        spline_type____control_vertices__ValidationData* parentValidationData = (spline_type____control_vertices__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 469:
        {
        surface_type____cylinder__ValidationData* parentValidationData = (surface_type____cylinder__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 334:
        {
        evaluate_scene__ValidationData* parentValidationData = (evaluate_scene__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 702:
        {
        imager__ValidationData* parentValidationData = (imager__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 242:
        {
        render__instance_material__ValidationData* parentValidationData = (render__instance_material__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 283:
        {
        joints__ValidationData* parentValidationData = (joints__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 2 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 166:
        {
        optics__ValidationData* parentValidationData = (optics__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 418:
        {
        profile_GLES__technique__pass__ValidationData* parentValidationData = (profile_GLES__technique__pass__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 419:
        {
        profile_GLSL__technique__pass__ValidationData* parentValidationData = (profile_GLSL__technique__pass__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 100:
        {
        ref_attachment__ValidationData* parentValidationData = (ref_attachment__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 98:
        {
        render__ValidationData* parentValidationData = (render__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 466:
        {
        scene__ValidationData* parentValidationData = (scene__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BOX) || (parentValidationData->validation_current_state == HASH_ELEMENT_CAPSULE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SPHERE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BOX) || (parentValidationData->validation_current_state == HASH_ELEMENT_CAPSULE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SPHERE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 95:
        {
        targets__ValidationData* parentValidationData = (targets__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 2 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 614:
        {
        profile_cg_type____technique__ValidationData* parentValidationData = (profile_cg_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->pass < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 615:
        {
        profile_common_type____technique__ValidationData* parentValidationData = (profile_common_type____technique__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLINN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONSTANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBERT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHONG) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 616:
        {
        profile_gles2_type____technique__ValidationData* parentValidationData = (profile_gles2_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->pass < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 617:
        {
        profile_gles_type____technique__ValidationData* parentValidationData = (profile_gles_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->pass < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 618:
        {
        profile_glsl_type____technique__ValidationData* parentValidationData = (profile_glsl_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->pass < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 482:
        {
        texture__ValidationData* parentValidationData = (texture__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 222:
        {
        vertex_weights__ValidationData* parentValidationData = (vertex_weights__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 2 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 320:
        {
        animation_clip__ValidationData* parentValidationData = (animation_clip__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_animation < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: instance_animation" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 12:
        {
        animation__ValidationData* parentValidationData = (animation__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANIMATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CHANNEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 536:
        {
        articulated_system__ValidationData* parentValidationData = (articulated_system__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_KINEMATICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOTION) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 575:
        {
        asset__ValidationData* parentValidationData = (asset__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->modified < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: modified" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 54:
        {
        bind_material__ValidationData* parentValidationData = (bind_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 665:
        {
        box__ValidationData* parentValidationData = (box__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->half_extents < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: half_extents" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 68:
        {
        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vertices < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 46:
        {
        camera__ValidationData* parentValidationData = (camera__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->optics < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: optics" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 59:
        {
        capsule__ValidationData* parentValidationData = (capsule__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 420:
        {
        pass____cg_pass_type__ValidationData* parentValidationData = (pass____cg_pass_type__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 738:
        {
        circle__ValidationData* parentValidationData = (circle__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 585:
        {
        cone__ValidationData* parentValidationData = (cone__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->angle < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: angle" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 84:
        {
        controller__ValidationData* parentValidationData = (controller__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_MORPH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKIN) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 97:
        {
        convex_mesh__ValidationData* parentValidationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 115:
        {
        curves__ValidationData* parentValidationData = (curves__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->curve < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: curve" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 470:
        {
        cylinder____cylinder_type__ValidationData* parentValidationData = (cylinder____cylinder_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 6:
        {
        edges__ValidationData* parentValidationData = (edges__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 4 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 69:
        {
        effect__ValidationData* parentValidationData = (effect__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_BRIDGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_CG) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_COMMON) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES2) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLSL) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 437:
        {
        ellipse__ValidationData* parentValidationData = (ellipse__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 157:
        {
        faces__ValidationData* parentValidationData = (faces__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 48:
        {
        force_field__ValidationData* parentValidationData = (force_field__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 613:
        {
        geometry__ValidationData* parentValidationData = (geometry__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BREP) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONVEX_MESH) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_MESH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SPLINE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 421:
        {
        pass____gles2_pass_type__ValidationData* parentValidationData = (pass____gles2_pass_type__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 578:
        {
        shader____gles2_shader_type__ValidationData* parentValidationData = (shader____gles2_shader_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->sources < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: sources" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 67:
        {
        sampler2D____gles_sampler_type__ValidationData* parentValidationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXCOORD) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 446:
        {
        value____gles_texture_pipeline_type__ValidationData* parentValidationData = (value____gles_texture_pipeline_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->texcombiner != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: texcombiner" ) )
                return false;
        }
        if ( parentValidationData->texenv != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: texenv" ) )
                return false;
        }
        if ( parentValidationData->extra >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 579:
        {
        shader____glsl_shader_type__ValidationData* parentValidationData = (shader____glsl_shader_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->sources < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: sources" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 244:
        {
        hyperbola__ValidationData* parentValidationData = (hyperbola__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 262:
        {
        image____image_type__ValidationData* parentValidationData = (image____image_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CREATE_2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_CREATE_3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_CREATE_CUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INIT_FROM) || (parentValidationData->validation_current_state == HASH_ELEMENT_RENDERABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 246:
        {
        instance_articulated_system__ValidationData* parentValidationData = (instance_articulated_system__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 623:
        {
        instance_camera__ValidationData* parentValidationData = (instance_camera__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 250:
        {
        instance_controller__ValidationData* parentValidationData = (instance_controller__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 576:
        {
        instance_effect__ValidationData* parentValidationData = (instance_effect__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 354:
        {
        instance_force_field__ValidationData* parentValidationData = (instance_force_field__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 33:
        {
        instance_geometry__ValidationData* parentValidationData = (instance_geometry__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 276:
        {
        instance_image__ValidationData* parentValidationData = (instance_image__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 711:
        {
        instance_joint__ValidationData* parentValidationData = (instance_joint__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 152:
        {
        instance_kinematics_model__ValidationData* parentValidationData = (instance_kinematics_model__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 17:
        {
        instance_kinematics_scene__ValidationData* parentValidationData = (instance_kinematics_scene__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 662:
        {
        instance_light__ValidationData* parentValidationData = (instance_light__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 243:
        {
        instance_material____instance_material_type__ValidationData* parentValidationData = (instance_material____instance_material_type__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 708:
        {
        instance_node__ValidationData* parentValidationData = (instance_node__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 212:
        {
        instance_physics_material__ValidationData* parentValidationData = (instance_physics_material__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 571:
        {
        instance_physics_model__ValidationData* parentValidationData = (instance_physics_model__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 533:
        {
        instance_rigid_body__ValidationData* parentValidationData = (instance_rigid_body__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 724:
        {
        instance_rigid_constraint__ValidationData* parentValidationData = (instance_rigid_constraint__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 388:
        {
        instance_animation__ValidationData* parentValidationData = (instance_animation__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 587:
        {
        joint__ValidationData* parentValidationData = (joint__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRISMATIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_REVOLUTE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 216:
        {
        kinematics_model__ValidationData* parentValidationData = (kinematics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 300:
        {
        kinematics_scene__ValidationData* parentValidationData = (kinematics_scene__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 465:
        {
        kinematics__ValidationData* parentValidationData = (kinematics__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 257:
        {
        library_animation_clips__ValidationData* parentValidationData = (library_animation_clips__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->animation_clip < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: animation_clip" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 239:
        {
        library_animations__ValidationData* parentValidationData = (library_animations__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->animation < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: animation" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 584:
        {
        library_articulated_systems__ValidationData* parentValidationData = (library_articulated_systems__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->articulated_system < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: articulated_system" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 565:
        {
        library_cameras__ValidationData* parentValidationData = (library_cameras__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->camera < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: camera" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 275:
        {
        library_controllers__ValidationData* parentValidationData = (library_controllers__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->controller < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: controller" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 206:
        {
        library_effects__ValidationData* parentValidationData = (library_effects__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->effect < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: effect" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 96:
        {
        library_force_fields__ValidationData* parentValidationData = (library_force_fields__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->force_field < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: force_field" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 101:
        {
        library_formulas__ValidationData* parentValidationData = (library_formulas__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->formula < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: formula" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 635:
        {
        library_geometries__ValidationData* parentValidationData = (library_geometries__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->geometry < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: geometry" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 369:
        {
        library_images__ValidationData* parentValidationData = (library_images__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->image < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: image" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 285:
        {
        library_joints__ValidationData* parentValidationData = (library_joints__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->joint < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: joint" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 227:
        {
        library_kinematics_models__ValidationData* parentValidationData = (library_kinematics_models__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->kinematics_model < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: kinematics_model" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 172:
        {
        library_kinematics_scenes__ValidationData* parentValidationData = (library_kinematics_scenes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->kinematics_scene < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: kinematics_scene" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 145:
        {
        library_lights__ValidationData* parentValidationData = (library_lights__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->light < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: light" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 45:
        {
        library_materials__ValidationData* parentValidationData = (library_materials__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->material < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: material" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 620:
        {
        library_nodes__ValidationData* parentValidationData = (library_nodes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->node < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: node" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 462:
        {
        library_physics_materials__ValidationData* parentValidationData = (library_physics_materials__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->physics_material < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: physics_material" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 7:
        {
        library_physics_models__ValidationData* parentValidationData = (library_physics_models__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->physics_model < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: physics_model" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 468:
        {
        library_physics_scenes__ValidationData* parentValidationData = (library_physics_scenes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->physics_scene < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: physics_scene" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 607:
        {
        library_visual_scenes__ValidationData* parentValidationData = (library_visual_scenes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->visual_scene < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: visual_scene" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 392:
        {
        light__ValidationData* parentValidationData = (light__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 223:
        {
        line__ValidationData* parentValidationData = (line__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->direction < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: direction" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 509:
        {
        lines__ValidationData* parentValidationData = (lines__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 611:
        {
        linestrips__ValidationData* parentValidationData = (linestrips__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 326:
        {
        material__ValidationData* parentValidationData = (material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_effect < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: instance_effect" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 704:
        {
        mesh__ValidationData* parentValidationData = (mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 304:
        {
        morph__ValidationData* parentValidationData = (morph__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->targets < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: targets" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 407:
        {
        motion__ValidationData* parentValidationData = (motion__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 36:
        {
        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CAMERA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CONTROLLER) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_LIGHT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_NODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_NODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 301:
        {
        nurbs_surface__ValidationData* parentValidationData = (nurbs_surface__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->control_vertices < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: control_vertices" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 596:
        {
        nurbs__ValidationData* parentValidationData = (nurbs__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->control_vertices < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: control_vertices" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 700:
        {
        parabola__ValidationData* parentValidationData = (parabola__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->focal < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: focal" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 258:
        {
        pcurves__ValidationData* parentValidationData = (pcurves__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 288:
        {
        physics_material__ValidationData* parentValidationData = (physics_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 241:
        {
        physics_model__ValidationData* parentValidationData = (physics_model__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 586:
        {
        physics_scene__ValidationData* parentValidationData = (physics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 350:
        {
        plane__ValidationData* parentValidationData = (plane__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->equation < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: equation" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 460:
        {
        polygons__ValidationData* parentValidationData = (polygons__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INPUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_PH) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 140:
        {
        polylist__ValidationData* parentValidationData = (polylist__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 42:
        {
        profile_BRIDGE__ValidationData* parentValidationData = (profile_BRIDGE__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 143:
        {
        profile_CG__ValidationData* parentValidationData = (profile_CG__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 480:
        {
        profile_COMMON__ValidationData* parentValidationData = (profile_COMMON__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 452:
        {
        profile_GLES2__ValidationData* parentValidationData = (profile_GLES2__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 196:
        {
        profile_GLES__ValidationData* parentValidationData = (profile_GLES__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 148:
        {
        profile_GLSL__ValidationData* parentValidationData = (profile_GLSL__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) || (parentValidationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 57:
        {
        rigid_body__ValidationData* parentValidationData = (rigid_body__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 647:
        {
        rigid_constraint__ValidationData* parentValidationData = (rigid_constraint__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 547:
        {
        shells__ValidationData* parentValidationData = (shells__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 483:
        {
        skin__ValidationData* parentValidationData = (skin__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vertex_weights < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: vertex_weights" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 601:
        {
        solids__ValidationData* parentValidationData = (solids__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 634:
        {
        sphere__ValidationData* parentValidationData = (sphere__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 65:
        {
        spline__ValidationData* parentValidationData = (spline__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->control_vertices < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: control_vertices" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 306:
        {
        surface_curves__ValidationData* parentValidationData = (surface_curves__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->curve < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: curve" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 318:
        {
        surfaces__ValidationData* parentValidationData = (surfaces__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->surface < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: surface" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 190:
        {
        swept_surface__ValidationData* parentValidationData = (swept_surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_AXIS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXTRA) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXTRA;
        }
//        parentValidationData->extra++;

        break;
        }

        case 15:
        {
        torus__ValidationData* parentValidationData = (torus__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 573:
        {
        triangles__ValidationData* parentValidationData = (triangles__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 729:
        {
        trifans__ValidationData* parentValidationData = (trifans__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 357:
        {
        tristrips__ValidationData* parentValidationData = (tristrips__ValidationData*)mValidationDataStack.top();
        parentValidationData->extra++;

        break;
        }

        case 508:
        {
        vertices__ValidationData* parentValidationData = (vertices__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 395:
        {
        visual_scene__ValidationData* parentValidationData = (visual_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->node < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: node" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        case 77:
        {
        wires__ValidationData* parentValidationData = (wires__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        parentValidationData->extra++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        extra__ValidationData* validationData = (extra__ValidationData*)mValidationDataStack.newObject(sizeof(extra__ValidationData));
        memset(validationData, 0, sizeof(extra__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__extra()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        extra__ValidationData* validationData = (extra__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_EXTRA, (const ParserChar*)0, "child: technique" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__technique____technique_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 702:
        {
        imager__ValidationData* parentValidationData = (imager__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 166:
        {
        optics__ValidationData* parentValidationData = (optics__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 54:
        {
        bind_material__ValidationData* parentValidationData = (bind_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 8:
        {
        extra__ValidationData* parentValidationData = (extra__ValidationData*)mValidationDataStack.top();
        parentValidationData->technique++;

        break;
        }

        case 48:
        {
        force_field__ValidationData* parentValidationData = (force_field__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 118:
        {
        formula__ValidationData* parentValidationData = (formula__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 533:
        {
        instance_rigid_body__ValidationData* parentValidationData = (instance_rigid_body__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 216:
        {
        kinematics_model__ValidationData* parentValidationData = (kinematics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 465:
        {
        kinematics__ValidationData* parentValidationData = (kinematics__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 392:
        {
        light__ValidationData* parentValidationData = (light__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 407:
        {
        motion__ValidationData* parentValidationData = (motion__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 288:
        {
        physics_material__ValidationData* parentValidationData = (physics_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 586:
        {
        physics_scene__ValidationData* parentValidationData = (physics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 57:
        {
        rigid_body__ValidationData* parentValidationData = (rigid_body__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 647:
        {
        rigid_constraint__ValidationData* parentValidationData = (rigid_constraint__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        break;
        }

        case 710:
        {
        source__ValidationData* parentValidationData = (source__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDREF_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_NAME_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIDREF_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TECHNIQUE_COMMON) || (parentValidationData->validation_current_state == HASH_ELEMENT_TOKEN_ARRAY) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TECHNIQUE;
        }
//        parentValidationData->technique++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        technique____technique_type__ValidationData* validationData = (technique____technique_type__ValidationData*)mValidationDataStack.newObject(sizeof(technique____technique_type__ValidationData));
        memset(validationData, 0, sizeof(technique____technique_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__technique____technique_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique____technique_type__ValidationData* validationData = (technique____technique_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_animations( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_ANIMATIONS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_ANIMATIONS;
        }
//        parentValidationData->library_animations++;

        library_animations__ValidationData* validationData = (library_animations__ValidationData*)mValidationDataStack.newObject(sizeof(library_animations__ValidationData));
        memset(validationData, 0, sizeof(library_animations__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_animations()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_animations__ValidationData* validationData = (library_animations__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->animation < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_ANIMATIONS, (const ParserChar*)0, "child: animation" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__animation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 12:
        {
        animation__ValidationData* parentValidationData = (animation__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANIMATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CHANNEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_SOURCE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ANIMATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ANIMATION;
        }
//        parentValidationData->animation++;

        break;
        }

        case 239:
        {
        library_animations__ValidationData* parentValidationData = (library_animations__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANIMATION, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->animation++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        animation__ValidationData* validationData = (animation__ValidationData*)mValidationDataStack.newObject(sizeof(animation__ValidationData));
        memset(validationData, 0, sizeof(animation__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__animation()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        animation__ValidationData* validationData = (animation__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ANIMATION) || (validationData->validation_current_state == HASH_ELEMENT_CHANNEL) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_ANIMATION, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__source( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 12:
        {
        animation__ValidationData* parentValidationData = (animation__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SOURCE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SOURCE;
        }
//        parentValidationData->source++;

        break;
        }

        case 68:
        {
        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vertices != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        if ( parentValidationData->edges != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: edges" ) )
                return false;
        }
        if ( parentValidationData->wires != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: wires" ) )
                return false;
        }
        if ( parentValidationData->faces != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: faces" ) )
                return false;
        }
        if ( parentValidationData->pcurves != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: pcurves" ) )
                return false;
        }
        if ( parentValidationData->shells != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: shells" ) )
                return false;
        }
        if ( parentValidationData->solids != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: solids" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->source++;

        break;
        }

        case 97:
        {
        convex_mesh__ValidationData* parentValidationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SOURCE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SOURCE;
        }
//        parentValidationData->source++;

        break;
        }

        case 704:
        {
        mesh__ValidationData* parentValidationData = (mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SOURCE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SOURCE;
        }
//        parentValidationData->source++;

        break;
        }

        case 304:
        {
        morph__ValidationData* parentValidationData = (morph__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->targets != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: targets" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->source++;

        break;
        }

        case 301:
        {
        nurbs_surface__ValidationData* parentValidationData = (nurbs_surface__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->control_vertices != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: control_vertices" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->source++;

        break;
        }

        case 596:
        {
        nurbs__ValidationData* parentValidationData = (nurbs__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->control_vertices != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: control_vertices" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->source++;

        break;
        }

        case 483:
        {
        skin__ValidationData* parentValidationData = (skin__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->joints != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: joints" ) )
                return false;
        }
        if ( parentValidationData->vertex_weights != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: vertex_weights" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->source++;

        break;
        }

        case 65:
        {
        spline__ValidationData* parentValidationData = (spline__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->control_vertices != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: control_vertices" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->source++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        source__ValidationData* validationData = (source__ValidationData*)mValidationDataStack.newObject(sizeof(source__ValidationData));
        memset(validationData, 0, sizeof(source__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__source()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source__ValidationData* validationData = (source__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ASSET) || (validationData->validation_current_state == HASH_ELEMENT_BOOL_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_IDREF_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_INT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_NAME_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_SIDREF_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) || (validationData->validation_current_state == HASH_ELEMENT_TECHNIQUE_COMMON) || (validationData->validation_current_state == HASH_ELEMENT_TOKEN_ARRAY) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SOURCE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__token_array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source__ValidationData* parentValidationData = (source__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TOKEN_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TOKEN_ARRAY;
        }
//        parentValidationData->token_array++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__token_array()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__IDREFS( const ParserString*value, size_t length )
{
    if( length < 1 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__IDREFS__stream( const ParserString*value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__IDREFS__streamEnd( const ParserString*value, size_t length )
{
    if( length < 1 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__IDREF_array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source__ValidationData* parentValidationData = (source__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDREF_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDREF_ARRAY;
        }
//        parentValidationData->IDREF_array++;

        IDREF_array__ValidationData* validationData = (IDREF_array__ValidationData*)mValidationDataStack.newObject(sizeof(IDREF_array__ValidationData));
        memset(validationData, 0, sizeof(IDREF_array__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__IDREF_array()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__Name( ParserString value )
{
    return validate__Name( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__Name( const ParserChar* value, size_t length )
{
// regular expression: "^([A-Z]|[_]|[a-z]|[:])([A-Z]|[_]|[a-z]|[:]|[-]|[.]|[0-9])*$"
static const COLLADABU::PcreCompiledPattern compiledPattern("^([A-Z]|[_]|[a-z]|[:])([A-Z]|[_]|[a-z]|[:]|[-]|[.]|[0-9])*$");
pcre* pattern = compiledPattern.getCompiledPattern();
int ovector[ PCRE_OVECCOUNT ];
int pcre_result = pcre_exec(
    pattern,
    0,
    value,
    (int)length,
    0,
    0,
    ovector,
    PCRE_OVECCOUNT);
if (pcre_result < 0)
    return ParserError::ERROR_VALIDATION_PATTERN;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__Name__stream( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__Name__streamEnd( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__Name_array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source__ValidationData* parentValidationData = (source__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NAME_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NAME_ARRAY;
        }
//        parentValidationData->Name_array++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__Name_array()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool_array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source__ValidationData* parentValidationData = (source__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL_ARRAY;
        }
//        parentValidationData->bool_array++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool_array()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__digits_type( const uint8 value )
{
    if( value > 17 )
        return ParserError::ERROR_VALIDATION_MAX_INCLUSIVE;

    if( value < 1 )
        return ParserError::ERROR_VALIDATION_MIN_INCLUSIVE;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__magnitude_type( const sint16 value )
{
    if( value > 308 )
        return ParserError::ERROR_VALIDATION_MAX_INCLUSIVE;

    if( value < -324 )
        return ParserError::ERROR_VALIDATION_MIN_INCLUSIVE;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float_array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source__ValidationData* parentValidationData = (source__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT_ARRAY;
        }
//        parentValidationData->float_array++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float_array()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int_array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source__ValidationData* parentValidationData = (source__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT_ARRAY;
        }
//        parentValidationData->int_array++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int_array()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sidref_type( ParserString value )
{
    return validate__sidref_type( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sidref_type( const ParserChar* value, size_t length )
{
// regular expression: "^((([A-Z]|[_]|[a-z])([A-Z]|[_]|[a-z]|[-]|[.]|[0-9])*)|.)(/([A-Z]|[_]|[a-z])([A-Z]|[_]|[a-z]|[-]|[.]|[0-9])*)+((\.([A-Z]|[_]|[a-z])([A-Z]|[_]|[a-z]|[-]|[.]|[0-9])*)|(\([0-9]+\)){1,2})?$"
static const COLLADABU::PcreCompiledPattern compiledPattern("^((([A-Z]|[_]|[a-z])([A-Z]|[_]|[a-z]|[-]|[.]|[0-9])*)|.)(/([A-Z]|[_]|[a-z])([A-Z]|[_]|[a-z]|[-]|[.]|[0-9])*)+((\\.([A-Z]|[_]|[a-z])([A-Z]|[_]|[a-z]|[-]|[.]|[0-9])*)|(\\([0-9]+\\)){1,2})?$");pcre* pattern = compiledPattern.getCompiledPattern();
int ovector[ PCRE_OVECCOUNT ];
int pcre_result = pcre_exec(
    pattern,
    0,
    value,
    (int)length,
    0,
    0,
    ovector,
    PCRE_OVECCOUNT);
if (pcre_result < 0)
    return ParserError::ERROR_VALIDATION_PATTERN;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sidref_type__stream( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sidref_type__streamEnd( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__SIDREF_array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source__ValidationData* parentValidationData = (source__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIDREF_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIDREF_ARRAY;
        }
//        parentValidationData->SIDREF_array++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__SIDREF_array()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__source_type____technique_common( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source__ValidationData* parentValidationData = (source__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDREF_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_NAME_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIDREF_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_TOKEN_ARRAY) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TECHNIQUE_COMMON;
        }
//        parentValidationData->technique_common++;

        source_type____technique_common__ValidationData* validationData = (source_type____technique_common__ValidationData*)mValidationDataStack.newObject(sizeof(source_type____technique_common__ValidationData));
        memset(validationData, 0, sizeof(source_type____technique_common__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__source_type____technique_common()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source_type____technique_common__ValidationData* validationData = (source_type____technique_common__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->accessor < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: accessor" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__accessor( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        source_type____technique_common__ValidationData* parentValidationData = (source_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->accessor >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ACCESSOR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->accessor++;

        accessor__ValidationData* validationData = (accessor__ValidationData*)mValidationDataStack.newObject(sizeof(accessor__ValidationData));
        memset(validationData, 0, sizeof(accessor__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__accessor()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        accessor__ValidationData* validationData = (accessor__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sid_type( ParserString value )
{
    return validate__sid_type( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sid_type( const ParserChar* value, size_t length )
{
// regular expression: "^([A-Z]|[_]|[a-z])([A-Z]|[_]|[a-z]|[-]|[.]|[0-9])*$"
static const COLLADABU::PcreCompiledPattern compiledPattern("^([A-Z]|[_]|[a-z])([A-Z]|[_]|[a-z]|[-]|[.]|[0-9])*$");
pcre* pattern = compiledPattern.getCompiledPattern();
int ovector[ PCRE_OVECCOUNT ];
int pcre_result = pcre_exec(
    pattern,
    0,
    value,
    (int)length,
    0,
    0,
    ovector,
    PCRE_OVECCOUNT);
if (pcre_result < 0)
    return ParserError::ERROR_VALIDATION_PATTERN;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sid_type__stream( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sid_type__streamEnd( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__param____param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 603:
        {
        accessor__ValidationData* parentValidationData = (accessor__ValidationData*)mValidationDataStack.top();
        parentValidationData->param++;

        break;
        }

        case 54:
        {
        bind_material__ValidationData* parentValidationData = (bind_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__param____param_type()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sampler_behavior_enum( ParserString value )
{
    return validate__sampler_behavior_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sampler_behavior_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sampler_behavior_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__sampler_behavior_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sampler( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        animation__ValidationData* parentValidationData = (animation__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SOURCE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER;
        }
//        parentValidationData->sampler++;

        sampler__ValidationData* validationData = (sampler__ValidationData*)mValidationDataStack.newObject(sizeof(sampler__ValidationData));
        memset(validationData, 0, sizeof(sampler__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sampler()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler__ValidationData* validationData = (sampler__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SAMPLER, (const ParserChar*)0, "child: input" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__urifragment_type( ParserString value )
{
    return validate__urifragment_type( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__urifragment_type( const ParserChar* value, size_t length )
{
// regular expression: "^(#(.*))$"
static const COLLADABU::PcreCompiledPattern compiledPattern("^(#(.*))$");
pcre* pattern = compiledPattern.getCompiledPattern();
int ovector[ PCRE_OVECCOUNT ];
int pcre_result = pcre_exec(
    pattern,
    0,
    value,
    (int)length,
    0,
    0,
    ovector,
    PCRE_OVECCOUNT);
if (pcre_result < 0)
    return ParserError::ERROR_VALIDATION_PATTERN;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__urifragment_type__stream( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__urifragment_type__streamEnd( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__input____input_local_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 207:
        {
        nurbs_surface_type____control_vertices__ValidationData* parentValidationData = (nurbs_surface_type____control_vertices__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 208:
        {
        nurbs_type____control_vertices__ValidationData* parentValidationData = (nurbs_type____control_vertices__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 209:
        {
        spline_type____control_vertices__ValidationData* parentValidationData = (spline_type____control_vertices__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 283:
        {
        joints__ValidationData* parentValidationData = (joints__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 95:
        {
        targets__ValidationData* parentValidationData = (targets__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 557:
        {
        sampler__ValidationData* parentValidationData = (sampler__ValidationData*)mValidationDataStack.top();
        parentValidationData->input++;

        break;
        }

        case 508:
        {
        vertices__ValidationData* parentValidationData = (vertices__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__input____input_local_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__channel( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        animation__ValidationData* parentValidationData = (animation__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_CHANNEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CHANNEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CHANNEL;
        }
//        parentValidationData->channel++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__channel()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_animation_clips( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS;
        }
//        parentValidationData->library_animation_clips++;

        library_animation_clips__ValidationData* validationData = (library_animation_clips__ValidationData*)mValidationDataStack.newObject(sizeof(library_animation_clips__ValidationData));
        memset(validationData, 0, sizeof(library_animation_clips__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_animation_clips()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_animation_clips__ValidationData* validationData = (library_animation_clips__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->animation_clip < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS, (const ParserChar*)0, "child: animation_clip" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__animation_clip( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_animation_clips__ValidationData* parentValidationData = (library_animation_clips__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANIMATION_CLIP, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->animation_clip++;

        animation_clip__ValidationData* validationData = (animation_clip__ValidationData*)mValidationDataStack.newObject(sizeof(animation_clip__ValidationData));
        memset(validationData, 0, sizeof(animation_clip__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__animation_clip()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        animation_clip__ValidationData* validationData = (animation_clip__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->instance_animation < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ANIMATION_CLIP, (const ParserChar*)0, "child: instance_animation" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_animation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        animation_clip__ValidationData* parentValidationData = (animation_clip__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_formula != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_ANIMATION, (const ParserChar*)0, "sibling: instance_formula" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_ANIMATION, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_animation++;

        instance_animation__ValidationData* validationData = (instance_animation__ValidationData*)mValidationDataStack.newObject(sizeof(instance_animation__ValidationData));
        memset(validationData, 0, sizeof(instance_animation__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_animation()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_animation__ValidationData* validationData = (instance_animation__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_formula( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 320:
        {
        animation_clip__ValidationData* parentValidationData = (animation_clip__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_animation < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_INSTANCE_FORMULA, (const ParserChar*)0, "sibling: instance_animation" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_FORMULA, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_formula++;

        break;
        }

        case 726:
        {
        axis_info____kinematics_axis_info_type__ValidationData* parentValidationData = (axis_info____kinematics_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ACTIVE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORMULA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INDEX) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_FORMULA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMITS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOCKED) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_FORMULA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_FORMULA;
        }
//        parentValidationData->instance_formula++;

        break;
        }

        case 685:
        {
        technique_common____kinematics_model_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_model_technique_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_FORMULA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_FORMULA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINK) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_FORMULA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_FORMULA;
        }
//        parentValidationData->instance_formula++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        instance_formula__ValidationData* validationData = (instance_formula__ValidationData*)mValidationDataStack.newObject(sizeof(instance_formula__ValidationData));
        memset(validationData, 0, sizeof(instance_formula__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_formula()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_formula__ValidationData* validationData = (instance_formula__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__setparam____formula_setparam_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_formula__ValidationData* parentValidationData = (instance_formula__ValidationData*)mValidationDataStack.top();
        parentValidationData->setparam++;

        setparam____formula_setparam_type__ValidationData* validationData = (setparam____formula_setparam_type__ValidationData*)mValidationDataStack.newObject(sizeof(setparam____formula_setparam_type__ValidationData));
        memset(validationData, 0, sizeof(setparam____formula_setparam_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__setparam____formula_setparam_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        setparam____formula_setparam_type__ValidationData* validationData = (setparam____formula_setparam_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->_int != 0 )
            if ( validationData->_int < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "child: int" ) )
                    return false;
        if ( validationData->SIDREF != 0 )
            if ( validationData->SIDREF < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "child: SIDREF" ) )
                    return false;
        if ( validationData->_bool != 0 )
            if ( validationData->_bool < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "child: bool" ) )
                    return false;
        if ( validationData->connect_param != 0 )
            if ( validationData->connect_param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "child: connect_param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->_int == 0 && validationData->SIDREF == 0 && validationData->_bool == 0 && validationData->connect_param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float____float_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 27:
        {
        target__ValidationData* parentValidationData = (target__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->_float >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_float++;

        break;
        }

        case 656:
        {
        newparam____formula_newparam_type__ValidationData* parentValidationData = (newparam____formula_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->_float >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_float++;

        break;
        }

        case 742:
        {
        setparam____formula_setparam_type__ValidationData* parentValidationData = (setparam____formula_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->connect_param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: connect_param" ) )
                return false;
        }
        if ( parentValidationData->_float >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_float++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 657:
        {
        newparam____fx_common_newparam_type__ValidationData* parentValidationData = (newparam____fx_common_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT;
        }
//        parentValidationData->_float++;

        break;
        }

        case 341:
        {
        bind____kinematics_bind_type__ValidationData* parentValidationData = (bind____kinematics_bind_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_float >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_float++;

        break;
        }

        case 661:
        {
        newparam____kinematics_newparam_type__ValidationData* parentValidationData = (newparam____kinematics_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->_float >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_float++;

        break;
        }

        case 743:
        {
        setparam____kinematics_setparam_type__ValidationData* parentValidationData = (setparam____kinematics_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->connect_param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: connect_param" ) )
                return false;
        }
        if ( parentValidationData->_float >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_float++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float____float_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int____int_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 656:
        {
        newparam____formula_newparam_type__ValidationData* parentValidationData = (newparam____formula_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->_int >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_int++;

        break;
        }

        case 742:
        {
        setparam____formula_setparam_type__ValidationData* parentValidationData = (setparam____formula_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->connect_param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: connect_param" ) )
                return false;
        }
        if ( parentValidationData->_int >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_int++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 341:
        {
        bind____kinematics_bind_type__ValidationData* parentValidationData = (bind____kinematics_bind_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_int >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_int++;

        break;
        }

        case 594:
        {
        index__ValidationData* parentValidationData = (index__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->_int >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_int++;

        break;
        }

        case 661:
        {
        newparam____kinematics_newparam_type__ValidationData* parentValidationData = (newparam____kinematics_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->_int >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_int++;

        break;
        }

        case 743:
        {
        setparam____kinematics_setparam_type__ValidationData* parentValidationData = (setparam____kinematics_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->connect_param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INT, (const ParserChar*)0, "sibling: connect_param" ) )
                return false;
        }
        if ( parentValidationData->_int >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_int++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int____int_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__SIDREF( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 282:
        {
        bind_kinematics_model__ValidationData* parentValidationData = (bind_kinematics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->SIDREF >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SIDREF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->SIDREF++;

        break;
        }

        case 449:
        {
        axis____common_sidref_or_param_type__ValidationData* parentValidationData = (axis____common_sidref_or_param_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->SIDREF >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SIDREF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->SIDREF++;

        break;
        }

        case 656:
        {
        newparam____formula_newparam_type__ValidationData* parentValidationData = (newparam____formula_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->SIDREF >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SIDREF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->SIDREF++;

        break;
        }

        case 742:
        {
        setparam____formula_setparam_type__ValidationData* parentValidationData = (setparam____formula_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->connect_param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: connect_param" ) )
                return false;
        }
        if ( parentValidationData->SIDREF >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SIDREF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->SIDREF++;

        break;
        }

        case 341:
        {
        bind____kinematics_bind_type__ValidationData* parentValidationData = (bind____kinematics_bind_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SIDREF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->SIDREF++;

        break;
        }

        case 661:
        {
        newparam____kinematics_newparam_type__ValidationData* parentValidationData = (newparam____kinematics_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->SIDREF >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SIDREF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->SIDREF++;

        break;
        }

        case 743:
        {
        setparam____kinematics_setparam_type__ValidationData* parentValidationData = (setparam____kinematics_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->connect_param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIDREF, (const ParserChar*)0, "sibling: connect_param" ) )
                return false;
        }
        if ( parentValidationData->SIDREF >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SIDREF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->SIDREF++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__SIDREF()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 229:
        {
        active__ValidationData* parentValidationData = (active__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->_bool >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_bool++;

        break;
        }

        case 656:
        {
        newparam____formula_newparam_type__ValidationData* parentValidationData = (newparam____formula_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_bool++;

        break;
        }

        case 742:
        {
        setparam____formula_setparam_type__ValidationData* parentValidationData = (setparam____formula_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->connect_param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: connect_param" ) )
                return false;
        }
        if ( parentValidationData->_bool >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_bool++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL;
        }
//        parentValidationData->_bool++;

        break;
        }

        case 341:
        {
        bind____kinematics_bind_type__ValidationData* parentValidationData = (bind____kinematics_bind_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_bool++;

        break;
        }

        case 661:
        {
        newparam____kinematics_newparam_type__ValidationData* parentValidationData = (newparam____kinematics_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_bool++;

        break;
        }

        case 743:
        {
        setparam____kinematics_setparam_type__ValidationData* parentValidationData = (setparam____kinematics_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->connect_param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BOOL, (const ParserChar*)0, "sibling: connect_param" ) )
                return false;
        }
        if ( parentValidationData->_bool >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BOOL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_bool++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__connect_param( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 742:
        {
        setparam____formula_setparam_type__ValidationData* parentValidationData = (setparam____formula_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONNECT_PARAM, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONNECT_PARAM, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONNECT_PARAM, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONNECT_PARAM, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->connect_param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CONNECT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->connect_param++;

        break;
        }

        case 743:
        {
        setparam____kinematics_setparam_type__ValidationData* parentValidationData = (setparam____kinematics_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONNECT_PARAM, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONNECT_PARAM, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONNECT_PARAM, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONNECT_PARAM, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->connect_param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CONNECT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->connect_param++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__connect_param()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_cameras( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_CAMERAS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_CAMERAS;
        }
//        parentValidationData->library_cameras++;

        library_cameras__ValidationData* validationData = (library_cameras__ValidationData*)mValidationDataStack.newObject(sizeof(library_cameras__ValidationData));
        memset(validationData, 0, sizeof(library_cameras__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_cameras()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_cameras__ValidationData* validationData = (library_cameras__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->camera < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_CAMERAS, (const ParserChar*)0, "child: camera" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__camera( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_cameras__ValidationData* parentValidationData = (library_cameras__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CAMERA, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->camera++;

        camera__ValidationData* validationData = (camera__ValidationData*)mValidationDataStack.newObject(sizeof(camera__ValidationData));
        memset(validationData, 0, sizeof(camera__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__camera()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        camera__ValidationData* validationData = (camera__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->optics < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CAMERA, (const ParserChar*)0, "child: optics" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__optics( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        camera__ValidationData* parentValidationData = (camera__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->imager != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_OPTICS, (const ParserChar*)0, "sibling: imager" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_OPTICS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->optics >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_OPTICS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->optics++;

        optics__ValidationData* validationData = (optics__ValidationData*)mValidationDataStack.newObject(sizeof(optics__ValidationData));
        memset(validationData, 0, sizeof(optics__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__optics()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        optics__ValidationData* validationData = (optics__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_OPTICS, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__optics__technique_common( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        optics__ValidationData* parentValidationData = (optics__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        optics__technique_common__ValidationData* validationData = (optics__technique_common__ValidationData*)mValidationDataStack.newObject(sizeof(optics__technique_common__ValidationData));
        memset(validationData, 0, sizeof(optics__technique_common__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__optics__technique_common()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        optics__technique_common__ValidationData* validationData = (optics__technique_common__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->orthographic != 0 )
            if ( validationData->orthographic < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: orthographic" ) )
                    return false;
        if ( validationData->perspective != 0 )
            if ( validationData->perspective < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: perspective" ) )
                    return false;
        if ( validationData->orthographic == 0 && validationData->perspective == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__orthographic( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        optics__technique_common__ValidationData* parentValidationData = (optics__technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->perspective != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ORTHOGRAPHIC, (const ParserChar*)0, "sibling: perspective" ) )
                return false;
        }
        if ( parentValidationData->orthographic >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ORTHOGRAPHIC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->orthographic++;

        orthographic__ValidationData* validationData = (orthographic__ValidationData*)mValidationDataStack.newObject(sizeof(orthographic__ValidationData));
        memset(validationData, 0, sizeof(orthographic__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__orthographic()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        orthographic__ValidationData* validationData = (orthographic__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ZFAR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_ORTHOGRAPHIC, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__xmag( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        orthographic__ValidationData* parentValidationData = (orthographic__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XMAG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XMAG;
        }
//        parentValidationData->xmag++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__xmag()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ymag( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        orthographic__ValidationData* parentValidationData = (orthographic__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_XMAG) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_YMAG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_YMAG;
        }
//        parentValidationData->ymag++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ymag()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__aspect_ratio( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 397:
        {
        orthographic__ValidationData* parentValidationData = (orthographic__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_XMAG) || (parentValidationData->validation_current_state == HASH_ELEMENT_YMAG) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASPECT_RATIO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASPECT_RATIO;
        }
//        parentValidationData->aspect_ratio++;

        break;
        }

        case 213:
        {
        perspective__ValidationData* parentValidationData = (perspective__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_XFOV) || (parentValidationData->validation_current_state == HASH_ELEMENT_YFOV) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ASPECT_RATIO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ASPECT_RATIO;
        }
//        parentValidationData->aspect_ratio++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__aspect_ratio()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__znear( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 397:
        {
        orthographic__ValidationData* parentValidationData = (orthographic__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASPECT_RATIO) || (parentValidationData->validation_current_state == HASH_ELEMENT_XMAG) || (parentValidationData->validation_current_state == HASH_ELEMENT_YMAG) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ZNEAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ZNEAR;
        }
//        parentValidationData->znear++;

        break;
        }

        case 213:
        {
        perspective__ValidationData* parentValidationData = (perspective__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASPECT_RATIO) || (parentValidationData->validation_current_state == HASH_ELEMENT_XFOV) || (parentValidationData->validation_current_state == HASH_ELEMENT_YFOV) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ZNEAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ZNEAR;
        }
//        parentValidationData->znear++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__znear()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__zfar( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 397:
        {
        orthographic__ValidationData* parentValidationData = (orthographic__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ZNEAR) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ZFAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ZFAR;
        }
//        parentValidationData->zfar++;

        break;
        }

        case 213:
        {
        perspective__ValidationData* parentValidationData = (perspective__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ZNEAR) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ZFAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ZFAR;
        }
//        parentValidationData->zfar++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__zfar()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__perspective( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        optics__technique_common__ValidationData* parentValidationData = (optics__technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->orthographic != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PERSPECTIVE, (const ParserChar*)0, "sibling: orthographic" ) )
                return false;
        }
        if ( parentValidationData->perspective >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PERSPECTIVE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->perspective++;

        perspective__ValidationData* validationData = (perspective__ValidationData*)mValidationDataStack.newObject(sizeof(perspective__ValidationData));
        memset(validationData, 0, sizeof(perspective__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__perspective()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        perspective__ValidationData* validationData = (perspective__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ZFAR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_PERSPECTIVE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__xfov( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        perspective__ValidationData* parentValidationData = (perspective__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XFOV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XFOV;
        }
//        parentValidationData->xfov++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__xfov()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__yfov( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        perspective__ValidationData* parentValidationData = (perspective__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_XFOV) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_YFOV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_YFOV;
        }
//        parentValidationData->yfov++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__yfov()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__imager( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        camera__ValidationData* parentValidationData = (camera__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->optics < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_IMAGER, (const ParserChar*)0, "sibling: optics" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_IMAGER, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->imager >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_IMAGER, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->imager++;

        imager__ValidationData* validationData = (imager__ValidationData*)mValidationDataStack.newObject(sizeof(imager__ValidationData));
        memset(validationData, 0, sizeof(imager__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__imager()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        imager__ValidationData* validationData = (imager__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_IMAGER, (const ParserChar*)0, "child: technique" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_controllers( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_CONTROLLERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_CONTROLLERS;
        }
//        parentValidationData->library_controllers++;

        library_controllers__ValidationData* validationData = (library_controllers__ValidationData*)mValidationDataStack.newObject(sizeof(library_controllers__ValidationData));
        memset(validationData, 0, sizeof(library_controllers__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_controllers()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_controllers__ValidationData* validationData = (library_controllers__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->controller < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_CONTROLLERS, (const ParserChar*)0, "child: controller" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__controller( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_controllers__ValidationData* parentValidationData = (library_controllers__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTROLLER, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->controller++;

        controller__ValidationData* validationData = (controller__ValidationData*)mValidationDataStack.newObject(sizeof(controller__ValidationData));
        memset(validationData, 0, sizeof(controller__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__controller()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        controller__ValidationData* validationData = (controller__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_MORPH) || (validationData->validation_current_state == HASH_ELEMENT_SKIN) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_CONTROLLER, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__skin( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        controller__ValidationData* parentValidationData = (controller__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SKIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SKIN;
        }
//        parentValidationData->skin++;

        skin__ValidationData* validationData = (skin__ValidationData*)mValidationDataStack.newObject(sizeof(skin__ValidationData));
        memset(validationData, 0, sizeof(skin__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__skin()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        skin__ValidationData* validationData = (skin__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->source < 3 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SKIN, (const ParserChar*)0, "child: source" ) )
                return false;
        }
        if ( validationData->joints < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SKIN, (const ParserChar*)0, "child: joints" ) )
                return false;
        }
        if ( validationData->vertex_weights < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SKIN, (const ParserChar*)0, "child: vertex_weights" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4x4_type( const float* value, size_t length )
{
    if( length < 16 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 16 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4x4_type__stream( const float* value, size_t length )
{

    if( length > 16 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4x4_type__streamEnd( const float* value, size_t length )
{
    if( length < 16 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 16 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bind_shape_matrix( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        skin__ValidationData* parentValidationData = (skin__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->source != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_SHAPE_MATRIX, (const ParserChar*)0, "sibling: source" ) )
                return false;
        }
        if ( parentValidationData->joints != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_SHAPE_MATRIX, (const ParserChar*)0, "sibling: joints" ) )
                return false;
        }
        if ( parentValidationData->vertex_weights != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_SHAPE_MATRIX, (const ParserChar*)0, "sibling: vertex_weights" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_SHAPE_MATRIX, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->bind_shape_matrix >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BIND_SHAPE_MATRIX, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->bind_shape_matrix++;

        bind_shape_matrix__ValidationData* validationData = (bind_shape_matrix__ValidationData*)mValidationDataStack.newObject(sizeof(bind_shape_matrix__ValidationData));
        memset(validationData, 0, sizeof(bind_shape_matrix__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bind_shape_matrix()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__joints( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        skin__ValidationData* parentValidationData = (skin__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->source < 3 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_JOINTS, (const ParserChar*)0, "sibling: source" ) )
                return false;
        }
        if ( parentValidationData->vertex_weights != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_JOINTS, (const ParserChar*)0, "sibling: vertex_weights" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_JOINTS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->joints >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_JOINTS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->joints++;

        joints__ValidationData* validationData = (joints__ValidationData*)mValidationDataStack.newObject(sizeof(joints__ValidationData));
        memset(validationData, 0, sizeof(joints__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__joints()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        joints__ValidationData* validationData = (joints__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 2 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_JOINTS, (const ParserChar*)0, "child: input" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__vertex_weights( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        skin__ValidationData* parentValidationData = (skin__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->joints < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VERTEX_WEIGHTS, (const ParserChar*)0, "sibling: joints" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VERTEX_WEIGHTS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->vertex_weights >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VERTEX_WEIGHTS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->vertex_weights++;

        vertex_weights__ValidationData* validationData = (vertex_weights__ValidationData*)mValidationDataStack.newObject(sizeof(vertex_weights__ValidationData));
        memset(validationData, 0, sizeof(vertex_weights__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__vertex_weights()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        vertex_weights__ValidationData* validationData = (vertex_weights__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 2 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_VERTEX_WEIGHTS, (const ParserChar*)0, "child: input" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__input____input_local_offset_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 222:
        {
        vertex_weights__ValidationData* parentValidationData = (vertex_weights__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->v != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: v" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 6:
        {
        edges__ValidationData* parentValidationData = (edges__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 157:
        {
        faces__ValidationData* parentValidationData = (faces__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 509:
        {
        lines__ValidationData* parentValidationData = (lines__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 611:
        {
        linestrips__ValidationData* parentValidationData = (linestrips__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 258:
        {
        pcurves__ValidationData* parentValidationData = (pcurves__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 460:
        {
        polygons__ValidationData* parentValidationData = (polygons__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INPUT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INPUT;
        }
//        parentValidationData->input++;

        break;
        }

        case 140:
        {
        polylist__ValidationData* parentValidationData = (polylist__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 547:
        {
        shells__ValidationData* parentValidationData = (shells__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 601:
        {
        solids__ValidationData* parentValidationData = (solids__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 573:
        {
        triangles__ValidationData* parentValidationData = (triangles__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 729:
        {
        trifans__ValidationData* parentValidationData = (trifans__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 357:
        {
        tristrips__ValidationData* parentValidationData = (tristrips__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        case 77:
        {
        wires__ValidationData* parentValidationData = (wires__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->input++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__input____input_local_offset_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__vcount( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 222:
        {
        vertex_weights__ValidationData* parentValidationData = (vertex_weights__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 2 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        if ( parentValidationData->v != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: v" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->vcount >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->vcount++;

        break;
        }

        case 157:
        {
        faces__ValidationData* parentValidationData = (faces__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 3 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->vcount >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->vcount++;

        break;
        }

        case 258:
        {
        pcurves__ValidationData* parentValidationData = (pcurves__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 3 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->vcount >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->vcount++;

        break;
        }

        case 140:
        {
        polylist__ValidationData* parentValidationData = (polylist__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->vcount >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->vcount++;

        break;
        }

        case 547:
        {
        shells__ValidationData* parentValidationData = (shells__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 2 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->vcount >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->vcount++;

        break;
        }

        case 601:
        {
        solids__ValidationData* parentValidationData = (solids__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 2 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->vcount >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->vcount++;

        break;
        }

        case 77:
        {
        wires__ValidationData* parentValidationData = (wires__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 2 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        if ( parentValidationData->p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->vcount >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VCOUNT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->vcount++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__vcount()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__v( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        vertex_weights__ValidationData* parentValidationData = (vertex_weights__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 2 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_V, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_V, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->v >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_V, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->v++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__v()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__morph_method_enum( ParserString value )
{
    return validate__morph_method_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__morph_method_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__morph_method_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__morph_method_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__morph( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        controller__ValidationData* parentValidationData = (controller__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MORPH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MORPH;
        }
//        parentValidationData->morph++;

        morph__ValidationData* validationData = (morph__ValidationData*)mValidationDataStack.newObject(sizeof(morph__ValidationData));
        memset(validationData, 0, sizeof(morph__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__morph()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        morph__ValidationData* validationData = (morph__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->source < 2 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MORPH, (const ParserChar*)0, "child: source" ) )
                return false;
        }
        if ( validationData->targets < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MORPH, (const ParserChar*)0, "child: targets" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__targets( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        morph__ValidationData* parentValidationData = (morph__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->source < 2 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TARGETS, (const ParserChar*)0, "sibling: source" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TARGETS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->targets >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TARGETS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->targets++;

        targets__ValidationData* validationData = (targets__ValidationData*)mValidationDataStack.newObject(sizeof(targets__ValidationData));
        memset(validationData, 0, sizeof(targets__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__targets()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        targets__ValidationData* validationData = (targets__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 2 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TARGETS, (const ParserChar*)0, "child: input" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_geometries( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_GEOMETRIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_GEOMETRIES;
        }
//        parentValidationData->library_geometries++;

        library_geometries__ValidationData* validationData = (library_geometries__ValidationData*)mValidationDataStack.newObject(sizeof(library_geometries__ValidationData));
        memset(validationData, 0, sizeof(library_geometries__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_geometries()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_geometries__ValidationData* validationData = (library_geometries__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->geometry < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_GEOMETRIES, (const ParserChar*)0, "child: geometry" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__geometry( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_geometries__ValidationData* parentValidationData = (library_geometries__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_GEOMETRY, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->geometry++;

        geometry__ValidationData* validationData = (geometry__ValidationData*)mValidationDataStack.newObject(sizeof(geometry__ValidationData));
        memset(validationData, 0, sizeof(geometry__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__geometry()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        geometry__ValidationData* validationData = (geometry__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BREP) || (validationData->validation_current_state == HASH_ELEMENT_CONVEX_MESH) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_MESH) || (validationData->validation_current_state == HASH_ELEMENT_SPLINE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_GEOMETRY, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__convex_mesh( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        geometry__ValidationData* parentValidationData = (geometry__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONVEX_MESH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONVEX_MESH;
        }
//        parentValidationData->convex_mesh++;

        convex_mesh__ValidationData* validationData = (convex_mesh__ValidationData*)mValidationDataStack.newObject(sizeof(convex_mesh__ValidationData));
        memset(validationData, 0, sizeof(convex_mesh__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__convex_mesh()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        convex_mesh__ValidationData* validationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_LINES) || (validationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (validationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (validationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (validationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (validationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (validationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (validationData->validation_current_state == HASH_ELEMENT_VERTICES) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_CONVEX_MESH, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__vertices( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 68:
        {
        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->source < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VERTICES, (const ParserChar*)0, "sibling: source" ) )
                return false;
        }
        if ( parentValidationData->edges != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VERTICES, (const ParserChar*)0, "sibling: edges" ) )
                return false;
        }
        if ( parentValidationData->wires != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VERTICES, (const ParserChar*)0, "sibling: wires" ) )
                return false;
        }
        if ( parentValidationData->faces != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VERTICES, (const ParserChar*)0, "sibling: faces" ) )
                return false;
        }
        if ( parentValidationData->pcurves != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VERTICES, (const ParserChar*)0, "sibling: pcurves" ) )
                return false;
        }
        if ( parentValidationData->shells != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VERTICES, (const ParserChar*)0, "sibling: shells" ) )
                return false;
        }
        if ( parentValidationData->solids != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VERTICES, (const ParserChar*)0, "sibling: solids" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VERTICES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->vertices >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VERTICES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->vertices++;

        break;
        }

        case 97:
        {
        convex_mesh__ValidationData* parentValidationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SOURCE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VERTICES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VERTICES;
        }
//        parentValidationData->vertices++;

        break;
        }

        case 704:
        {
        mesh__ValidationData* parentValidationData = (mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SOURCE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VERTICES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VERTICES;
        }
//        parentValidationData->vertices++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        vertices__ValidationData* validationData = (vertices__ValidationData*)mValidationDataStack.newObject(sizeof(vertices__ValidationData));
        memset(validationData, 0, sizeof(vertices__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__vertices()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        vertices__ValidationData* validationData = (vertices__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_VERTICES, (const ParserChar*)0, "child: input" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NCName( ParserString value )
{
    return validate__NCName( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NCName( const ParserChar* value, size_t length )
{
// regular expression: "^([A-Z]|[_]|[a-z])([A-Z]|[_]|[a-z]|[-]|[.]|[0-9])*$"
static const COLLADABU::PcreCompiledPattern compiledPattern("^([A-Z]|[_]|[a-z])([A-Z]|[_]|[a-z]|[-]|[.]|[0-9])*$");
pcre* pattern = compiledPattern.getCompiledPattern();
int ovector[ PCRE_OVECCOUNT ];
int pcre_result = pcre_exec(
    pattern,
    0,
    value,
    (int)length,
    0,
    0,
    ovector,
    PCRE_OVECCOUNT);
if (pcre_result < 0)
    return ParserError::ERROR_VALIDATION_PATTERN;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NCName__stream( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NCName__streamEnd( const ParserChar* value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__lines( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 97:
        {
        convex_mesh__ValidationData* parentValidationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINES;
        }
//        parentValidationData->lines++;

        break;
        }

        case 704:
        {
        mesh__ValidationData* parentValidationData = (mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINES;
        }
//        parentValidationData->lines++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        lines__ValidationData* validationData = (lines__ValidationData*)mValidationDataStack.newObject(sizeof(lines__ValidationData));
        memset(validationData, 0, sizeof(lines__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__lines()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        lines__ValidationData* validationData = (lines__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__p( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 490:
        {
        ph__ValidationData* parentValidationData = (ph__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->h != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: h" ) )
                return false;
        }
        if ( parentValidationData->p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 6:
        {
        edges__ValidationData* parentValidationData = (edges__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->input < 4 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: input" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 157:
        {
        faces__ValidationData* parentValidationData = (faces__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 509:
        {
        lines__ValidationData* parentValidationData = (lines__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 611:
        {
        linestrips__ValidationData* parentValidationData = (linestrips__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 258:
        {
        pcurves__ValidationData* parentValidationData = (pcurves__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 460:
        {
        polygons__ValidationData* parentValidationData = (polygons__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INPUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_PH) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_P, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_P;
        }
//        parentValidationData->p++;

        break;
        }

        case 140:
        {
        polylist__ValidationData* parentValidationData = (polylist__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 547:
        {
        shells__ValidationData* parentValidationData = (shells__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 601:
        {
        solids__ValidationData* parentValidationData = (solids__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 573:
        {
        triangles__ValidationData* parentValidationData = (triangles__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 729:
        {
        trifans__ValidationData* parentValidationData = (trifans__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 357:
        {
        tristrips__ValidationData* parentValidationData = (tristrips__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        case 77:
        {
        wires__ValidationData* parentValidationData = (wires__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vcount < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: vcount" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->p++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__p()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__linestrips( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 97:
        {
        convex_mesh__ValidationData* parentValidationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINESTRIPS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINESTRIPS;
        }
//        parentValidationData->linestrips++;

        break;
        }

        case 704:
        {
        mesh__ValidationData* parentValidationData = (mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINESTRIPS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINESTRIPS;
        }
//        parentValidationData->linestrips++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        linestrips__ValidationData* validationData = (linestrips__ValidationData*)mValidationDataStack.newObject(sizeof(linestrips__ValidationData));
        memset(validationData, 0, sizeof(linestrips__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__linestrips()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        linestrips__ValidationData* validationData = (linestrips__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__polygons( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 97:
        {
        convex_mesh__ValidationData* parentValidationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGONS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGONS;
        }
//        parentValidationData->polygons++;

        break;
        }

        case 704:
        {
        mesh__ValidationData* parentValidationData = (mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGONS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGONS;
        }
//        parentValidationData->polygons++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        polygons__ValidationData* validationData = (polygons__ValidationData*)mValidationDataStack.newObject(sizeof(polygons__ValidationData));
        memset(validationData, 0, sizeof(polygons__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__polygons()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        polygons__ValidationData* validationData = (polygons__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INPUT) || (validationData->validation_current_state == HASH_ELEMENT_P) || (validationData->validation_current_state == HASH_ELEMENT_PH) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_POLYGONS, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ph( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        polygons__ValidationData* parentValidationData = (polygons__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INPUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_PH) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PH;
        }
//        parentValidationData->ph++;

        ph__ValidationData* validationData = (ph__ValidationData*)mValidationDataStack.newObject(sizeof(ph__ValidationData));
        memset(validationData, 0, sizeof(ph__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ph()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        ph__ValidationData* validationData = (ph__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->p < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PH, (const ParserChar*)0, "child: p" ) )
                return false;
        }
        if ( validationData->h < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PH, (const ParserChar*)0, "child: h" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__h( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        ph__ValidationData* parentValidationData = (ph__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->p < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_H, (const ParserChar*)0, "sibling: p" ) )
                return false;
        }
        parentValidationData->h++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__h()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__polylist( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 97:
        {
        convex_mesh__ValidationData* parentValidationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYLIST, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYLIST;
        }
//        parentValidationData->polylist++;

        break;
        }

        case 704:
        {
        mesh__ValidationData* parentValidationData = (mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYLIST, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYLIST;
        }
//        parentValidationData->polylist++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        polylist__ValidationData* validationData = (polylist__ValidationData*)mValidationDataStack.newObject(sizeof(polylist__ValidationData));
        memset(validationData, 0, sizeof(polylist__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__polylist()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        polylist__ValidationData* validationData = (polylist__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__triangles( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 97:
        {
        convex_mesh__ValidationData* parentValidationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRIANGLES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRIANGLES;
        }
//        parentValidationData->triangles++;

        break;
        }

        case 704:
        {
        mesh__ValidationData* parentValidationData = (mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRIANGLES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRIANGLES;
        }
//        parentValidationData->triangles++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        triangles__ValidationData* validationData = (triangles__ValidationData*)mValidationDataStack.newObject(sizeof(triangles__ValidationData));
        memset(validationData, 0, sizeof(triangles__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__triangles()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        triangles__ValidationData* validationData = (triangles__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__trifans( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 97:
        {
        convex_mesh__ValidationData* parentValidationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRIFANS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRIFANS;
        }
//        parentValidationData->trifans++;

        break;
        }

        case 704:
        {
        mesh__ValidationData* parentValidationData = (mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRIFANS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRIFANS;
        }
//        parentValidationData->trifans++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        trifans__ValidationData* validationData = (trifans__ValidationData*)mValidationDataStack.newObject(sizeof(trifans__ValidationData));
        memset(validationData, 0, sizeof(trifans__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__trifans()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        trifans__ValidationData* validationData = (trifans__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__tristrips( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 97:
        {
        convex_mesh__ValidationData* parentValidationData = (convex_mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRISTRIPS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRISTRIPS;
        }
//        parentValidationData->tristrips++;

        break;
        }

        case 704:
        {
        mesh__ValidationData* parentValidationData = (mesh__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_LINES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRISTRIPS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRISTRIPS;
        }
//        parentValidationData->tristrips++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        tristrips__ValidationData* validationData = (tristrips__ValidationData*)mValidationDataStack.newObject(sizeof(tristrips__ValidationData));
        memset(validationData, 0, sizeof(tristrips__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__tristrips()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        tristrips__ValidationData* validationData = (tristrips__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mesh( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        geometry__ValidationData* parentValidationData = (geometry__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MESH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MESH;
        }
//        parentValidationData->mesh++;

        mesh__ValidationData* validationData = (mesh__ValidationData*)mValidationDataStack.newObject(sizeof(mesh__ValidationData));
        memset(validationData, 0, sizeof(mesh__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mesh()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        mesh__ValidationData* validationData = (mesh__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_LINES) || (validationData->validation_current_state == HASH_ELEMENT_LINESTRIPS) || (validationData->validation_current_state == HASH_ELEMENT_POLYGONS) || (validationData->validation_current_state == HASH_ELEMENT_POLYLIST) || (validationData->validation_current_state == HASH_ELEMENT_TRIANGLES) || (validationData->validation_current_state == HASH_ELEMENT_TRIFANS) || (validationData->validation_current_state == HASH_ELEMENT_TRISTRIPS) || (validationData->validation_current_state == HASH_ELEMENT_VERTICES) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_MESH, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__spline( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        geometry__ValidationData* parentValidationData = (geometry__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SPLINE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SPLINE;
        }
//        parentValidationData->spline++;

        spline__ValidationData* validationData = (spline__ValidationData*)mValidationDataStack.newObject(sizeof(spline__ValidationData));
        memset(validationData, 0, sizeof(spline__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__spline()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        spline__ValidationData* validationData = (spline__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->source < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPLINE, (const ParserChar*)0, "child: source" ) )
                return false;
        }
        if ( validationData->control_vertices < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPLINE, (const ParserChar*)0, "child: control_vertices" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__spline_type____control_vertices( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        spline__ValidationData* parentValidationData = (spline__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->source < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, "sibling: source" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->control_vertices >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->control_vertices++;

        spline_type____control_vertices__ValidationData* validationData = (spline_type____control_vertices__ValidationData*)mValidationDataStack.newObject(sizeof(spline_type____control_vertices__ValidationData));
        memset(validationData, 0, sizeof(spline_type____control_vertices__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__spline_type____control_vertices()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        spline_type____control_vertices__ValidationData* validationData = (spline_type____control_vertices__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, "child: input" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__brep( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        geometry__ValidationData* parentValidationData = (geometry__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BREP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BREP;
        }
//        parentValidationData->brep++;

        brep__ValidationData* validationData = (brep__ValidationData*)mValidationDataStack.newObject(sizeof(brep__ValidationData));
        memset(validationData, 0, sizeof(brep__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__brep()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        brep__ValidationData* validationData = (brep__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->source < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BREP, (const ParserChar*)0, "child: source" ) )
                return false;
        }
        if ( validationData->vertices < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BREP, (const ParserChar*)0, "child: vertices" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__curves( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->surface_curves != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: surface_curves" ) )
                return false;
        }
        if ( parentValidationData->surfaces != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: surfaces" ) )
                return false;
        }
        if ( parentValidationData->source != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: source" ) )
                return false;
        }
        if ( parentValidationData->vertices != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        if ( parentValidationData->edges != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: edges" ) )
                return false;
        }
        if ( parentValidationData->wires != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: wires" ) )
                return false;
        }
        if ( parentValidationData->faces != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: faces" ) )
                return false;
        }
        if ( parentValidationData->pcurves != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: pcurves" ) )
                return false;
        }
        if ( parentValidationData->shells != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: shells" ) )
                return false;
        }
        if ( parentValidationData->solids != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: solids" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->curves >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CURVES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->curves++;

        curves__ValidationData* validationData = (curves__ValidationData*)mValidationDataStack.newObject(sizeof(curves__ValidationData));
        memset(validationData, 0, sizeof(curves__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__curves()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        curves__ValidationData* validationData = (curves__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->curve < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CURVES, (const ParserChar*)0, "child: curve" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__curve( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 115:
        {
        curves__ValidationData* parentValidationData = (curves__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->curve++;

        break;
        }

        case 306:
        {
        surface_curves__ValidationData* parentValidationData = (surface_curves__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CURVE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->curve++;

        break;
        }

        case 190:
        {
        swept_surface__ValidationData* parentValidationData = (swept_surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURVE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURVE;
        }
//        parentValidationData->curve++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        curve__ValidationData* validationData = (curve__ValidationData*)mValidationDataStack.newObject(sizeof(curve__ValidationData));
        memset(validationData, 0, sizeof(curve__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__curve()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        curve__ValidationData* validationData = (curve__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_CIRCLE) || (validationData->validation_current_state == HASH_ELEMENT_ELLIPSE) || (validationData->validation_current_state == HASH_ELEMENT_HYPERBOLA) || (validationData->validation_current_state == HASH_ELEMENT_LINE) || (validationData->validation_current_state == HASH_ELEMENT_NURBS) || (validationData->validation_current_state == HASH_ELEMENT_ORIENT) || (validationData->validation_current_state == HASH_ELEMENT_ORIGIN) || (validationData->validation_current_state == HASH_ELEMENT_PARABOLA) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_CURVE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__line( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        curve__ValidationData* parentValidationData = (curve__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE;
        }
//        parentValidationData->line++;

        line__ValidationData* validationData = (line__ValidationData*)mValidationDataStack.newObject(sizeof(line__ValidationData));
        memset(validationData, 0, sizeof(line__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__line()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        line__ValidationData* validationData = (line__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->origin < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LINE, (const ParserChar*)0, "child: origin" ) )
                return false;
        }
        if ( validationData->direction < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LINE, (const ParserChar*)0, "child: direction" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3_type( const float* value, size_t length )
{
    if( length < 3 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3_type__stream( const float* value, size_t length )
{

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3_type__streamEnd( const float* value, size_t length )
{
    if( length < 3 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__origin____float3_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 223:
        {
        line__ValidationData* parentValidationData = (line__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->direction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ORIGIN, (const ParserChar*)0, "sibling: direction" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ORIGIN, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->origin >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ORIGIN, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->origin++;

        break;
        }

        case 190:
        {
        swept_surface__ValidationData* parentValidationData = (swept_surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_CURVE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ORIGIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ORIGIN;
        }
//        parentValidationData->origin++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__origin____float3_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__direction( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 223:
        {
        line__ValidationData* parentValidationData = (line__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->origin < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_DIRECTION, (const ParserChar*)0, "sibling: origin" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIRECTION, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->direction >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DIRECTION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->direction++;

        break;
        }

        case 190:
        {
        swept_surface__ValidationData* parentValidationData = (swept_surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_CURVE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIRECTION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIRECTION;
        }
//        parentValidationData->direction++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__direction()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__circle( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        curve__ValidationData* parentValidationData = (curve__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CIRCLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CIRCLE;
        }
//        parentValidationData->circle++;

        circle__ValidationData* validationData = (circle__ValidationData*)mValidationDataStack.newObject(sizeof(circle__ValidationData));
        memset(validationData, 0, sizeof(circle__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__circle()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        circle__ValidationData* validationData = (circle__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->radius < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CIRCLE, (const ParserChar*)0, "child: radius" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__radius____float_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 469:
        {
        surface_type____cylinder__ValidationData* parentValidationData = (surface_type____cylinder__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->radius >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RADIUS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->radius++;

        break;
        }

        case 738:
        {
        circle__ValidationData* parentValidationData = (circle__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->radius >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RADIUS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->radius++;

        break;
        }

        case 585:
        {
        cone__ValidationData* parentValidationData = (cone__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->angle != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: angle" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->radius >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RADIUS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->radius++;

        break;
        }

        case 634:
        {
        sphere__ValidationData* parentValidationData = (sphere__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->radius >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RADIUS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->radius++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__radius____float_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ellipse( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        curve__ValidationData* parentValidationData = (curve__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ELLIPSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ELLIPSE;
        }
//        parentValidationData->ellipse++;

        ellipse__ValidationData* validationData = (ellipse__ValidationData*)mValidationDataStack.newObject(sizeof(ellipse__ValidationData));
        memset(validationData, 0, sizeof(ellipse__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ellipse()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        ellipse__ValidationData* validationData = (ellipse__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->radius < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ELLIPSE, (const ParserChar*)0, "child: radius" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2_type( const float* value, size_t length )
{
    if( length < 2 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 2 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2_type__stream( const float* value, size_t length )
{

    if( length > 2 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2_type__streamEnd( const float* value, size_t length )
{
    if( length < 2 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 2 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__radius____float2_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 470:
        {
        cylinder____cylinder_type__ValidationData* parentValidationData = (cylinder____cylinder_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->height < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: height" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->radius >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RADIUS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->radius++;

        break;
        }

        case 437:
        {
        ellipse__ValidationData* parentValidationData = (ellipse__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->radius >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RADIUS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->radius++;

        break;
        }

        case 244:
        {
        hyperbola__ValidationData* parentValidationData = (hyperbola__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->radius >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RADIUS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->radius++;

        break;
        }

        case 15:
        {
        torus__ValidationData* parentValidationData = (torus__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->radius >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RADIUS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->radius++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        radius____float2_type__ValidationData* validationData = (radius____float2_type__ValidationData*)mValidationDataStack.newObject(sizeof(radius____float2_type__ValidationData));
        memset(validationData, 0, sizeof(radius____float2_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__radius____float2_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__parabola( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        curve__ValidationData* parentValidationData = (curve__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARABOLA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARABOLA;
        }
//        parentValidationData->parabola++;

        parabola__ValidationData* validationData = (parabola__ValidationData*)mValidationDataStack.newObject(sizeof(parabola__ValidationData));
        memset(validationData, 0, sizeof(parabola__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__parabola()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        parabola__ValidationData* validationData = (parabola__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->focal < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PARABOLA, (const ParserChar*)0, "child: focal" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__focal( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        parabola__ValidationData* parentValidationData = (parabola__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FOCAL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->focal >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FOCAL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->focal++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__focal()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__hyperbola( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        curve__ValidationData* parentValidationData = (curve__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HYPERBOLA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HYPERBOLA;
        }
//        parentValidationData->hyperbola++;

        hyperbola__ValidationData* validationData = (hyperbola__ValidationData*)mValidationDataStack.newObject(sizeof(hyperbola__ValidationData));
        memset(validationData, 0, sizeof(hyperbola__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__hyperbola()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        hyperbola__ValidationData* validationData = (hyperbola__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->radius < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_HYPERBOLA, (const ParserChar*)0, "child: radius" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__nurbs( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        curve__ValidationData* parentValidationData = (curve__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NURBS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NURBS;
        }
//        parentValidationData->nurbs++;

        nurbs__ValidationData* validationData = (nurbs__ValidationData*)mValidationDataStack.newObject(sizeof(nurbs__ValidationData));
        memset(validationData, 0, sizeof(nurbs__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__nurbs()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        nurbs__ValidationData* validationData = (nurbs__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->source < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NURBS, (const ParserChar*)0, "child: source" ) )
                return false;
        }
        if ( validationData->control_vertices < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NURBS, (const ParserChar*)0, "child: control_vertices" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__nurbs_type____control_vertices( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        nurbs__ValidationData* parentValidationData = (nurbs__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->source < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, "sibling: source" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->control_vertices >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->control_vertices++;

        nurbs_type____control_vertices__ValidationData* validationData = (nurbs_type____control_vertices__ValidationData*)mValidationDataStack.newObject(sizeof(nurbs_type____control_vertices__ValidationData));
        memset(validationData, 0, sizeof(nurbs_type____control_vertices__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__nurbs_type____control_vertices()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        nurbs_type____control_vertices__ValidationData* validationData = (nurbs_type____control_vertices__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, "child: input" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4_type( const float* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4_type__stream( const float* value, size_t length )
{

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4_type__streamEnd( const float* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__orient( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 109:
        {
        curve__ValidationData* parentValidationData = (curve__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_CIRCLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ELLIPSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_HYPERBOLA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NURBS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ORIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARABOLA) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ORIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ORIENT;
        }
//        parentValidationData->orient++;

        break;
        }

        case 263:
        {
        surface__ValidationData* parentValidationData = (surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_CONE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NURBS_SURFACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ORIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SPHERE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SWEPT_SURFACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TORUS) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ORIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ORIENT;
        }
//        parentValidationData->orient++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        orient__ValidationData* validationData = (orient__ValidationData*)mValidationDataStack.newObject(sizeof(orient__ValidationData));
        memset(validationData, 0, sizeof(orient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__orient()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__origin____origin_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 109:
        {
        curve__ValidationData* parentValidationData = (curve__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_CIRCLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ELLIPSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_HYPERBOLA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NURBS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ORIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARABOLA) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ORIGIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ORIGIN;
        }
//        parentValidationData->origin++;

        break;
        }

        case 263:
        {
        surface__ValidationData* parentValidationData = (surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_CONE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NURBS_SURFACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ORIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SPHERE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SWEPT_SURFACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TORUS) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ORIGIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ORIGIN;
        }
//        parentValidationData->origin++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        origin____origin_type__ValidationData* validationData = (origin____origin_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____origin_type__ValidationData));
        memset(validationData, 0, sizeof(origin____origin_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__origin____origin_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__surface_curves( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->surfaces != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "sibling: surfaces" ) )
                return false;
        }
        if ( parentValidationData->source != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "sibling: source" ) )
                return false;
        }
        if ( parentValidationData->vertices != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        if ( parentValidationData->edges != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "sibling: edges" ) )
                return false;
        }
        if ( parentValidationData->wires != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "sibling: wires" ) )
                return false;
        }
        if ( parentValidationData->faces != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "sibling: faces" ) )
                return false;
        }
        if ( parentValidationData->pcurves != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "sibling: pcurves" ) )
                return false;
        }
        if ( parentValidationData->shells != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "sibling: shells" ) )
                return false;
        }
        if ( parentValidationData->solids != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "sibling: solids" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->surface_curves >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->surface_curves++;

        surface_curves__ValidationData* validationData = (surface_curves__ValidationData*)mValidationDataStack.newObject(sizeof(surface_curves__ValidationData));
        memset(validationData, 0, sizeof(surface_curves__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__surface_curves()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        surface_curves__ValidationData* validationData = (surface_curves__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->curve < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SURFACE_CURVES, (const ParserChar*)0, "child: curve" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__surfaces( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->source != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACES, (const ParserChar*)0, "sibling: source" ) )
                return false;
        }
        if ( parentValidationData->vertices != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACES, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        if ( parentValidationData->edges != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACES, (const ParserChar*)0, "sibling: edges" ) )
                return false;
        }
        if ( parentValidationData->wires != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACES, (const ParserChar*)0, "sibling: wires" ) )
                return false;
        }
        if ( parentValidationData->faces != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACES, (const ParserChar*)0, "sibling: faces" ) )
                return false;
        }
        if ( parentValidationData->pcurves != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACES, (const ParserChar*)0, "sibling: pcurves" ) )
                return false;
        }
        if ( parentValidationData->shells != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACES, (const ParserChar*)0, "sibling: shells" ) )
                return false;
        }
        if ( parentValidationData->solids != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACES, (const ParserChar*)0, "sibling: solids" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->surfaces >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SURFACES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->surfaces++;

        surfaces__ValidationData* validationData = (surfaces__ValidationData*)mValidationDataStack.newObject(sizeof(surfaces__ValidationData));
        memset(validationData, 0, sizeof(surfaces__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__surfaces()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        surfaces__ValidationData* validationData = (surfaces__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->surface < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SURFACES, (const ParserChar*)0, "child: surface" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__surface( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        surfaces__ValidationData* parentValidationData = (surfaces__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SURFACE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->surface++;

        surface__ValidationData* validationData = (surface__ValidationData*)mValidationDataStack.newObject(sizeof(surface__ValidationData));
        memset(validationData, 0, sizeof(surface__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__surface()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        surface__ValidationData* validationData = (surface__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_CONE) || (validationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (validationData->validation_current_state == HASH_ELEMENT_NURBS_SURFACE) || (validationData->validation_current_state == HASH_ELEMENT_ORIENT) || (validationData->validation_current_state == HASH_ELEMENT_ORIGIN) || (validationData->validation_current_state == HASH_ELEMENT_PLANE) || (validationData->validation_current_state == HASH_ELEMENT_SPHERE) || (validationData->validation_current_state == HASH_ELEMENT_SWEPT_SURFACE) || (validationData->validation_current_state == HASH_ELEMENT_TORUS) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SURFACE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__plane( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLANE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLANE;
        }
//        parentValidationData->plane++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLANE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLANE;
        }
//        parentValidationData->plane++;

        break;
        }

        case 263:
        {
        surface__ValidationData* parentValidationData = (surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLANE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLANE;
        }
//        parentValidationData->plane++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        plane__ValidationData* validationData = (plane__ValidationData*)mValidationDataStack.newObject(sizeof(plane__ValidationData));
        memset(validationData, 0, sizeof(plane__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__plane()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        plane__ValidationData* validationData = (plane__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->equation < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PLANE, (const ParserChar*)0, "child: equation" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__equation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        plane__ValidationData* parentValidationData = (plane__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EQUATION, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->equation >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EQUATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->equation++;

        orient__ValidationData* validationData = (orient__ValidationData*)mValidationDataStack.newObject(sizeof(orient__ValidationData));
        memset(validationData, 0, sizeof(orient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__equation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sphere( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SPHERE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SPHERE;
        }
//        parentValidationData->sphere++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SPHERE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SPHERE;
        }
//        parentValidationData->sphere++;

        break;
        }

        case 263:
        {
        surface__ValidationData* parentValidationData = (surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SPHERE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SPHERE;
        }
//        parentValidationData->sphere++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        sphere__ValidationData* validationData = (sphere__ValidationData*)mValidationDataStack.newObject(sizeof(sphere__ValidationData));
        memset(validationData, 0, sizeof(sphere__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sphere()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sphere__ValidationData* validationData = (sphere__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->radius < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPHERE, (const ParserChar*)0, "child: radius" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__torus( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        surface__ValidationData* parentValidationData = (surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TORUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TORUS;
        }
//        parentValidationData->torus++;

        torus__ValidationData* validationData = (torus__ValidationData*)mValidationDataStack.newObject(sizeof(torus__ValidationData));
        memset(validationData, 0, sizeof(torus__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__torus()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        torus__ValidationData* validationData = (torus__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->radius < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TORUS, (const ParserChar*)0, "child: radius" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__swept_surface( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        surface__ValidationData* parentValidationData = (surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SWEPT_SURFACE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SWEPT_SURFACE;
        }
//        parentValidationData->swept_surface++;

        swept_surface__ValidationData* validationData = (swept_surface__ValidationData*)mValidationDataStack.newObject(sizeof(swept_surface__ValidationData));
        memset(validationData, 0, sizeof(swept_surface__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__swept_surface()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        swept_surface__ValidationData* validationData = (swept_surface__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_AXIS) || (validationData->validation_current_state == HASH_ELEMENT_DIRECTION) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SWEPT_SURFACE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__axis____float3_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        swept_surface__ValidationData* parentValidationData = (swept_surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ORIGIN) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AXIS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AXIS;
        }
//        parentValidationData->axis++;

        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__axis____float3_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__nurbs_surface( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        surface__ValidationData* parentValidationData = (surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NURBS_SURFACE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NURBS_SURFACE;
        }
//        parentValidationData->nurbs_surface++;

        nurbs_surface__ValidationData* validationData = (nurbs_surface__ValidationData*)mValidationDataStack.newObject(sizeof(nurbs_surface__ValidationData));
        memset(validationData, 0, sizeof(nurbs_surface__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__nurbs_surface()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        nurbs_surface__ValidationData* validationData = (nurbs_surface__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->source < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NURBS_SURFACE, (const ParserChar*)0, "child: source" ) )
                return false;
        }
        if ( validationData->control_vertices < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NURBS_SURFACE, (const ParserChar*)0, "child: control_vertices" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__nurbs_surface_type____control_vertices( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        nurbs_surface__ValidationData* parentValidationData = (nurbs_surface__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->source < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, "sibling: source" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->control_vertices >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->control_vertices++;

        nurbs_surface_type____control_vertices__ValidationData* validationData = (nurbs_surface_type____control_vertices__ValidationData*)mValidationDataStack.newObject(sizeof(nurbs_surface_type____control_vertices__ValidationData));
        memset(validationData, 0, sizeof(nurbs_surface_type____control_vertices__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__nurbs_surface_type____control_vertices()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        nurbs_surface_type____control_vertices__ValidationData* validationData = (nurbs_surface_type____control_vertices__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CONTROL_VERTICES, (const ParserChar*)0, "child: input" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__cone( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        surface__ValidationData* parentValidationData = (surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONE;
        }
//        parentValidationData->cone++;

        cone__ValidationData* validationData = (cone__ValidationData*)mValidationDataStack.newObject(sizeof(cone__ValidationData));
        memset(validationData, 0, sizeof(cone__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__cone()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        cone__ValidationData* validationData = (cone__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->radius < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CONE, (const ParserChar*)0, "child: radius" ) )
                return false;
        }
        if ( validationData->angle < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CONE, (const ParserChar*)0, "child: angle" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__angle( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        cone__ValidationData* parentValidationData = (cone__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ANGLE, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANGLE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->angle >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ANGLE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->angle++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__angle()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__surface_type____cylinder( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        surface__ValidationData* parentValidationData = (surface__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CYLINDER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CYLINDER;
        }
//        parentValidationData->cylinder++;

        surface_type____cylinder__ValidationData* validationData = (surface_type____cylinder__ValidationData*)mValidationDataStack.newObject(sizeof(surface_type____cylinder__ValidationData));
        memset(validationData, 0, sizeof(surface_type____cylinder__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__surface_type____cylinder()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        surface_type____cylinder__ValidationData* validationData = (surface_type____cylinder__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->radius < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CYLINDER, (const ParserChar*)0, "child: radius" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__edges( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vertices < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EDGES, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        if ( parentValidationData->wires != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EDGES, (const ParserChar*)0, "sibling: wires" ) )
                return false;
        }
        if ( parentValidationData->faces != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EDGES, (const ParserChar*)0, "sibling: faces" ) )
                return false;
        }
        if ( parentValidationData->pcurves != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EDGES, (const ParserChar*)0, "sibling: pcurves" ) )
                return false;
        }
        if ( parentValidationData->shells != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EDGES, (const ParserChar*)0, "sibling: shells" ) )
                return false;
        }
        if ( parentValidationData->solids != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EDGES, (const ParserChar*)0, "sibling: solids" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EDGES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->edges >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EDGES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->edges++;

        edges__ValidationData* validationData = (edges__ValidationData*)mValidationDataStack.newObject(sizeof(edges__ValidationData));
        memset(validationData, 0, sizeof(edges__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__edges()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        edges__ValidationData* validationData = (edges__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 4 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_EDGES, (const ParserChar*)0, "child: input" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__wires( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vertices < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_WIRES, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        if ( parentValidationData->faces != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WIRES, (const ParserChar*)0, "sibling: faces" ) )
                return false;
        }
        if ( parentValidationData->pcurves != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WIRES, (const ParserChar*)0, "sibling: pcurves" ) )
                return false;
        }
        if ( parentValidationData->shells != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WIRES, (const ParserChar*)0, "sibling: shells" ) )
                return false;
        }
        if ( parentValidationData->solids != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WIRES, (const ParserChar*)0, "sibling: solids" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WIRES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->wires >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_WIRES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->wires++;

        wires__ValidationData* validationData = (wires__ValidationData*)mValidationDataStack.newObject(sizeof(wires__ValidationData));
        memset(validationData, 0, sizeof(wires__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__wires()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        wires__ValidationData* validationData = (wires__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 2 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_WIRES, (const ParserChar*)0, "child: input" ) )
                return false;
        }
        if ( validationData->vcount < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_WIRES, (const ParserChar*)0, "child: vcount" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__faces( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vertices < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_FACES, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        if ( parentValidationData->pcurves != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACES, (const ParserChar*)0, "sibling: pcurves" ) )
                return false;
        }
        if ( parentValidationData->shells != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACES, (const ParserChar*)0, "sibling: shells" ) )
                return false;
        }
        if ( parentValidationData->solids != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACES, (const ParserChar*)0, "sibling: solids" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->faces >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FACES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->faces++;

        faces__ValidationData* validationData = (faces__ValidationData*)mValidationDataStack.newObject(sizeof(faces__ValidationData));
        memset(validationData, 0, sizeof(faces__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__faces()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        faces__ValidationData* validationData = (faces__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 3 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FACES, (const ParserChar*)0, "child: input" ) )
                return false;
        }
        if ( validationData->vcount < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FACES, (const ParserChar*)0, "child: vcount" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__pcurves( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vertices < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_PCURVES, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        if ( parentValidationData->shells != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PCURVES, (const ParserChar*)0, "sibling: shells" ) )
                return false;
        }
        if ( parentValidationData->solids != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PCURVES, (const ParserChar*)0, "sibling: solids" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PCURVES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->pcurves >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PCURVES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->pcurves++;

        pcurves__ValidationData* validationData = (pcurves__ValidationData*)mValidationDataStack.newObject(sizeof(pcurves__ValidationData));
        memset(validationData, 0, sizeof(pcurves__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__pcurves()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        pcurves__ValidationData* validationData = (pcurves__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 3 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PCURVES, (const ParserChar*)0, "child: input" ) )
                return false;
        }
        if ( validationData->vcount < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PCURVES, (const ParserChar*)0, "child: vcount" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__shells( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vertices < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_SHELLS, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        if ( parentValidationData->solids != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHELLS, (const ParserChar*)0, "sibling: solids" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHELLS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->shells >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SHELLS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->shells++;

        shells__ValidationData* validationData = (shells__ValidationData*)mValidationDataStack.newObject(sizeof(shells__ValidationData));
        memset(validationData, 0, sizeof(shells__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__shells()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        shells__ValidationData* validationData = (shells__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 2 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SHELLS, (const ParserChar*)0, "child: input" ) )
                return false;
        }
        if ( validationData->vcount < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SHELLS, (const ParserChar*)0, "child: vcount" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__solids( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        brep__ValidationData* parentValidationData = (brep__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->vertices < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_SOLIDS, (const ParserChar*)0, "sibling: vertices" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOLIDS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->solids >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SOLIDS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->solids++;

        solids__ValidationData* validationData = (solids__ValidationData*)mValidationDataStack.newObject(sizeof(solids__ValidationData));
        memset(validationData, 0, sizeof(solids__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__solids()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        solids__ValidationData* validationData = (solids__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->input < 2 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SOLIDS, (const ParserChar*)0, "child: input" ) )
                return false;
        }
        if ( validationData->vcount < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SOLIDS, (const ParserChar*)0, "child: vcount" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_effects( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_EFFECTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_EFFECTS;
        }
//        parentValidationData->library_effects++;

        library_effects__ValidationData* validationData = (library_effects__ValidationData*)mValidationDataStack.newObject(sizeof(library_effects__ValidationData));
        memset(validationData, 0, sizeof(library_effects__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_effects()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_effects__ValidationData* validationData = (library_effects__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->effect < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_EFFECTS, (const ParserChar*)0, "child: effect" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__effect( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_effects__ValidationData* parentValidationData = (library_effects__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EFFECT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->effect++;

        effect__ValidationData* validationData = (effect__ValidationData*)mValidationDataStack.newObject(sizeof(effect__ValidationData));
        memset(validationData, 0, sizeof(effect__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__effect()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effect__ValidationData* validationData = (effect__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_PROFILE_BRIDGE) || (validationData->validation_current_state == HASH_ELEMENT_PROFILE_CG) || (validationData->validation_current_state == HASH_ELEMENT_PROFILE_COMMON) || (validationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES) || (validationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES2) || (validationData->validation_current_state == HASH_ELEMENT_PROFILE_GLSL) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_EFFECT, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__annotate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ANNOTATE;
        }
//        parentValidationData->annotate++;

        break;
        }

        case 418:
        {
        profile_GLES__technique__pass__ValidationData* parentValidationData = (profile_GLES__technique__pass__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->states != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: states" ) )
                return false;
        }
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->annotate++;

        break;
        }

        case 419:
        {
        profile_GLSL__technique__pass__ValidationData* parentValidationData = (profile_GLSL__technique__pass__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->states != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: states" ) )
                return false;
        }
        if ( parentValidationData->program != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: program" ) )
                return false;
        }
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->annotate++;

        break;
        }

        case 614:
        {
        profile_cg_type____technique__ValidationData* parentValidationData = (profile_cg_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->pass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->annotate++;

        break;
        }

        case 616:
        {
        profile_gles2_type____technique__ValidationData* parentValidationData = (profile_gles2_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->pass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->annotate++;

        break;
        }

        case 617:
        {
        profile_gles_type____technique__ValidationData* parentValidationData = (profile_gles_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->pass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->annotate++;

        break;
        }

        case 618:
        {
        profile_glsl_type____technique__ValidationData* parentValidationData = (profile_glsl_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->pass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: pass" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->annotate++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ANNOTATE;
        }
//        parentValidationData->annotate++;

        break;
        }

        case 420:
        {
        pass____cg_pass_type__ValidationData* parentValidationData = (pass____cg_pass_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->states != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: states" ) )
                return false;
        }
        if ( parentValidationData->program != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: program" ) )
                return false;
        }
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->annotate++;

        break;
        }

        case 69:
        {
        effect__ValidationData* parentValidationData = (effect__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ANNOTATE;
        }
//        parentValidationData->annotate++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ANNOTATE;
        }
//        parentValidationData->annotate++;

        break;
        }

        case 421:
        {
        pass____gles2_pass_type__ValidationData* parentValidationData = (pass____gles2_pass_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->states != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: states" ) )
                return false;
        }
        if ( parentValidationData->program != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: program" ) )
                return false;
        }
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->annotate++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ANNOTATE;
        }
//        parentValidationData->annotate++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ANNOTATE;
        }
//        parentValidationData->annotate++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        annotate__ValidationData* validationData = (annotate__ValidationData*)mValidationDataStack.newObject(sizeof(annotate__ValidationData));
        memset(validationData, 0, sizeof(annotate__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__annotate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        annotate__ValidationData* validationData = (annotate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_STRING) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_ANNOTATE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2_type( const bool* value, size_t length )
{
    if( length < 2 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 2 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2_type__stream( const bool* value, size_t length )
{

    if( length > 2 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2_type__streamEnd( const bool* value, size_t length )
{
    if( length < 2 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 2 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2;
        }
//        parentValidationData->bool2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool2__ValidationData* validationData = (bool2__ValidationData*)mValidationDataStack.newObject(sizeof(bool2__ValidationData));
        memset(validationData, 0, sizeof(bool2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3_type( const bool* value, size_t length )
{
    if( length < 3 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3_type__stream( const bool* value, size_t length )
{

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3_type__streamEnd( const bool* value, size_t length )
{
    if( length < 3 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3;
        }
//        parentValidationData->bool3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool3__ValidationData* validationData = (bool3__ValidationData*)mValidationDataStack.newObject(sizeof(bool3__ValidationData));
        memset(validationData, 0, sizeof(bool3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool3()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4_type( const bool* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4_type__stream( const bool* value, size_t length )
{

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4_type__streamEnd( const bool* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4;
        }
//        parentValidationData->bool4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool4__ValidationData* validationData = (bool4__ValidationData*)mValidationDataStack.newObject(sizeof(bool4__ValidationData));
        memset(validationData, 0, sizeof(bool4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool4()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2_type( const sint64* value, size_t length )
{
    if( length < 2 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 2 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2_type__stream( const sint64* value, size_t length )
{

    if( length > 2 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 2 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 2 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2;
        }
//        parentValidationData->int2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int2__ValidationData* validationData = (int2__ValidationData*)mValidationDataStack.newObject(sizeof(int2__ValidationData));
        memset(validationData, 0, sizeof(int2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3_type( const sint64* value, size_t length )
{
    if( length < 3 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3_type__stream( const sint64* value, size_t length )
{

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 3 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3;
        }
//        parentValidationData->int3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int3__ValidationData* validationData = (int3__ValidationData*)mValidationDataStack.newObject(sizeof(int3__ValidationData));
        memset(validationData, 0, sizeof(int3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int3()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4_type( const sint64* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4_type__stream( const sint64* value, size_t length )
{

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4;
        }
//        parentValidationData->int4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int4__ValidationData* validationData = (int4__ValidationData*)mValidationDataStack.newObject(sizeof(int4__ValidationData));
        memset(validationData, 0, sizeof(int4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 26:
        {
        speed____common_float2_or_param_type__ValidationData* parentValidationData = (speed____common_float2_or_param_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->float2 >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->float2++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 657:
        {
        newparam____fx_common_newparam_type__ValidationData* parentValidationData = (newparam____fx_common_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2;
        }
//        parentValidationData->float2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        radius____float2_type__ValidationData* validationData = (radius____float2_type__ValidationData*)mValidationDataStack.newObject(sizeof(radius____float2_type__ValidationData));
        memset(validationData, 0, sizeof(radius____float2_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 657:
        {
        newparam____fx_common_newparam_type__ValidationData* parentValidationData = (newparam____fx_common_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3;
        }
//        parentValidationData->float3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 657:
        {
        newparam____fx_common_newparam_type__ValidationData* parentValidationData = (newparam____fx_common_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4;
        }
//        parentValidationData->float4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        orient__ValidationData* validationData = (orient__ValidationData*)mValidationDataStack.newObject(sizeof(orient__ValidationData));
        memset(validationData, 0, sizeof(orient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float4()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2x2_type( const float* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2x2_type__stream( const float* value, size_t length )
{

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2x2_type__streamEnd( const float* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float2x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X2;
        }
//        parentValidationData->float2x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float2x2__ValidationData* validationData = (float2x2__ValidationData*)mValidationDataStack.newObject(sizeof(float2x2__ValidationData));
        memset(validationData, 0, sizeof(float2x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float2x2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3x3_type( const float* value, size_t length )
{
    if( length < 9 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 9 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3x3_type__stream( const float* value, size_t length )
{

    if( length > 9 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3x3_type__streamEnd( const float* value, size_t length )
{
    if( length < 9 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 9 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float3x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X3;
        }
//        parentValidationData->float3x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float3x3__ValidationData* validationData = (float3x3__ValidationData*)mValidationDataStack.newObject(sizeof(float3x3__ValidationData));
        memset(validationData, 0, sizeof(float3x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float3x3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float4x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X4;
        }
//        parentValidationData->float4x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bind_shape_matrix__ValidationData* validationData = (bind_shape_matrix__ValidationData*)mValidationDataStack.newObject(sizeof(bind_shape_matrix__ValidationData));
        memset(validationData, 0, sizeof(bind_shape_matrix__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float4x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__string( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STRING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STRING;
        }
//        parentValidationData->string++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STRING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STRING;
        }
//        parentValidationData->string++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STRING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STRING;
        }
//        parentValidationData->string++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STRING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STRING;
        }
//        parentValidationData->string++;

        break;
        }

        case 498:
        {
        annotate__ValidationData* parentValidationData = (annotate__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STRING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STRING;
        }
//        parentValidationData->string++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__string()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__newparam____fx_newparam_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effect__ValidationData* parentValidationData = (effect__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEWPARAM;
        }
//        parentValidationData->newparam++;

        newparam____fx_newparam_type__ValidationData* validationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.newObject(sizeof(newparam____fx_newparam_type__ValidationData));
        memset(validationData, 0, sizeof(newparam____fx_newparam_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__newparam____fx_newparam_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____fx_newparam_type__ValidationData* validationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__semantic____NCName( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTIC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTIC;
        }
//        parentValidationData->semantic++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTIC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTIC;
        }
//        parentValidationData->semantic++;

        break;
        }

        case 657:
        {
        newparam____fx_common_newparam_type__ValidationData* parentValidationData = (newparam____fx_common_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTIC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTIC;
        }
//        parentValidationData->semantic++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTIC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTIC;
        }
//        parentValidationData->semantic++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTIC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTIC;
        }
//        parentValidationData->semantic++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTIC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTIC;
        }
//        parentValidationData->semantic++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__semantic____NCName()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_modifier_enum( ParserString value )
{
    return validate__fx_modifier_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_modifier_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_modifier_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_modifier_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__modifier( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODIFIER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODIFIER;
        }
//        parentValidationData->modifier++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODIFIER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODIFIER;
        }
//        parentValidationData->modifier++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODIFIER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODIFIER;
        }
//        parentValidationData->modifier++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODIFIER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODIFIER;
        }
//        parentValidationData->modifier++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODIFIER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODIFIER;
        }
//        parentValidationData->modifier++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__modifier()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float2x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X1;
        }
//        parentValidationData->float2x1++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X1;
        }
//        parentValidationData->float2x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X1;
        }
//        parentValidationData->float2x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X1;
        }
//        parentValidationData->float2x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X1;
        }
//        parentValidationData->float2x1++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X1;
        }
//        parentValidationData->float2x1++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X1;
        }
//        parentValidationData->float2x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        radius____float2_type__ValidationData* validationData = (radius____float2_type__ValidationData*)mValidationDataStack.newObject(sizeof(radius____float2_type__ValidationData));
        memset(validationData, 0, sizeof(radius____float2_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float2x1()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2x3_type( const float* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2x3_type__stream( const float* value, size_t length )
{

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2x3_type__streamEnd( const float* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float2x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X3;
        }
//        parentValidationData->float2x3++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X3;
        }
//        parentValidationData->float2x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X3;
        }
//        parentValidationData->float2x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X3;
        }
//        parentValidationData->float2x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X3;
        }
//        parentValidationData->float2x3++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X3;
        }
//        parentValidationData->float2x3++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X3;
        }
//        parentValidationData->float2x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float2x3__ValidationData* validationData = (float2x3__ValidationData*)mValidationDataStack.newObject(sizeof(float2x3__ValidationData));
        memset(validationData, 0, sizeof(float2x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float2x3()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2x4_type( const float* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2x4_type__stream( const float* value, size_t length )
{

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float2x4_type__streamEnd( const float* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float2x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X4;
        }
//        parentValidationData->float2x4++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X4;
        }
//        parentValidationData->float2x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X4;
        }
//        parentValidationData->float2x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X4;
        }
//        parentValidationData->float2x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X4;
        }
//        parentValidationData->float2x4++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X4;
        }
//        parentValidationData->float2x4++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT2X4;
        }
//        parentValidationData->float2x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float2x4__ValidationData* validationData = (float2x4__ValidationData*)mValidationDataStack.newObject(sizeof(float2x4__ValidationData));
        memset(validationData, 0, sizeof(float2x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float2x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float3x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X1;
        }
//        parentValidationData->float3x1++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X1;
        }
//        parentValidationData->float3x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X1;
        }
//        parentValidationData->float3x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X1;
        }
//        parentValidationData->float3x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X1;
        }
//        parentValidationData->float3x1++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X1;
        }
//        parentValidationData->float3x1++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X1;
        }
//        parentValidationData->float3x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float3x1()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3x2_type( const float* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3x2_type__stream( const float* value, size_t length )
{

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3x2_type__streamEnd( const float* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float3x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X2;
        }
//        parentValidationData->float3x2++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X2;
        }
//        parentValidationData->float3x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X2;
        }
//        parentValidationData->float3x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X2;
        }
//        parentValidationData->float3x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X2;
        }
//        parentValidationData->float3x2++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X2;
        }
//        parentValidationData->float3x2++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X2;
        }
//        parentValidationData->float3x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float3x2__ValidationData* validationData = (float3x2__ValidationData*)mValidationDataStack.newObject(sizeof(float3x2__ValidationData));
        memset(validationData, 0, sizeof(float3x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float3x2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3x4_type( const float* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3x4_type__stream( const float* value, size_t length )
{

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float3x4_type__streamEnd( const float* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float3x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X4;
        }
//        parentValidationData->float3x4++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X4;
        }
//        parentValidationData->float3x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X4;
        }
//        parentValidationData->float3x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X4;
        }
//        parentValidationData->float3x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X4;
        }
//        parentValidationData->float3x4++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X4;
        }
//        parentValidationData->float3x4++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT3X4;
        }
//        parentValidationData->float3x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float3x4__ValidationData* validationData = (float3x4__ValidationData*)mValidationDataStack.newObject(sizeof(float3x4__ValidationData));
        memset(validationData, 0, sizeof(float3x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float3x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float4x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X1;
        }
//        parentValidationData->float4x1++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X1;
        }
//        parentValidationData->float4x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X1;
        }
//        parentValidationData->float4x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X1;
        }
//        parentValidationData->float4x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X1;
        }
//        parentValidationData->float4x1++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X1;
        }
//        parentValidationData->float4x1++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X1;
        }
//        parentValidationData->float4x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        orient__ValidationData* validationData = (orient__ValidationData*)mValidationDataStack.newObject(sizeof(orient__ValidationData));
        memset(validationData, 0, sizeof(orient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float4x1()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4x2_type( const float* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4x2_type__stream( const float* value, size_t length )
{

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4x2_type__streamEnd( const float* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float4x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X2;
        }
//        parentValidationData->float4x2++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X2;
        }
//        parentValidationData->float4x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X2;
        }
//        parentValidationData->float4x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X2;
        }
//        parentValidationData->float4x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X2;
        }
//        parentValidationData->float4x2++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X2;
        }
//        parentValidationData->float4x2++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X2;
        }
//        parentValidationData->float4x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float4x2__ValidationData* validationData = (float4x2__ValidationData*)mValidationDataStack.newObject(sizeof(float4x2__ValidationData));
        memset(validationData, 0, sizeof(float4x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float4x2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4x3_type( const float* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4x3_type__stream( const float* value, size_t length )
{

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float4x3_type__streamEnd( const float* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float4x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X3;
        }
//        parentValidationData->float4x3++;

        break;
        }

        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X3;
        }
//        parentValidationData->float4x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X3;
        }
//        parentValidationData->float4x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X3;
        }
//        parentValidationData->float4x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X3;
        }
//        parentValidationData->float4x3++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X3;
        }
//        parentValidationData->float4x3++;

        break;
        }

        case 659:
        {
        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT4X3;
        }
//        parentValidationData->float4x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float4x3__ValidationData* validationData = (float4x3__ValidationData*)mValidationDataStack.newObject(sizeof(float4x3__ValidationData));
        memset(validationData, 0, sizeof(float4x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float4x3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sampler1D( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER1D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER1D;
        }
//        parentValidationData->sampler1D++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER1D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER1D;
        }
//        parentValidationData->sampler1D++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER1D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER1D;
        }
//        parentValidationData->sampler1D++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER1D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER1D;
        }
//        parentValidationData->sampler1D++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER1D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER1D;
        }
//        parentValidationData->sampler1D++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER1D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER1D;
        }
//        parentValidationData->sampler1D++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER1D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER1D;
        }
//        parentValidationData->sampler1D++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER1D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER1D;
        }
//        parentValidationData->sampler1D++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        sampler1D__ValidationData* validationData = (sampler1D__ValidationData*)mValidationDataStack.newObject(sizeof(sampler1D__ValidationData));
        memset(validationData, 0, sizeof(sampler1D__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sampler1D()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler1D__ValidationData* validationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SAMPLER1D, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_image( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 486:
        {
        color_target__ValidationData* parentValidationData = (color_target__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->instance_image >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->instance_image++;

        break;
        }

        case 735:
        {
        depth_target__ValidationData* parentValidationData = (depth_target__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->instance_image >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->instance_image++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_IMAGE;
        }
//        parentValidationData->instance_image++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_IMAGE;
        }
//        parentValidationData->instance_image++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_IMAGE;
        }
//        parentValidationData->instance_image++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_IMAGE;
        }
//        parentValidationData->instance_image++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_IMAGE;
        }
//        parentValidationData->instance_image++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_IMAGE;
        }
//        parentValidationData->instance_image++;

        break;
        }

        case 44:
        {
        stencil_target__ValidationData* parentValidationData = (stencil_target__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->instance_image >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->instance_image++;

        break;
        }

        case 67:
        {
        sampler2D____gles_sampler_type__ValidationData* parentValidationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_IMAGE;
        }
//        parentValidationData->instance_image++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        instance_image__ValidationData* validationData = (instance_image__ValidationData*)mValidationDataStack.newObject(sizeof(instance_image__ValidationData));
        memset(validationData, 0, sizeof(instance_image__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_image()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_image__ValidationData* validationData = (instance_image__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_wrap_enum( ParserString value )
{
    return validate__fx_sampler_wrap_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_wrap_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_wrap_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_wrap_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__wrap_s____fx_sampler_wrap_enum( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->wrap_t != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: wrap_t" ) )
                return false;
        }
        if ( parentValidationData->wrap_p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: wrap_p" ) )
                return false;
        }
        if ( parentValidationData->minfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: minfilter" ) )
                return false;
        }
        if ( parentValidationData->magfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: magfilter" ) )
                return false;
        }
        if ( parentValidationData->mipfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: mipfilter" ) )
                return false;
        }
        if ( parentValidationData->border_color != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: border_color" ) )
                return false;
        }
        if ( parentValidationData->mip_max_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: mip_max_level" ) )
                return false;
        }
        if ( parentValidationData->mip_min_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: mip_min_level" ) )
                return false;
        }
        if ( parentValidationData->mip_bias != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: mip_bias" ) )
                return false;
        }
        if ( parentValidationData->max_anisotropy != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: max_anisotropy" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->wrap_s >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->wrap_s++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_S;
        }
//        parentValidationData->wrap_s++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_S;
        }
//        parentValidationData->wrap_s++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_S;
        }
//        parentValidationData->wrap_s++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_S;
        }
//        parentValidationData->wrap_s++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_S;
        }
//        parentValidationData->wrap_s++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_S;
        }
//        parentValidationData->wrap_s++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__wrap_s____fx_sampler_wrap_enum()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__wrap_t____fx_sampler_wrap_enum( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->wrap_p != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, "sibling: wrap_p" ) )
                return false;
        }
        if ( parentValidationData->minfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, "sibling: minfilter" ) )
                return false;
        }
        if ( parentValidationData->magfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, "sibling: magfilter" ) )
                return false;
        }
        if ( parentValidationData->mipfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, "sibling: mipfilter" ) )
                return false;
        }
        if ( parentValidationData->border_color != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, "sibling: border_color" ) )
                return false;
        }
        if ( parentValidationData->mip_max_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, "sibling: mip_max_level" ) )
                return false;
        }
        if ( parentValidationData->mip_min_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, "sibling: mip_min_level" ) )
                return false;
        }
        if ( parentValidationData->mip_bias != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, "sibling: mip_bias" ) )
                return false;
        }
        if ( parentValidationData->max_anisotropy != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, "sibling: max_anisotropy" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->wrap_t >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->wrap_t++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_T;
        }
//        parentValidationData->wrap_t++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_T;
        }
//        parentValidationData->wrap_t++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_T;
        }
//        parentValidationData->wrap_t++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_T;
        }
//        parentValidationData->wrap_t++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_T;
        }
//        parentValidationData->wrap_t++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_T;
        }
//        parentValidationData->wrap_t++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__wrap_t____fx_sampler_wrap_enum()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__wrap_p( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->minfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, "sibling: minfilter" ) )
                return false;
        }
        if ( parentValidationData->magfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, "sibling: magfilter" ) )
                return false;
        }
        if ( parentValidationData->mipfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, "sibling: mipfilter" ) )
                return false;
        }
        if ( parentValidationData->border_color != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, "sibling: border_color" ) )
                return false;
        }
        if ( parentValidationData->mip_max_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, "sibling: mip_max_level" ) )
                return false;
        }
        if ( parentValidationData->mip_min_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, "sibling: mip_min_level" ) )
                return false;
        }
        if ( parentValidationData->mip_bias != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, "sibling: mip_bias" ) )
                return false;
        }
        if ( parentValidationData->max_anisotropy != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, "sibling: max_anisotropy" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->wrap_p >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->wrap_p++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_P;
        }
//        parentValidationData->wrap_p++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_P;
        }
//        parentValidationData->wrap_p++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_P;
        }
//        parentValidationData->wrap_p++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_P;
        }
//        parentValidationData->wrap_p++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_P;
        }
//        parentValidationData->wrap_p++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_P, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_P;
        }
//        parentValidationData->wrap_p++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__wrap_p()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_min_filter_enum( ParserString value )
{
    return validate__fx_sampler_min_filter_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_min_filter_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_min_filter_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_min_filter_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__minfilter( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->magfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, "sibling: magfilter" ) )
                return false;
        }
        if ( parentValidationData->mipfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, "sibling: mipfilter" ) )
                return false;
        }
        if ( parentValidationData->border_color != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, "sibling: border_color" ) )
                return false;
        }
        if ( parentValidationData->mip_max_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, "sibling: mip_max_level" ) )
                return false;
        }
        if ( parentValidationData->mip_min_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, "sibling: mip_min_level" ) )
                return false;
        }
        if ( parentValidationData->mip_bias != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, "sibling: mip_bias" ) )
                return false;
        }
        if ( parentValidationData->max_anisotropy != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, "sibling: max_anisotropy" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->minfilter >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->minfilter++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINFILTER;
        }
//        parentValidationData->minfilter++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINFILTER;
        }
//        parentValidationData->minfilter++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINFILTER;
        }
//        parentValidationData->minfilter++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINFILTER;
        }
//        parentValidationData->minfilter++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINFILTER;
        }
//        parentValidationData->minfilter++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINFILTER;
        }
//        parentValidationData->minfilter++;

        break;
        }

        case 67:
        {
        sampler2D____gles_sampler_type__ValidationData* parentValidationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXCOORD) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINFILTER;
        }
//        parentValidationData->minfilter++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__minfilter()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_mag_filter_enum( ParserString value )
{
    return validate__fx_sampler_mag_filter_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_mag_filter_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_mag_filter_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_mag_filter_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__magfilter( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mipfilter != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, "sibling: mipfilter" ) )
                return false;
        }
        if ( parentValidationData->border_color != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, "sibling: border_color" ) )
                return false;
        }
        if ( parentValidationData->mip_max_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, "sibling: mip_max_level" ) )
                return false;
        }
        if ( parentValidationData->mip_min_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, "sibling: mip_min_level" ) )
                return false;
        }
        if ( parentValidationData->mip_bias != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, "sibling: mip_bias" ) )
                return false;
        }
        if ( parentValidationData->max_anisotropy != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, "sibling: max_anisotropy" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->magfilter >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->magfilter++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAGFILTER;
        }
//        parentValidationData->magfilter++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAGFILTER;
        }
//        parentValidationData->magfilter++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAGFILTER;
        }
//        parentValidationData->magfilter++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAGFILTER;
        }
//        parentValidationData->magfilter++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAGFILTER;
        }
//        parentValidationData->magfilter++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAGFILTER;
        }
//        parentValidationData->magfilter++;

        break;
        }

        case 67:
        {
        sampler2D____gles_sampler_type__ValidationData* parentValidationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXCOORD) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAGFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAGFILTER;
        }
//        parentValidationData->magfilter++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__magfilter()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_mip_filter_enum( ParserString value )
{
    return validate__fx_sampler_mip_filter_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_mip_filter_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_mip_filter_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_sampler_mip_filter_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mipfilter( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->border_color != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, "sibling: border_color" ) )
                return false;
        }
        if ( parentValidationData->mip_max_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, "sibling: mip_max_level" ) )
                return false;
        }
        if ( parentValidationData->mip_min_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, "sibling: mip_min_level" ) )
                return false;
        }
        if ( parentValidationData->mip_bias != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, "sibling: mip_bias" ) )
                return false;
        }
        if ( parentValidationData->max_anisotropy != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, "sibling: max_anisotropy" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->mipfilter >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mipfilter++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIPFILTER;
        }
//        parentValidationData->mipfilter++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIPFILTER;
        }
//        parentValidationData->mipfilter++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIPFILTER;
        }
//        parentValidationData->mipfilter++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIPFILTER;
        }
//        parentValidationData->mipfilter++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIPFILTER;
        }
//        parentValidationData->mipfilter++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIPFILTER;
        }
//        parentValidationData->mipfilter++;

        break;
        }

        case 67:
        {
        sampler2D____gles_sampler_type__ValidationData* parentValidationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXCOORD) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIPFILTER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIPFILTER;
        }
//        parentValidationData->mipfilter++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mipfilter()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_color_type( const float* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_color_type__stream( const float* value, size_t length )
{

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_color_type__streamEnd( const float* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__border_color( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mip_max_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, "sibling: mip_max_level" ) )
                return false;
        }
        if ( parentValidationData->mip_min_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, "sibling: mip_min_level" ) )
                return false;
        }
        if ( parentValidationData->mip_bias != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, "sibling: mip_bias" ) )
                return false;
        }
        if ( parentValidationData->max_anisotropy != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, "sibling: max_anisotropy" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->border_color >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->border_color++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BORDER_COLOR;
        }
//        parentValidationData->border_color++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BORDER_COLOR;
        }
//        parentValidationData->border_color++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BORDER_COLOR;
        }
//        parentValidationData->border_color++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BORDER_COLOR;
        }
//        parentValidationData->border_color++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BORDER_COLOR;
        }
//        parentValidationData->border_color++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BORDER_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BORDER_COLOR;
        }
//        parentValidationData->border_color++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        border_color__ValidationData* validationData = (border_color__ValidationData*)mValidationDataStack.newObject(sizeof(border_color__ValidationData));
        memset(validationData, 0, sizeof(border_color__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__border_color()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mip_max_level( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mip_min_level != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, "sibling: mip_min_level" ) )
                return false;
        }
        if ( parentValidationData->mip_bias != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, "sibling: mip_bias" ) )
                return false;
        }
        if ( parentValidationData->max_anisotropy != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, "sibling: max_anisotropy" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->mip_max_level >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mip_max_level++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MAX_LEVEL;
        }
//        parentValidationData->mip_max_level++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MAX_LEVEL;
        }
//        parentValidationData->mip_max_level++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MAX_LEVEL;
        }
//        parentValidationData->mip_max_level++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MAX_LEVEL;
        }
//        parentValidationData->mip_max_level++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MAX_LEVEL;
        }
//        parentValidationData->mip_max_level++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MAX_LEVEL;
        }
//        parentValidationData->mip_max_level++;

        break;
        }

        case 67:
        {
        sampler2D____gles_sampler_type__ValidationData* parentValidationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXCOORD) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MAX_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MAX_LEVEL;
        }
//        parentValidationData->mip_max_level++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mip_max_level()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mip_min_level( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mip_bias != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIP_MIN_LEVEL, (const ParserChar*)0, "sibling: mip_bias" ) )
                return false;
        }
        if ( parentValidationData->max_anisotropy != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIP_MIN_LEVEL, (const ParserChar*)0, "sibling: max_anisotropy" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIP_MIN_LEVEL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->mip_min_level >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MIP_MIN_LEVEL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mip_min_level++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MIN_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MIN_LEVEL;
        }
//        parentValidationData->mip_min_level++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MIN_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MIN_LEVEL;
        }
//        parentValidationData->mip_min_level++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MIN_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MIN_LEVEL;
        }
//        parentValidationData->mip_min_level++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MIN_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MIN_LEVEL;
        }
//        parentValidationData->mip_min_level++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MIN_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MIN_LEVEL;
        }
//        parentValidationData->mip_min_level++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_MIN_LEVEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_MIN_LEVEL;
        }
//        parentValidationData->mip_min_level++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mip_min_level()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mip_bias( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->max_anisotropy != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIP_BIAS, (const ParserChar*)0, "sibling: max_anisotropy" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIP_BIAS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->mip_bias >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MIP_BIAS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mip_bias++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_BIAS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_BIAS;
        }
//        parentValidationData->mip_bias++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_BIAS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_BIAS;
        }
//        parentValidationData->mip_bias++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_BIAS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_BIAS;
        }
//        parentValidationData->mip_bias++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_BIAS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_BIAS;
        }
//        parentValidationData->mip_bias++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_BIAS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_BIAS;
        }
//        parentValidationData->mip_bias++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_BIAS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_BIAS;
        }
//        parentValidationData->mip_bias++;

        break;
        }

        case 67:
        {
        sampler2D____gles_sampler_type__ValidationData* parentValidationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXCOORD) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIP_BIAS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIP_BIAS;
        }
//        parentValidationData->mip_bias++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mip_bias()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__max_anisotropy( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 87:
        {
        sampler_states__ValidationData* parentValidationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MAX_ANISOTROPY, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->max_anisotropy >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MAX_ANISOTROPY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->max_anisotropy++;

        break;
        }

        case 416:
        {
        sampler1D__ValidationData* parentValidationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX_ANISOTROPY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX_ANISOTROPY;
        }
//        parentValidationData->max_anisotropy++;

        break;
        }

        case 66:
        {
        sampler2D____fx_sampler2D_type__ValidationData* parentValidationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX_ANISOTROPY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX_ANISOTROPY;
        }
//        parentValidationData->max_anisotropy++;

        break;
        }

        case 444:
        {
        sampler3D__ValidationData* parentValidationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX_ANISOTROPY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX_ANISOTROPY;
        }
//        parentValidationData->max_anisotropy++;

        break;
        }

        case 445:
        {
        samplerCUBE__ValidationData* parentValidationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX_ANISOTROPY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX_ANISOTROPY;
        }
//        parentValidationData->max_anisotropy++;

        break;
        }

        case 305:
        {
        samplerDEPTH__ValidationData* parentValidationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX_ANISOTROPY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX_ANISOTROPY;
        }
//        parentValidationData->max_anisotropy++;

        break;
        }

        case 110:
        {
        samplerRECT__ValidationData* parentValidationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX_ANISOTROPY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX_ANISOTROPY;
        }
//        parentValidationData->max_anisotropy++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__max_anisotropy()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sampler2D____fx_sampler2D_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 657:
        {
        newparam____fx_common_newparam_type__ValidationData* parentValidationData = (newparam____fx_common_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        sampler2D____fx_sampler2D_type__ValidationData* validationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.newObject(sizeof(sampler2D____fx_sampler2D_type__ValidationData));
        memset(validationData, 0, sizeof(sampler2D____fx_sampler2D_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sampler2D____fx_sampler2D_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler2D____fx_sampler2D_type__ValidationData* validationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sampler3D( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER3D;
        }
//        parentValidationData->sampler3D++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        sampler3D__ValidationData* validationData = (sampler3D__ValidationData*)mValidationDataStack.newObject(sizeof(sampler3D__ValidationData));
        memset(validationData, 0, sizeof(sampler3D__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sampler3D()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler3D__ValidationData* validationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SAMPLER3D, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__samplerCUBE( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERCUBE;
        }
//        parentValidationData->samplerCUBE++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        samplerCUBE__ValidationData* validationData = (samplerCUBE__ValidationData*)mValidationDataStack.newObject(sizeof(samplerCUBE__ValidationData));
        memset(validationData, 0, sizeof(samplerCUBE__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__samplerCUBE()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        samplerCUBE__ValidationData* validationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SAMPLERCUBE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__samplerRECT( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERRECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERRECT;
        }
//        parentValidationData->samplerRECT++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERRECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERRECT;
        }
//        parentValidationData->samplerRECT++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERRECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERRECT;
        }
//        parentValidationData->samplerRECT++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERRECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERRECT;
        }
//        parentValidationData->samplerRECT++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERRECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERRECT;
        }
//        parentValidationData->samplerRECT++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERRECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERRECT;
        }
//        parentValidationData->samplerRECT++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERRECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERRECT;
        }
//        parentValidationData->samplerRECT++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERRECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERRECT;
        }
//        parentValidationData->samplerRECT++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        samplerRECT__ValidationData* validationData = (samplerRECT__ValidationData*)mValidationDataStack.newObject(sizeof(samplerRECT__ValidationData));
        memset(validationData, 0, sizeof(samplerRECT__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__samplerRECT()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        samplerRECT__ValidationData* validationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SAMPLERRECT, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__samplerDEPTH( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLERDEPTH;
        }
//        parentValidationData->samplerDEPTH++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        samplerDEPTH__ValidationData* validationData = (samplerDEPTH__ValidationData*)mValidationDataStack.newObject(sizeof(samplerDEPTH__ValidationData));
        memset(validationData, 0, sizeof(samplerDEPTH__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__samplerDEPTH()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        samplerDEPTH__ValidationData* validationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SAMPLERDEPTH, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__enum____string( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 739:
        {
        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 658:
        {
        newparam____fx_newparam_type__ValidationData* parentValidationData = (newparam____fx_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__enum____string()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_COMMON( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effect__ValidationData* parentValidationData = (effect__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_BRIDGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_CG) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_COMMON) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES2) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLSL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PROFILE_COMMON, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PROFILE_COMMON;
        }
//        parentValidationData->profile_COMMON++;

        profile_COMMON__ValidationData* validationData = (profile_COMMON__ValidationData*)mValidationDataStack.newObject(sizeof(profile_COMMON__ValidationData));
        memset(validationData, 0, sizeof(profile_COMMON__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_COMMON()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_COMMON__ValidationData* validationData = (profile_COMMON__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PROFILE_COMMON, (const ParserChar*)0, "child: technique" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__newparam____fx_common_newparam_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_COMMON__ValidationData* parentValidationData = (profile_COMMON__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->newparam++;

        newparam____fx_common_newparam_type__ValidationData* validationData = (newparam____fx_common_newparam_type__ValidationData*)mValidationDataStack.newObject(sizeof(newparam____fx_common_newparam_type__ValidationData));
        memset(validationData, 0, sizeof(newparam____fx_common_newparam_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__newparam____fx_common_newparam_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____fx_common_newparam_type__ValidationData* validationData = (newparam____fx_common_newparam_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_common_type____technique( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_COMMON__ValidationData* parentValidationData = (profile_COMMON__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique++;

        profile_common_type____technique__ValidationData* validationData = (profile_common_type____technique__ValidationData*)mValidationDataStack.newObject(sizeof(profile_common_type____technique__ValidationData));
        memset(validationData, 0, sizeof(profile_common_type____technique__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_common_type____technique()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_common_type____technique__ValidationData* validationData = (profile_common_type____technique__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BLINN) || (validationData->validation_current_state == HASH_ELEMENT_CONSTANT) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_LAMBERT) || (validationData->validation_current_state == HASH_ELEMENT_PHONG) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_COMMON__technique__constant( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_common_type____technique__ValidationData* parentValidationData = (profile_common_type____technique__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONSTANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONSTANT;
        }
//        parentValidationData->constant++;

        profile_COMMON__technique__constant__ValidationData* validationData = (profile_COMMON__technique__constant__ValidationData*)mValidationDataStack.newObject(sizeof(profile_COMMON__technique__constant__ValidationData));
        memset(validationData, 0, sizeof(profile_COMMON__technique__constant__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_COMMON__technique__constant()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_COMMON__technique__constant__ValidationData* validationData = (profile_COMMON__technique__constant__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__emission( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 728:
        {
        blinn__ValidationData* parentValidationData = (blinn__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ambient != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: ambient" ) )
                return false;
        }
        if ( parentValidationData->diffuse != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: diffuse" ) )
                return false;
        }
        if ( parentValidationData->specular != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: specular" ) )
                return false;
        }
        if ( parentValidationData->shininess != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: shininess" ) )
                return false;
        }
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->emission >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EMISSION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->emission++;

        break;
        }

        case 188:
        {
        profile_COMMON__technique__constant__ValidationData* parentValidationData = (profile_COMMON__technique__constant__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->emission >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EMISSION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->emission++;

        break;
        }

        case 317:
        {
        lambert__ValidationData* parentValidationData = (lambert__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ambient != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: ambient" ) )
                return false;
        }
        if ( parentValidationData->diffuse != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: diffuse" ) )
                return false;
        }
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->emission >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EMISSION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->emission++;

        break;
        }

        case 722:
        {
        phong__ValidationData* parentValidationData = (phong__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ambient != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: ambient" ) )
                return false;
        }
        if ( parentValidationData->diffuse != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: diffuse" ) )
                return false;
        }
        if ( parentValidationData->specular != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: specular" ) )
                return false;
        }
        if ( parentValidationData->shininess != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: shininess" ) )
                return false;
        }
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->emission >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EMISSION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->emission++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        emission__ValidationData* validationData = (emission__ValidationData*)mValidationDataStack.newObject(sizeof(emission__ValidationData));
        memset(validationData, 0, sizeof(emission__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__emission()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        emission__ValidationData* validationData = (emission__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->color != 0 )
            if ( validationData->color < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "child: color" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->texture != 0 )
            if ( validationData->texture < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_EMISSION, (const ParserChar*)0, "child: texture" ) )
                    return false;
        if ( validationData->color == 0 && validationData->param == 0 && validationData->texture == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_EMISSION, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fx_common_color_or_texture_type____color( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 34:
        {
        emission__ValidationData* parentValidationData = (emission__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->texture != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: texture" ) )
                return false;
        }
        if ( parentValidationData->color >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_COLOR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->color++;

        break;
        }

        case 251:
        {
        transparent__ValidationData* parentValidationData = (transparent__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->texture != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: texture" ) )
                return false;
        }
        if ( parentValidationData->color >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_COLOR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->color++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        fx_common_color_or_texture_type____color__ValidationData* validationData = (fx_common_color_or_texture_type____color__ValidationData*)mValidationDataStack.newObject(sizeof(fx_common_color_or_texture_type____color__ValidationData));
        memset(validationData, 0, sizeof(fx_common_color_or_texture_type____color__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fx_common_color_or_texture_type____color()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fx_common_color_or_texture_type____param( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 34:
        {
        emission__ValidationData* parentValidationData = (emission__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->texture != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: texture" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 251:
        {
        transparent__ValidationData* parentValidationData = (transparent__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->texture != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: texture" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fx_common_color_or_texture_type____param()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texture( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 34:
        {
        emission__ValidationData* parentValidationData = (emission__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TEXTURE, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TEXTURE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->texture >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TEXTURE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->texture++;

        break;
        }

        case 251:
        {
        transparent__ValidationData* parentValidationData = (transparent__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TEXTURE, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TEXTURE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->texture >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TEXTURE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->texture++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        texture__ValidationData* validationData = (texture__ValidationData*)mValidationDataStack.newObject(sizeof(texture__ValidationData));
        memset(validationData, 0, sizeof(texture__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texture()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texture__ValidationData* validationData = (texture__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__reflective( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 728:
        {
        blinn__ValidationData* parentValidationData = (blinn__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->reflective >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->reflective++;

        break;
        }

        case 188:
        {
        profile_COMMON__technique__constant__ValidationData* parentValidationData = (profile_COMMON__technique__constant__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->reflective >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->reflective++;

        break;
        }

        case 317:
        {
        lambert__ValidationData* parentValidationData = (lambert__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->reflective >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->reflective++;

        break;
        }

        case 722:
        {
        phong__ValidationData* parentValidationData = (phong__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->reflective >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->reflective++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        emission__ValidationData* validationData = (emission__ValidationData*)mValidationDataStack.newObject(sizeof(emission__ValidationData));
        memset(validationData, 0, sizeof(emission__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__reflective()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        emission__ValidationData* validationData = (emission__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->color != 0 )
            if ( validationData->color < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "child: color" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->texture != 0 )
            if ( validationData->texture < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, "child: texture" ) )
                    return false;
        if ( validationData->color == 0 && validationData->param == 0 && validationData->texture == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_REFLECTIVE, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__reflectivity( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 728:
        {
        blinn__ValidationData* parentValidationData = (blinn__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->reflectivity >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->reflectivity++;

        break;
        }

        case 188:
        {
        profile_COMMON__technique__constant__ValidationData* parentValidationData = (profile_COMMON__technique__constant__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->reflectivity >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->reflectivity++;

        break;
        }

        case 317:
        {
        lambert__ValidationData* parentValidationData = (lambert__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->reflectivity >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->reflectivity++;

        break;
        }

        case 722:
        {
        phong__ValidationData* parentValidationData = (phong__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->reflectivity >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->reflectivity++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        reflectivity__ValidationData* validationData = (reflectivity__ValidationData*)mValidationDataStack.newObject(sizeof(reflectivity__ValidationData));
        memset(validationData, 0, sizeof(reflectivity__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__reflectivity()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        reflectivity__ValidationData* validationData = (reflectivity__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_REFLECTIVITY, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fx_common_float_or_param_type____float( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        reflectivity__ValidationData* parentValidationData = (reflectivity__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FLOAT, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->_float >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FLOAT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->_float++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fx_common_float_or_param_type____float()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fx_common_float_or_param_type____param( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        reflectivity__ValidationData* parentValidationData = (reflectivity__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fx_common_float_or_param_type____param()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_opaque_enum( ParserString value )
{
    return validate__fx_opaque_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_opaque_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_opaque_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_opaque_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__transparent( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 728:
        {
        blinn__ValidationData* parentValidationData = (blinn__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->transparent >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->transparent++;

        break;
        }

        case 188:
        {
        profile_COMMON__technique__constant__ValidationData* parentValidationData = (profile_COMMON__technique__constant__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->transparent >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->transparent++;

        break;
        }

        case 317:
        {
        lambert__ValidationData* parentValidationData = (lambert__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->transparent >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->transparent++;

        break;
        }

        case 722:
        {
        phong__ValidationData* parentValidationData = (phong__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->transparent >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->transparent++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        transparent__ValidationData* validationData = (transparent__ValidationData*)mValidationDataStack.newObject(sizeof(transparent__ValidationData));
        memset(validationData, 0, sizeof(transparent__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__transparent()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        transparent__ValidationData* validationData = (transparent__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->color != 0 )
            if ( validationData->color < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "child: color" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->texture != 0 )
            if ( validationData->texture < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, "child: texture" ) )
                    return false;
        if ( validationData->color == 0 && validationData->param == 0 && validationData->texture == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TRANSPARENT, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__transparency( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 728:
        {
        blinn__ValidationData* parentValidationData = (blinn__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->transparency >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->transparency++;

        break;
        }

        case 188:
        {
        profile_COMMON__technique__constant__ValidationData* parentValidationData = (profile_COMMON__technique__constant__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->transparency >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->transparency++;

        break;
        }

        case 317:
        {
        lambert__ValidationData* parentValidationData = (lambert__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->transparency >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->transparency++;

        break;
        }

        case 722:
        {
        phong__ValidationData* parentValidationData = (phong__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->transparency >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->transparency++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        reflectivity__ValidationData* validationData = (reflectivity__ValidationData*)mValidationDataStack.newObject(sizeof(reflectivity__ValidationData));
        memset(validationData, 0, sizeof(reflectivity__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__transparency()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        reflectivity__ValidationData* validationData = (reflectivity__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TRANSPARENCY, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__index_of_refraction( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 728:
        {
        blinn__ValidationData* parentValidationData = (blinn__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->index_of_refraction >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INDEX_OF_REFRACTION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->index_of_refraction++;

        break;
        }

        case 188:
        {
        profile_COMMON__technique__constant__ValidationData* parentValidationData = (profile_COMMON__technique__constant__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->index_of_refraction >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INDEX_OF_REFRACTION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->index_of_refraction++;

        break;
        }

        case 317:
        {
        lambert__ValidationData* parentValidationData = (lambert__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->index_of_refraction >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INDEX_OF_REFRACTION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->index_of_refraction++;

        break;
        }

        case 722:
        {
        phong__ValidationData* parentValidationData = (phong__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->index_of_refraction >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INDEX_OF_REFRACTION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->index_of_refraction++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        reflectivity__ValidationData* validationData = (reflectivity__ValidationData*)mValidationDataStack.newObject(sizeof(reflectivity__ValidationData));
        memset(validationData, 0, sizeof(reflectivity__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__index_of_refraction()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        reflectivity__ValidationData* validationData = (reflectivity__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INDEX_OF_REFRACTION, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INDEX_OF_REFRACTION, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_INDEX_OF_REFRACTION, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__lambert( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_common_type____technique__ValidationData* parentValidationData = (profile_common_type____technique__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBERT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBERT;
        }
//        parentValidationData->lambert++;

        lambert__ValidationData* validationData = (lambert__ValidationData*)mValidationDataStack.newObject(sizeof(lambert__ValidationData));
        memset(validationData, 0, sizeof(lambert__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__lambert()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        lambert__ValidationData* validationData = (lambert__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ambient____fx_common_color_or_texture_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 728:
        {
        blinn__ValidationData* parentValidationData = (blinn__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->diffuse != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: diffuse" ) )
                return false;
        }
        if ( parentValidationData->specular != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: specular" ) )
                return false;
        }
        if ( parentValidationData->shininess != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: shininess" ) )
                return false;
        }
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->ambient >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ambient++;

        break;
        }

        case 317:
        {
        lambert__ValidationData* parentValidationData = (lambert__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->diffuse != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: diffuse" ) )
                return false;
        }
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->ambient >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ambient++;

        break;
        }

        case 722:
        {
        phong__ValidationData* parentValidationData = (phong__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->diffuse != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: diffuse" ) )
                return false;
        }
        if ( parentValidationData->specular != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: specular" ) )
                return false;
        }
        if ( parentValidationData->shininess != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: shininess" ) )
                return false;
        }
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->ambient >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ambient++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        emission__ValidationData* validationData = (emission__ValidationData*)mValidationDataStack.newObject(sizeof(emission__ValidationData));
        memset(validationData, 0, sizeof(emission__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ambient____fx_common_color_or_texture_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        emission__ValidationData* validationData = (emission__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->color != 0 )
            if ( validationData->color < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "child: color" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->texture != 0 )
            if ( validationData->texture < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "child: texture" ) )
                    return false;
        if ( validationData->color == 0 && validationData->param == 0 && validationData->texture == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__diffuse( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 728:
        {
        blinn__ValidationData* parentValidationData = (blinn__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->specular != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: specular" ) )
                return false;
        }
        if ( parentValidationData->shininess != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: shininess" ) )
                return false;
        }
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->diffuse >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->diffuse++;

        break;
        }

        case 317:
        {
        lambert__ValidationData* parentValidationData = (lambert__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->diffuse >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->diffuse++;

        break;
        }

        case 722:
        {
        phong__ValidationData* parentValidationData = (phong__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->specular != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: specular" ) )
                return false;
        }
        if ( parentValidationData->shininess != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: shininess" ) )
                return false;
        }
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->diffuse >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->diffuse++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        emission__ValidationData* validationData = (emission__ValidationData*)mValidationDataStack.newObject(sizeof(emission__ValidationData));
        memset(validationData, 0, sizeof(emission__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__diffuse()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        emission__ValidationData* validationData = (emission__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->color != 0 )
            if ( validationData->color < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "child: color" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->texture != 0 )
            if ( validationData->texture < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, "child: texture" ) )
                    return false;
        if ( validationData->color == 0 && validationData->param == 0 && validationData->texture == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_DIFFUSE, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__phong( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_common_type____technique__ValidationData* parentValidationData = (profile_common_type____technique__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PHONG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PHONG;
        }
//        parentValidationData->phong++;

        phong__ValidationData* validationData = (phong__ValidationData*)mValidationDataStack.newObject(sizeof(phong__ValidationData));
        memset(validationData, 0, sizeof(phong__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__phong()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        phong__ValidationData* validationData = (phong__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__specular( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 728:
        {
        blinn__ValidationData* parentValidationData = (blinn__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->shininess != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: shininess" ) )
                return false;
        }
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->specular >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->specular++;

        break;
        }

        case 722:
        {
        phong__ValidationData* parentValidationData = (phong__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->shininess != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: shininess" ) )
                return false;
        }
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->specular >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->specular++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        emission__ValidationData* validationData = (emission__ValidationData*)mValidationDataStack.newObject(sizeof(emission__ValidationData));
        memset(validationData, 0, sizeof(emission__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__specular()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        emission__ValidationData* validationData = (emission__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->color != 0 )
            if ( validationData->color < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "child: color" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->texture != 0 )
            if ( validationData->texture < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, "child: texture" ) )
                    return false;
        if ( validationData->color == 0 && validationData->param == 0 && validationData->texture == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_SPECULAR, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__shininess( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 728:
        {
        blinn__ValidationData* parentValidationData = (blinn__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->shininess >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SHININESS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->shininess++;

        break;
        }

        case 722:
        {
        phong__ValidationData* parentValidationData = (phong__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->reflective != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "sibling: reflective" ) )
                return false;
        }
        if ( parentValidationData->reflectivity != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "sibling: reflectivity" ) )
                return false;
        }
        if ( parentValidationData->transparent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "sibling: transparent" ) )
                return false;
        }
        if ( parentValidationData->transparency != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "sibling: transparency" ) )
                return false;
        }
        if ( parentValidationData->index_of_refraction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "sibling: index_of_refraction" ) )
                return false;
        }
        if ( parentValidationData->shininess >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SHININESS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->shininess++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        reflectivity__ValidationData* validationData = (reflectivity__ValidationData*)mValidationDataStack.newObject(sizeof(reflectivity__ValidationData));
        memset(validationData, 0, sizeof(reflectivity__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__shininess()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        reflectivity__ValidationData* validationData = (reflectivity__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SHININESS, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_SHININESS, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__blinn( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_common_type____technique__ValidationData* parentValidationData = (profile_common_type____technique__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLINN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLINN;
        }
//        parentValidationData->blinn++;

        blinn__ValidationData* validationData = (blinn__ValidationData*)mValidationDataStack.newObject(sizeof(blinn__ValidationData));
        memset(validationData, 0, sizeof(blinn__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__blinn()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        blinn__ValidationData* validationData = (blinn__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_BRIDGE( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effect__ValidationData* parentValidationData = (effect__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_BRIDGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_CG) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_COMMON) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES2) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLSL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PROFILE_BRIDGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PROFILE_BRIDGE;
        }
//        parentValidationData->profile_BRIDGE++;

        profile_BRIDGE__ValidationData* validationData = (profile_BRIDGE__ValidationData*)mValidationDataStack.newObject(sizeof(profile_BRIDGE__ValidationData));
        memset(validationData, 0, sizeof(profile_BRIDGE__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_BRIDGE()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_BRIDGE__ValidationData* validationData = (profile_BRIDGE__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effect__ValidationData* parentValidationData = (effect__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_BRIDGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_CG) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_COMMON) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES2) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLSL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PROFILE_GLES2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PROFILE_GLES2;
        }
//        parentValidationData->profile_GLES2++;

        profile_GLES2__ValidationData* validationData = (profile_GLES2__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__ValidationData* validationData = (profile_GLES2__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_PROFILE_GLES2, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__include( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 143:
        {
        profile_CG__ValidationData* parentValidationData = (profile_CG__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INCLUDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INCLUDE;
        }
//        parentValidationData->include++;

        break;
        }

        case 452:
        {
        profile_GLES2__ValidationData* parentValidationData = (profile_GLES2__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INCLUDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INCLUDE;
        }
//        parentValidationData->include++;

        break;
        }

        case 148:
        {
        profile_GLSL__ValidationData* parentValidationData = (profile_GLSL__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INCLUDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INCLUDE;
        }
//        parentValidationData->include++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__include()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__code( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 143:
        {
        profile_CG__ValidationData* parentValidationData = (profile_CG__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODE;
        }
//        parentValidationData->code++;

        break;
        }

        case 452:
        {
        profile_GLES2__ValidationData* parentValidationData = (profile_GLES2__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODE;
        }
//        parentValidationData->code++;

        break;
        }

        case 148:
        {
        profile_GLSL__ValidationData* parentValidationData = (profile_GLSL__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODE;
        }
//        parentValidationData->code++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__code()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_gles2_type____newparam( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__ValidationData* parentValidationData = (profile_GLES2__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEWPARAM;
        }
//        parentValidationData->newparam++;

        profile_gles2_type____newparam__ValidationData* validationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.newObject(sizeof(profile_gles2_type____newparam__ValidationData));
        memset(validationData, 0, sizeof(profile_gles2_type____newparam__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_gles2_type____newparam()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_gles2_type____newparam__ValidationData* validationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BVEC2) || (validationData->validation_current_state == HASH_ELEMENT_BVEC3) || (validationData->validation_current_state == HASH_ELEMENT_BVEC4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_IVEC2) || (validationData->validation_current_state == HASH_ELEMENT_IVEC3) || (validationData->validation_current_state == HASH_ELEMENT_IVEC4) || (validationData->validation_current_state == HASH_ELEMENT_MAT2) || (validationData->validation_current_state == HASH_ELEMENT_MAT3) || (validationData->validation_current_state == HASH_ELEMENT_MAT4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (validationData->validation_current_state == HASH_ELEMENT_VEC2) || (validationData->validation_current_state == HASH_ELEMENT_VEC3) || (validationData->validation_current_state == HASH_ELEMENT_VEC4) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bvec2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC2;
        }
//        parentValidationData->bvec2++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC2;
        }
//        parentValidationData->bvec2++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC2;
        }
//        parentValidationData->bvec2++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC2;
        }
//        parentValidationData->bvec2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool2__ValidationData* validationData = (bool2__ValidationData*)mValidationDataStack.newObject(sizeof(bool2__ValidationData));
        memset(validationData, 0, sizeof(bool2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bvec2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bvec3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC3;
        }
//        parentValidationData->bvec3++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC3;
        }
//        parentValidationData->bvec3++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC3;
        }
//        parentValidationData->bvec3++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC3;
        }
//        parentValidationData->bvec3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool3__ValidationData* validationData = (bool3__ValidationData*)mValidationDataStack.newObject(sizeof(bool3__ValidationData));
        memset(validationData, 0, sizeof(bool3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bvec3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bvec4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC4;
        }
//        parentValidationData->bvec4++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC4;
        }
//        parentValidationData->bvec4++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC4;
        }
//        parentValidationData->bvec4++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVEC4;
        }
//        parentValidationData->bvec4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool4__ValidationData* validationData = (bool4__ValidationData*)mValidationDataStack.newObject(sizeof(bool4__ValidationData));
        memset(validationData, 0, sizeof(bool4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bvec4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__vec2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC2;
        }
//        parentValidationData->vec2++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC2;
        }
//        parentValidationData->vec2++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC2;
        }
//        parentValidationData->vec2++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC2;
        }
//        parentValidationData->vec2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        radius____float2_type__ValidationData* validationData = (radius____float2_type__ValidationData*)mValidationDataStack.newObject(sizeof(radius____float2_type__ValidationData));
        memset(validationData, 0, sizeof(radius____float2_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__vec2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__vec3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC3;
        }
//        parentValidationData->vec3++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC3;
        }
//        parentValidationData->vec3++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC3;
        }
//        parentValidationData->vec3++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC3;
        }
//        parentValidationData->vec3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__vec3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__vec4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC4;
        }
//        parentValidationData->vec4++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC4;
        }
//        parentValidationData->vec4++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC4;
        }
//        parentValidationData->vec4++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VEC4;
        }
//        parentValidationData->vec4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        orient__ValidationData* validationData = (orient__ValidationData*)mValidationDataStack.newObject(sizeof(orient__ValidationData));
        memset(validationData, 0, sizeof(orient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__vec4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mat2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT2;
        }
//        parentValidationData->mat2++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT2;
        }
//        parentValidationData->mat2++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT2;
        }
//        parentValidationData->mat2++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT2;
        }
//        parentValidationData->mat2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float2x2__ValidationData* validationData = (float2x2__ValidationData*)mValidationDataStack.newObject(sizeof(float2x2__ValidationData));
        memset(validationData, 0, sizeof(float2x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mat2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mat3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT3;
        }
//        parentValidationData->mat3++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT3;
        }
//        parentValidationData->mat3++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT3;
        }
//        parentValidationData->mat3++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT3;
        }
//        parentValidationData->mat3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float3x3__ValidationData* validationData = (float3x3__ValidationData*)mValidationDataStack.newObject(sizeof(float3x3__ValidationData));
        memset(validationData, 0, sizeof(float3x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mat3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mat4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT4;
        }
//        parentValidationData->mat4++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT4;
        }
//        parentValidationData->mat4++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT4;
        }
//        parentValidationData->mat4++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAT4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAT4;
        }
//        parentValidationData->mat4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bind_shape_matrix__ValidationData* validationData = (bind_shape_matrix__ValidationData*)mValidationDataStack.newObject(sizeof(bind_shape_matrix__ValidationData));
        memset(validationData, 0, sizeof(bind_shape_matrix__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mat4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ivec2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC2;
        }
//        parentValidationData->ivec2++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC2;
        }
//        parentValidationData->ivec2++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC2;
        }
//        parentValidationData->ivec2++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC2;
        }
//        parentValidationData->ivec2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int2__ValidationData* validationData = (int2__ValidationData*)mValidationDataStack.newObject(sizeof(int2__ValidationData));
        memset(validationData, 0, sizeof(int2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ivec2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ivec3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC3;
        }
//        parentValidationData->ivec3++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC3;
        }
//        parentValidationData->ivec3++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC3;
        }
//        parentValidationData->ivec3++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC3;
        }
//        parentValidationData->ivec3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int3__ValidationData* validationData = (int3__ValidationData*)mValidationDataStack.newObject(sizeof(int3__ValidationData));
        memset(validationData, 0, sizeof(int3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ivec3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ivec4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC4;
        }
//        parentValidationData->ivec4++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC4;
        }
//        parentValidationData->ivec4++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC4;
        }
//        parentValidationData->ivec4++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IVEC4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IVEC4;
        }
//        parentValidationData->ivec4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int4__ValidationData* validationData = (int4__ValidationData*)mValidationDataStack.newObject(sizeof(int4__ValidationData));
        memset(validationData, 0, sizeof(int4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ivec4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__enum____gl_enumeration_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__enum____gl_enumeration_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__newparam__usertype( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_USERTYPE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_USERTYPE;
        }
//        parentValidationData->usertype++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_USERTYPE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_USERTYPE;
        }
//        parentValidationData->usertype++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLES2__newparam__usertype__ValidationData* validationData = (profile_GLES2__newparam__usertype__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__newparam__usertype__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__newparam__usertype__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__newparam__usertype()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__newparam__usertype__ValidationData* validationData = (profile_GLES2__newparam__usertype__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__newparam__usertype__setparam( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__newparam__usertype__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__ValidationData*)mValidationDataStack.top();
        parentValidationData->setparam++;

        profile_GLES2__newparam__usertype__setparam__ValidationData* validationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__newparam__usertype__setparam__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__newparam__usertype__setparam__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__newparam__usertype__setparam()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__newparam__usertype__setparam__ValidationData* validationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BVEC2) || (validationData->validation_current_state == HASH_ELEMENT_BVEC3) || (validationData->validation_current_state == HASH_ELEMENT_BVEC4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_IVEC2) || (validationData->validation_current_state == HASH_ELEMENT_IVEC3) || (validationData->validation_current_state == HASH_ELEMENT_IVEC4) || (validationData->validation_current_state == HASH_ELEMENT_MAT2) || (validationData->validation_current_state == HASH_ELEMENT_MAT3) || (validationData->validation_current_state == HASH_ELEMENT_MAT4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (validationData->validation_current_state == HASH_ELEMENT_VEC2) || (validationData->validation_current_state == HASH_ELEMENT_VEC3) || (validationData->validation_current_state == HASH_ELEMENT_VEC4) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__positiveInteger( const uint64 value )
{
    if( value < 1 )
        return ParserError::ERROR_VALIDATION_MIN_INCLUSIVE;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__newparam__usertype__setparam__array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 403:
        {
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

        break;
        }

        case 373:
        {
        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

        break;
        }

        case 654:
        {
        profile_gles2_type____newparam__ValidationData* parentValidationData = (profile_gles2_type____newparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

        break;
        }

        case 740:
        {
        profile_GLES2__newparam__usertype__setparam__ValidationData* parentValidationData = (profile_GLES2__newparam__usertype__setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_IVEC4) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC2) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC3) || (parentValidationData->validation_current_state == HASH_ELEMENT_VEC4) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLES2__newparam__usertype__setparam__array__ValidationData* validationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__newparam__usertype__setparam__array__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__newparam__usertype__setparam__array__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__newparam__usertype__setparam__array()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__newparam__usertype__setparam__array__ValidationData* validationData = (profile_GLES2__newparam__usertype__setparam__array__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BVEC2) || (validationData->validation_current_state == HASH_ELEMENT_BVEC3) || (validationData->validation_current_state == HASH_ELEMENT_BVEC4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_IVEC2) || (validationData->validation_current_state == HASH_ELEMENT_IVEC3) || (validationData->validation_current_state == HASH_ELEMENT_IVEC4) || (validationData->validation_current_state == HASH_ELEMENT_MAT2) || (validationData->validation_current_state == HASH_ELEMENT_MAT3) || (validationData->validation_current_state == HASH_ELEMENT_MAT4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (validationData->validation_current_state == HASH_ELEMENT_VEC2) || (validationData->validation_current_state == HASH_ELEMENT_VEC3) || (validationData->validation_current_state == HASH_ELEMENT_VEC4) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_gles2_type____technique( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__ValidationData* parentValidationData = (profile_GLES2__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TECHNIQUE;
        }
//        parentValidationData->technique++;

        profile_gles2_type____technique__ValidationData* validationData = (profile_gles2_type____technique__ValidationData*)mValidationDataStack.newObject(sizeof(profile_gles2_type____technique__ValidationData));
        memset(validationData, 0, sizeof(profile_gles2_type____technique__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_gles2_type____technique()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_gles2_type____technique__ValidationData* validationData = (profile_gles2_type____technique__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->pass < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "child: pass" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__pass____gles2_pass_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_gles2_type____technique__ValidationData* parentValidationData = (profile_gles2_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PASS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->pass++;

        pass____gles2_pass_type__ValidationData* validationData = (pass____gles2_pass_type__ValidationData*)mValidationDataStack.newObject(sizeof(pass____gles2_pass_type__ValidationData));
        memset(validationData, 0, sizeof(pass____gles2_pass_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__pass____gles2_pass_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        pass____gles2_pass_type__ValidationData* validationData = (pass____gles2_pass_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__gles2_pass_type____states( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        pass____gles2_pass_type__ValidationData* parentValidationData = (pass____gles2_pass_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->program != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: program" ) )
                return false;
        }
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->states >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_STATES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->states++;

        gles2_pass_type____states__ValidationData* validationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.newObject(sizeof(gles2_pass_type____states__ValidationData));
        memset(validationData, 0, sizeof(gles2_pass_type____states__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__gles2_pass_type____states()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* validationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (validationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (validationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (validationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (validationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_STATES, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_color( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_COLOR;
        }
//        parentValidationData->blend_color++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_color()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_blend_equation_enum( ParserString value )
{
    return validate__gl_blend_equation_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_blend_equation_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_blend_equation_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_blend_equation_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_equation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_EQUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_EQUATION;
        }
//        parentValidationData->blend_equation++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_equation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_equation_separate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_EQUATION_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_EQUATION_SEPARATE;
        }
//        parentValidationData->blend_equation_separate++;

        profile_GLES2__technique__pass__states__blend_equation_separate__ValidationData* validationData = (profile_GLES2__technique__pass__states__blend_equation_separate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__technique__pass__states__blend_equation_separate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__technique__pass__states__blend_equation_separate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_equation_separate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_equation_separate__ValidationData* validationData = (profile_GLES2__technique__pass__states__blend_equation_separate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->rgb < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_EQUATION_SEPARATE, (const ParserChar*)0, "child: rgb" ) )
                return false;
        }
        if ( validationData->alpha < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_EQUATION_SEPARATE, (const ParserChar*)0, "child: alpha" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_equation_separate__rgb( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_equation_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__blend_equation_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RGB, (const ParserChar*)0, "sibling: alpha" ) )
                return false;
        }
        if ( parentValidationData->rgb >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RGB, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->rgb++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_equation_separate__rgb()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_equation_separate__alpha( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_equation_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__blend_equation_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->rgb < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ALPHA, (const ParserChar*)0, "sibling: rgb" ) )
                return false;
        }
        if ( parentValidationData->alpha >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ALPHA, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->alpha++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_equation_separate__alpha()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_FUNC;
        }
//        parentValidationData->blend_func++;

        profile_GLES2__technique__pass__states__blend_func__ValidationData* validationData = (profile_GLES2__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__technique__pass__states__blend_func__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__technique__pass__states__blend_func__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_func()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_func__ValidationData* validationData = (profile_GLES2__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->src < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC, (const ParserChar*)0, "child: src" ) )
                return false;
        }
        if ( validationData->dest < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC, (const ParserChar*)0, "child: dest" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_blend_enum( ParserString value )
{
    return validate__gl_blend_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_blend_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_blend_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_blend_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_func__src( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_func__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->dest != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC, (const ParserChar*)0, "sibling: dest" ) )
                return false;
        }
        if ( parentValidationData->src >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SRC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->src++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_func__src()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_func__dest( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_func__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->src < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_DEST, (const ParserChar*)0, "sibling: src" ) )
                return false;
        }
        if ( parentValidationData->dest >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DEST, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->dest++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_func__dest()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_func_separate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_FUNC_SEPARATE;
        }
//        parentValidationData->blend_func_separate++;

        profile_GLES2__technique__pass__states__blend_func_separate__ValidationData* validationData = (profile_GLES2__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__technique__pass__states__blend_func_separate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__technique__pass__states__blend_func_separate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_func_separate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_func_separate__ValidationData* validationData = (profile_GLES2__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->src_rgb < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, "child: src_rgb" ) )
                return false;
        }
        if ( validationData->dest_rgb < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, "child: dest_rgb" ) )
                return false;
        }
        if ( validationData->src_alpha < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, "child: src_alpha" ) )
                return false;
        }
        if ( validationData->dest_alpha < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, "child: dest_alpha" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_func_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->dest_rgb != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC_RGB, (const ParserChar*)0, "sibling: dest_rgb" ) )
                return false;
        }
        if ( parentValidationData->src_alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC_RGB, (const ParserChar*)0, "sibling: src_alpha" ) )
                return false;
        }
        if ( parentValidationData->dest_alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC_RGB, (const ParserChar*)0, "sibling: dest_alpha" ) )
                return false;
        }
        if ( parentValidationData->src_rgb >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SRC_RGB, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->src_rgb++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_func_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->src_rgb < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_DEST_RGB, (const ParserChar*)0, "sibling: src_rgb" ) )
                return false;
        }
        if ( parentValidationData->src_alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEST_RGB, (const ParserChar*)0, "sibling: src_alpha" ) )
                return false;
        }
        if ( parentValidationData->dest_alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEST_RGB, (const ParserChar*)0, "sibling: dest_alpha" ) )
                return false;
        }
        if ( parentValidationData->dest_rgb >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DEST_RGB, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->dest_rgb++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_func_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->dest_rgb < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_SRC_ALPHA, (const ParserChar*)0, "sibling: dest_rgb" ) )
                return false;
        }
        if ( parentValidationData->dest_alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC_ALPHA, (const ParserChar*)0, "sibling: dest_alpha" ) )
                return false;
        }
        if ( parentValidationData->src_alpha >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SRC_ALPHA, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->src_alpha++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__blend_func_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->src_alpha < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_DEST_ALPHA, (const ParserChar*)0, "sibling: src_alpha" ) )
                return false;
        }
        if ( parentValidationData->dest_alpha >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DEST_ALPHA, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->dest_alpha++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__color_mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_MASK;
        }
//        parentValidationData->color_mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__color_mask()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_face_enum( ParserString value )
{
    return validate__gl_face_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_face_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_face_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_face_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__cull_face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CULL_FACE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CULL_FACE;
        }
//        parentValidationData->cull_face++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__cull_face()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_func_enum( ParserString value )
{
    return validate__gl_func_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_func_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_func_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_func_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__depth_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_FUNC;
        }
//        parentValidationData->depth_func++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__depth_func()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__depth_mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_MASK;
        }
//        parentValidationData->depth_mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__depth_mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__depth_range( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_RANGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_RANGE;
        }
//        parentValidationData->depth_range++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__depth_range()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_front_face_enum( ParserString value )
{
    return validate__gl_front_face_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_front_face_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_front_face_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_front_face_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__front_face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FRONT_FACE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FRONT_FACE;
        }
//        parentValidationData->front_face++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__front_face()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__line_width( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_WIDTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_WIDTH;
        }
//        parentValidationData->line_width++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__line_width()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__polygon_offset( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET;
        }
//        parentValidationData->polygon_offset++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__polygon_offset()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__point_size( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE;
        }
//        parentValidationData->point_size++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__point_size()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sample_coverage( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_COVERAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_COVERAGE;
        }
//        parentValidationData->sample_coverage++;

        sample_coverage__ValidationData* validationData = (sample_coverage__ValidationData*)mValidationDataStack.newObject(sizeof(sample_coverage__ValidationData));
        memset(validationData, 0, sizeof(sample_coverage__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sample_coverage()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sample_coverage__ValidationData* validationData = (sample_coverage__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->value < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SAMPLE_COVERAGE, (const ParserChar*)0, "child: value" ) )
                return false;
        }
        if ( validationData->invert < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SAMPLE_COVERAGE, (const ParserChar*)0, "child: invert" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sample_coverage__value( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sample_coverage__ValidationData* parentValidationData = (sample_coverage__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->invert != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, "sibling: invert" ) )
                return false;
        }
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sample_coverage__value()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__invert( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sample_coverage__ValidationData* parentValidationData = (sample_coverage__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_INVERT, (const ParserChar*)0, "sibling: value" ) )
                return false;
        }
        if ( parentValidationData->invert >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INVERT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->invert++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__invert()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__scissor( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCISSOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCISSOR;
        }
//        parentValidationData->scissor++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__scissor()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_FUNC;
        }
//        parentValidationData->stencil_func++;

        profile_GLES2__technique__pass__states__stencil_func__ValidationData* validationData = (profile_GLES2__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__technique__pass__states__stencil_func__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__technique__pass__states__stencil_func__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_func()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_func__ValidationData* validationData = (profile_GLES2__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->func < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, "child: func" ) )
                return false;
        }
        if ( validationData->ref < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, "child: ref" ) )
                return false;
        }
        if ( validationData->mask < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, "child: mask" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_func__func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_func__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FUNC, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FUNC, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->func >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FUNC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->func++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_func__func()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_func__ref( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_func__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->func < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: func" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->ref >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_func__ref()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_func__mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_func__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MASK, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MASK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_func__mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_func_separate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_FUNC_SEPARATE;
        }
//        parentValidationData->stencil_func_separate++;

        profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData* validationData = (profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_func_separate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData* validationData = (profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->front < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, "child: front" ) )
                return false;
        }
        if ( validationData->back < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, "child: back" ) )
                return false;
        }
        if ( validationData->ref < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, "child: ref" ) )
                return false;
        }
        if ( validationData->mask < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, "child: mask" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_func_separate__front( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->back != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRONT, (const ParserChar*)0, "sibling: back" ) )
                return false;
        }
        if ( parentValidationData->ref != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRONT, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRONT, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->front >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FRONT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->front++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_func_separate__front()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_func_separate__back( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->front < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_BACK, (const ParserChar*)0, "sibling: front" ) )
                return false;
        }
        if ( parentValidationData->ref != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BACK, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BACK, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->back >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BACK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->back++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_func_separate__back()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_func_separate__ref( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->back < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: back" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->ref >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_func_separate__ref()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_func_separate__mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MASK, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MASK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_func_separate__mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_MASK;
        }
//        parentValidationData->stencil_mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_mask_separate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_MASK_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_MASK_SEPARATE;
        }
//        parentValidationData->stencil_mask_separate++;

        profile_GLES2__technique__pass__states__stencil_mask_separate__ValidationData* validationData = (profile_GLES2__technique__pass__states__stencil_mask_separate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__technique__pass__states__stencil_mask_separate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__technique__pass__states__stencil_mask_separate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_mask_separate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_mask_separate__ValidationData* validationData = (profile_GLES2__technique__pass__states__stencil_mask_separate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->face < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_MASK_SEPARATE, (const ParserChar*)0, "child: face" ) )
                return false;
        }
        if ( validationData->mask < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_MASK_SEPARATE, (const ParserChar*)0, "child: mask" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_mask_separate__face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_mask_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_mask_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACE, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->face >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FACE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->face++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_mask_separate__face()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_mask_separate__mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_mask_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_mask_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->face < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MASK, (const ParserChar*)0, "sibling: face" ) )
                return false;
        }
        if ( parentValidationData->mask >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MASK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_mask_separate__mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_op( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_OP;
        }
//        parentValidationData->stencil_op++;

        profile_GLES2__technique__pass__states__stencil_op__ValidationData* validationData = (profile_GLES2__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__technique__pass__states__stencil_op__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__technique__pass__states__stencil_op__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_op()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_op__ValidationData* validationData = (profile_GLES2__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->fail < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, "child: fail" ) )
                return false;
        }
        if ( validationData->zfail < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, "child: zfail" ) )
                return false;
        }
        if ( validationData->zpass < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, "child: zpass" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_stencil_op_enum( ParserString value )
{
    return validate__gl_stencil_op_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_stencil_op_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_stencil_op_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_stencil_op_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_op__fail( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_op__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->zfail != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->fail >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->fail++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_op__fail()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_op__zfail( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_op__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->fail < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, "sibling: fail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->zfail >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->zfail++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_op__zfail()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_op__zpass( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_op__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->zfail < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ZPASS, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ZPASS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->zpass++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_op__zpass()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_op_separate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_OP_SEPARATE;
        }
//        parentValidationData->stencil_op_separate++;

        profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData* validationData = (profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_op_separate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData* validationData = (profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->face < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, "child: face" ) )
                return false;
        }
        if ( validationData->fail < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, "child: fail" ) )
                return false;
        }
        if ( validationData->zfail < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, "child: zfail" ) )
                return false;
        }
        if ( validationData->zpass < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, "child: zpass" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_op_separate__face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->fail != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACE, (const ParserChar*)0, "sibling: fail" ) )
                return false;
        }
        if ( parentValidationData->zfail != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACE, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACE, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->face >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FACE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->face++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_op_separate__face()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_op_separate__fail( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->face < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: face" ) )
                return false;
        }
        if ( parentValidationData->zfail != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->fail >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->fail++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_op_separate__fail()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_op_separate__zfail( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->fail < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, "sibling: fail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->zfail >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->zfail++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_op_separate__zfail()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_op_separate__zpass( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData* parentValidationData = (profile_GLES2__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->zfail < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ZPASS, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ZPASS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->zpass++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_op_separate__zpass()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__blend_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_ENABLE;
        }
//        parentValidationData->blend_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__blend_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__cull_face_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CULL_FACE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CULL_FACE_ENABLE;
        }
//        parentValidationData->cull_face_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__cull_face_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__depth_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_TEST_ENABLE;
        }
//        parentValidationData->depth_test_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__depth_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__dither_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DITHER_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DITHER_ENABLE;
        }
//        parentValidationData->dither_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__dither_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__polygon_offset_fill_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE;
        }
//        parentValidationData->polygon_offset_fill_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__polygon_offset_fill_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__point_size_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE_ENABLE;
        }
//        parentValidationData->point_size_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__point_size_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE;
        }
//        parentValidationData->sample_alpha_to_coverage_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__sample_coverage_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE;
        }
//        parentValidationData->sample_coverage_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__sample_coverage_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__scissor_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCISSOR_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCISSOR_TEST_ENABLE;
        }
//        parentValidationData->scissor_test_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__scissor_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__states__stencil_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____states__ValidationData* parentValidationData = (gles2_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_TEST_ENABLE;
        }
//        parentValidationData->stencil_test_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__states__stencil_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__program____gles2_program_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        pass____gles2_pass_type__ValidationData* parentValidationData = (pass____gles2_pass_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PROGRAM, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PROGRAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->program >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PROGRAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->program++;

        program____gles2_program_type__ValidationData* validationData = (program____gles2_program_type__ValidationData*)mValidationDataStack.newObject(sizeof(program____gles2_program_type__ValidationData));
        memset(validationData, 0, sizeof(program____gles2_program_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__program____gles2_program_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        program____gles2_program_type__ValidationData* validationData = (program____gles2_program_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_pipeline_stage_enum( ParserString value )
{
    return validate__fx_pipeline_stage_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_pipeline_stage_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_pipeline_stage_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__fx_pipeline_stage_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__shader____gles2_shader_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        program____gles2_program_type__ValidationData* parentValidationData = (program____gles2_program_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->linker != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHADER, (const ParserChar*)0, "sibling: linker" ) )
                return false;
        }
        if ( parentValidationData->bind_attribute != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHADER, (const ParserChar*)0, "sibling: bind_attribute" ) )
                return false;
        }
        if ( parentValidationData->bind_uniform != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHADER, (const ParserChar*)0, "sibling: bind_uniform" ) )
                return false;
        }
        parentValidationData->shader++;

        shader____gles2_shader_type__ValidationData* validationData = (shader____gles2_shader_type__ValidationData*)mValidationDataStack.newObject(sizeof(shader____gles2_shader_type__ValidationData));
        memset(validationData, 0, sizeof(shader____gles2_shader_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__shader____gles2_shader_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        shader____gles2_shader_type__ValidationData* validationData = (shader____gles2_shader_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->sources < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SHADER, (const ParserChar*)0, "child: sources" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__gles2_shader_type____sources( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        shader____gles2_shader_type__ValidationData* parentValidationData = (shader____gles2_shader_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->compiler != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCES, (const ParserChar*)0, "sibling: compiler" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->sources >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SOURCES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->sources++;

        gles2_shader_type____sources__ValidationData* validationData = (gles2_shader_type____sources__ValidationData*)mValidationDataStack.newObject(sizeof(gles2_shader_type____sources__ValidationData));
        memset(validationData, 0, sizeof(gles2_shader_type____sources__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__gles2_shader_type____sources()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_shader_type____sources__ValidationData* validationData = (gles2_shader_type____sources__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_IMPORT) || (validationData->validation_current_state == HASH_ELEMENT_INLINE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SOURCES, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__inline( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 232:
        {
        gles2_shader_type____sources__ValidationData* parentValidationData = (gles2_shader_type____sources__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_IMPORT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INLINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INLINE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INLINE;
        }
//        parentValidationData->inline_++;

        break;
        }

        case 233:
        {
        profile_CG__technique__pass__program__shader__sources__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__sources__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_IMPORT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INLINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INLINE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INLINE;
        }
//        parentValidationData->inline_++;

        break;
        }

        case 234:
        {
        sources____fx_sources_type__ValidationData* parentValidationData = (sources____fx_sources_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_IMPORT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INLINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INLINE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INLINE;
        }
//        parentValidationData->inline_++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__inline()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__import( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 232:
        {
        gles2_shader_type____sources__ValidationData* parentValidationData = (gles2_shader_type____sources__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_IMPORT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INLINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPORT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPORT;
        }
//        parentValidationData->import++;

        break;
        }

        case 233:
        {
        profile_CG__technique__pass__program__shader__sources__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__sources__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_IMPORT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INLINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPORT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPORT;
        }
//        parentValidationData->import++;

        break;
        }

        case 234:
        {
        sources____fx_sources_type__ValidationData* parentValidationData = (sources____fx_sources_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_IMPORT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INLINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPORT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPORT;
        }
//        parentValidationData->import++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__import()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__compiler( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 577:
        {
        profile_CG__technique__pass__program__shader__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->sources < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_COMPILER, (const ParserChar*)0, "sibling: sources" ) )
                return false;
        }
        if ( parentValidationData->bind_uniform != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COMPILER, (const ParserChar*)0, "sibling: bind_uniform" ) )
                return false;
        }
        parentValidationData->compiler++;

        break;
        }

        case 578:
        {
        shader____gles2_shader_type__ValidationData* parentValidationData = (shader____gles2_shader_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->sources < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_COMPILER, (const ParserChar*)0, "sibling: sources" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COMPILER, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->compiler++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        compiler__ValidationData* validationData = (compiler__ValidationData*)mValidationDataStack.newObject(sizeof(compiler__ValidationData));
        memset(validationData, 0, sizeof(compiler__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__compiler()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        compiler__ValidationData* validationData = (compiler__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__binary( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        compiler__ValidationData* parentValidationData = (compiler__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->binary >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BINARY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->binary++;

        binary__ValidationData* validationData = (binary__ValidationData*)mValidationDataStack.newObject(sizeof(binary__ValidationData));
        memset(validationData, 0, sizeof(binary__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__binary()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        binary__ValidationData* validationData = (binary__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->ref != 0 )
            if ( validationData->ref < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BINARY, (const ParserChar*)0, "child: ref" ) )
                    return false;
        if ( validationData->hex != 0 )
            if ( validationData->hex < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BINARY, (const ParserChar*)0, "child: hex" ) )
                    return false;
        if ( validationData->ref == 0 && validationData->hex == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_BINARY, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ref____anyURI( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 366:
        {
        binary__ValidationData* parentValidationData = (binary__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->hex != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: hex" ) )
                return false;
        }
        if ( parentValidationData->ref >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref++;

        break;
        }

        case 422:
        {
        create_2d__init_from__ValidationData* parentValidationData = (create_2d__init_from__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->hex != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: hex" ) )
                return false;
        }
        if ( parentValidationData->ref >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref++;

        break;
        }

        case 423:
        {
        create_3d__init_from__ValidationData* parentValidationData = (create_3d__init_from__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->hex != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: hex" ) )
                return false;
        }
        if ( parentValidationData->ref >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref++;

        break;
        }

        case 424:
        {
        create_cube__init_from__ValidationData* parentValidationData = (create_cube__init_from__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->hex != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: hex" ) )
                return false;
        }
        if ( parentValidationData->ref >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref++;

        break;
        }

        case 425:
        {
        image_type____init_from__ValidationData* parentValidationData = (image_type____init_from__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->hex != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: hex" ) )
                return false;
        }
        if ( parentValidationData->ref >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ref____anyURI()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__binary__hex( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        binary__ValidationData* parentValidationData = (binary__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HEX, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->hex >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HEX, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->hex++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__binary__hex()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__linker( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        program____gles2_program_type__ValidationData* parentValidationData = (program____gles2_program_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->bind_attribute != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LINKER, (const ParserChar*)0, "sibling: bind_attribute" ) )
                return false;
        }
        if ( parentValidationData->bind_uniform != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LINKER, (const ParserChar*)0, "sibling: bind_uniform" ) )
                return false;
        }
        parentValidationData->linker++;

        compiler__ValidationData* validationData = (compiler__ValidationData*)mValidationDataStack.newObject(sizeof(compiler__ValidationData));
        memset(validationData, 0, sizeof(compiler__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__linker()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        compiler__ValidationData* validationData = (compiler__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__gles2_program_type____bind_attribute( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        program____gles2_program_type__ValidationData* parentValidationData = (program____gles2_program_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->bind_uniform != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_ATTRIBUTE, (const ParserChar*)0, "sibling: bind_uniform" ) )
                return false;
        }
        parentValidationData->bind_attribute++;

        gles2_program_type____bind_attribute__ValidationData* validationData = (gles2_program_type____bind_attribute__ValidationData*)mValidationDataStack.newObject(sizeof(gles2_program_type____bind_attribute__ValidationData));
        memset(validationData, 0, sizeof(gles2_program_type____bind_attribute__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__gles2_program_type____bind_attribute()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_program_type____bind_attribute__ValidationData* validationData = (gles2_program_type____bind_attribute__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->semantic != 0 )
            if ( validationData->semantic < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND_ATTRIBUTE, (const ParserChar*)0, "child: semantic" ) )
                    return false;
        if ( validationData->semantic == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_BIND_ATTRIBUTE, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__semantic____token( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 163:
        {
        gles2_program_type____bind_attribute__ValidationData* parentValidationData = (gles2_program_type____bind_attribute__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->semantic >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SEMANTIC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->semantic++;

        break;
        }

        case 164:
        {
        glsl_program_type____bind_attribute__ValidationData* parentValidationData = (glsl_program_type____bind_attribute__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->semantic >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SEMANTIC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->semantic++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__semantic____token()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__gles2_program_type____bind_uniform( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        program____gles2_program_type__ValidationData* parentValidationData = (program____gles2_program_type__ValidationData*)mValidationDataStack.top();
        parentValidationData->bind_uniform++;

        gles2_program_type____bind_uniform__ValidationData* validationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.newObject(sizeof(gles2_program_type____bind_uniform__ValidationData));
        memset(validationData, 0, sizeof(gles2_program_type____bind_uniform__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__gles2_program_type____bind_uniform()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_program_type____bind_uniform__ValidationData* validationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BVEC2) || (validationData->validation_current_state == HASH_ELEMENT_BVEC3) || (validationData->validation_current_state == HASH_ELEMENT_BVEC4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_IVEC2) || (validationData->validation_current_state == HASH_ELEMENT_IVEC3) || (validationData->validation_current_state == HASH_ELEMENT_IVEC4) || (validationData->validation_current_state == HASH_ELEMENT_MAT2) || (validationData->validation_current_state == HASH_ELEMENT_MAT3) || (validationData->validation_current_state == HASH_ELEMENT_MAT4) || (validationData->validation_current_state == HASH_ELEMENT_PARAM) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (validationData->validation_current_state == HASH_ELEMENT_VEC2) || (validationData->validation_current_state == HASH_ELEMENT_VEC3) || (validationData->validation_current_state == HASH_ELEMENT_VEC4) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_BIND_UNIFORM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES2__technique__pass__program__bind_uniform__param( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_program_type____bind_uniform__ValidationData* parentValidationData = (gles2_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARAM;
        }
//        parentValidationData->param++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES2__technique__pass__program__bind_uniform__param()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__gles2_pass_type____evaluate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        pass____gles2_pass_type__ValidationData* parentValidationData = (pass____gles2_pass_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EVALUATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->evaluate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EVALUATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->evaluate++;

        gles2_pass_type____evaluate__ValidationData* validationData = (gles2_pass_type____evaluate__ValidationData*)mValidationDataStack.newObject(sizeof(gles2_pass_type____evaluate__ValidationData));
        memset(validationData, 0, sizeof(gles2_pass_type____evaluate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__gles2_pass_type____evaluate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        gles2_pass_type____evaluate__ValidationData* validationData = (gles2_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_face_enum( ParserString value )
{
    return validate__image_face_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_face_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_face_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_face_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__color_target( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 168:
        {
        cg_pass_type____evaluate__ValidationData* parentValidationData = (cg_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->depth_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: depth_target" ) )
                return false;
        }
        if ( parentValidationData->stencil_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: stencil_target" ) )
                return false;
        }
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->color_target++;

        break;
        }

        case 169:
        {
        gles2_pass_type____evaluate__ValidationData* parentValidationData = (gles2_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->depth_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: depth_target" ) )
                return false;
        }
        if ( parentValidationData->stencil_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: stencil_target" ) )
                return false;
        }
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->color_target++;

        break;
        }

        case 170:
        {
        profile_GLES__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLES__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->depth_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: depth_target" ) )
                return false;
        }
        if ( parentValidationData->stencil_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: stencil_target" ) )
                return false;
        }
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->color_target++;

        break;
        }

        case 171:
        {
        profile_GLSL__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->depth_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: depth_target" ) )
                return false;
        }
        if ( parentValidationData->stencil_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: stencil_target" ) )
                return false;
        }
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->color_target++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        color_target__ValidationData* validationData = (color_target__ValidationData*)mValidationDataStack.newObject(sizeof(color_target__ValidationData));
        memset(validationData, 0, sizeof(color_target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__color_target()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        color_target__ValidationData* validationData = (color_target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->instance_image != 0 )
            if ( validationData->instance_image < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, "child: instance_image" ) )
                    return false;
        if ( validationData->param == 0 && validationData->instance_image == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_COLOR_TARGET, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fx_colortarget_type____param( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 486:
        {
        color_target__ValidationData* parentValidationData = (color_target__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_image != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: instance_image" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 735:
        {
        depth_target__ValidationData* parentValidationData = (depth_target__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_image != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: instance_image" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 44:
        {
        stencil_target__ValidationData* parentValidationData = (stencil_target__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_image != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: instance_image" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fx_colortarget_type____param()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__depth_target( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 168:
        {
        cg_pass_type____evaluate__ValidationData* parentValidationData = (cg_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->stencil_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: stencil_target" ) )
                return false;
        }
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->depth_target++;

        break;
        }

        case 169:
        {
        gles2_pass_type____evaluate__ValidationData* parentValidationData = (gles2_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->stencil_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: stencil_target" ) )
                return false;
        }
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->depth_target++;

        break;
        }

        case 170:
        {
        profile_GLES__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLES__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->stencil_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: stencil_target" ) )
                return false;
        }
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->depth_target++;

        break;
        }

        case 171:
        {
        profile_GLSL__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->stencil_target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: stencil_target" ) )
                return false;
        }
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->depth_target++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        depth_target__ValidationData* validationData = (depth_target__ValidationData*)mValidationDataStack.newObject(sizeof(depth_target__ValidationData));
        memset(validationData, 0, sizeof(depth_target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__depth_target()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        depth_target__ValidationData* validationData = (depth_target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->instance_image != 0 )
            if ( validationData->instance_image < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, "child: instance_image" ) )
                    return false;
        if ( validationData->param == 0 && validationData->instance_image == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_DEPTH_TARGET, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__stencil_target( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 168:
        {
        cg_pass_type____evaluate__ValidationData* parentValidationData = (cg_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->stencil_target++;

        break;
        }

        case 169:
        {
        gles2_pass_type____evaluate__ValidationData* parentValidationData = (gles2_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->stencil_target++;

        break;
        }

        case 170:
        {
        profile_GLES__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLES__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->stencil_target++;

        break;
        }

        case 171:
        {
        profile_GLSL__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: color_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->stencil_target++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        stencil_target__ValidationData* validationData = (stencil_target__ValidationData*)mValidationDataStack.newObject(sizeof(stencil_target__ValidationData));
        memset(validationData, 0, sizeof(stencil_target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__stencil_target()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        stencil_target__ValidationData* validationData = (stencil_target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->instance_image != 0 )
            if ( validationData->instance_image < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, "child: instance_image" ) )
                    return false;
        if ( validationData->param == 0 && validationData->instance_image == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_STENCIL_TARGET, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__color_clear( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 168:
        {
        cg_pass_type____evaluate__ValidationData* parentValidationData = (cg_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->color_clear++;

        break;
        }

        case 169:
        {
        gles2_pass_type____evaluate__ValidationData* parentValidationData = (gles2_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->color_clear++;

        break;
        }

        case 170:
        {
        profile_GLES__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLES__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->color_clear++;

        break;
        }

        case 171:
        {
        profile_GLSL__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->color_clear++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        color_clear__ValidationData* validationData = (color_clear__ValidationData*)mValidationDataStack.newObject(sizeof(color_clear__ValidationData));
        memset(validationData, 0, sizeof(color_clear__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__color_clear()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__stencil_clear( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 168:
        {
        cg_pass_type____evaluate__ValidationData* parentValidationData = (cg_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->stencil_clear++;

        break;
        }

        case 169:
        {
        gles2_pass_type____evaluate__ValidationData* parentValidationData = (gles2_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->depth_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_CLEAR, (const ParserChar*)0, "sibling: depth_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->stencil_clear++;

        break;
        }

        case 170:
        {
        profile_GLES__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLES__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->stencil_clear++;

        break;
        }

        case 171:
        {
        profile_GLSL__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STENCIL_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->stencil_clear++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__stencil_clear()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__depth_clear( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 168:
        {
        cg_pass_type____evaluate__ValidationData* parentValidationData = (cg_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_CLEAR, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->depth_clear++;

        break;
        }

        case 169:
        {
        gles2_pass_type____evaluate__ValidationData* parentValidationData = (gles2_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->depth_clear++;

        break;
        }

        case 170:
        {
        profile_GLES__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLES__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_CLEAR, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->depth_clear++;

        break;
        }

        case 171:
        {
        profile_GLSL__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->stencil_clear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_CLEAR, (const ParserChar*)0, "sibling: stencil_clear" ) )
                return false;
        }
        if ( parentValidationData->draw != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEPTH_CLEAR, (const ParserChar*)0, "sibling: draw" ) )
                return false;
        }
        parentValidationData->depth_clear++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__depth_clear()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__draw( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 168:
        {
        cg_pass_type____evaluate__ValidationData* parentValidationData = (cg_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->draw >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DRAW, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->draw++;

        break;
        }

        case 169:
        {
        gles2_pass_type____evaluate__ValidationData* parentValidationData = (gles2_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->draw >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DRAW, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->draw++;

        break;
        }

        case 170:
        {
        profile_GLES__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLES__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->draw >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DRAW, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->draw++;

        break;
        }

        case 171:
        {
        profile_GLSL__technique__pass__evaluate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->draw >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DRAW, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->draw++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__draw()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effect__ValidationData* parentValidationData = (effect__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_BRIDGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_CG) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_COMMON) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES2) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLSL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PROFILE_GLSL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PROFILE_GLSL;
        }
//        parentValidationData->profile_GLSL++;

        profile_GLSL__ValidationData* validationData = (profile_GLSL__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__ValidationData* validationData = (profile_GLSL__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_PROFILE_GLSL, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__newparam____glsl_newparam_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__ValidationData* parentValidationData = (profile_GLSL__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEWPARAM;
        }
//        parentValidationData->newparam++;

        newparam____glsl_newparam_type__ValidationData* validationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.newObject(sizeof(newparam____glsl_newparam_type__ValidationData));
        memset(validationData, 0, sizeof(newparam____glsl_newparam_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__newparam____glsl_newparam_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____glsl_newparam_type__ValidationData* validationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__array____glsl_array_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 374:
        {
        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

        break;
        }

        case 405:
        {
        array____glsl_array_type__ValidationData* parentValidationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

        break;
        }

        case 660:
        {
        newparam____glsl_newparam_type__ValidationData* parentValidationData = (newparam____glsl_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        array____glsl_array_type__ValidationData* validationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.newObject(sizeof(array____glsl_array_type__ValidationData));
        memset(validationData, 0, sizeof(array____glsl_array_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__array____glsl_array_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        array____glsl_array_type__ValidationData* validationData = (array____glsl_array_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_glsl_type____technique( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__ValidationData* parentValidationData = (profile_GLSL__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TECHNIQUE;
        }
//        parentValidationData->technique++;

        profile_glsl_type____technique__ValidationData* validationData = (profile_glsl_type____technique__ValidationData*)mValidationDataStack.newObject(sizeof(profile_glsl_type____technique__ValidationData));
        memset(validationData, 0, sizeof(profile_glsl_type____technique__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_glsl_type____technique()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_glsl_type____technique__ValidationData* validationData = (profile_glsl_type____technique__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->pass < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "child: pass" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_glsl_type____technique__ValidationData* parentValidationData = (profile_glsl_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PASS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->pass++;

        profile_GLSL__technique__pass__ValidationData* validationData = (profile_GLSL__technique__pass__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__ValidationData* validationData = (profile_GLSL__technique__pass__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__ValidationData* parentValidationData = (profile_GLSL__technique__pass__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->program != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: program" ) )
                return false;
        }
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->states >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_STATES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->states++;

        profile_GLSL__technique__pass__states__ValidationData* validationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__states__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__states__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__ValidationData* validationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (validationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (validationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (validationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (validationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_FOG_END) || (validationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (validationData->validation_current_state == HASH_ELEMENT_FOG_START) || (validationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (validationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (validationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (validationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (validationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (validationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (validationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_STATES, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__alpha_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ALPHA_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ALPHA_FUNC;
        }
//        parentValidationData->alpha_func++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ALPHA_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ALPHA_FUNC;
        }
//        parentValidationData->alpha_func++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLSL__technique__pass__states__alpha_func__ValidationData* validationData = (profile_GLSL__technique__pass__states__alpha_func__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__states__alpha_func__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__states__alpha_func__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__alpha_func()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__alpha_func__ValidationData* validationData = (profile_GLSL__technique__pass__states__alpha_func__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->func < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ALPHA_FUNC, (const ParserChar*)0, "child: func" ) )
                return false;
        }
        if ( validationData->value < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ALPHA_FUNC, (const ParserChar*)0, "child: value" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__alpha_func__func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__alpha_func__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__alpha_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FUNC, (const ParserChar*)0, "sibling: value" ) )
                return false;
        }
        if ( parentValidationData->func >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FUNC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->func++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__alpha_func__func()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_alpha_value_type( const float value )
{
    if( value > 1.0E0 )
        return ParserError::ERROR_VALIDATION_MAX_INCLUSIVE;

    if( value < 0.0E1 )
        return ParserError::ERROR_VALIDATION_MIN_INCLUSIVE;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__alpha_func__value( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__alpha_func__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__alpha_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->func < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, "sibling: func" ) )
                return false;
        }
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__alpha_func__value()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_FUNC;
        }
//        parentValidationData->blend_func++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_FUNC;
        }
//        parentValidationData->blend_func++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLSL__technique__pass__states__blend_func__ValidationData* validationData = (profile_GLSL__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__states__blend_func__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__states__blend_func__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_func()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_func__ValidationData* validationData = (profile_GLSL__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->src < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC, (const ParserChar*)0, "child: src" ) )
                return false;
        }
        if ( validationData->dest < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC, (const ParserChar*)0, "child: dest" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_func__src( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_func__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->dest != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC, (const ParserChar*)0, "sibling: dest" ) )
                return false;
        }
        if ( parentValidationData->src >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SRC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->src++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_func__src()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_func__dest( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_func__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->src < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_DEST, (const ParserChar*)0, "sibling: src" ) )
                return false;
        }
        if ( parentValidationData->dest >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DEST, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->dest++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_func__dest()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_func_separate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_FUNC_SEPARATE;
        }
//        parentValidationData->blend_func_separate++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_FUNC_SEPARATE;
        }
//        parentValidationData->blend_func_separate++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLSL__technique__pass__states__blend_func_separate__ValidationData* validationData = (profile_GLSL__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__states__blend_func_separate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__states__blend_func_separate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_func_separate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_func_separate__ValidationData* validationData = (profile_GLSL__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->src_rgb < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, "child: src_rgb" ) )
                return false;
        }
        if ( validationData->dest_rgb < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, "child: dest_rgb" ) )
                return false;
        }
        if ( validationData->src_alpha < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, "child: src_alpha" ) )
                return false;
        }
        if ( validationData->dest_alpha < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC_SEPARATE, (const ParserChar*)0, "child: dest_alpha" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_func_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->dest_rgb != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC_RGB, (const ParserChar*)0, "sibling: dest_rgb" ) )
                return false;
        }
        if ( parentValidationData->src_alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC_RGB, (const ParserChar*)0, "sibling: src_alpha" ) )
                return false;
        }
        if ( parentValidationData->dest_alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC_RGB, (const ParserChar*)0, "sibling: dest_alpha" ) )
                return false;
        }
        if ( parentValidationData->src_rgb >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SRC_RGB, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->src_rgb++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_func_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->src_rgb < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_DEST_RGB, (const ParserChar*)0, "sibling: src_rgb" ) )
                return false;
        }
        if ( parentValidationData->src_alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEST_RGB, (const ParserChar*)0, "sibling: src_alpha" ) )
                return false;
        }
        if ( parentValidationData->dest_alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DEST_RGB, (const ParserChar*)0, "sibling: dest_alpha" ) )
                return false;
        }
        if ( parentValidationData->dest_rgb >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DEST_RGB, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->dest_rgb++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_func_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->dest_rgb < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_SRC_ALPHA, (const ParserChar*)0, "sibling: dest_rgb" ) )
                return false;
        }
        if ( parentValidationData->dest_alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC_ALPHA, (const ParserChar*)0, "sibling: dest_alpha" ) )
                return false;
        }
        if ( parentValidationData->src_alpha >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SRC_ALPHA, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->src_alpha++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_func_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__blend_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->src_alpha < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_DEST_ALPHA, (const ParserChar*)0, "sibling: src_alpha" ) )
                return false;
        }
        if ( parentValidationData->dest_alpha >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DEST_ALPHA, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->dest_alpha++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_equation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_EQUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_EQUATION;
        }
//        parentValidationData->blend_equation++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_EQUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_EQUATION;
        }
//        parentValidationData->blend_equation++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_equation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_equation_separate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_EQUATION_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_EQUATION_SEPARATE;
        }
//        parentValidationData->blend_equation_separate++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_EQUATION_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_EQUATION_SEPARATE;
        }
//        parentValidationData->blend_equation_separate++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLSL__technique__pass__states__blend_equation_separate__ValidationData* validationData = (profile_GLSL__technique__pass__states__blend_equation_separate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__states__blend_equation_separate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__states__blend_equation_separate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_equation_separate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_equation_separate__ValidationData* validationData = (profile_GLSL__technique__pass__states__blend_equation_separate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->rgb < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_EQUATION_SEPARATE, (const ParserChar*)0, "child: rgb" ) )
                return false;
        }
        if ( validationData->alpha < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_EQUATION_SEPARATE, (const ParserChar*)0, "child: alpha" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_equation_separate__rgb( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_equation_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__blend_equation_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RGB, (const ParserChar*)0, "sibling: alpha" ) )
                return false;
        }
        if ( parentValidationData->rgb >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RGB, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->rgb++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_equation_separate__rgb()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_equation_separate__alpha( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__blend_equation_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__blend_equation_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->rgb < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ALPHA, (const ParserChar*)0, "sibling: rgb" ) )
                return false;
        }
        if ( parentValidationData->alpha >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ALPHA, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->alpha++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_equation_separate__alpha()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__color_material( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_MATERIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_MATERIAL;
        }
//        parentValidationData->color_material++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_MATERIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_MATERIAL;
        }
//        parentValidationData->color_material++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        color_material__ValidationData* validationData = (color_material__ValidationData*)mValidationDataStack.newObject(sizeof(color_material__ValidationData));
        memset(validationData, 0, sizeof(color_material__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__color_material()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        color_material__ValidationData* validationData = (color_material__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->face < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_COLOR_MATERIAL, (const ParserChar*)0, "child: face" ) )
                return false;
        }
        if ( validationData->mode < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_COLOR_MATERIAL, (const ParserChar*)0, "child: mode" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__color_material__face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        color_material__ValidationData* parentValidationData = (color_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mode != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACE, (const ParserChar*)0, "sibling: mode" ) )
                return false;
        }
        if ( parentValidationData->face >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FACE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->face++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__color_material__face()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_material_enum( ParserString value )
{
    return validate__gl_material_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_material_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_material_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_material_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__color_material__mode( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        color_material__ValidationData* parentValidationData = (color_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->face < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MODE, (const ParserChar*)0, "sibling: face" ) )
                return false;
        }
        if ( parentValidationData->mode >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MODE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mode++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__color_material__mode()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__cull_face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CULL_FACE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CULL_FACE;
        }
//        parentValidationData->cull_face++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CULL_FACE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CULL_FACE;
        }
//        parentValidationData->cull_face++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__cull_face()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__depth_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_FUNC;
        }
//        parentValidationData->depth_func++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_FUNC;
        }
//        parentValidationData->depth_func++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__depth_func()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_fog_enum( ParserString value )
{
    return validate__gl_fog_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_fog_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_fog_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_fog_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__fog_mode( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_MODE;
        }
//        parentValidationData->fog_mode++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_MODE;
        }
//        parentValidationData->fog_mode++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__fog_mode()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_fog_coord_src_enum( ParserString value )
{
    return validate__gl_fog_coord_src_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_fog_coord_src_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_fog_coord_src_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_fog_coord_src_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fog_coord_src( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_COORD_SRC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_COORD_SRC;
        }
//        parentValidationData->fog_coord_src++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_COORD_SRC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_COORD_SRC;
        }
//        parentValidationData->fog_coord_src++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fog_coord_src()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__front_face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FRONT_FACE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FRONT_FACE;
        }
//        parentValidationData->front_face++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FRONT_FACE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FRONT_FACE;
        }
//        parentValidationData->front_face++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__front_face()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_light_model_color_control_enum( ParserString value )
{
    return validate__gl_light_model_color_control_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_light_model_color_control_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_light_model_color_control_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_light_model_color_control_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__light_model_color_control( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL;
        }
//        parentValidationData->light_model_color_control++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL;
        }
//        parentValidationData->light_model_color_control++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__light_model_color_control()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_logic_op_enum( ParserString value )
{
    return validate__gl_logic_op_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_logic_op_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_logic_op_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_logic_op_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__logic_op( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGIC_OP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGIC_OP;
        }
//        parentValidationData->logic_op++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGIC_OP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGIC_OP;
        }
//        parentValidationData->logic_op++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__logic_op()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__polygon_mode( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_MODE;
        }
//        parentValidationData->polygon_mode++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_MODE;
        }
//        parentValidationData->polygon_mode++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        polygon_mode__ValidationData* validationData = (polygon_mode__ValidationData*)mValidationDataStack.newObject(sizeof(polygon_mode__ValidationData));
        memset(validationData, 0, sizeof(polygon_mode__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__polygon_mode()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        polygon_mode__ValidationData* validationData = (polygon_mode__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->face < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_POLYGON_MODE, (const ParserChar*)0, "child: face" ) )
                return false;
        }
        if ( validationData->mode < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_POLYGON_MODE, (const ParserChar*)0, "child: mode" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__polygon_mode__face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        polygon_mode__ValidationData* parentValidationData = (polygon_mode__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mode != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACE, (const ParserChar*)0, "sibling: mode" ) )
                return false;
        }
        if ( parentValidationData->face >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FACE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->face++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__polygon_mode__face()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_polygon_mode_enum( ParserString value )
{
    return validate__gl_polygon_mode_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_polygon_mode_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_polygon_mode_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_polygon_mode_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__polygon_mode__mode( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        polygon_mode__ValidationData* parentValidationData = (polygon_mode__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->face < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MODE, (const ParserChar*)0, "sibling: face" ) )
                return false;
        }
        if ( parentValidationData->mode >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MODE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mode++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__polygon_mode__mode()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_shade_model_enum( ParserString value )
{
    return validate__gl_shade_model_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_shade_model_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_shade_model_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gl_shade_model_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__shade_model( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SHADE_MODEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SHADE_MODEL;
        }
//        parentValidationData->shade_model++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SHADE_MODEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SHADE_MODEL;
        }
//        parentValidationData->shade_model++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__shade_model()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_FUNC;
        }
//        parentValidationData->stencil_func++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_FUNC;
        }
//        parentValidationData->stencil_func++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLSL__technique__pass__states__stencil_func__ValidationData* validationData = (profile_GLSL__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__states__stencil_func__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__states__stencil_func__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_func()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_func__ValidationData* validationData = (profile_GLSL__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->func < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, "child: func" ) )
                return false;
        }
        if ( validationData->ref < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, "child: ref" ) )
                return false;
        }
        if ( validationData->mask < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, "child: mask" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_func__func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_func__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FUNC, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FUNC, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->func >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FUNC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->func++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_func__func()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_func__ref( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_func__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->func < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: func" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->ref >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_func__ref()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_func__mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_func__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MASK, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MASK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_func__mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_op( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_OP;
        }
//        parentValidationData->stencil_op++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_OP;
        }
//        parentValidationData->stencil_op++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLSL__technique__pass__states__stencil_op__ValidationData* validationData = (profile_GLSL__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__states__stencil_op__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__states__stencil_op__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_op()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_op__ValidationData* validationData = (profile_GLSL__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->fail < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, "child: fail" ) )
                return false;
        }
        if ( validationData->zfail < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, "child: zfail" ) )
                return false;
        }
        if ( validationData->zpass < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, "child: zpass" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_op__fail( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_op__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->zfail != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->fail >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->fail++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_op__fail()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_op__zfail( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_op__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->fail < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, "sibling: fail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->zfail >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->zfail++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_op__zfail()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_op__zpass( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_op__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->zfail < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ZPASS, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ZPASS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->zpass++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_op__zpass()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_func_separate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_FUNC_SEPARATE;
        }
//        parentValidationData->stencil_func_separate++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_FUNC_SEPARATE;
        }
//        parentValidationData->stencil_func_separate++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData* validationData = (profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_func_separate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData* validationData = (profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->front < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, "child: front" ) )
                return false;
        }
        if ( validationData->back < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, "child: back" ) )
                return false;
        }
        if ( validationData->ref < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, "child: ref" ) )
                return false;
        }
        if ( validationData->mask < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC_SEPARATE, (const ParserChar*)0, "child: mask" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_func_separate__front( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->back != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRONT, (const ParserChar*)0, "sibling: back" ) )
                return false;
        }
        if ( parentValidationData->ref != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRONT, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRONT, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->front >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FRONT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->front++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_func_separate__front()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_func_separate__back( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->front < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_BACK, (const ParserChar*)0, "sibling: front" ) )
                return false;
        }
        if ( parentValidationData->ref != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BACK, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BACK, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->back >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BACK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->back++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_func_separate__back()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_func_separate__ref( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->back < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: back" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->ref >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_func_separate__ref()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_func_separate__mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_func_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MASK, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MASK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_func_separate__mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_op_separate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_OP_SEPARATE;
        }
//        parentValidationData->stencil_op_separate++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_OP_SEPARATE;
        }
//        parentValidationData->stencil_op_separate++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData* validationData = (profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_op_separate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData* validationData = (profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->face < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, "child: face" ) )
                return false;
        }
        if ( validationData->fail < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, "child: fail" ) )
                return false;
        }
        if ( validationData->zfail < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, "child: zfail" ) )
                return false;
        }
        if ( validationData->zpass < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP_SEPARATE, (const ParserChar*)0, "child: zpass" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_op_separate__face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->fail != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACE, (const ParserChar*)0, "sibling: fail" ) )
                return false;
        }
        if ( parentValidationData->zfail != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACE, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACE, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->face >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FACE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->face++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_op_separate__face()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_op_separate__fail( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->face < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: face" ) )
                return false;
        }
        if ( parentValidationData->zfail != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->fail >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->fail++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_op_separate__fail()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_op_separate__zfail( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->fail < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, "sibling: fail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->zfail >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->zfail++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_op_separate__zfail()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_op_separate__zpass( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_op_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->zfail < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ZPASS, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ZPASS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->zpass++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_op_separate__zpass()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_mask_separate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_MASK_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_MASK_SEPARATE;
        }
//        parentValidationData->stencil_mask_separate++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_MASK_SEPARATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_MASK_SEPARATE;
        }
//        parentValidationData->stencil_mask_separate++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        profile_GLSL__technique__pass__states__stencil_mask_separate__ValidationData* validationData = (profile_GLSL__technique__pass__states__stencil_mask_separate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__states__stencil_mask_separate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__states__stencil_mask_separate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_mask_separate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_mask_separate__ValidationData* validationData = (profile_GLSL__technique__pass__states__stencil_mask_separate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->face < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_MASK_SEPARATE, (const ParserChar*)0, "child: face" ) )
                return false;
        }
        if ( validationData->mask < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_MASK_SEPARATE, (const ParserChar*)0, "child: mask" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_mask_separate__face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_mask_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_mask_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FACE, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->face >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FACE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->face++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_mask_separate__face()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_mask_separate__mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__states__stencil_mask_separate__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__stencil_mask_separate__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->face < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MASK, (const ParserChar*)0, "sibling: face" ) )
                return false;
        }
        if ( parentValidationData->mask >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MASK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_mask_separate__mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_ENABLE;
        }
//        parentValidationData->light_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_ENABLE;
        }
//        parentValidationData->light_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_ambient( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_AMBIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_AMBIENT;
        }
//        parentValidationData->light_ambient++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_AMBIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_AMBIENT;
        }
//        parentValidationData->light_ambient++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_ambient()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_diffuse( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_DIFFUSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_DIFFUSE;
        }
//        parentValidationData->light_diffuse++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_DIFFUSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_DIFFUSE;
        }
//        parentValidationData->light_diffuse++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_diffuse()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_specular( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPECULAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPECULAR;
        }
//        parentValidationData->light_specular++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPECULAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPECULAR;
        }
//        parentValidationData->light_specular++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_specular()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_position( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_POSITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_POSITION;
        }
//        parentValidationData->light_position++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_POSITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_POSITION;
        }
//        parentValidationData->light_position++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_position()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_constant_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION;
        }
//        parentValidationData->light_constant_attenuation++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION;
        }
//        parentValidationData->light_constant_attenuation++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_constant_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_linear_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION;
        }
//        parentValidationData->light_linear_attenuation++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION;
        }
//        parentValidationData->light_linear_attenuation++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_linear_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_quadratic_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION;
        }
//        parentValidationData->light_quadratic_attenuation++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION;
        }
//        parentValidationData->light_quadratic_attenuation++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_quadratic_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_spot_cutoff( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPOT_CUTOFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPOT_CUTOFF;
        }
//        parentValidationData->light_spot_cutoff++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPOT_CUTOFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPOT_CUTOFF;
        }
//        parentValidationData->light_spot_cutoff++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_spot_cutoff()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_spot_direction( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPOT_DIRECTION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPOT_DIRECTION;
        }
//        parentValidationData->light_spot_direction++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPOT_DIRECTION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPOT_DIRECTION;
        }
//        parentValidationData->light_spot_direction++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_spot_direction()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_spot_exponent( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPOT_EXPONENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPOT_EXPONENT;
        }
//        parentValidationData->light_spot_exponent++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPOT_EXPONENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPOT_EXPONENT;
        }
//        parentValidationData->light_spot_exponent++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_spot_exponent()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texture1D( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE1D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE1D;
        }
//        parentValidationData->texture1D++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE1D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE1D;
        }
//        parentValidationData->texture1D++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        texture1D__ValidationData* validationData = (texture1D__ValidationData*)mValidationDataStack.newObject(sizeof(texture1D__ValidationData));
        memset(validationData, 0, sizeof(texture1D__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texture1D()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texture1D__ValidationData* validationData = (texture1D__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->value != 0 )
            if ( validationData->value < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTURE1D, (const ParserChar*)0, "child: value" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTURE1D, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->value == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TEXTURE1D, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__value____fx_sampler1D_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texture1D__ValidationData* parentValidationData = (texture1D__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

        sampler1D__ValidationData* validationData = (sampler1D__ValidationData*)mValidationDataStack.newObject(sizeof(sampler1D__ValidationData));
        memset(validationData, 0, sizeof(sampler1D__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__value____fx_sampler1D_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler1D__ValidationData* validationData = (sampler1D__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__param____NCName( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 376:
        {
        texture1D__ValidationData* parentValidationData = (texture1D__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: value" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 165:
        {
        texture2D__ValidationData* parentValidationData = (texture2D__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: value" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 736:
        {
        texture3D__ValidationData* parentValidationData = (texture3D__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: value" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 701:
        {
        textureCUBE__ValidationData* parentValidationData = (textureCUBE__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: value" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 609:
        {
        textureDEPTH__ValidationData* parentValidationData = (textureDEPTH__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: value" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 273:
        {
        textureRECT__ValidationData* parentValidationData = (textureRECT__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: value" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__param____NCName()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texture2D( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE2D;
        }
//        parentValidationData->texture2D++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE2D;
        }
//        parentValidationData->texture2D++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        texture2D__ValidationData* validationData = (texture2D__ValidationData*)mValidationDataStack.newObject(sizeof(texture2D__ValidationData));
        memset(validationData, 0, sizeof(texture2D__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texture2D()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texture2D__ValidationData* validationData = (texture2D__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->value != 0 )
            if ( validationData->value < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTURE2D, (const ParserChar*)0, "child: value" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTURE2D, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->value == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TEXTURE2D, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__value____fx_sampler2D_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texture2D__ValidationData* parentValidationData = (texture2D__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

        sampler2D____fx_sampler2D_type__ValidationData* validationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.newObject(sizeof(sampler2D____fx_sampler2D_type__ValidationData));
        memset(validationData, 0, sizeof(sampler2D____fx_sampler2D_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__value____fx_sampler2D_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler2D____fx_sampler2D_type__ValidationData* validationData = (sampler2D____fx_sampler2D_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texture3D( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE3D;
        }
//        parentValidationData->texture3D++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE3D;
        }
//        parentValidationData->texture3D++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        texture3D__ValidationData* validationData = (texture3D__ValidationData*)mValidationDataStack.newObject(sizeof(texture3D__ValidationData));
        memset(validationData, 0, sizeof(texture3D__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texture3D()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texture3D__ValidationData* validationData = (texture3D__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->value != 0 )
            if ( validationData->value < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTURE3D, (const ParserChar*)0, "child: value" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTURE3D, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->value == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TEXTURE3D, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__value____fx_sampler3D_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texture3D__ValidationData* parentValidationData = (texture3D__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

        sampler3D__ValidationData* validationData = (sampler3D__ValidationData*)mValidationDataStack.newObject(sizeof(sampler3D__ValidationData));
        memset(validationData, 0, sizeof(sampler3D__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__value____fx_sampler3D_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler3D__ValidationData* validationData = (sampler3D__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__textureCUBE( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURECUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURECUBE;
        }
//        parentValidationData->textureCUBE++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURECUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURECUBE;
        }
//        parentValidationData->textureCUBE++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        textureCUBE__ValidationData* validationData = (textureCUBE__ValidationData*)mValidationDataStack.newObject(sizeof(textureCUBE__ValidationData));
        memset(validationData, 0, sizeof(textureCUBE__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__textureCUBE()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        textureCUBE__ValidationData* validationData = (textureCUBE__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->value != 0 )
            if ( validationData->value < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTURECUBE, (const ParserChar*)0, "child: value" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTURECUBE, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->value == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TEXTURECUBE, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__value____fx_samplerCUBE_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        textureCUBE__ValidationData* parentValidationData = (textureCUBE__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

        samplerCUBE__ValidationData* validationData = (samplerCUBE__ValidationData*)mValidationDataStack.newObject(sizeof(samplerCUBE__ValidationData));
        memset(validationData, 0, sizeof(samplerCUBE__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__value____fx_samplerCUBE_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        samplerCUBE__ValidationData* validationData = (samplerCUBE__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__textureRECT( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURERECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURERECT;
        }
//        parentValidationData->textureRECT++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURERECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURERECT;
        }
//        parentValidationData->textureRECT++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        textureRECT__ValidationData* validationData = (textureRECT__ValidationData*)mValidationDataStack.newObject(sizeof(textureRECT__ValidationData));
        memset(validationData, 0, sizeof(textureRECT__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__textureRECT()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        textureRECT__ValidationData* validationData = (textureRECT__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->value != 0 )
            if ( validationData->value < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTURERECT, (const ParserChar*)0, "child: value" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTURERECT, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->value == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TEXTURERECT, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__value____fx_samplerRECT_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        textureRECT__ValidationData* parentValidationData = (textureRECT__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

        samplerRECT__ValidationData* validationData = (samplerRECT__ValidationData*)mValidationDataStack.newObject(sizeof(samplerRECT__ValidationData));
        memset(validationData, 0, sizeof(samplerRECT__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__value____fx_samplerRECT_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        samplerRECT__ValidationData* validationData = (samplerRECT__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__textureDEPTH( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTUREDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTUREDEPTH;
        }
//        parentValidationData->textureDEPTH++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTUREDEPTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTUREDEPTH;
        }
//        parentValidationData->textureDEPTH++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        textureDEPTH__ValidationData* validationData = (textureDEPTH__ValidationData*)mValidationDataStack.newObject(sizeof(textureDEPTH__ValidationData));
        memset(validationData, 0, sizeof(textureDEPTH__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__textureDEPTH()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        textureDEPTH__ValidationData* validationData = (textureDEPTH__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->value != 0 )
            if ( validationData->value < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTUREDEPTH, (const ParserChar*)0, "child: value" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TEXTUREDEPTH, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->value == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TEXTUREDEPTH, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__value____fx_samplerDEPTH_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        textureDEPTH__ValidationData* parentValidationData = (textureDEPTH__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->param != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, "sibling: param" ) )
                return false;
        }
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

        samplerDEPTH__ValidationData* validationData = (samplerDEPTH__ValidationData*)mValidationDataStack.newObject(sizeof(samplerDEPTH__ValidationData));
        memset(validationData, 0, sizeof(samplerDEPTH__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__value____fx_samplerDEPTH_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        samplerDEPTH__ValidationData* validationData = (samplerDEPTH__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BORDER_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MAX_ANISOTROPY) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MIN_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_P) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texture1D_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE1D_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE1D_ENABLE;
        }
//        parentValidationData->texture1D_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE1D_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE1D_ENABLE;
        }
//        parentValidationData->texture1D_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texture1D_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texture2D_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE2D_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE2D_ENABLE;
        }
//        parentValidationData->texture2D_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE2D_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE2D_ENABLE;
        }
//        parentValidationData->texture2D_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texture2D_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texture3D_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE3D_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE3D_ENABLE;
        }
//        parentValidationData->texture3D_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE3D_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE3D_ENABLE;
        }
//        parentValidationData->texture3D_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texture3D_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__textureCUBE_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURECUBE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURECUBE_ENABLE;
        }
//        parentValidationData->textureCUBE_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURECUBE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURECUBE_ENABLE;
        }
//        parentValidationData->textureCUBE_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__textureCUBE_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__textureRECT_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURERECT_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURERECT_ENABLE;
        }
//        parentValidationData->textureRECT_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURERECT_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURERECT_ENABLE;
        }
//        parentValidationData->textureRECT_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__textureRECT_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__textureDEPTH_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTUREDEPTH_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTUREDEPTH_ENABLE;
        }
//        parentValidationData->textureDEPTH_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTUREDEPTH_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTUREDEPTH_ENABLE;
        }
//        parentValidationData->textureDEPTH_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__textureDEPTH_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texture_env_color( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE_ENV_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE_ENV_COLOR;
        }
//        parentValidationData->texture_env_color++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE_ENV_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE_ENV_COLOR;
        }
//        parentValidationData->texture_env_color++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texture_env_color()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texture_env_mode( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE_ENV_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE_ENV_MODE;
        }
//        parentValidationData->texture_env_mode++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE_ENV_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE_ENV_MODE;
        }
//        parentValidationData->texture_env_mode++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texture_env_mode()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__clip_plane( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CLIP_PLANE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CLIP_PLANE;
        }
//        parentValidationData->clip_plane++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CLIP_PLANE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CLIP_PLANE;
        }
//        parentValidationData->clip_plane++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__clip_plane()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__clip_plane_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CLIP_PLANE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CLIP_PLANE_ENABLE;
        }
//        parentValidationData->clip_plane_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CLIP_PLANE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CLIP_PLANE_ENABLE;
        }
//        parentValidationData->clip_plane_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__clip_plane_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_color( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_COLOR;
        }
//        parentValidationData->blend_color++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_COLOR;
        }
//        parentValidationData->blend_color++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_color()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__color_mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_MASK;
        }
//        parentValidationData->color_mask++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_MASK;
        }
//        parentValidationData->color_mask++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__color_mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__depth_bounds( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_BOUNDS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_BOUNDS;
        }
//        parentValidationData->depth_bounds++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_BOUNDS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_BOUNDS;
        }
//        parentValidationData->depth_bounds++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__depth_bounds()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__depth_mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_MASK;
        }
//        parentValidationData->depth_mask++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_MASK;
        }
//        parentValidationData->depth_mask++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__depth_mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__depth_range( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_RANGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_RANGE;
        }
//        parentValidationData->depth_range++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_RANGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_RANGE;
        }
//        parentValidationData->depth_range++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__depth_range()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__fog_density( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_DENSITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_DENSITY;
        }
//        parentValidationData->fog_density++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_DENSITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_DENSITY;
        }
//        parentValidationData->fog_density++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__fog_density()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__fog_start( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_START, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_START;
        }
//        parentValidationData->fog_start++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_START, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_START;
        }
//        parentValidationData->fog_start++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__fog_start()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__fog_end( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_END, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_END;
        }
//        parentValidationData->fog_end++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_END, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_END;
        }
//        parentValidationData->fog_end++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__fog_end()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__fog_color( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_COLOR;
        }
//        parentValidationData->fog_color++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_COLOR;
        }
//        parentValidationData->fog_color++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__fog_color()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_model_ambient( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_MODEL_AMBIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_MODEL_AMBIENT;
        }
//        parentValidationData->light_model_ambient++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_MODEL_AMBIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_MODEL_AMBIENT;
        }
//        parentValidationData->light_model_ambient++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_model_ambient()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__lighting_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHTING_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHTING_ENABLE;
        }
//        parentValidationData->lighting_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHTING_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHTING_ENABLE;
        }
//        parentValidationData->lighting_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__lighting_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__line_stipple( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_STIPPLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_STIPPLE;
        }
//        parentValidationData->line_stipple++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_STIPPLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_STIPPLE;
        }
//        parentValidationData->line_stipple++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__line_stipple()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__line_width( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_WIDTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_WIDTH;
        }
//        parentValidationData->line_width++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_WIDTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_WIDTH;
        }
//        parentValidationData->line_width++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__line_width()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__material_ambient( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_AMBIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_AMBIENT;
        }
//        parentValidationData->material_ambient++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_AMBIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_AMBIENT;
        }
//        parentValidationData->material_ambient++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__material_ambient()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__material_diffuse( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_DIFFUSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_DIFFUSE;
        }
//        parentValidationData->material_diffuse++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_DIFFUSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_DIFFUSE;
        }
//        parentValidationData->material_diffuse++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__material_diffuse()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__material_emission( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_EMISSION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_EMISSION;
        }
//        parentValidationData->material_emission++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_EMISSION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_EMISSION;
        }
//        parentValidationData->material_emission++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__material_emission()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__material_shininess( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_SHININESS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_SHININESS;
        }
//        parentValidationData->material_shininess++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_SHININESS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_SHININESS;
        }
//        parentValidationData->material_shininess++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__material_shininess()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__material_specular( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_SPECULAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_SPECULAR;
        }
//        parentValidationData->material_specular++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_SPECULAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_SPECULAR;
        }
//        parentValidationData->material_specular++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__material_specular()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__model_view_matrix( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODEL_VIEW_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODEL_VIEW_MATRIX;
        }
//        parentValidationData->model_view_matrix++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODEL_VIEW_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODEL_VIEW_MATRIX;
        }
//        parentValidationData->model_view_matrix++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__model_view_matrix()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__point_distance_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_DISTANCE_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_DISTANCE_ATTENUATION;
        }
//        parentValidationData->point_distance_attenuation++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_DISTANCE_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_DISTANCE_ATTENUATION;
        }
//        parentValidationData->point_distance_attenuation++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__point_distance_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__point_fade_threshold_size( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE;
        }
//        parentValidationData->point_fade_threshold_size++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE;
        }
//        parentValidationData->point_fade_threshold_size++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__point_fade_threshold_size()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__point_size( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE;
        }
//        parentValidationData->point_size++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE;
        }
//        parentValidationData->point_size++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__point_size()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__point_size_min( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE_MIN;
        }
//        parentValidationData->point_size_min++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE_MIN;
        }
//        parentValidationData->point_size_min++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__point_size_min()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__point_size_max( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE_MAX;
        }
//        parentValidationData->point_size_max++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE_MAX;
        }
//        parentValidationData->point_size_max++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__point_size_max()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__polygon_offset( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET;
        }
//        parentValidationData->polygon_offset++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET;
        }
//        parentValidationData->polygon_offset++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__polygon_offset()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__projection_matrix( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PROJECTION_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PROJECTION_MATRIX;
        }
//        parentValidationData->projection_matrix++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PROJECTION_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PROJECTION_MATRIX;
        }
//        parentValidationData->projection_matrix++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__projection_matrix()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__scissor( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCISSOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCISSOR;
        }
//        parentValidationData->scissor++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCISSOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCISSOR;
        }
//        parentValidationData->scissor++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__scissor()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_MASK;
        }
//        parentValidationData->stencil_mask++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_MASK;
        }
//        parentValidationData->stencil_mask++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__alpha_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ALPHA_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ALPHA_TEST_ENABLE;
        }
//        parentValidationData->alpha_test_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ALPHA_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ALPHA_TEST_ENABLE;
        }
//        parentValidationData->alpha_test_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__alpha_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__blend_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_ENABLE;
        }
//        parentValidationData->blend_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_ENABLE;
        }
//        parentValidationData->blend_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__blend_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__color_logic_op_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE;
        }
//        parentValidationData->color_logic_op_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE;
        }
//        parentValidationData->color_logic_op_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__color_logic_op_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__color_material_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_MATERIAL_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_MATERIAL_ENABLE;
        }
//        parentValidationData->color_material_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_MATERIAL_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_MATERIAL_ENABLE;
        }
//        parentValidationData->color_material_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__color_material_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__cull_face_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CULL_FACE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CULL_FACE_ENABLE;
        }
//        parentValidationData->cull_face_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CULL_FACE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CULL_FACE_ENABLE;
        }
//        parentValidationData->cull_face_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__cull_face_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__depth_bounds_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_BOUNDS_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_BOUNDS_ENABLE;
        }
//        parentValidationData->depth_bounds_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_BOUNDS_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_BOUNDS_ENABLE;
        }
//        parentValidationData->depth_bounds_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__depth_bounds_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__depth_clamp_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_CLAMP_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_CLAMP_ENABLE;
        }
//        parentValidationData->depth_clamp_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_CLAMP_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_CLAMP_ENABLE;
        }
//        parentValidationData->depth_clamp_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__depth_clamp_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__depth_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_TEST_ENABLE;
        }
//        parentValidationData->depth_test_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_TEST_ENABLE;
        }
//        parentValidationData->depth_test_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__depth_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__dither_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DITHER_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DITHER_ENABLE;
        }
//        parentValidationData->dither_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DITHER_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DITHER_ENABLE;
        }
//        parentValidationData->dither_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__dither_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__fog_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_ENABLE;
        }
//        parentValidationData->fog_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_ENABLE;
        }
//        parentValidationData->fog_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__fog_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__light_model_local_viewer_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE;
        }
//        parentValidationData->light_model_local_viewer_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE;
        }
//        parentValidationData->light_model_local_viewer_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__light_model_local_viewer_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__light_model_two_side_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE;
        }
//        parentValidationData->light_model_two_side_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE;
        }
//        parentValidationData->light_model_two_side_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__light_model_two_side_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__line_smooth_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_SMOOTH_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_SMOOTH_ENABLE;
        }
//        parentValidationData->line_smooth_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_SMOOTH_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_SMOOTH_ENABLE;
        }
//        parentValidationData->line_smooth_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__line_smooth_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__line_stipple_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_STIPPLE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_STIPPLE_ENABLE;
        }
//        parentValidationData->line_stipple_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_STIPPLE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_STIPPLE_ENABLE;
        }
//        parentValidationData->line_stipple_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__line_stipple_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__logic_op_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGIC_OP_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGIC_OP_ENABLE;
        }
//        parentValidationData->logic_op_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGIC_OP_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGIC_OP_ENABLE;
        }
//        parentValidationData->logic_op_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__logic_op_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__multisample_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MULTISAMPLE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MULTISAMPLE_ENABLE;
        }
//        parentValidationData->multisample_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MULTISAMPLE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MULTISAMPLE_ENABLE;
        }
//        parentValidationData->multisample_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__multisample_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__normalize_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NORMALIZE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NORMALIZE_ENABLE;
        }
//        parentValidationData->normalize_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NORMALIZE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NORMALIZE_ENABLE;
        }
//        parentValidationData->normalize_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__normalize_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__point_smooth_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SMOOTH_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SMOOTH_ENABLE;
        }
//        parentValidationData->point_smooth_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SMOOTH_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SMOOTH_ENABLE;
        }
//        parentValidationData->point_smooth_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__point_smooth_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__polygon_offset_fill_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE;
        }
//        parentValidationData->polygon_offset_fill_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE;
        }
//        parentValidationData->polygon_offset_fill_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__polygon_offset_fill_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__polygon_offset_line_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE;
        }
//        parentValidationData->polygon_offset_line_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE;
        }
//        parentValidationData->polygon_offset_line_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__polygon_offset_line_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__polygon_offset_point_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE;
        }
//        parentValidationData->polygon_offset_point_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE;
        }
//        parentValidationData->polygon_offset_point_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__polygon_offset_point_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__polygon_smooth_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_SMOOTH_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_SMOOTH_ENABLE;
        }
//        parentValidationData->polygon_smooth_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_SMOOTH_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_SMOOTH_ENABLE;
        }
//        parentValidationData->polygon_smooth_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__polygon_smooth_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__polygon_stipple_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_STIPPLE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_STIPPLE_ENABLE;
        }
//        parentValidationData->polygon_stipple_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_STIPPLE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_STIPPLE_ENABLE;
        }
//        parentValidationData->polygon_stipple_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__polygon_stipple_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__rescale_normal_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RESCALE_NORMAL_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RESCALE_NORMAL_ENABLE;
        }
//        parentValidationData->rescale_normal_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RESCALE_NORMAL_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RESCALE_NORMAL_ENABLE;
        }
//        parentValidationData->rescale_normal_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__rescale_normal_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE;
        }
//        parentValidationData->sample_alpha_to_coverage_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE;
        }
//        parentValidationData->sample_alpha_to_coverage_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE;
        }
//        parentValidationData->sample_alpha_to_one_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE;
        }
//        parentValidationData->sample_alpha_to_one_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__sample_coverage_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE;
        }
//        parentValidationData->sample_coverage_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE;
        }
//        parentValidationData->sample_coverage_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__sample_coverage_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__scissor_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCISSOR_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCISSOR_TEST_ENABLE;
        }
//        parentValidationData->scissor_test_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCISSOR_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCISSOR_TEST_ENABLE;
        }
//        parentValidationData->scissor_test_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__scissor_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__states__stencil_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 525:
        {
        cg_pass_type____states__ValidationData* parentValidationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_TEST_ENABLE;
        }
//        parentValidationData->stencil_test_enable++;

        break;
        }

        case 528:
        {
        profile_GLSL__technique__pass__states__ValidationData* parentValidationData = (profile_GLSL__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_TEST_ENABLE;
        }
//        parentValidationData->stencil_test_enable++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__states__stencil_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__program____glsl_program_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__ValidationData* parentValidationData = (profile_GLSL__technique__pass__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PROGRAM, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PROGRAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->program >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PROGRAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->program++;

        program____glsl_program_type__ValidationData* validationData = (program____glsl_program_type__ValidationData*)mValidationDataStack.newObject(sizeof(program____glsl_program_type__ValidationData));
        memset(validationData, 0, sizeof(program____glsl_program_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__program____glsl_program_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        program____glsl_program_type__ValidationData* validationData = (program____glsl_program_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__shader____glsl_shader_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        program____glsl_program_type__ValidationData* parentValidationData = (program____glsl_program_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->bind_attribute != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHADER, (const ParserChar*)0, "sibling: bind_attribute" ) )
                return false;
        }
        if ( parentValidationData->bind_uniform != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SHADER, (const ParserChar*)0, "sibling: bind_uniform" ) )
                return false;
        }
        parentValidationData->shader++;

        shader____glsl_shader_type__ValidationData* validationData = (shader____glsl_shader_type__ValidationData*)mValidationDataStack.newObject(sizeof(shader____glsl_shader_type__ValidationData));
        memset(validationData, 0, sizeof(shader____glsl_shader_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__shader____glsl_shader_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        shader____glsl_shader_type__ValidationData* validationData = (shader____glsl_shader_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->sources < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SHADER, (const ParserChar*)0, "child: sources" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sources____fx_sources_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        shader____glsl_shader_type__ValidationData* parentValidationData = (shader____glsl_shader_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->sources >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SOURCES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->sources++;

        sources____fx_sources_type__ValidationData* validationData = (sources____fx_sources_type__ValidationData*)mValidationDataStack.newObject(sizeof(sources____fx_sources_type__ValidationData));
        memset(validationData, 0, sizeof(sources____fx_sources_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sources____fx_sources_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sources____fx_sources_type__ValidationData* validationData = (sources____fx_sources_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_IMPORT) || (validationData->validation_current_state == HASH_ELEMENT_INLINE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SOURCES, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__glsl_program_type____bind_attribute( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        program____glsl_program_type__ValidationData* parentValidationData = (program____glsl_program_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->bind_uniform != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_ATTRIBUTE, (const ParserChar*)0, "sibling: bind_uniform" ) )
                return false;
        }
        parentValidationData->bind_attribute++;

        glsl_program_type____bind_attribute__ValidationData* validationData = (glsl_program_type____bind_attribute__ValidationData*)mValidationDataStack.newObject(sizeof(glsl_program_type____bind_attribute__ValidationData));
        memset(validationData, 0, sizeof(glsl_program_type____bind_attribute__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__glsl_program_type____bind_attribute()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        glsl_program_type____bind_attribute__ValidationData* validationData = (glsl_program_type____bind_attribute__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->semantic != 0 )
            if ( validationData->semantic < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND_ATTRIBUTE, (const ParserChar*)0, "child: semantic" ) )
                    return false;
        if ( validationData->semantic == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_BIND_ATTRIBUTE, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__glsl_program_type____bind_uniform( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        program____glsl_program_type__ValidationData* parentValidationData = (program____glsl_program_type__ValidationData*)mValidationDataStack.top();
        parentValidationData->bind_uniform++;

        glsl_program_type____bind_uniform__ValidationData* validationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.newObject(sizeof(glsl_program_type____bind_uniform__ValidationData));
        memset(validationData, 0, sizeof(glsl_program_type____bind_uniform__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__glsl_program_type____bind_uniform()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        glsl_program_type____bind_uniform__ValidationData* validationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_PARAM) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_BIND_UNIFORM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__program__bind_uniform__param( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        glsl_program_type____bind_uniform__ValidationData* parentValidationData = (glsl_program_type____bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARAM;
        }
//        parentValidationData->param++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__program__bind_uniform__param()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLSL__technique__pass__evaluate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__ValidationData* parentValidationData = (profile_GLSL__technique__pass__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EVALUATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->evaluate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EVALUATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->evaluate++;

        profile_GLSL__technique__pass__evaluate__ValidationData* validationData = (profile_GLSL__technique__pass__evaluate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLSL__technique__pass__evaluate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLSL__technique__pass__evaluate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLSL__technique__pass__evaluate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLSL__technique__pass__evaluate__ValidationData* validationData = (profile_GLSL__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_CG( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effect__ValidationData* parentValidationData = (effect__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_BRIDGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_CG) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_COMMON) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES2) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLSL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PROFILE_CG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PROFILE_CG;
        }
//        parentValidationData->profile_CG++;

        profile_CG__ValidationData* validationData = (profile_CG__ValidationData*)mValidationDataStack.newObject(sizeof(profile_CG__ValidationData));
        memset(validationData, 0, sizeof(profile_CG__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_CG()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_CG__ValidationData* validationData = (profile_CG__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_PROFILE_CG, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__newparam____cg_newparam_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_CG__ValidationData* parentValidationData = (profile_CG__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEWPARAM;
        }
//        parentValidationData->newparam++;

        newparam____cg_newparam_type__ValidationData* validationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.newObject(sizeof(newparam____cg_newparam_type__ValidationData));
        memset(validationData, 0, sizeof(newparam____cg_newparam_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__newparam____cg_newparam_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____cg_newparam_type__ValidationData* validationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FIXED) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF) || (validationData->validation_current_state == HASH_ELEMENT_HALF2) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF3) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF4) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT2X1) || (validationData->validation_current_state == HASH_ELEMENT_INT2X2) || (validationData->validation_current_state == HASH_ELEMENT_INT2X3) || (validationData->validation_current_state == HASH_ELEMENT_INT2X4) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT3X1) || (validationData->validation_current_state == HASH_ELEMENT_INT3X2) || (validationData->validation_current_state == HASH_ELEMENT_INT3X3) || (validationData->validation_current_state == HASH_ELEMENT_INT3X4) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_INT4X1) || (validationData->validation_current_state == HASH_ELEMENT_INT4X2) || (validationData->validation_current_state == HASH_ELEMENT_INT4X3) || (validationData->validation_current_state == HASH_ELEMENT_INT4X4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (validationData->validation_current_state == HASH_ELEMENT_STRING) || (validationData->validation_current_state == HASH_ELEMENT_USERTYPE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool2x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X1;
        }
//        parentValidationData->bool2x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X1;
        }
//        parentValidationData->bool2x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X1;
        }
//        parentValidationData->bool2x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X1;
        }
//        parentValidationData->bool2x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool2__ValidationData* validationData = (bool2__ValidationData*)mValidationDataStack.newObject(sizeof(bool2__ValidationData));
        memset(validationData, 0, sizeof(bool2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool2x1()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2x2_type( const bool* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2x2_type__stream( const bool* value, size_t length )
{

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2x2_type__streamEnd( const bool* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool2x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X2;
        }
//        parentValidationData->bool2x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X2;
        }
//        parentValidationData->bool2x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X2;
        }
//        parentValidationData->bool2x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X2;
        }
//        parentValidationData->bool2x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool2x2__ValidationData* validationData = (bool2x2__ValidationData*)mValidationDataStack.newObject(sizeof(bool2x2__ValidationData));
        memset(validationData, 0, sizeof(bool2x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool2x2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2x3_type( const bool* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2x3_type__stream( const bool* value, size_t length )
{

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2x3_type__streamEnd( const bool* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool2x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X3;
        }
//        parentValidationData->bool2x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X3;
        }
//        parentValidationData->bool2x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X3;
        }
//        parentValidationData->bool2x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X3;
        }
//        parentValidationData->bool2x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool2x3__ValidationData* validationData = (bool2x3__ValidationData*)mValidationDataStack.newObject(sizeof(bool2x3__ValidationData));
        memset(validationData, 0, sizeof(bool2x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool2x3()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2x4_type( const bool* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2x4_type__stream( const bool* value, size_t length )
{

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool2x4_type__streamEnd( const bool* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool2x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X4;
        }
//        parentValidationData->bool2x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X4;
        }
//        parentValidationData->bool2x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X4;
        }
//        parentValidationData->bool2x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL2X4;
        }
//        parentValidationData->bool2x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool2x4__ValidationData* validationData = (bool2x4__ValidationData*)mValidationDataStack.newObject(sizeof(bool2x4__ValidationData));
        memset(validationData, 0, sizeof(bool2x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool2x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool3x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X1;
        }
//        parentValidationData->bool3x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X1;
        }
//        parentValidationData->bool3x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X1;
        }
//        parentValidationData->bool3x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X1;
        }
//        parentValidationData->bool3x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool3__ValidationData* validationData = (bool3__ValidationData*)mValidationDataStack.newObject(sizeof(bool3__ValidationData));
        memset(validationData, 0, sizeof(bool3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool3x1()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3x2_type( const bool* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3x2_type__stream( const bool* value, size_t length )
{

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3x2_type__streamEnd( const bool* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool3x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X2;
        }
//        parentValidationData->bool3x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X2;
        }
//        parentValidationData->bool3x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X2;
        }
//        parentValidationData->bool3x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X2;
        }
//        parentValidationData->bool3x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool3x2__ValidationData* validationData = (bool3x2__ValidationData*)mValidationDataStack.newObject(sizeof(bool3x2__ValidationData));
        memset(validationData, 0, sizeof(bool3x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool3x2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3x3_type( const bool* value, size_t length )
{
    if( length < 9 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 9 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3x3_type__stream( const bool* value, size_t length )
{

    if( length > 9 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3x3_type__streamEnd( const bool* value, size_t length )
{
    if( length < 9 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 9 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool3x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X3;
        }
//        parentValidationData->bool3x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X3;
        }
//        parentValidationData->bool3x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X3;
        }
//        parentValidationData->bool3x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X3;
        }
//        parentValidationData->bool3x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool3x3__ValidationData* validationData = (bool3x3__ValidationData*)mValidationDataStack.newObject(sizeof(bool3x3__ValidationData));
        memset(validationData, 0, sizeof(bool3x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool3x3()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3x4_type( const bool* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3x4_type__stream( const bool* value, size_t length )
{

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool3x4_type__streamEnd( const bool* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool3x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X4;
        }
//        parentValidationData->bool3x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X4;
        }
//        parentValidationData->bool3x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X4;
        }
//        parentValidationData->bool3x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL3X4;
        }
//        parentValidationData->bool3x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool3x4__ValidationData* validationData = (bool3x4__ValidationData*)mValidationDataStack.newObject(sizeof(bool3x4__ValidationData));
        memset(validationData, 0, sizeof(bool3x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool3x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool4x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X1;
        }
//        parentValidationData->bool4x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X1;
        }
//        parentValidationData->bool4x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X1;
        }
//        parentValidationData->bool4x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X1;
        }
//        parentValidationData->bool4x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool4__ValidationData* validationData = (bool4__ValidationData*)mValidationDataStack.newObject(sizeof(bool4__ValidationData));
        memset(validationData, 0, sizeof(bool4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool4x1()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4x2_type( const bool* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4x2_type__stream( const bool* value, size_t length )
{

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4x2_type__streamEnd( const bool* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool4x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X2;
        }
//        parentValidationData->bool4x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X2;
        }
//        parentValidationData->bool4x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X2;
        }
//        parentValidationData->bool4x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X2;
        }
//        parentValidationData->bool4x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool4x2__ValidationData* validationData = (bool4x2__ValidationData*)mValidationDataStack.newObject(sizeof(bool4x2__ValidationData));
        memset(validationData, 0, sizeof(bool4x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool4x2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4x3_type( const bool* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4x3_type__stream( const bool* value, size_t length )
{

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4x3_type__streamEnd( const bool* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool4x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X3;
        }
//        parentValidationData->bool4x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X3;
        }
//        parentValidationData->bool4x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X3;
        }
//        parentValidationData->bool4x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X3;
        }
//        parentValidationData->bool4x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool4x3__ValidationData* validationData = (bool4x3__ValidationData*)mValidationDataStack.newObject(sizeof(bool4x3__ValidationData));
        memset(validationData, 0, sizeof(bool4x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool4x3()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4x4_type( const bool* value, size_t length )
{
    if( length < 16 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 16 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4x4_type__stream( const bool* value, size_t length )
{

    if( length > 16 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__bool4x4_type__streamEnd( const bool* value, size_t length )
{
    if( length < 16 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 16 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bool4x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X4;
        }
//        parentValidationData->bool4x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X4;
        }
//        parentValidationData->bool4x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X4;
        }
//        parentValidationData->bool4x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOOL4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOOL4X4;
        }
//        parentValidationData->bool4x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bool4x4__ValidationData* validationData = (bool4x4__ValidationData*)mValidationDataStack.newObject(sizeof(bool4x4__ValidationData));
        memset(validationData, 0, sizeof(bool4x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bool4x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int2x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X1;
        }
//        parentValidationData->int2x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X1;
        }
//        parentValidationData->int2x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X1;
        }
//        parentValidationData->int2x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X1;
        }
//        parentValidationData->int2x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int2__ValidationData* validationData = (int2__ValidationData*)mValidationDataStack.newObject(sizeof(int2__ValidationData));
        memset(validationData, 0, sizeof(int2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int2x1()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2x2_type( const sint64* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2x2_type__stream( const sint64* value, size_t length )
{

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2x2_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int2x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X2;
        }
//        parentValidationData->int2x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X2;
        }
//        parentValidationData->int2x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X2;
        }
//        parentValidationData->int2x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X2;
        }
//        parentValidationData->int2x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int2x2__ValidationData* validationData = (int2x2__ValidationData*)mValidationDataStack.newObject(sizeof(int2x2__ValidationData));
        memset(validationData, 0, sizeof(int2x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int2x2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2x3_type( const sint64* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2x3_type__stream( const sint64* value, size_t length )
{

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2x3_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int2x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X3;
        }
//        parentValidationData->int2x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X3;
        }
//        parentValidationData->int2x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X3;
        }
//        parentValidationData->int2x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X3;
        }
//        parentValidationData->int2x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int2x3__ValidationData* validationData = (int2x3__ValidationData*)mValidationDataStack.newObject(sizeof(int2x3__ValidationData));
        memset(validationData, 0, sizeof(int2x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int2x3()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2x4_type( const sint64* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2x4_type__stream( const sint64* value, size_t length )
{

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int2x4_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int2x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X4;
        }
//        parentValidationData->int2x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X4;
        }
//        parentValidationData->int2x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X4;
        }
//        parentValidationData->int2x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT2X4;
        }
//        parentValidationData->int2x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int2x4__ValidationData* validationData = (int2x4__ValidationData*)mValidationDataStack.newObject(sizeof(int2x4__ValidationData));
        memset(validationData, 0, sizeof(int2x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int2x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int3x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X1;
        }
//        parentValidationData->int3x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X1;
        }
//        parentValidationData->int3x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X1;
        }
//        parentValidationData->int3x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X1;
        }
//        parentValidationData->int3x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int3__ValidationData* validationData = (int3__ValidationData*)mValidationDataStack.newObject(sizeof(int3__ValidationData));
        memset(validationData, 0, sizeof(int3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int3x1()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3x2_type( const sint64* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3x2_type__stream( const sint64* value, size_t length )
{

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3x2_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 6 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 6 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int3x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X2;
        }
//        parentValidationData->int3x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X2;
        }
//        parentValidationData->int3x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X2;
        }
//        parentValidationData->int3x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X2;
        }
//        parentValidationData->int3x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int3x2__ValidationData* validationData = (int3x2__ValidationData*)mValidationDataStack.newObject(sizeof(int3x2__ValidationData));
        memset(validationData, 0, sizeof(int3x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int3x2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3x3_type( const sint64* value, size_t length )
{
    if( length < 9 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 9 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3x3_type__stream( const sint64* value, size_t length )
{

    if( length > 9 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3x3_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 9 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 9 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int3x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X3;
        }
//        parentValidationData->int3x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X3;
        }
//        parentValidationData->int3x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X3;
        }
//        parentValidationData->int3x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X3;
        }
//        parentValidationData->int3x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int3x3__ValidationData* validationData = (int3x3__ValidationData*)mValidationDataStack.newObject(sizeof(int3x3__ValidationData));
        memset(validationData, 0, sizeof(int3x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int3x3()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3x4_type( const sint64* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3x4_type__stream( const sint64* value, size_t length )
{

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int3x4_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int3x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X4;
        }
//        parentValidationData->int3x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X4;
        }
//        parentValidationData->int3x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X4;
        }
//        parentValidationData->int3x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT3X4;
        }
//        parentValidationData->int3x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int3x4__ValidationData* validationData = (int3x4__ValidationData*)mValidationDataStack.newObject(sizeof(int3x4__ValidationData));
        memset(validationData, 0, sizeof(int3x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int3x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int4x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X1;
        }
//        parentValidationData->int4x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X1;
        }
//        parentValidationData->int4x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X1;
        }
//        parentValidationData->int4x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X1;
        }
//        parentValidationData->int4x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int4__ValidationData* validationData = (int4__ValidationData*)mValidationDataStack.newObject(sizeof(int4__ValidationData));
        memset(validationData, 0, sizeof(int4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int4x1()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4x2_type( const sint64* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4x2_type__stream( const sint64* value, size_t length )
{

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4x2_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 8 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 8 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int4x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X2;
        }
//        parentValidationData->int4x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X2;
        }
//        parentValidationData->int4x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X2;
        }
//        parentValidationData->int4x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X2;
        }
//        parentValidationData->int4x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int4x2__ValidationData* validationData = (int4x2__ValidationData*)mValidationDataStack.newObject(sizeof(int4x2__ValidationData));
        memset(validationData, 0, sizeof(int4x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int4x2()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4x3_type( const sint64* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4x3_type__stream( const sint64* value, size_t length )
{

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4x3_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 12 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 12 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int4x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X3;
        }
//        parentValidationData->int4x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X3;
        }
//        parentValidationData->int4x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X3;
        }
//        parentValidationData->int4x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X3;
        }
//        parentValidationData->int4x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int4x3__ValidationData* validationData = (int4x3__ValidationData*)mValidationDataStack.newObject(sizeof(int4x3__ValidationData));
        memset(validationData, 0, sizeof(int4x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int4x3()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4x4_type( const sint64* value, size_t length )
{
    if( length < 16 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 16 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4x4_type__stream( const sint64* value, size_t length )
{

    if( length > 16 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__int4x4_type__streamEnd( const sint64* value, size_t length )
{
    if( length < 16 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 16 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int4x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X4;
        }
//        parentValidationData->int4x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X4;
        }
//        parentValidationData->int4x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X4;
        }
//        parentValidationData->int4x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT4X4;
        }
//        parentValidationData->int4x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        int4x4__ValidationData* validationData = (int4x4__ValidationData*)mValidationDataStack.newObject(sizeof(int4x4__ValidationData));
        memset(validationData, 0, sizeof(int4x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int4x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF;
        }
//        parentValidationData->half++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF;
        }
//        parentValidationData->half++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF;
        }
//        parentValidationData->half++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF;
        }
//        parentValidationData->half++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2;
        }
//        parentValidationData->half2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2;
        }
//        parentValidationData->half2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2;
        }
//        parentValidationData->half2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2;
        }
//        parentValidationData->half2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        radius____float2_type__ValidationData* validationData = (radius____float2_type__ValidationData*)mValidationDataStack.newObject(sizeof(radius____float2_type__ValidationData));
        memset(validationData, 0, sizeof(radius____float2_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3;
        }
//        parentValidationData->half3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3;
        }
//        parentValidationData->half3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3;
        }
//        parentValidationData->half3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3;
        }
//        parentValidationData->half3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4;
        }
//        parentValidationData->half4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4;
        }
//        parentValidationData->half4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4;
        }
//        parentValidationData->half4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4;
        }
//        parentValidationData->half4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        orient__ValidationData* validationData = (orient__ValidationData*)mValidationDataStack.newObject(sizeof(orient__ValidationData));
        memset(validationData, 0, sizeof(orient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half2x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X1;
        }
//        parentValidationData->half2x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X1;
        }
//        parentValidationData->half2x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X1;
        }
//        parentValidationData->half2x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X1;
        }
//        parentValidationData->half2x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        radius____float2_type__ValidationData* validationData = (radius____float2_type__ValidationData*)mValidationDataStack.newObject(sizeof(radius____float2_type__ValidationData));
        memset(validationData, 0, sizeof(radius____float2_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half2x1()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half2x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X2;
        }
//        parentValidationData->half2x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X2;
        }
//        parentValidationData->half2x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X2;
        }
//        parentValidationData->half2x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X2;
        }
//        parentValidationData->half2x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float2x2__ValidationData* validationData = (float2x2__ValidationData*)mValidationDataStack.newObject(sizeof(float2x2__ValidationData));
        memset(validationData, 0, sizeof(float2x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half2x2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half2x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X3;
        }
//        parentValidationData->half2x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X3;
        }
//        parentValidationData->half2x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X3;
        }
//        parentValidationData->half2x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X3;
        }
//        parentValidationData->half2x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float2x3__ValidationData* validationData = (float2x3__ValidationData*)mValidationDataStack.newObject(sizeof(float2x3__ValidationData));
        memset(validationData, 0, sizeof(float2x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half2x3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half2x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X4;
        }
//        parentValidationData->half2x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X4;
        }
//        parentValidationData->half2x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X4;
        }
//        parentValidationData->half2x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF2X4;
        }
//        parentValidationData->half2x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float2x4__ValidationData* validationData = (float2x4__ValidationData*)mValidationDataStack.newObject(sizeof(float2x4__ValidationData));
        memset(validationData, 0, sizeof(float2x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half2x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half3x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X1;
        }
//        parentValidationData->half3x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X1;
        }
//        parentValidationData->half3x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X1;
        }
//        parentValidationData->half3x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X1;
        }
//        parentValidationData->half3x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half3x1()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half3x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X2;
        }
//        parentValidationData->half3x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X2;
        }
//        parentValidationData->half3x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X2;
        }
//        parentValidationData->half3x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X2;
        }
//        parentValidationData->half3x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float3x2__ValidationData* validationData = (float3x2__ValidationData*)mValidationDataStack.newObject(sizeof(float3x2__ValidationData));
        memset(validationData, 0, sizeof(float3x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half3x2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half3x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X3;
        }
//        parentValidationData->half3x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X3;
        }
//        parentValidationData->half3x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X3;
        }
//        parentValidationData->half3x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X3;
        }
//        parentValidationData->half3x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float3x3__ValidationData* validationData = (float3x3__ValidationData*)mValidationDataStack.newObject(sizeof(float3x3__ValidationData));
        memset(validationData, 0, sizeof(float3x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half3x3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half3x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X4;
        }
//        parentValidationData->half3x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X4;
        }
//        parentValidationData->half3x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X4;
        }
//        parentValidationData->half3x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF3X4;
        }
//        parentValidationData->half3x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float3x4__ValidationData* validationData = (float3x4__ValidationData*)mValidationDataStack.newObject(sizeof(float3x4__ValidationData));
        memset(validationData, 0, sizeof(float3x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half3x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half4x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X1;
        }
//        parentValidationData->half4x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X1;
        }
//        parentValidationData->half4x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X1;
        }
//        parentValidationData->half4x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X1;
        }
//        parentValidationData->half4x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        orient__ValidationData* validationData = (orient__ValidationData*)mValidationDataStack.newObject(sizeof(orient__ValidationData));
        memset(validationData, 0, sizeof(orient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half4x1()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half4x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X2;
        }
//        parentValidationData->half4x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X2;
        }
//        parentValidationData->half4x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X2;
        }
//        parentValidationData->half4x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X2;
        }
//        parentValidationData->half4x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float4x2__ValidationData* validationData = (float4x2__ValidationData*)mValidationDataStack.newObject(sizeof(float4x2__ValidationData));
        memset(validationData, 0, sizeof(float4x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half4x2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half4x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X3;
        }
//        parentValidationData->half4x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X3;
        }
//        parentValidationData->half4x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X3;
        }
//        parentValidationData->half4x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X3;
        }
//        parentValidationData->half4x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float4x3__ValidationData* validationData = (float4x3__ValidationData*)mValidationDataStack.newObject(sizeof(float4x3__ValidationData));
        memset(validationData, 0, sizeof(float4x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half4x3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half4x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X4;
        }
//        parentValidationData->_half4x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X4;
        }
//        parentValidationData->_half4x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X4;
        }
//        parentValidationData->_half4x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HALF4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HALF4X4;
        }
//        parentValidationData->_half4x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bind_shape_matrix__ValidationData* validationData = (bind_shape_matrix__ValidationData*)mValidationDataStack.newObject(sizeof(bind_shape_matrix__ValidationData));
        memset(validationData, 0, sizeof(bind_shape_matrix__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half4x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED;
        }
//        parentValidationData->fixed++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED;
        }
//        parentValidationData->fixed++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED;
        }
//        parentValidationData->fixed++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED;
        }
//        parentValidationData->fixed++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2;
        }
//        parentValidationData->fixed2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2;
        }
//        parentValidationData->fixed2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2;
        }
//        parentValidationData->fixed2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2;
        }
//        parentValidationData->fixed2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        radius____float2_type__ValidationData* validationData = (radius____float2_type__ValidationData*)mValidationDataStack.newObject(sizeof(radius____float2_type__ValidationData));
        memset(validationData, 0, sizeof(radius____float2_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3;
        }
//        parentValidationData->fixed3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3;
        }
//        parentValidationData->fixed3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3;
        }
//        parentValidationData->fixed3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3;
        }
//        parentValidationData->fixed3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4;
        }
//        parentValidationData->fixed4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4;
        }
//        parentValidationData->fixed4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4;
        }
//        parentValidationData->fixed4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4;
        }
//        parentValidationData->fixed4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        orient__ValidationData* validationData = (orient__ValidationData*)mValidationDataStack.newObject(sizeof(orient__ValidationData));
        memset(validationData, 0, sizeof(orient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed2x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X1;
        }
//        parentValidationData->fixed2x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X1;
        }
//        parentValidationData->fixed2x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X1;
        }
//        parentValidationData->fixed2x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X1;
        }
//        parentValidationData->fixed2x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        radius____float2_type__ValidationData* validationData = (radius____float2_type__ValidationData*)mValidationDataStack.newObject(sizeof(radius____float2_type__ValidationData));
        memset(validationData, 0, sizeof(radius____float2_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed2x1()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed2x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X2;
        }
//        parentValidationData->fixed2x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X2;
        }
//        parentValidationData->fixed2x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X2;
        }
//        parentValidationData->fixed2x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X2;
        }
//        parentValidationData->fixed2x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float2x2__ValidationData* validationData = (float2x2__ValidationData*)mValidationDataStack.newObject(sizeof(float2x2__ValidationData));
        memset(validationData, 0, sizeof(float2x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed2x2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed2x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X3;
        }
//        parentValidationData->fixed2x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X3;
        }
//        parentValidationData->fixed2x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X3;
        }
//        parentValidationData->fixed2x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X3;
        }
//        parentValidationData->fixed2x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float2x3__ValidationData* validationData = (float2x3__ValidationData*)mValidationDataStack.newObject(sizeof(float2x3__ValidationData));
        memset(validationData, 0, sizeof(float2x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed2x3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed2x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X4;
        }
//        parentValidationData->fixed2x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X4;
        }
//        parentValidationData->fixed2x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X4;
        }
//        parentValidationData->fixed2x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED2X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED2X4;
        }
//        parentValidationData->fixed2x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float2x4__ValidationData* validationData = (float2x4__ValidationData*)mValidationDataStack.newObject(sizeof(float2x4__ValidationData));
        memset(validationData, 0, sizeof(float2x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed2x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed3x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X1;
        }
//        parentValidationData->fixed3x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X1;
        }
//        parentValidationData->fixed3x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X1;
        }
//        parentValidationData->fixed3x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X1;
        }
//        parentValidationData->fixed3x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed3x1()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed3x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X2;
        }
//        parentValidationData->fixed3x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X2;
        }
//        parentValidationData->fixed3x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X2;
        }
//        parentValidationData->fixed3x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X2;
        }
//        parentValidationData->fixed3x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float3x2__ValidationData* validationData = (float3x2__ValidationData*)mValidationDataStack.newObject(sizeof(float3x2__ValidationData));
        memset(validationData, 0, sizeof(float3x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed3x2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed3x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X3;
        }
//        parentValidationData->fixed3x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X3;
        }
//        parentValidationData->fixed3x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X3;
        }
//        parentValidationData->fixed3x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X3;
        }
//        parentValidationData->fixed3x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float3x3__ValidationData* validationData = (float3x3__ValidationData*)mValidationDataStack.newObject(sizeof(float3x3__ValidationData));
        memset(validationData, 0, sizeof(float3x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed3x3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed3x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X4;
        }
//        parentValidationData->fixed3x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X4;
        }
//        parentValidationData->fixed3x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X4;
        }
//        parentValidationData->fixed3x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED3X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED3X4;
        }
//        parentValidationData->fixed3x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float3x4__ValidationData* validationData = (float3x4__ValidationData*)mValidationDataStack.newObject(sizeof(float3x4__ValidationData));
        memset(validationData, 0, sizeof(float3x4__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed3x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed4x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X1;
        }
//        parentValidationData->fixed4x1++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X1;
        }
//        parentValidationData->fixed4x1++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X1;
        }
//        parentValidationData->fixed4x1++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X1;
        }
//        parentValidationData->fixed4x1++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        orient__ValidationData* validationData = (orient__ValidationData*)mValidationDataStack.newObject(sizeof(orient__ValidationData));
        memset(validationData, 0, sizeof(orient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed4x1()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed4x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X2;
        }
//        parentValidationData->fixed4x2++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X2;
        }
//        parentValidationData->fixed4x2++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X2;
        }
//        parentValidationData->fixed4x2++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X2;
        }
//        parentValidationData->fixed4x2++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float4x2__ValidationData* validationData = (float4x2__ValidationData*)mValidationDataStack.newObject(sizeof(float4x2__ValidationData));
        memset(validationData, 0, sizeof(float4x2__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed4x2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed4x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X3;
        }
//        parentValidationData->fixed4x3++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X3;
        }
//        parentValidationData->fixed4x3++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X3;
        }
//        parentValidationData->fixed4x3++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X3;
        }
//        parentValidationData->fixed4x3++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        float4x3__ValidationData* validationData = (float4x3__ValidationData*)mValidationDataStack.newObject(sizeof(float4x3__ValidationData));
        memset(validationData, 0, sizeof(float4x3__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed4x3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__fixed4x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X4;
        }
//        parentValidationData->fixed4x4++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X4;
        }
//        parentValidationData->fixed4x4++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X4;
        }
//        parentValidationData->fixed4x4++;

        break;
        }

        case 741:
        {
        setparam____cg_setparam_type__ValidationData* parentValidationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FIXED4X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FIXED4X4;
        }
//        parentValidationData->fixed4x4++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bind_shape_matrix__ValidationData* validationData = (bind_shape_matrix__ValidationData*)mValidationDataStack.newObject(sizeof(bind_shape_matrix__ValidationData));
        memset(validationData, 0, sizeof(bind_shape_matrix__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__fixed4x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__array____cg_array_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        array____cg_array_type__ValidationData* validationData = (array____cg_array_type__ValidationData*)mValidationDataStack.newObject(sizeof(array____cg_array_type__ValidationData));
        memset(validationData, 0, sizeof(array____cg_array_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__array____cg_array_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        array____cg_array_type__ValidationData* validationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FIXED) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF) || (validationData->validation_current_state == HASH_ELEMENT_HALF2) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF3) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF4) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT2X1) || (validationData->validation_current_state == HASH_ELEMENT_INT2X2) || (validationData->validation_current_state == HASH_ELEMENT_INT2X3) || (validationData->validation_current_state == HASH_ELEMENT_INT2X4) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT3X1) || (validationData->validation_current_state == HASH_ELEMENT_INT3X2) || (validationData->validation_current_state == HASH_ELEMENT_INT3X3) || (validationData->validation_current_state == HASH_ELEMENT_INT3X4) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_INT4X1) || (validationData->validation_current_state == HASH_ELEMENT_INT4X2) || (validationData->validation_current_state == HASH_ELEMENT_INT4X3) || (validationData->validation_current_state == HASH_ELEMENT_INT4X4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (validationData->validation_current_state == HASH_ELEMENT_STRING) || (validationData->validation_current_state == HASH_ELEMENT_USERTYPE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__usertype____cg_user_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 375:
        {
        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_USERTYPE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_USERTYPE;
        }
//        parentValidationData->usertype++;

        break;
        }

        case 404:
        {
        array____cg_array_type__ValidationData* parentValidationData = (array____cg_array_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_ENUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT2X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT3X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X1) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X2) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X3) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT4X4) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_STRING) || (parentValidationData->validation_current_state == HASH_ELEMENT_USERTYPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_USERTYPE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_USERTYPE;
        }
//        parentValidationData->usertype++;

        break;
        }

        case 655:
        {
        newparam____cg_newparam_type__ValidationData* parentValidationData = (newparam____cg_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_USERTYPE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_USERTYPE;
        }
//        parentValidationData->usertype++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        usertype____cg_user_type__ValidationData* validationData = (usertype____cg_user_type__ValidationData*)mValidationDataStack.newObject(sizeof(usertype____cg_user_type__ValidationData));
        memset(validationData, 0, sizeof(usertype____cg_user_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__usertype____cg_user_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        usertype____cg_user_type__ValidationData* validationData = (usertype____cg_user_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__setparam____cg_setparam_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        usertype____cg_user_type__ValidationData* parentValidationData = (usertype____cg_user_type__ValidationData*)mValidationDataStack.top();
        parentValidationData->setparam++;

        setparam____cg_setparam_type__ValidationData* validationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.newObject(sizeof(setparam____cg_setparam_type__ValidationData));
        memset(validationData, 0, sizeof(setparam____cg_setparam_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__setparam____cg_setparam_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        setparam____cg_setparam_type__ValidationData* validationData = (setparam____cg_setparam_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FIXED) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF) || (validationData->validation_current_state == HASH_ELEMENT_HALF2) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF3) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF4) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT2X1) || (validationData->validation_current_state == HASH_ELEMENT_INT2X2) || (validationData->validation_current_state == HASH_ELEMENT_INT2X3) || (validationData->validation_current_state == HASH_ELEMENT_INT2X4) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT3X1) || (validationData->validation_current_state == HASH_ELEMENT_INT3X2) || (validationData->validation_current_state == HASH_ELEMENT_INT3X3) || (validationData->validation_current_state == HASH_ELEMENT_INT3X4) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_INT4X1) || (validationData->validation_current_state == HASH_ELEMENT_INT4X2) || (validationData->validation_current_state == HASH_ELEMENT_INT4X3) || (validationData->validation_current_state == HASH_ELEMENT_INT4X4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (validationData->validation_current_state == HASH_ELEMENT_STRING) || (validationData->validation_current_state == HASH_ELEMENT_USERTYPE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_cg_type____technique( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_CG__ValidationData* parentValidationData = (profile_CG__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INCLUDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TECHNIQUE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TECHNIQUE;
        }
//        parentValidationData->technique++;

        profile_cg_type____technique__ValidationData* validationData = (profile_cg_type____technique__ValidationData*)mValidationDataStack.newObject(sizeof(profile_cg_type____technique__ValidationData));
        memset(validationData, 0, sizeof(profile_cg_type____technique__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_cg_type____technique()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_cg_type____technique__ValidationData* validationData = (profile_cg_type____technique__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->pass < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "child: pass" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__pass____cg_pass_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_cg_type____technique__ValidationData* parentValidationData = (profile_cg_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PASS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->pass++;

        pass____cg_pass_type__ValidationData* validationData = (pass____cg_pass_type__ValidationData*)mValidationDataStack.newObject(sizeof(pass____cg_pass_type__ValidationData));
        memset(validationData, 0, sizeof(pass____cg_pass_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__pass____cg_pass_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        pass____cg_pass_type__ValidationData* validationData = (pass____cg_pass_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__cg_pass_type____states( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        pass____cg_pass_type__ValidationData* parentValidationData = (pass____cg_pass_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->program != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: program" ) )
                return false;
        }
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->states >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_STATES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->states++;

        cg_pass_type____states__ValidationData* validationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.newObject(sizeof(cg_pass_type____states__ValidationData));
        memset(validationData, 0, sizeof(cg_pass_type____states__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__cg_pass_type____states()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        cg_pass_type____states__ValidationData* validationData = (cg_pass_type____states__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_EQUATION_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (validationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (validationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_BOUNDS_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_CLAMP_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_FOG_COORD_SRC) || (validationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (validationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_FOG_END) || (validationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (validationData->validation_current_state == HASH_ELEMENT_FOG_START) || (validationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (validationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE) || (validationData->validation_current_state == HASH_ELEMENT_LINE_STIPPLE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (validationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (validationData->validation_current_state == HASH_ELEMENT_LOGIC_OP_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (validationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_MODE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_SMOOTH_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_STIPPLE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (validationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_OP_SEPARATE) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE1D) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE1D_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE2D) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE2D_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE3D) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE3D_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURECUBE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_TEXTUREDEPTH_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURERECT) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURERECT_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE_ENV_MODE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_STATES, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__cg_pass_type____program( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        pass____cg_pass_type__ValidationData* parentValidationData = (pass____cg_pass_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PROGRAM, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PROGRAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->program >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PROGRAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->program++;

        cg_pass_type____program__ValidationData* validationData = (cg_pass_type____program__ValidationData*)mValidationDataStack.newObject(sizeof(cg_pass_type____program__ValidationData));
        memset(validationData, 0, sizeof(cg_pass_type____program__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__cg_pass_type____program()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        cg_pass_type____program__ValidationData* validationData = (cg_pass_type____program__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_CG__technique__pass__program__shader( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        cg_pass_type____program__ValidationData* parentValidationData = (cg_pass_type____program__ValidationData*)mValidationDataStack.top();
        parentValidationData->shader++;

        profile_CG__technique__pass__program__shader__ValidationData* validationData = (profile_CG__technique__pass__program__shader__ValidationData*)mValidationDataStack.newObject(sizeof(profile_CG__technique__pass__program__shader__ValidationData));
        memset(validationData, 0, sizeof(profile_CG__technique__pass__program__shader__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_CG__technique__pass__program__shader()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_CG__technique__pass__program__shader__ValidationData* validationData = (profile_CG__technique__pass__program__shader__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->sources < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SHADER, (const ParserChar*)0, "child: sources" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_CG__technique__pass__program__shader__sources( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_CG__technique__pass__program__shader__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->compiler != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCES, (const ParserChar*)0, "sibling: compiler" ) )
                return false;
        }
        if ( parentValidationData->bind_uniform != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SOURCES, (const ParserChar*)0, "sibling: bind_uniform" ) )
                return false;
        }
        if ( parentValidationData->sources >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SOURCES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->sources++;

        profile_CG__technique__pass__program__shader__sources__ValidationData* validationData = (profile_CG__technique__pass__program__shader__sources__ValidationData*)mValidationDataStack.newObject(sizeof(profile_CG__technique__pass__program__shader__sources__ValidationData));
        memset(validationData, 0, sizeof(profile_CG__technique__pass__program__shader__sources__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_CG__technique__pass__program__shader__sources()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_CG__technique__pass__program__shader__sources__ValidationData* validationData = (profile_CG__technique__pass__program__shader__sources__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_IMPORT) || (validationData->validation_current_state == HASH_ELEMENT_INLINE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SOURCES, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_CG__technique__pass__program__shader__bind_uniform( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_CG__technique__pass__program__shader__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->sources < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_BIND_UNIFORM, (const ParserChar*)0, "sibling: sources" ) )
                return false;
        }
        parentValidationData->bind_uniform++;

        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* validationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.newObject(sizeof(profile_CG__technique__pass__program__shader__bind_uniform__ValidationData));
        memset(validationData, 0, sizeof(profile_CG__technique__pass__program__shader__bind_uniform__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_CG__technique__pass__program__shader__bind_uniform()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* validationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2X4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3X4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X1) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4X4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FIXED) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED2X4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED3X4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X1) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X2) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X3) || (validationData->validation_current_state == HASH_ELEMENT_FIXED4X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF) || (validationData->validation_current_state == HASH_ELEMENT_HALF2) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF2X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF3) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF3X4) || (validationData->validation_current_state == HASH_ELEMENT_HALF4) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X1) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X2) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X3) || (validationData->validation_current_state == HASH_ELEMENT_HALF4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT2X1) || (validationData->validation_current_state == HASH_ELEMENT_INT2X2) || (validationData->validation_current_state == HASH_ELEMENT_INT2X3) || (validationData->validation_current_state == HASH_ELEMENT_INT2X4) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT3X1) || (validationData->validation_current_state == HASH_ELEMENT_INT3X2) || (validationData->validation_current_state == HASH_ELEMENT_INT3X3) || (validationData->validation_current_state == HASH_ELEMENT_INT3X4) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_INT4X1) || (validationData->validation_current_state == HASH_ELEMENT_INT4X2) || (validationData->validation_current_state == HASH_ELEMENT_INT4X3) || (validationData->validation_current_state == HASH_ELEMENT_INT4X4) || (validationData->validation_current_state == HASH_ELEMENT_PARAM) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER1D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER3D) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERCUBE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERDEPTH) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLERRECT) || (validationData->validation_current_state == HASH_ELEMENT_STRING) || (validationData->validation_current_state == HASH_ELEMENT_USERTYPE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_BIND_UNIFORM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_CG__technique__pass__program__shader__bind_uniform__param( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_CG__technique__pass__program__shader__bind_uniform__ValidationData* parentValidationData = (profile_CG__technique__pass__program__shader__bind_uniform__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARAM;
        }
//        parentValidationData->param++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_CG__technique__pass__program__shader__bind_uniform__param()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__cg_pass_type____evaluate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        pass____cg_pass_type__ValidationData* parentValidationData = (pass____cg_pass_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EVALUATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->evaluate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EVALUATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->evaluate++;

        cg_pass_type____evaluate__ValidationData* validationData = (cg_pass_type____evaluate__ValidationData*)mValidationDataStack.newObject(sizeof(cg_pass_type____evaluate__ValidationData));
        memset(validationData, 0, sizeof(cg_pass_type____evaluate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__cg_pass_type____evaluate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        cg_pass_type____evaluate__ValidationData* validationData = (cg_pass_type____evaluate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effect__ValidationData* parentValidationData = (effect__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_BRIDGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_CG) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_COMMON) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLES2) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROFILE_GLSL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PROFILE_GLES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PROFILE_GLES;
        }
//        parentValidationData->profile_GLES++;

        profile_GLES__ValidationData* validationData = (profile_GLES__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__ValidationData* validationData = (profile_GLES__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PROFILE_GLES, (const ParserChar*)0, "child: technique" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__newparam____gles_newparam_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__ValidationData* parentValidationData = (profile_GLES__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->newparam++;

        newparam____gles_newparam_type__ValidationData* validationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.newObject(sizeof(newparam____gles_newparam_type__ValidationData));
        memset(validationData, 0, sizeof(newparam____gles_newparam_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__newparam____gles_newparam_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____gles_newparam_type__ValidationData* validationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT1X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT1X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT1X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT1X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER2D) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float1x1( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT1X1, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT1X1;
        }
//        parentValidationData->float1x1++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float1x1()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float1x2( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT1X2, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT1X2;
        }
//        parentValidationData->float1x2++;

        radius____float2_type__ValidationData* validationData = (radius____float2_type__ValidationData*)mValidationDataStack.newObject(sizeof(radius____float2_type__ValidationData));
        memset(validationData, 0, sizeof(radius____float2_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float1x2()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float1x3( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT1X3, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT1X3;
        }
//        parentValidationData->float1x3++;

        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float1x3()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__float1x4( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOAT1X4, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOAT1X4;
        }
//        parentValidationData->float1x4++;

        orient__ValidationData* validationData = (orient__ValidationData*)mValidationDataStack.newObject(sizeof(orient__ValidationData));
        memset(validationData, 0, sizeof(orient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__float1x4()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sampler2D____gles_sampler_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER2D;
        }
//        parentValidationData->sampler2D++;

        sampler2D____gles_sampler_type__ValidationData* validationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.newObject(sizeof(sampler2D____gles_sampler_type__ValidationData));
        memset(validationData, 0, sizeof(sampler2D____gles_sampler_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sampler2D____gles_sampler_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler2D____gles_sampler_type__ValidationData* validationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_MAGFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MINFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIPFILTER) || (validationData->validation_current_state == HASH_ELEMENT_MIP_BIAS) || (validationData->validation_current_state == HASH_ELEMENT_MIP_MAX_LEVEL) || (validationData->validation_current_state == HASH_ELEMENT_TEXCOORD) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (validationData->validation_current_state == HASH_ELEMENT_WRAP_T) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SAMPLER2D, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texcoord( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler2D____gles_sampler_type__ValidationData* parentValidationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXCOORD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXCOORD;
        }
//        parentValidationData->texcoord++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texcoord()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_sampler_wrap_enum( ParserString value )
{
    return validate__gles_sampler_wrap_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_sampler_wrap_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_sampler_wrap_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_sampler_wrap_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__wrap_s____gles_sampler_wrap_enum( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler2D____gles_sampler_type__ValidationData* parentValidationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXCOORD) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_S, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_S;
        }
//        parentValidationData->wrap_s++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__wrap_s____gles_sampler_wrap_enum()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__wrap_t____gles_sampler_wrap_enum( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler2D____gles_sampler_type__ValidationData* parentValidationData = (sampler2D____gles_sampler_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXCOORD) || (parentValidationData->validation_current_state == HASH_ELEMENT_WRAP_S) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_WRAP_T, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_WRAP_T;
        }
//        parentValidationData->wrap_t++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__wrap_t____gles_sampler_wrap_enum()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__enum____gles_enumeration_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____gles_newparam_type__ValidationData* parentValidationData = (newparam____gles_newparam_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODIFIER) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ENUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ENUM;
        }
//        parentValidationData->_enum++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__enum____gles_enumeration_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_gles_type____technique( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__ValidationData* parentValidationData = (profile_GLES__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique++;

        profile_gles_type____technique__ValidationData* validationData = (profile_gles_type____technique__ValidationData*)mValidationDataStack.newObject(sizeof(profile_gles_type____technique__ValidationData));
        memset(validationData, 0, sizeof(profile_gles_type____technique__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_gles_type____technique()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_gles_type____technique__ValidationData* validationData = (profile_gles_type____technique__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->pass < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE, (const ParserChar*)0, "child: pass" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_gles_type____technique__ValidationData* parentValidationData = (profile_gles_type____technique__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PASS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->pass++;

        profile_GLES__technique__pass__ValidationData* validationData = (profile_GLES__technique__pass__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES__technique__pass__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES__technique__pass__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__ValidationData* validationData = (profile_GLES__technique__pass__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__ValidationData* parentValidationData = (profile_GLES__technique__pass__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->evaluate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: evaluate" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STATES, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->states >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_STATES, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->states++;

        profile_GLES__technique__pass__states__ValidationData* validationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES__technique__pass__states__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES__technique__pass__states__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* validationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (validationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (validationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (validationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (validationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (validationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (validationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_FOG_END) || (validationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (validationData->validation_current_state == HASH_ELEMENT_FOG_START) || (validationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (validationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (validationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (validationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (validationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (validationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (validationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (validationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (validationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (validationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (validationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (validationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_STATES, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__alpha_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ALPHA_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ALPHA_FUNC;
        }
//        parentValidationData->alpha_func++;

        profile_GLES__technique__pass__states__alpha_func__ValidationData* validationData = (profile_GLES__technique__pass__states__alpha_func__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES__technique__pass__states__alpha_func__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES__technique__pass__states__alpha_func__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__alpha_func()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__alpha_func__ValidationData* validationData = (profile_GLES__technique__pass__states__alpha_func__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->func < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ALPHA_FUNC, (const ParserChar*)0, "child: func" ) )
                return false;
        }
        if ( validationData->value < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ALPHA_FUNC, (const ParserChar*)0, "child: value" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__alpha_func__func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__alpha_func__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__alpha_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FUNC, (const ParserChar*)0, "sibling: value" ) )
                return false;
        }
        if ( parentValidationData->func >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FUNC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->func++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__alpha_func__func()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__alpha_func__value( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__alpha_func__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__alpha_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->func < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, "sibling: func" ) )
                return false;
        }
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__alpha_func__value()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__blend_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_FUNC;
        }
//        parentValidationData->blend_func++;

        profile_GLES__technique__pass__states__blend_func__ValidationData* validationData = (profile_GLES__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES__technique__pass__states__blend_func__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES__technique__pass__states__blend_func__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__blend_func()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__blend_func__ValidationData* validationData = (profile_GLES__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->src < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC, (const ParserChar*)0, "child: src" ) )
                return false;
        }
        if ( validationData->dest < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BLEND_FUNC, (const ParserChar*)0, "child: dest" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__blend_func__src( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__blend_func__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->dest != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SRC, (const ParserChar*)0, "sibling: dest" ) )
                return false;
        }
        if ( parentValidationData->src >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SRC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->src++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__blend_func__src()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__blend_func__dest( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__blend_func__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__blend_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->src < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_DEST, (const ParserChar*)0, "sibling: src" ) )
                return false;
        }
        if ( parentValidationData->dest >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DEST, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->dest++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__blend_func__dest()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_max_clip_planes_index_type( const uint64 value )
{
    if( value >= 5 )
        return ParserError::ERROR_VALIDATION_MAX_EXCLUSIVE;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__clip_plane( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CLIP_PLANE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CLIP_PLANE;
        }
//        parentValidationData->clip_plane++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__clip_plane()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__color_mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_MASK;
        }
//        parentValidationData->color_mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__color_mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__cull_face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CULL_FACE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CULL_FACE;
        }
//        parentValidationData->cull_face++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__cull_face()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__depth_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_FUNC;
        }
//        parentValidationData->depth_func++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__depth_func()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__depth_mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_MASK;
        }
//        parentValidationData->depth_mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__depth_mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__depth_range( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_RANGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_RANGE;
        }
//        parentValidationData->depth_range++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__depth_range()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__fog_color( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_COLOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_COLOR;
        }
//        parentValidationData->fog_color++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__fog_color()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__fog_density( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_DENSITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_DENSITY;
        }
//        parentValidationData->fog_density++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__fog_density()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__fog_mode( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_MODE;
        }
//        parentValidationData->fog_mode++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__fog_mode()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__fog_start( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_START, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_START;
        }
//        parentValidationData->fog_start++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__fog_start()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__fog_end( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_END, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_END;
        }
//        parentValidationData->fog_end++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__fog_end()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__front_face( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FRONT_FACE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FRONT_FACE;
        }
//        parentValidationData->front_face++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__front_face()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__logic_op( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGIC_OP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGIC_OP;
        }
//        parentValidationData->logic_op++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__logic_op()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_max_lights_index_type( const uint64 value )
{
    if( value >= 7 )
        return ParserError::ERROR_VALIDATION_MAX_EXCLUSIVE;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_ambient( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_AMBIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_AMBIENT;
        }
//        parentValidationData->light_ambient++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_ambient()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_diffuse( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_DIFFUSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_DIFFUSE;
        }
//        parentValidationData->light_diffuse++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_diffuse()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_specular( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPECULAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPECULAR;
        }
//        parentValidationData->light_specular++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_specular()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_position( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_POSITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_POSITION;
        }
//        parentValidationData->light_position++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_position()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_constant_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION;
        }
//        parentValidationData->light_constant_attenuation++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_constant_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_linear_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION;
        }
//        parentValidationData->light_linear_attenuation++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_linear_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_quadratic_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION;
        }
//        parentValidationData->light_quadratic_attenuation++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_quadratic_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_spot_cutoff( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPOT_CUTOFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPOT_CUTOFF;
        }
//        parentValidationData->light_spot_cutoff++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_spot_cutoff()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_spot_direction( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPOT_DIRECTION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPOT_DIRECTION;
        }
//        parentValidationData->light_spot_direction++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_spot_direction()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_spot_exponent( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_SPOT_EXPONENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_SPOT_EXPONENT;
        }
//        parentValidationData->light_spot_exponent++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_spot_exponent()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_model_ambient( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_MODEL_AMBIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_MODEL_AMBIENT;
        }
//        parentValidationData->light_model_ambient++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_model_ambient()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__line_width( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_WIDTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_WIDTH;
        }
//        parentValidationData->line_width++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__line_width()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__material_ambient( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_AMBIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_AMBIENT;
        }
//        parentValidationData->material_ambient++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__material_ambient()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__material_diffuse( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_DIFFUSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_DIFFUSE;
        }
//        parentValidationData->material_diffuse++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__material_diffuse()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__material_emission( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_EMISSION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_EMISSION;
        }
//        parentValidationData->material_emission++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__material_emission()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__material_shininess( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_SHININESS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_SHININESS;
        }
//        parentValidationData->material_shininess++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__material_shininess()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__material_specular( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATERIAL_SPECULAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATERIAL_SPECULAR;
        }
//        parentValidationData->material_specular++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__material_specular()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__model_view_matrix( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODEL_VIEW_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODEL_VIEW_MATRIX;
        }
//        parentValidationData->model_view_matrix++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__model_view_matrix()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__point_distance_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_DISTANCE_ATTENUATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_DISTANCE_ATTENUATION;
        }
//        parentValidationData->point_distance_attenuation++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__point_distance_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__point_fade_threshold_size( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE;
        }
//        parentValidationData->point_fade_threshold_size++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__point_fade_threshold_size()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__point_size( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE;
        }
//        parentValidationData->point_size++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__point_size()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__point_size_min( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE_MIN;
        }
//        parentValidationData->point_size_min++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__point_size_min()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__point_size_max( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SIZE_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SIZE_MAX;
        }
//        parentValidationData->point_size_max++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__point_size_max()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__polygon_offset( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET;
        }
//        parentValidationData->polygon_offset++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__polygon_offset()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__projection_matrix( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PROJECTION_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PROJECTION_MATRIX;
        }
//        parentValidationData->projection_matrix++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__projection_matrix()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__scissor( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCISSOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCISSOR;
        }
//        parentValidationData->scissor++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__scissor()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__shade_model( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SHADE_MODEL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SHADE_MODEL;
        }
//        parentValidationData->shade_model++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__shade_model()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__stencil_func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_FUNC;
        }
//        parentValidationData->stencil_func++;

        profile_GLES__technique__pass__states__stencil_func__ValidationData* validationData = (profile_GLES__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES__technique__pass__states__stencil_func__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES__technique__pass__states__stencil_func__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__stencil_func()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__stencil_func__ValidationData* validationData = (profile_GLES__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->func < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, "child: func" ) )
                return false;
        }
        if ( validationData->ref < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, "child: ref" ) )
                return false;
        }
        if ( validationData->mask < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_FUNC, (const ParserChar*)0, "child: mask" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__stencil_func__func( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__stencil_func__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FUNC, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FUNC, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->func >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FUNC, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->func++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__stencil_func__func()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__stencil_func__ref( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__stencil_func__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->func < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: func" ) )
                return false;
        }
        if ( parentValidationData->mask != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF, (const ParserChar*)0, "sibling: mask" ) )
                return false;
        }
        if ( parentValidationData->ref >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__stencil_func__ref()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__stencil_func__mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__stencil_func__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__stencil_func__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MASK, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->mask >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MASK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__stencil_func__mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__stencil_mask( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_MASK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_MASK;
        }
//        parentValidationData->stencil_mask++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__stencil_mask()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__stencil_op( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_OP;
        }
//        parentValidationData->stencil_op++;

        profile_GLES__technique__pass__states__stencil_op__ValidationData* validationData = (profile_GLES__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES__technique__pass__states__stencil_op__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES__technique__pass__states__stencil_op__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__stencil_op()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__stencil_op__ValidationData* validationData = (profile_GLES__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->fail < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, "child: fail" ) )
                return false;
        }
        if ( validationData->zfail < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, "child: zfail" ) )
                return false;
        }
        if ( validationData->zpass < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_STENCIL_OP, (const ParserChar*)0, "child: zpass" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_stencil_op_enum( ParserString value )
{
    return validate__gles_stencil_op_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_stencil_op_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_stencil_op_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_stencil_op_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__stencil_op__fail( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__stencil_op__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->zfail != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FAIL, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->fail >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->fail++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__stencil_op__fail()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__stencil_op__zfail( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__stencil_op__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->fail < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, "sibling: fail" ) )
                return false;
        }
        if ( parentValidationData->zpass != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, "sibling: zpass" ) )
                return false;
        }
        if ( parentValidationData->zfail >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ZFAIL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->zfail++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__stencil_op__zfail()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__stencil_op__zpass( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__stencil_op__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__stencil_op__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->zfail < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ZPASS, (const ParserChar*)0, "sibling: zfail" ) )
                return false;
        }
        if ( parentValidationData->zpass >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ZPASS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->zpass++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__stencil_op__zpass()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texture_pipeline( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TEXTURE_PIPELINE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TEXTURE_PIPELINE;
        }
//        parentValidationData->texture_pipeline++;

        texture_pipeline__ValidationData* validationData = (texture_pipeline__ValidationData*)mValidationDataStack.newObject(sizeof(texture_pipeline__ValidationData));
        memset(validationData, 0, sizeof(texture_pipeline__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texture_pipeline()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texture_pipeline__ValidationData* validationData = (texture_pipeline__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__value____gles_texture_pipeline_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texture_pipeline__ValidationData* parentValidationData = (texture_pipeline__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

        value____gles_texture_pipeline_type__ValidationData* validationData = (value____gles_texture_pipeline_type__ValidationData*)mValidationDataStack.newObject(sizeof(value____gles_texture_pipeline_type__ValidationData));
        memset(validationData, 0, sizeof(value____gles_texture_pipeline_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__value____gles_texture_pipeline_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        value____gles_texture_pipeline_type__ValidationData* validationData = (value____gles_texture_pipeline_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->texcombiner != 0 )
            if ( validationData->texcombiner < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_VALUE, (const ParserChar*)0, "child: texcombiner" ) )
                    return false;
        if ( validationData->texenv != 0 )
            if ( validationData->texenv < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_VALUE, (const ParserChar*)0, "child: texenv" ) )
                    return false;
        if ( validationData->extra != 0 )
            if ( validationData->extra < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_VALUE, (const ParserChar*)0, "child: extra" ) )
                    return false;
        if ( validationData->texcombiner == 0 && validationData->texenv == 0 && validationData->extra == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texcombiner( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        value____gles_texture_pipeline_type__ValidationData* parentValidationData = (value____gles_texture_pipeline_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->texenv != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TEXCOMBINER, (const ParserChar*)0, "sibling: texenv" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TEXCOMBINER, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->texcombiner >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TEXCOMBINER, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->texcombiner++;

        texcombiner__ValidationData* validationData = (texcombiner__ValidationData*)mValidationDataStack.newObject(sizeof(texcombiner__ValidationData));
        memset(validationData, 0, sizeof(texcombiner__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texcombiner()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texcombiner__ValidationData* validationData = (texcombiner__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__constant____gles_texture_constant_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 142:
        {
        texcombiner__ValidationData* parentValidationData = (texcombiner__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->RGB != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONSTANT, (const ParserChar*)0, "sibling: RGB" ) )
                return false;
        }
        if ( parentValidationData->alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONSTANT, (const ParserChar*)0, "sibling: alpha" ) )
                return false;
        }
        if ( parentValidationData->constant >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CONSTANT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->constant++;

        break;
        }

        case 582:
        {
        texenv__ValidationData* parentValidationData = (texenv__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->constant >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CONSTANT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->constant++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__constant____gles_texture_constant_type()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operator_rgb_enum( ParserString value )
{
    return validate__gles_texcombiner_operator_rgb_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operator_rgb_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operator_rgb_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operator_rgb_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__RGB( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texcombiner__ValidationData* parentValidationData = (texcombiner__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->alpha != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RGB, (const ParserChar*)0, "sibling: alpha" ) )
                return false;
        }
        if ( parentValidationData->RGB >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RGB, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->RGB++;

        RGB__ValidationData* validationData = (RGB__ValidationData*)mValidationDataStack.newObject(sizeof(RGB__ValidationData));
        memset(validationData, 0, sizeof(RGB__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__RGB()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        RGB__ValidationData* validationData = (RGB__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->argument < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_RGB, (const ParserChar*)0, "child: argument" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_source_enum( ParserString value )
{
    return validate__gles_texcombiner_source_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_source_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_source_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_source_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operand_rgb_enum( ParserString value )
{
    return validate__gles_texcombiner_operand_rgb_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operand_rgb_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operand_rgb_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operand_rgb_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__argument____gles_texcombiner_argument_rgb_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        RGB__ValidationData* parentValidationData = (RGB__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->argument >= 3 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ARGUMENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->argument++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__argument____gles_texcombiner_argument_rgb_type()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operator_alpha_enum( ParserString value )
{
    return validate__gles_texcombiner_operator_alpha_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operator_alpha_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operator_alpha_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operator_alpha_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__alpha____gles_texcombiner_command_alpha_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texcombiner__ValidationData* parentValidationData = (texcombiner__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->alpha >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ALPHA, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->alpha++;

        alpha____gles_texcombiner_command_alpha_type__ValidationData* validationData = (alpha____gles_texcombiner_command_alpha_type__ValidationData*)mValidationDataStack.newObject(sizeof(alpha____gles_texcombiner_command_alpha_type__ValidationData));
        memset(validationData, 0, sizeof(alpha____gles_texcombiner_command_alpha_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__alpha____gles_texcombiner_command_alpha_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        alpha____gles_texcombiner_command_alpha_type__ValidationData* validationData = (alpha____gles_texcombiner_command_alpha_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->argument < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ALPHA, (const ParserChar*)0, "child: argument" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operand_alpha_enum( ParserString value )
{
    return validate__gles_texcombiner_operand_alpha_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operand_alpha_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operand_alpha_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texcombiner_operand_alpha_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__argument____gles_texcombiner_argument_alpha_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        alpha____gles_texcombiner_command_alpha_type__ValidationData* parentValidationData = (alpha____gles_texcombiner_command_alpha_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->argument >= 3 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ARGUMENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->argument++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__argument____gles_texcombiner_argument_alpha_type()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texenv_mode_enum( ParserString value )
{
    return validate__gles_texenv_mode_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texenv_mode_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texenv_mode_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__gles_texenv_mode_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__texenv( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        value____gles_texture_pipeline_type__ValidationData* parentValidationData = (value____gles_texture_pipeline_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->texcombiner != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TEXENV, (const ParserChar*)0, "sibling: texcombiner" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TEXENV, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->texenv >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TEXENV, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->texenv++;

        texenv__ValidationData* validationData = (texenv__ValidationData*)mValidationDataStack.newObject(sizeof(texenv__ValidationData));
        memset(validationData, 0, sizeof(texenv__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__texenv()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        texenv__ValidationData* validationData = (texenv__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__alpha_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ALPHA_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ALPHA_TEST_ENABLE;
        }
//        parentValidationData->alpha_test_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__alpha_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__blend_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BLEND_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BLEND_ENABLE;
        }
//        parentValidationData->blend_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__blend_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__clip_plane_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CLIP_PLANE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CLIP_PLANE_ENABLE;
        }
//        parentValidationData->clip_plane_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__clip_plane_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__color_logic_op_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE;
        }
//        parentValidationData->color_logic_op_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__color_logic_op_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__color_material_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COLOR_MATERIAL_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COLOR_MATERIAL_ENABLE;
        }
//        parentValidationData->color_material_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__color_material_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__cull_face_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CULL_FACE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CULL_FACE_ENABLE;
        }
//        parentValidationData->cull_face_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__cull_face_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__depth_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEPTH_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEPTH_TEST_ENABLE;
        }
//        parentValidationData->depth_test_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__depth_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__dither_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DITHER_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DITHER_ENABLE;
        }
//        parentValidationData->dither_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__dither_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__fog_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FOG_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FOG_ENABLE;
        }
//        parentValidationData->fog_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__fog_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_ENABLE;
        }
//        parentValidationData->light_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__lighting_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHTING_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHTING_ENABLE;
        }
//        parentValidationData->lighting_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__lighting_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__light_model_two_side_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE;
        }
//        parentValidationData->light_model_two_side_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__light_model_two_side_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__line_smooth_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINE_SMOOTH_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINE_SMOOTH_ENABLE;
        }
//        parentValidationData->line_smooth_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__line_smooth_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__multisample_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MULTISAMPLE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MULTISAMPLE_ENABLE;
        }
//        parentValidationData->multisample_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__multisample_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__normalize_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NORMALIZE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NORMALIZE_ENABLE;
        }
//        parentValidationData->normalize_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__normalize_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__point_smooth_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POINT_SMOOTH_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POINT_SMOOTH_ENABLE;
        }
//        parentValidationData->point_smooth_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__point_smooth_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__polygon_offset_fill_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE;
        }
//        parentValidationData->polygon_offset_fill_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__polygon_offset_fill_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__rescale_normal_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RESCALE_NORMAL_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RESCALE_NORMAL_ENABLE;
        }
//        parentValidationData->rescale_normal_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__rescale_normal_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE;
        }
//        parentValidationData->sample_alpha_to_coverage_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__sample_alpha_to_one_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE;
        }
//        parentValidationData->sample_alpha_to_one_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__sample_alpha_to_one_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__sample_coverage_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE;
        }
//        parentValidationData->sample_coverage_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__sample_coverage_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__scissor_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCISSOR_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCISSOR_TEST_ENABLE;
        }
//        parentValidationData->scissor_test_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__scissor_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__states__stencil_test_enable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__states__ValidationData* parentValidationData = (profile_GLES__technique__pass__states__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ALPHA_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_BLEND_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CLIP_PLANE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_LOGIC_OP_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_COLOR_MATERIAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CULL_FACE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_RANGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEPTH_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DITHER_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_COLOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FOG_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_FRONT_FACE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHTING_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_CONSTANT_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_LINEAR_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_MODEL_TWO_SIDE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_POSITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_QUADRATIC_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_CUTOFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_DIRECTION) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIGHT_SPOT_EXPONENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINE_WIDTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGIC_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_AMBIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_DIFFUSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_EMISSION) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SHININESS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATERIAL_SPECULAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODEL_VIEW_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MULTISAMPLE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NORMALIZE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_DISTANCE_ATTENUATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_FADE_THRESHOLD_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SIZE_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_POINT_SMOOTH_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_POLYGON_OFFSET_FILL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PROJECTION_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_RESCALE_NORMAL_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_ALPHA_TO_ONE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SAMPLE_COVERAGE_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCISSOR_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHADE_MODEL) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_FUNC) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_MASK) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_OP) || (parentValidationData->validation_current_state == HASH_ELEMENT_STENCIL_TEST_ENABLE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TEXTURE_PIPELINE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_STENCIL_TEST_ENABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_STENCIL_TEST_ENABLE;
        }
//        parentValidationData->stencil_test_enable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__states__stencil_test_enable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__profile_GLES__technique__pass__evaluate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__ValidationData* parentValidationData = (profile_GLES__technique__pass__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EVALUATE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->evaluate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EVALUATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->evaluate++;

        profile_GLES__technique__pass__evaluate__ValidationData* validationData = (profile_GLES__technique__pass__evaluate__ValidationData*)mValidationDataStack.newObject(sizeof(profile_GLES__technique__pass__evaluate__ValidationData));
        memset(validationData, 0, sizeof(profile_GLES__technique__pass__evaluate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__profile_GLES__technique__pass__evaluate()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        profile_GLES__technique__pass__evaluate__ValidationData* validationData = (profile_GLES__technique__pass__evaluate__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_force_fields( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_FORCE_FIELDS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_FORCE_FIELDS;
        }
//        parentValidationData->library_force_fields++;

        library_force_fields__ValidationData* validationData = (library_force_fields__ValidationData*)mValidationDataStack.newObject(sizeof(library_force_fields__ValidationData));
        memset(validationData, 0, sizeof(library_force_fields__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_force_fields()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_force_fields__ValidationData* validationData = (library_force_fields__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->force_field < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_FORCE_FIELDS, (const ParserChar*)0, "child: force_field" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__force_field( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_force_fields__ValidationData* parentValidationData = (library_force_fields__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FORCE_FIELD, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->force_field++;

        force_field__ValidationData* validationData = (force_field__ValidationData*)mValidationDataStack.newObject(sizeof(force_field__ValidationData));
        memset(validationData, 0, sizeof(force_field__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__force_field()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        force_field__ValidationData* validationData = (force_field__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FORCE_FIELD, (const ParserChar*)0, "child: technique" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_images( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_IMAGES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_IMAGES;
        }
//        parentValidationData->library_images++;

        library_images__ValidationData* validationData = (library_images__ValidationData*)mValidationDataStack.newObject(sizeof(library_images__ValidationData));
        memset(validationData, 0, sizeof(library_images__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_images()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_images__ValidationData* validationData = (library_images__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->image < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_IMAGES, (const ParserChar*)0, "child: image" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__image____image_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_images__ValidationData* parentValidationData = (library_images__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->image++;

        image____image_type__ValidationData* validationData = (image____image_type__ValidationData*)mValidationDataStack.newObject(sizeof(image____image_type__ValidationData));
        memset(validationData, 0, sizeof(image____image_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__image____image_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        image____image_type__ValidationData* validationData = (image____image_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ASSET) || (validationData->validation_current_state == HASH_ELEMENT_CREATE_2D) || (validationData->validation_current_state == HASH_ELEMENT_CREATE_3D) || (validationData->validation_current_state == HASH_ELEMENT_CREATE_CUBE) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INIT_FROM) || (validationData->validation_current_state == HASH_ELEMENT_RENDERABLE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__renderable( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        image____image_type__ValidationData* parentValidationData = (image____image_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RENDERABLE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RENDERABLE;
        }
//        parentValidationData->renderable++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__renderable()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__image_type____init_from( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        image____image_type__ValidationData* parentValidationData = (image____image_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_RENDERABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INIT_FROM;
        }
//        parentValidationData->init_from++;

        image_type____init_from__ValidationData* validationData = (image_type____init_from__ValidationData*)mValidationDataStack.newObject(sizeof(image_type____init_from__ValidationData));
        memset(validationData, 0, sizeof(image_type____init_from__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__image_type____init_from()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        image_type____init_from__ValidationData* validationData = (image_type____init_from__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->ref != 0 )
            if ( validationData->ref < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, "child: ref" ) )
                    return false;
        if ( validationData->hex != 0 )
            if ( validationData->hex < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, "child: hex" ) )
                    return false;
        if ( validationData->ref == 0 && validationData->hex == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_images__image__init_from__hex( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 422:
        {
        create_2d__init_from__ValidationData* parentValidationData = (create_2d__init_from__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HEX, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->hex >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HEX, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->hex++;

        break;
        }

        case 423:
        {
        create_3d__init_from__ValidationData* parentValidationData = (create_3d__init_from__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HEX, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->hex >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HEX, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->hex++;

        break;
        }

        case 424:
        {
        create_cube__init_from__ValidationData* parentValidationData = (create_cube__init_from__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HEX, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->hex >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HEX, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->hex++;

        break;
        }

        case 425:
        {
        image_type____init_from__ValidationData* parentValidationData = (image_type____init_from__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HEX, (const ParserChar*)0, "sibling: ref" ) )
                return false;
        }
        if ( parentValidationData->hex >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HEX, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->hex++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_images__image__init_from__hex()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_2d( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        image____image_type__ValidationData* parentValidationData = (image____image_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_RENDERABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CREATE_2D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CREATE_2D;
        }
//        parentValidationData->create_2d++;

        create_2d__ValidationData* validationData = (create_2d__ValidationData*)mValidationDataStack.newObject(sizeof(create_2d__ValidationData));
        memset(validationData, 0, sizeof(create_2d__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_2d()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_2d__ValidationData* validationData = (create_2d__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ARRAY) || (validationData->validation_current_state == HASH_ELEMENT_FORMAT) || (validationData->validation_current_state == HASH_ELEMENT_INIT_FROM) || (validationData->validation_current_state == HASH_ELEMENT_MIPS) || (validationData->validation_current_state == HASH_ELEMENT_UNNORMALIZED) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_CREATE_2D, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__size_exact( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_2d__ValidationData* parentValidationData = (create_2d__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIZE_EXACT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIZE_EXACT;
        }
//        parentValidationData->size_exact++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__size_exact()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__size_ratio( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_2d__ValidationData* parentValidationData = (create_2d__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIZE_RATIO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIZE_RATIO;
        }
//        parentValidationData->size_ratio++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__size_ratio()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mips( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 259:
        {
        create_2d__ValidationData* parentValidationData = (create_2d__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SIZE_EXACT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIZE_RATIO) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIPS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIPS;
        }
//        parentValidationData->mips++;

        break;
        }

        case 286:
        {
        create_3d__ValidationData* parentValidationData = (create_3d__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->size < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MIPS, (const ParserChar*)0, "sibling: size" ) )
                return false;
        }
        if ( parentValidationData->array != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPS, (const ParserChar*)0, "sibling: array" ) )
                return false;
        }
        if ( parentValidationData->format != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPS, (const ParserChar*)0, "sibling: format" ) )
                return false;
        }
        if ( parentValidationData->init_from != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPS, (const ParserChar*)0, "sibling: init_from" ) )
                return false;
        }
        if ( parentValidationData->mips >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MIPS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mips++;

        break;
        }

        case 347:
        {
        create_cube__ValidationData* parentValidationData = (create_cube__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->size < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MIPS, (const ParserChar*)0, "sibling: size" ) )
                return false;
        }
        if ( parentValidationData->array != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPS, (const ParserChar*)0, "sibling: array" ) )
                return false;
        }
        if ( parentValidationData->format != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPS, (const ParserChar*)0, "sibling: format" ) )
                return false;
        }
        if ( parentValidationData->init_from != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIPS, (const ParserChar*)0, "sibling: init_from" ) )
                return false;
        }
        if ( parentValidationData->mips >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MIPS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->mips++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mips()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__unnormalized( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_2d__ValidationData* parentValidationData = (create_2d__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_SIZE_EXACT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIZE_RATIO) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNNORMALIZED, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNNORMALIZED;
        }
//        parentValidationData->unnormalized++;

        unnormalized__ValidationData* validationData = (unnormalized__ValidationData*)mValidationDataStack.newObject(sizeof(unnormalized__ValidationData));
        memset(validationData, 0, sizeof(unnormalized__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__unnormalized()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        unnormalized__ValidationData* validationData = (unnormalized__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_2d__array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_2d__ValidationData* parentValidationData = (create_2d__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_MIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNNORMALIZED) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARRAY;
        }
//        parentValidationData->array++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_2d__array()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_2d__format( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_2d__ValidationData* parentValidationData = (create_2d__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNNORMALIZED) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORMAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORMAT;
        }
//        parentValidationData->format++;

        create_2d__format__ValidationData* validationData = (create_2d__format__ValidationData*)mValidationDataStack.newObject(sizeof(create_2d__format__ValidationData));
        memset(validationData, 0, sizeof(create_2d__format__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_2d__format()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_2d__format__ValidationData* validationData = (create_2d__format__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->hint < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FORMAT, (const ParserChar*)0, "child: hint" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_channels_enum( ParserString value )
{
    return validate__image_format_hint_channels_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_channels_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_channels_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_channels_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_range_enum( ParserString value )
{
    return validate__image_format_hint_range_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_range_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_range_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_range_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_precision_enum( ParserString value )
{
    return validate__image_format_hint_precision_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_precision_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_precision_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__image_format_hint_precision_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_2d__format__hint( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_2d__format__ValidationData* parentValidationData = (create_2d__format__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->exact != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HINT, (const ParserChar*)0, "sibling: exact" ) )
                return false;
        }
        if ( parentValidationData->hint >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HINT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->hint++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_2d__format__hint()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__exact( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 51:
        {
        create_2d__format__ValidationData* parentValidationData = (create_2d__format__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->hint < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXACT, (const ParserChar*)0, "sibling: hint" ) )
                return false;
        }
        if ( parentValidationData->exact >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EXACT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->exact++;

        break;
        }

        case 52:
        {
        create_3d__format__ValidationData* parentValidationData = (create_3d__format__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->hint < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXACT, (const ParserChar*)0, "sibling: hint" ) )
                return false;
        }
        if ( parentValidationData->exact >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EXACT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->exact++;

        break;
        }

        case 53:
        {
        create_cube__format__ValidationData* parentValidationData = (create_cube__format__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->hint < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EXACT, (const ParserChar*)0, "sibling: hint" ) )
                return false;
        }
        if ( parentValidationData->exact >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EXACT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->exact++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__exact()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_2d__init_from( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_2d__ValidationData* parentValidationData = (create_2d__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ARRAY) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORMAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INIT_FROM) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNNORMALIZED) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INIT_FROM;
        }
//        parentValidationData->init_from++;

        create_2d__init_from__ValidationData* validationData = (create_2d__init_from__ValidationData*)mValidationDataStack.newObject(sizeof(create_2d__init_from__ValidationData));
        memset(validationData, 0, sizeof(create_2d__init_from__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_2d__init_from()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_2d__init_from__ValidationData* validationData = (create_2d__init_from__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->ref != 0 )
            if ( validationData->ref < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, "child: ref" ) )
                    return false;
        if ( validationData->hex != 0 )
            if ( validationData->hex < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, "child: hex" ) )
                    return false;
        if ( validationData->ref == 0 && validationData->hex == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_3d( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        image____image_type__ValidationData* parentValidationData = (image____image_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_RENDERABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CREATE_3D, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CREATE_3D;
        }
//        parentValidationData->create_3d++;

        create_3d__ValidationData* validationData = (create_3d__ValidationData*)mValidationDataStack.newObject(sizeof(create_3d__ValidationData));
        memset(validationData, 0, sizeof(create_3d__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_3d()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_3d__ValidationData* validationData = (create_3d__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->size < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CREATE_3D, (const ParserChar*)0, "child: size" ) )
                return false;
        }
        if ( validationData->mips < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CREATE_3D, (const ParserChar*)0, "child: mips" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_3d__size( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_3d__ValidationData* parentValidationData = (create_3d__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mips != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIZE, (const ParserChar*)0, "sibling: mips" ) )
                return false;
        }
        if ( parentValidationData->array != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIZE, (const ParserChar*)0, "sibling: array" ) )
                return false;
        }
        if ( parentValidationData->format != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIZE, (const ParserChar*)0, "sibling: format" ) )
                return false;
        }
        if ( parentValidationData->init_from != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIZE, (const ParserChar*)0, "sibling: init_from" ) )
                return false;
        }
        if ( parentValidationData->size >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SIZE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->size++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_3d__size()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_3d__array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_3d__ValidationData* parentValidationData = (create_3d__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mips < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, "sibling: mips" ) )
                return false;
        }
        if ( parentValidationData->format != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, "sibling: format" ) )
                return false;
        }
        if ( parentValidationData->init_from != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, "sibling: init_from" ) )
                return false;
        }
        if ( parentValidationData->array >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->array++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_3d__array()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_3d__format( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_3d__ValidationData* parentValidationData = (create_3d__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mips < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_FORMAT, (const ParserChar*)0, "sibling: mips" ) )
                return false;
        }
        if ( parentValidationData->init_from != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FORMAT, (const ParserChar*)0, "sibling: init_from" ) )
                return false;
        }
        if ( parentValidationData->format >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FORMAT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->format++;

        create_3d__format__ValidationData* validationData = (create_3d__format__ValidationData*)mValidationDataStack.newObject(sizeof(create_3d__format__ValidationData));
        memset(validationData, 0, sizeof(create_3d__format__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_3d__format()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_3d__format__ValidationData* validationData = (create_3d__format__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->hint < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FORMAT, (const ParserChar*)0, "child: hint" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_3d__format__hint( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_3d__format__ValidationData* parentValidationData = (create_3d__format__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->exact != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HINT, (const ParserChar*)0, "sibling: exact" ) )
                return false;
        }
        if ( parentValidationData->hint >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HINT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->hint++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_3d__format__hint()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_3d__init_from( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_3d__ValidationData* parentValidationData = (create_3d__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mips < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, "sibling: mips" ) )
                return false;
        }
        parentValidationData->init_from++;

        create_3d__init_from__ValidationData* validationData = (create_3d__init_from__ValidationData*)mValidationDataStack.newObject(sizeof(create_3d__init_from__ValidationData));
        memset(validationData, 0, sizeof(create_3d__init_from__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_3d__init_from()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_3d__init_from__ValidationData* validationData = (create_3d__init_from__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->ref != 0 )
            if ( validationData->ref < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, "child: ref" ) )
                    return false;
        if ( validationData->hex != 0 )
            if ( validationData->hex < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, "child: hex" ) )
                    return false;
        if ( validationData->ref == 0 && validationData->hex == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_cube( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        image____image_type__ValidationData* parentValidationData = (image____image_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_RENDERABLE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CREATE_CUBE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CREATE_CUBE;
        }
//        parentValidationData->create_cube++;

        create_cube__ValidationData* validationData = (create_cube__ValidationData*)mValidationDataStack.newObject(sizeof(create_cube__ValidationData));
        memset(validationData, 0, sizeof(create_cube__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_cube()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_cube__ValidationData* validationData = (create_cube__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->size < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CREATE_CUBE, (const ParserChar*)0, "child: size" ) )
                return false;
        }
        if ( validationData->mips < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CREATE_CUBE, (const ParserChar*)0, "child: mips" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_cube__size( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_cube__ValidationData* parentValidationData = (create_cube__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mips != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIZE, (const ParserChar*)0, "sibling: mips" ) )
                return false;
        }
        if ( parentValidationData->array != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIZE, (const ParserChar*)0, "sibling: array" ) )
                return false;
        }
        if ( parentValidationData->format != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIZE, (const ParserChar*)0, "sibling: format" ) )
                return false;
        }
        if ( parentValidationData->init_from != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SIZE, (const ParserChar*)0, "sibling: init_from" ) )
                return false;
        }
        if ( parentValidationData->size >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SIZE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->size++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_cube__size()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_cube__array( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_cube__ValidationData* parentValidationData = (create_cube__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mips < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, "sibling: mips" ) )
                return false;
        }
        if ( parentValidationData->format != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, "sibling: format" ) )
                return false;
        }
        if ( parentValidationData->init_from != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ARRAY, (const ParserChar*)0, "sibling: init_from" ) )
                return false;
        }
        if ( parentValidationData->array >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ARRAY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->array++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_cube__array()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_cube__format( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_cube__ValidationData* parentValidationData = (create_cube__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mips < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_FORMAT, (const ParserChar*)0, "sibling: mips" ) )
                return false;
        }
        if ( parentValidationData->init_from != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FORMAT, (const ParserChar*)0, "sibling: init_from" ) )
                return false;
        }
        if ( parentValidationData->format >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FORMAT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->format++;

        create_cube__format__ValidationData* validationData = (create_cube__format__ValidationData*)mValidationDataStack.newObject(sizeof(create_cube__format__ValidationData));
        memset(validationData, 0, sizeof(create_cube__format__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_cube__format()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_cube__format__ValidationData* validationData = (create_cube__format__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->hint < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FORMAT, (const ParserChar*)0, "child: hint" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_cube__format__hint( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_cube__format__ValidationData* parentValidationData = (create_cube__format__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->exact != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HINT, (const ParserChar*)0, "sibling: exact" ) )
                return false;
        }
        if ( parentValidationData->hint >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HINT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->hint++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_cube__format__hint()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__create_cube__init_from( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_cube__ValidationData* parentValidationData = (create_cube__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->mips < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, "sibling: mips" ) )
                return false;
        }
        parentValidationData->init_from++;

        create_cube__init_from__ValidationData* validationData = (create_cube__init_from__ValidationData*)mValidationDataStack.newObject(sizeof(create_cube__init_from__ValidationData));
        memset(validationData, 0, sizeof(create_cube__init_from__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__create_cube__init_from()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        create_cube__init_from__ValidationData* validationData = (create_cube__init_from__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->ref != 0 )
            if ( validationData->ref < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, "child: ref" ) )
                    return false;
        if ( validationData->hex != 0 )
            if ( validationData->hex < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, "child: hex" ) )
                    return false;
        if ( validationData->ref == 0 && validationData->hex == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_INIT_FROM, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_lights( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_LIGHTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_LIGHTS;
        }
//        parentValidationData->library_lights++;

        library_lights__ValidationData* validationData = (library_lights__ValidationData*)mValidationDataStack.newObject(sizeof(library_lights__ValidationData));
        memset(validationData, 0, sizeof(library_lights__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_lights()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_lights__ValidationData* validationData = (library_lights__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->light < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_LIGHTS, (const ParserChar*)0, "child: light" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__light( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_lights__ValidationData* parentValidationData = (library_lights__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LIGHT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->light++;

        light__ValidationData* validationData = (light__ValidationData*)mValidationDataStack.newObject(sizeof(light__ValidationData));
        memset(validationData, 0, sizeof(light__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__light()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        light__ValidationData* validationData = (light__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIGHT, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__light_type____technique_common( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        light__ValidationData* parentValidationData = (light__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        light_type____technique_common__ValidationData* validationData = (light_type____technique_common__ValidationData*)mValidationDataStack.newObject(sizeof(light_type____technique_common__ValidationData));
        memset(validationData, 0, sizeof(light_type____technique_common__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__light_type____technique_common()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        light_type____technique_common__ValidationData* validationData = (light_type____technique_common__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->ambient != 0 )
            if ( validationData->ambient < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: ambient" ) )
                    return false;
        if ( validationData->directional != 0 )
            if ( validationData->directional < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: directional" ) )
                    return false;
        if ( validationData->point != 0 )
            if ( validationData->point < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: point" ) )
                    return false;
        if ( validationData->spot != 0 )
            if ( validationData->spot < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: spot" ) )
                    return false;
        if ( validationData->ambient == 0 && validationData->directional == 0 && validationData->point == 0 && validationData->spot == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__light__technique_common__ambient( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        light_type____technique_common__ValidationData* parentValidationData = (light_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->directional != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: directional" ) )
                return false;
        }
        if ( parentValidationData->point != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: point" ) )
                return false;
        }
        if ( parentValidationData->spot != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "sibling: spot" ) )
                return false;
        }
        if ( parentValidationData->ambient >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ambient++;

        light__technique_common__ambient__ValidationData* validationData = (light__technique_common__ambient__ValidationData*)mValidationDataStack.newObject(sizeof(light__technique_common__ambient__ValidationData));
        memset(validationData, 0, sizeof(light__technique_common__ambient__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__light__technique_common__ambient()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        light__technique_common__ambient__ValidationData* validationData = (light__technique_common__ambient__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->color < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_AMBIENT, (const ParserChar*)0, "child: color" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__color____targetable_float3_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 58:
        {
        light__technique_common__ambient__ValidationData* parentValidationData = (light__technique_common__ambient__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_COLOR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->color++;

        break;
        }

        case 349:
        {
        directional__ValidationData* parentValidationData = (directional__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_COLOR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->color++;

        break;
        }

        case 74:
        {
        point__ValidationData* parentValidationData = (point__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->constant_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: constant_attenuation" ) )
                return false;
        }
        if ( parentValidationData->linear_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: linear_attenuation" ) )
                return false;
        }
        if ( parentValidationData->quadratic_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: quadratic_attenuation" ) )
                return false;
        }
        if ( parentValidationData->color >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_COLOR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->color++;

        break;
        }

        case 361:
        {
        spot__ValidationData* parentValidationData = (spot__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->constant_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: constant_attenuation" ) )
                return false;
        }
        if ( parentValidationData->linear_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: linear_attenuation" ) )
                return false;
        }
        if ( parentValidationData->quadratic_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: quadratic_attenuation" ) )
                return false;
        }
        if ( parentValidationData->falloff_angle != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: falloff_angle" ) )
                return false;
        }
        if ( parentValidationData->falloff_exponent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_COLOR, (const ParserChar*)0, "sibling: falloff_exponent" ) )
                return false;
        }
        if ( parentValidationData->color >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_COLOR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->color++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        color____targetable_float3_type__ValidationData* validationData = (color____targetable_float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(color____targetable_float3_type__ValidationData));
        memset(validationData, 0, sizeof(color____targetable_float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__color____targetable_float3_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__directional( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        light_type____technique_common__ValidationData* parentValidationData = (light_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ambient != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIRECTIONAL, (const ParserChar*)0, "sibling: ambient" ) )
                return false;
        }
        if ( parentValidationData->point != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIRECTIONAL, (const ParserChar*)0, "sibling: point" ) )
                return false;
        }
        if ( parentValidationData->spot != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DIRECTIONAL, (const ParserChar*)0, "sibling: spot" ) )
                return false;
        }
        if ( parentValidationData->directional >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DIRECTIONAL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->directional++;

        directional__ValidationData* validationData = (directional__ValidationData*)mValidationDataStack.newObject(sizeof(directional__ValidationData));
        memset(validationData, 0, sizeof(directional__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__directional()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        directional__ValidationData* validationData = (directional__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->color < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_DIRECTIONAL, (const ParserChar*)0, "child: color" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__point( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        light_type____technique_common__ValidationData* parentValidationData = (light_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ambient != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_POINT, (const ParserChar*)0, "sibling: ambient" ) )
                return false;
        }
        if ( parentValidationData->directional != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_POINT, (const ParserChar*)0, "sibling: directional" ) )
                return false;
        }
        if ( parentValidationData->spot != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_POINT, (const ParserChar*)0, "sibling: spot" ) )
                return false;
        }
        if ( parentValidationData->point >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_POINT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->point++;

        point__ValidationData* validationData = (point__ValidationData*)mValidationDataStack.newObject(sizeof(point__ValidationData));
        memset(validationData, 0, sizeof(point__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__point()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        point__ValidationData* validationData = (point__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->color < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_POINT, (const ParserChar*)0, "child: color" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__constant_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 74:
        {
        point__ValidationData* parentValidationData = (point__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_CONSTANT_ATTENUATION, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->linear_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONSTANT_ATTENUATION, (const ParserChar*)0, "sibling: linear_attenuation" ) )
                return false;
        }
        if ( parentValidationData->quadratic_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONSTANT_ATTENUATION, (const ParserChar*)0, "sibling: quadratic_attenuation" ) )
                return false;
        }
        if ( parentValidationData->constant_attenuation >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CONSTANT_ATTENUATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->constant_attenuation++;

        break;
        }

        case 361:
        {
        spot__ValidationData* parentValidationData = (spot__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_CONSTANT_ATTENUATION, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->linear_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONSTANT_ATTENUATION, (const ParserChar*)0, "sibling: linear_attenuation" ) )
                return false;
        }
        if ( parentValidationData->quadratic_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONSTANT_ATTENUATION, (const ParserChar*)0, "sibling: quadratic_attenuation" ) )
                return false;
        }
        if ( parentValidationData->falloff_angle != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONSTANT_ATTENUATION, (const ParserChar*)0, "sibling: falloff_angle" ) )
                return false;
        }
        if ( parentValidationData->falloff_exponent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_CONSTANT_ATTENUATION, (const ParserChar*)0, "sibling: falloff_exponent" ) )
                return false;
        }
        if ( parentValidationData->constant_attenuation >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_CONSTANT_ATTENUATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->constant_attenuation++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__constant_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__linear_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 74:
        {
        point__ValidationData* parentValidationData = (point__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_LINEAR_ATTENUATION, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->quadratic_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LINEAR_ATTENUATION, (const ParserChar*)0, "sibling: quadratic_attenuation" ) )
                return false;
        }
        if ( parentValidationData->linear_attenuation >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_LINEAR_ATTENUATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->linear_attenuation++;

        break;
        }

        case 361:
        {
        spot__ValidationData* parentValidationData = (spot__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_LINEAR_ATTENUATION, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->quadratic_attenuation != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LINEAR_ATTENUATION, (const ParserChar*)0, "sibling: quadratic_attenuation" ) )
                return false;
        }
        if ( parentValidationData->falloff_angle != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LINEAR_ATTENUATION, (const ParserChar*)0, "sibling: falloff_angle" ) )
                return false;
        }
        if ( parentValidationData->falloff_exponent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LINEAR_ATTENUATION, (const ParserChar*)0, "sibling: falloff_exponent" ) )
                return false;
        }
        if ( parentValidationData->linear_attenuation >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_LINEAR_ATTENUATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->linear_attenuation++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__linear_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__quadratic_attenuation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 74:
        {
        point__ValidationData* parentValidationData = (point__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_QUADRATIC_ATTENUATION, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->quadratic_attenuation >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_QUADRATIC_ATTENUATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->quadratic_attenuation++;

        break;
        }

        case 361:
        {
        spot__ValidationData* parentValidationData = (spot__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_QUADRATIC_ATTENUATION, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->falloff_angle != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_QUADRATIC_ATTENUATION, (const ParserChar*)0, "sibling: falloff_angle" ) )
                return false;
        }
        if ( parentValidationData->falloff_exponent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_QUADRATIC_ATTENUATION, (const ParserChar*)0, "sibling: falloff_exponent" ) )
                return false;
        }
        if ( parentValidationData->quadratic_attenuation >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_QUADRATIC_ATTENUATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->quadratic_attenuation++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__quadratic_attenuation()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__spot( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        light_type____technique_common__ValidationData* parentValidationData = (light_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ambient != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPOT, (const ParserChar*)0, "sibling: ambient" ) )
                return false;
        }
        if ( parentValidationData->directional != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPOT, (const ParserChar*)0, "sibling: directional" ) )
                return false;
        }
        if ( parentValidationData->point != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPOT, (const ParserChar*)0, "sibling: point" ) )
                return false;
        }
        if ( parentValidationData->spot >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SPOT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->spot++;

        spot__ValidationData* validationData = (spot__ValidationData*)mValidationDataStack.newObject(sizeof(spot__ValidationData));
        memset(validationData, 0, sizeof(spot__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__spot()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        spot__ValidationData* validationData = (spot__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->color < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPOT, (const ParserChar*)0, "child: color" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__falloff_angle( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        spot__ValidationData* parentValidationData = (spot__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_FALLOFF_ANGLE, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->falloff_exponent != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FALLOFF_ANGLE, (const ParserChar*)0, "sibling: falloff_exponent" ) )
                return false;
        }
        if ( parentValidationData->falloff_angle >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FALLOFF_ANGLE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->falloff_angle++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__falloff_angle()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__falloff_exponent( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        spot__ValidationData* parentValidationData = (spot__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->color < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_FALLOFF_EXPONENT, (const ParserChar*)0, "sibling: color" ) )
                return false;
        }
        if ( parentValidationData->falloff_exponent >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FALLOFF_EXPONENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->falloff_exponent++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__falloff_exponent()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_materials( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_MATERIALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_MATERIALS;
        }
//        parentValidationData->library_materials++;

        library_materials__ValidationData* validationData = (library_materials__ValidationData*)mValidationDataStack.newObject(sizeof(library_materials__ValidationData));
        memset(validationData, 0, sizeof(library_materials__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_materials()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_materials__ValidationData* validationData = (library_materials__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->material < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_MATERIALS, (const ParserChar*)0, "child: material" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__material( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_materials__ValidationData* parentValidationData = (library_materials__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MATERIAL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->material++;

        material__ValidationData* validationData = (material__ValidationData*)mValidationDataStack.newObject(sizeof(material__ValidationData));
        memset(validationData, 0, sizeof(material__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__material()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        material__ValidationData* validationData = (material__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->instance_effect < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MATERIAL, (const ParserChar*)0, "child: instance_effect" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_effect( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        material__ValidationData* parentValidationData = (material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_EFFECT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->instance_effect >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INSTANCE_EFFECT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->instance_effect++;

        instance_effect__ValidationData* validationData = (instance_effect__ValidationData*)mValidationDataStack.newObject(sizeof(instance_effect__ValidationData));
        memset(validationData, 0, sizeof(instance_effect__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_effect()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_effect__ValidationData* validationData = (instance_effect__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__technique_hint( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_effect__ValidationData* parentValidationData = (instance_effect__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->setparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_HINT, (const ParserChar*)0, "sibling: setparam" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_HINT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->technique_hint++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__technique_hint()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_effect_type____setparam( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_effect__ValidationData* parentValidationData = (instance_effect__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->setparam++;

        instance_effect_type____setparam__ValidationData* validationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.newObject(sizeof(instance_effect_type____setparam__ValidationData));
        memset(validationData, 0, sizeof(instance_effect_type____setparam__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_effect_type____setparam()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_effect_type____setparam__ValidationData* validationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BOOL) || (validationData->validation_current_state == HASH_ELEMENT_BOOL2) || (validationData->validation_current_state == HASH_ELEMENT_BOOL3) || (validationData->validation_current_state == HASH_ELEMENT_BOOL4) || (validationData->validation_current_state == HASH_ELEMENT_ENUM) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT2X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT3X4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X1) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X2) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X3) || (validationData->validation_current_state == HASH_ELEMENT_FLOAT4X4) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INT2) || (validationData->validation_current_state == HASH_ELEMENT_INT3) || (validationData->validation_current_state == HASH_ELEMENT_INT4) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_SAMPLER_STATES) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sampler_image( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER_IMAGE;
        }
//        parentValidationData->sampler_image++;

        instance_image__ValidationData* validationData = (instance_image__ValidationData*)mValidationDataStack.newObject(sizeof(instance_image__ValidationData));
        memset(validationData, 0, sizeof(instance_image__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sampler_image()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_image__ValidationData* validationData = (instance_image__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sampler_states( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_effect_type____setparam__ValidationData* parentValidationData = (instance_effect_type____setparam__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SAMPLER_STATES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SAMPLER_STATES;
        }
//        parentValidationData->sampler_states++;

        sampler_states__ValidationData* validationData = (sampler_states__ValidationData*)mValidationDataStack.newObject(sizeof(sampler_states__ValidationData));
        memset(validationData, 0, sizeof(sampler_states__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sampler_states()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        sampler_states__ValidationData* validationData = (sampler_states__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_nodes( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_NODES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_NODES;
        }
//        parentValidationData->library_nodes++;

        library_nodes__ValidationData* validationData = (library_nodes__ValidationData*)mValidationDataStack.newObject(sizeof(library_nodes__ValidationData));
        memset(validationData, 0, sizeof(library_nodes__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_nodes()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_nodes__ValidationData* validationData = (library_nodes__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->node < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_NODES, (const ParserChar*)0, "child: node" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__node_enum( ParserString value )
{
    return validate__node_enum( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__node_enum( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__node_enum__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__node_enum__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__node( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 620:
        {
        library_nodes__ValidationData* parentValidationData = (library_nodes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NODE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->node++;

        break;
        }

        case 36:
        {
        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CAMERA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CONTROLLER) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_LIGHT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_NODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_NODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NODE;
        }
//        parentValidationData->node++;

        break;
        }

        case 395:
        {
        visual_scene__ValidationData* parentValidationData = (visual_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->evaluate_scene != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NODE, (const ParserChar*)0, "sibling: evaluate_scene" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NODE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->node++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        node__ValidationData* validationData = (node__ValidationData*)mValidationDataStack.newObject(sizeof(node__ValidationData));
        memset(validationData, 0, sizeof(node__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__node()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        node__ValidationData* validationData = (node__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ASSET) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_CAMERA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_CONTROLLER) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_LIGHT) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_NODE) || (validationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_NODE) || (validationData->validation_current_state == HASH_ELEMENT_ROTATE) || (validationData->validation_current_state == HASH_ELEMENT_SCALE) || (validationData->validation_current_state == HASH_ELEMENT_SKEW) || (validationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_NODE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__lookat( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOOKAT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOOKAT;
        }
//        parentValidationData->lookat++;

        lookat__ValidationData* validationData = (lookat__ValidationData*)mValidationDataStack.newObject(sizeof(lookat__ValidationData));
        memset(validationData, 0, sizeof(lookat__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__lookat()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__matrix____matrix_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        matrix____matrix_type__ValidationData* validationData = (matrix____matrix_type__ValidationData*)mValidationDataStack.newObject(sizeof(matrix____matrix_type__ValidationData));
        memset(validationData, 0, sizeof(matrix____matrix_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__matrix____matrix_type()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__rotate( const float* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__rotate__stream( const float* value, size_t length )
{

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__rotate__streamEnd( const float* value, size_t length )
{
    if( length < 4 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 4 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rotate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 280:
        {
        attachment_end__ValidationData* parentValidationData = (attachment_end__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->translate != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, "sibling: translate" ) )
                return false;
        }
        if ( parentValidationData->rotate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->rotate++;

        break;
        }

        case 299:
        {
        attachment_full__ValidationData* parentValidationData = (attachment_full__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROTATE;
        }
//        parentValidationData->rotate++;

        break;
        }

        case 9:
        {
        attachment_start__ValidationData* parentValidationData = (attachment_start__ValidationData*)mValidationDataStack.top();
        parentValidationData->rotate++;

        break;
        }

        case 194:
        {
        attachment__ValidationData* parentValidationData = (attachment__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROTATE;
        }
//        parentValidationData->rotate++;

        break;
        }

        case 127:
        {
        instance_rigid_body__technique_common__mass_frame__ValidationData* parentValidationData = (instance_rigid_body__technique_common__mass_frame__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->translate != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, "sibling: translate" ) )
                return false;
        }
        if ( parentValidationData->rotate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->rotate++;

        break;
        }

        case 128:
        {
        rigid_body__technique_common__mass_frame__ValidationData* parentValidationData = (rigid_body__technique_common__mass_frame__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->translate != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, "sibling: translate" ) )
                return false;
        }
        if ( parentValidationData->rotate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->rotate++;

        break;
        }

        case 100:
        {
        ref_attachment__ValidationData* parentValidationData = (ref_attachment__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROTATE;
        }
//        parentValidationData->rotate++;

        break;
        }

        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BOX) || (parentValidationData->validation_current_state == HASH_ELEMENT_CAPSULE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SPHERE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROTATE;
        }
//        parentValidationData->rotate++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BOX) || (parentValidationData->validation_current_state == HASH_ELEMENT_CAPSULE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SPHERE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROTATE;
        }
//        parentValidationData->rotate++;

        break;
        }

        case 230:
        {
        frame_origin__ValidationData* parentValidationData = (frame_origin__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->translate != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, "sibling: translate" ) )
                return false;
        }
        if ( parentValidationData->rotate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->rotate++;

        break;
        }

        case 622:
        {
        link__ValidationData* parentValidationData = (link__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROTATE;
        }
//        parentValidationData->rotate++;

        break;
        }

        case 36:
        {
        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROTATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROTATE;
        }
//        parentValidationData->rotate++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        rotate__ValidationData* validationData = (rotate__ValidationData*)mValidationDataStack.newObject(sizeof(rotate__ValidationData));
        memset(validationData, 0, sizeof(rotate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rotate()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__scale( const float* value, size_t length )
{
    if( length < 3 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__scale__stream( const float* value, size_t length )
{

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__scale__streamEnd( const float* value, size_t length )
{
    if( length < 3 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__scale( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALE;
        }
//        parentValidationData->scale++;

        scale__ValidationData* validationData = (scale__ValidationData*)mValidationDataStack.newObject(sizeof(scale__ValidationData));
        memset(validationData, 0, sizeof(scale__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__scale()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float7_type( const float* value, size_t length )
{
    if( length < 7 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 7 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float7_type__stream( const float* value, size_t length )
{

    if( length > 7 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__float7_type__streamEnd( const float* value, size_t length )
{
    if( length < 7 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 7 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__skew( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SKEW, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SKEW;
        }
//        parentValidationData->skew++;

        skew__ValidationData* validationData = (skew__ValidationData*)mValidationDataStack.newObject(sizeof(skew__ValidationData));
        memset(validationData, 0, sizeof(skew__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__skew()
{
    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__translate( const float* value, size_t length )
{
    if( length < 3 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__translate__stream( const float* value, size_t length )
{

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__translate__streamEnd( const float* value, size_t length )
{
    if( length < 3 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    if( length > 3 )
        return ParserError::ERROR_VALIDATION_MAX_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__translate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 280:
        {
        attachment_end__ValidationData* parentValidationData = (attachment_end__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->rotate != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, "sibling: rotate" ) )
                return false;
        }
        if ( parentValidationData->translate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->translate++;

        break;
        }

        case 299:
        {
        attachment_full__ValidationData* parentValidationData = (attachment_full__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSLATE;
        }
//        parentValidationData->translate++;

        break;
        }

        case 9:
        {
        attachment_start__ValidationData* parentValidationData = (attachment_start__ValidationData*)mValidationDataStack.top();
        parentValidationData->translate++;

        break;
        }

        case 194:
        {
        attachment__ValidationData* parentValidationData = (attachment__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSLATE;
        }
//        parentValidationData->translate++;

        break;
        }

        case 127:
        {
        instance_rigid_body__technique_common__mass_frame__ValidationData* parentValidationData = (instance_rigid_body__technique_common__mass_frame__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->rotate != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, "sibling: rotate" ) )
                return false;
        }
        if ( parentValidationData->translate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->translate++;

        break;
        }

        case 128:
        {
        rigid_body__technique_common__mass_frame__ValidationData* parentValidationData = (rigid_body__technique_common__mass_frame__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->rotate != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, "sibling: rotate" ) )
                return false;
        }
        if ( parentValidationData->translate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->translate++;

        break;
        }

        case 100:
        {
        ref_attachment__ValidationData* parentValidationData = (ref_attachment__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSLATE;
        }
//        parentValidationData->translate++;

        break;
        }

        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BOX) || (parentValidationData->validation_current_state == HASH_ELEMENT_CAPSULE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SPHERE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSLATE;
        }
//        parentValidationData->translate++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_BOX) || (parentValidationData->validation_current_state == HASH_ELEMENT_CAPSULE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLANE) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SPHERE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSLATE;
        }
//        parentValidationData->translate++;

        break;
        }

        case 230:
        {
        frame_origin__ValidationData* parentValidationData = (frame_origin__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->rotate != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, "sibling: rotate" ) )
                return false;
        }
        if ( parentValidationData->translate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->translate++;

        break;
        }

        case 622:
        {
        link__ValidationData* parentValidationData = (link__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSLATE;
        }
//        parentValidationData->translate++;

        break;
        }

        case 36:
        {
        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSLATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSLATE;
        }
//        parentValidationData->translate++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        translate__ValidationData* validationData = (translate__ValidationData*)mValidationDataStack.newObject(sizeof(translate__ValidationData));
        memset(validationData, 0, sizeof(translate__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__translate()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_camera( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CAMERA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_CAMERA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_CAMERA;
        }
//        parentValidationData->instance_camera++;

        instance_camera__ValidationData* validationData = (instance_camera__ValidationData*)mValidationDataStack.newObject(sizeof(instance_camera__ValidationData));
        memset(validationData, 0, sizeof(instance_camera__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_camera()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_camera__ValidationData* validationData = (instance_camera__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_controller( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CAMERA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CONTROLLER) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_CONTROLLER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_CONTROLLER;
        }
//        parentValidationData->instance_controller++;

        instance_controller__ValidationData* validationData = (instance_controller__ValidationData*)mValidationDataStack.newObject(sizeof(instance_controller__ValidationData));
        memset(validationData, 0, sizeof(instance_controller__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_controller()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_controller__ValidationData* validationData = (instance_controller__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__skeleton( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_controller__ValidationData* parentValidationData = (instance_controller__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->bind_material != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SKELETON, (const ParserChar*)0, "sibling: bind_material" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SKELETON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->skeleton++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__skeleton()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bind_material( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 250:
        {
        instance_controller__ValidationData* parentValidationData = (instance_controller__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_MATERIAL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->bind_material >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BIND_MATERIAL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->bind_material++;

        break;
        }

        case 33:
        {
        instance_geometry__ValidationData* parentValidationData = (instance_geometry__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_MATERIAL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->bind_material >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_BIND_MATERIAL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->bind_material++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bind_material__ValidationData* validationData = (bind_material__ValidationData*)mValidationDataStack.newObject(sizeof(bind_material__ValidationData));
        memset(validationData, 0, sizeof(bind_material__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bind_material()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bind_material__ValidationData* validationData = (bind_material__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND_MATERIAL, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bind_material_type____technique_common( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bind_material__ValidationData* parentValidationData = (bind_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        bind_material_type____technique_common__ValidationData* validationData = (bind_material_type____technique_common__ValidationData*)mValidationDataStack.newObject(sizeof(bind_material_type____technique_common__ValidationData));
        memset(validationData, 0, sizeof(bind_material_type____technique_common__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bind_material_type____technique_common()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bind_material_type____technique_common__ValidationData* validationData = (bind_material_type____technique_common__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->instance_material < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: instance_material" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_material____instance_material_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bind_material_type____technique_common__ValidationData* parentValidationData = (bind_material_type____technique_common__ValidationData*)mValidationDataStack.top();
        parentValidationData->instance_material++;

        instance_material____instance_material_type__ValidationData* validationData = (instance_material____instance_material_type__ValidationData*)mValidationDataStack.newObject(sizeof(instance_material____instance_material_type__ValidationData));
        memset(validationData, 0, sizeof(instance_material____instance_material_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_material____instance_material_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_material____instance_material_type__ValidationData* validationData = (instance_material____instance_material_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_material_type____bind( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_material____instance_material_type__ValidationData* parentValidationData = (instance_material____instance_material_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->bind_vertex_input != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: bind_vertex_input" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->bind++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_material_type____bind()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bind_vertex_input( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_material____instance_material_type__ValidationData* parentValidationData = (instance_material____instance_material_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_VERTEX_INPUT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->bind_vertex_input++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bind_vertex_input()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_geometry( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_GEOMETRY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_GEOMETRY;
        }
//        parentValidationData->instance_geometry++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_GEOMETRY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_GEOMETRY;
        }
//        parentValidationData->instance_geometry++;

        break;
        }

        case 36:
        {
        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CAMERA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CONTROLLER) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_GEOMETRY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_GEOMETRY;
        }
//        parentValidationData->instance_geometry++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        instance_geometry__ValidationData* validationData = (instance_geometry__ValidationData*)mValidationDataStack.newObject(sizeof(instance_geometry__ValidationData));
        memset(validationData, 0, sizeof(instance_geometry__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_geometry()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_geometry__ValidationData* validationData = (instance_geometry__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_light( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CAMERA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CONTROLLER) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_LIGHT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_LIGHT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_LIGHT;
        }
//        parentValidationData->instance_light++;

        instance_light__ValidationData* validationData = (instance_light__ValidationData*)mValidationDataStack.newObject(sizeof(instance_light__ValidationData));
        memset(validationData, 0, sizeof(instance_light__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_light()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_light__ValidationData* validationData = (instance_light__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_node( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        node__ValidationData* parentValidationData = (node__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CAMERA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_CONTROLLER) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_LIGHT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_NODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOOKAT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_SKEW) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_NODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_NODE;
        }
//        parentValidationData->instance_node++;

        instance_node__ValidationData* validationData = (instance_node__ValidationData*)mValidationDataStack.newObject(sizeof(instance_node__ValidationData));
        memset(validationData, 0, sizeof(instance_node__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_node()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_node__ValidationData* validationData = (instance_node__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_physics_materials( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS;
        }
//        parentValidationData->library_physics_materials++;

        library_physics_materials__ValidationData* validationData = (library_physics_materials__ValidationData*)mValidationDataStack.newObject(sizeof(library_physics_materials__ValidationData));
        memset(validationData, 0, sizeof(library_physics_materials__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_physics_materials()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_physics_materials__ValidationData* validationData = (library_physics_materials__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->physics_material < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS, (const ParserChar*)0, "child: physics_material" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__physics_material( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PHYSICS_MATERIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PHYSICS_MATERIAL;
        }
//        parentValidationData->physics_material++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PHYSICS_MATERIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PHYSICS_MATERIAL;
        }
//        parentValidationData->physics_material++;

        break;
        }

        case 676:
        {
        instance_rigid_body_type____technique_common__ValidationData* parentValidationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANGULAR_VELOCITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_INERTIA) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS_FRAME) || (parentValidationData->validation_current_state == HASH_ELEMENT_VELOCITY) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PHYSICS_MATERIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PHYSICS_MATERIAL;
        }
//        parentValidationData->physics_material++;

        break;
        }

        case 681:
        {
        rigid_body_type____technique_common__ValidationData* parentValidationData = (rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_INERTIA) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS_FRAME) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PHYSICS_MATERIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PHYSICS_MATERIAL;
        }
//        parentValidationData->physics_material++;

        break;
        }

        case 462:
        {
        library_physics_materials__ValidationData* parentValidationData = (library_physics_materials__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PHYSICS_MATERIAL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->physics_material++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        physics_material__ValidationData* validationData = (physics_material__ValidationData*)mValidationDataStack.newObject(sizeof(physics_material__ValidationData));
        memset(validationData, 0, sizeof(physics_material__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__physics_material()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_material__ValidationData* validationData = (physics_material__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PHYSICS_MATERIAL, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__physics_material_type____technique_common( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_material__ValidationData* parentValidationData = (physics_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        physics_material_type____technique_common__ValidationData* validationData = (physics_material_type____technique_common__ValidationData*)mValidationDataStack.newObject(sizeof(physics_material_type____technique_common__ValidationData));
        memset(validationData, 0, sizeof(physics_material_type____technique_common__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__physics_material_type____technique_common()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_material_type____technique_common__ValidationData* validationData = (physics_material_type____technique_common__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__dynamic_friction( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_material_type____technique_common__ValidationData* parentValidationData = (physics_material_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->restitution != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DYNAMIC_FRICTION, (const ParserChar*)0, "sibling: restitution" ) )
                return false;
        }
        if ( parentValidationData->static_friction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DYNAMIC_FRICTION, (const ParserChar*)0, "sibling: static_friction" ) )
                return false;
        }
        if ( parentValidationData->dynamic_friction >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DYNAMIC_FRICTION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->dynamic_friction++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__dynamic_friction()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__restitution( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_material_type____technique_common__ValidationData* parentValidationData = (physics_material_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->static_friction != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RESTITUTION, (const ParserChar*)0, "sibling: static_friction" ) )
                return false;
        }
        if ( parentValidationData->restitution >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RESTITUTION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->restitution++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__restitution()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__static_friction( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_material_type____technique_common__ValidationData* parentValidationData = (physics_material_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->static_friction >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_STATIC_FRICTION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->static_friction++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__static_friction()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_physics_models( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_PHYSICS_MODELS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_PHYSICS_MODELS;
        }
//        parentValidationData->library_physics_models++;

        library_physics_models__ValidationData* validationData = (library_physics_models__ValidationData*)mValidationDataStack.newObject(sizeof(library_physics_models__ValidationData));
        memset(validationData, 0, sizeof(library_physics_models__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_physics_models()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_physics_models__ValidationData* validationData = (library_physics_models__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->physics_model < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_PHYSICS_MODELS, (const ParserChar*)0, "child: physics_model" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__physics_model( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_physics_models__ValidationData* parentValidationData = (library_physics_models__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PHYSICS_MODEL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->physics_model++;

        physics_model__ValidationData* validationData = (physics_model__ValidationData*)mValidationDataStack.newObject(sizeof(physics_model__ValidationData));
        memset(validationData, 0, sizeof(physics_model__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__physics_model()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_model__ValidationData* validationData = (physics_model__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rigid_body( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_model__ValidationData* parentValidationData = (physics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->rigid_constraint != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RIGID_BODY, (const ParserChar*)0, "sibling: rigid_constraint" ) )
                return false;
        }
        if ( parentValidationData->instance_physics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RIGID_BODY, (const ParserChar*)0, "sibling: instance_physics_model" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RIGID_BODY, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->rigid_body++;

        rigid_body__ValidationData* validationData = (rigid_body__ValidationData*)mValidationDataStack.newObject(sizeof(rigid_body__ValidationData));
        memset(validationData, 0, sizeof(rigid_body__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rigid_body()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_body__ValidationData* validationData = (rigid_body__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_RIGID_BODY, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rigid_body_type____technique_common( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_body__ValidationData* parentValidationData = (rigid_body__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        rigid_body_type____technique_common__ValidationData* validationData = (rigid_body_type____technique_common__ValidationData*)mValidationDataStack.newObject(sizeof(rigid_body_type____technique_common__ValidationData));
        memset(validationData, 0, sizeof(rigid_body_type____technique_common__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rigid_body_type____technique_common()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_body_type____technique_common__ValidationData* validationData = (rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_SHAPE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rigid_body__technique_common__dynamic( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_body_type____technique_common__ValidationData* parentValidationData = (rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DYNAMIC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DYNAMIC;
        }
//        parentValidationData->dynamic++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rigid_body__technique_common__dynamic()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mass( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MASS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MASS;
        }
//        parentValidationData->mass++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MASS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MASS;
        }
//        parentValidationData->mass++;

        break;
        }

        case 676:
        {
        instance_rigid_body_type____technique_common__ValidationData* parentValidationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANGULAR_VELOCITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_VELOCITY) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MASS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MASS;
        }
//        parentValidationData->mass++;

        break;
        }

        case 681:
        {
        rigid_body_type____technique_common__ValidationData* parentValidationData = (rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MASS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MASS;
        }
//        parentValidationData->mass++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mass()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rigid_body__technique_common__mass_frame( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_body_type____technique_common__ValidationData* parentValidationData = (rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MASS_FRAME, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MASS_FRAME;
        }
//        parentValidationData->mass_frame++;

        rigid_body__technique_common__mass_frame__ValidationData* validationData = (rigid_body__technique_common__mass_frame__ValidationData*)mValidationDataStack.newObject(sizeof(rigid_body__technique_common__mass_frame__ValidationData));
        memset(validationData, 0, sizeof(rigid_body__technique_common__mass_frame__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rigid_body__technique_common__mass_frame()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_body__technique_common__mass_frame__ValidationData* validationData = (rigid_body__technique_common__mass_frame__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->translate != 0 )
            if ( validationData->translate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MASS_FRAME, (const ParserChar*)0, "child: translate" ) )
                    return false;
        if ( validationData->rotate != 0 )
            if ( validationData->rotate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MASS_FRAME, (const ParserChar*)0, "child: rotate" ) )
                    return false;
        if ( validationData->translate == 0 && validationData->rotate == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_MASS_FRAME, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__inertia( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 676:
        {
        instance_rigid_body_type____technique_common__ValidationData* parentValidationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANGULAR_VELOCITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS_FRAME) || (parentValidationData->validation_current_state == HASH_ELEMENT_VELOCITY) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INERTIA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INERTIA;
        }
//        parentValidationData->inertia++;

        break;
        }

        case 681:
        {
        rigid_body_type____technique_common__ValidationData* parentValidationData = (rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS_FRAME) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INERTIA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INERTIA;
        }
//        parentValidationData->inertia++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        color____targetable_float3_type__ValidationData* validationData = (color____targetable_float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(color____targetable_float3_type__ValidationData));
        memset(validationData, 0, sizeof(color____targetable_float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__inertia()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_physics_material( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL;
        }
//        parentValidationData->instance_physics_material++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL;
        }
//        parentValidationData->instance_physics_material++;

        break;
        }

        case 676:
        {
        instance_rigid_body_type____technique_common__ValidationData* parentValidationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANGULAR_VELOCITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_INERTIA) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS_FRAME) || (parentValidationData->validation_current_state == HASH_ELEMENT_VELOCITY) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL;
        }
//        parentValidationData->instance_physics_material++;

        break;
        }

        case 681:
        {
        rigid_body_type____technique_common__ValidationData* parentValidationData = (rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_INERTIA) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS_FRAME) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL;
        }
//        parentValidationData->instance_physics_material++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        instance_physics_material__ValidationData* validationData = (instance_physics_material__ValidationData*)mValidationDataStack.newObject(sizeof(instance_physics_material__ValidationData));
        memset(validationData, 0, sizeof(instance_physics_material__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_physics_material()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_physics_material__ValidationData* validationData = (instance_physics_material__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rigid_body__technique_common__shape( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_body_type____technique_common__ValidationData* parentValidationData = (rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_INERTIA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS_FRAME) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHAPE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SHAPE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SHAPE;
        }
//        parentValidationData->shape++;

        rigid_body__technique_common__shape__ValidationData* validationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.newObject(sizeof(rigid_body__technique_common__shape__ValidationData));
        memset(validationData, 0, sizeof(rigid_body__technique_common__shape__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rigid_body__technique_common__shape()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_body__technique_common__shape__ValidationData* validationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BOX) || (validationData->validation_current_state == HASH_ELEMENT_CAPSULE) || (validationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (validationData->validation_current_state == HASH_ELEMENT_PLANE) || (validationData->validation_current_state == HASH_ELEMENT_ROTATE) || (validationData->validation_current_state == HASH_ELEMENT_SPHERE) || (validationData->validation_current_state == HASH_ELEMENT_TRANSLATE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SHAPE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rigid_body__technique_common__shape__hollow( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HOLLOW, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HOLLOW;
        }
//        parentValidationData->hollow++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rigid_body__technique_common__shape__hollow()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__density( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DENSITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DENSITY;
        }
//        parentValidationData->density++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DENSITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DENSITY;
        }
//        parentValidationData->density++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__density()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__box( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOX;
        }
//        parentValidationData->box++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BOX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BOX;
        }
//        parentValidationData->box++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        box__ValidationData* validationData = (box__ValidationData*)mValidationDataStack.newObject(sizeof(box__ValidationData));
        memset(validationData, 0, sizeof(box__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__box()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        box__ValidationData* validationData = (box__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->half_extents < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BOX, (const ParserChar*)0, "child: half_extents" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__half_extents( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        box__ValidationData* parentValidationData = (box__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HALF_EXTENTS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->half_extents >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HALF_EXTENTS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->half_extents++;

        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__half_extents()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__cylinder____cylinder_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CYLINDER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CYLINDER;
        }
//        parentValidationData->cylinder++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CYLINDER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CYLINDER;
        }
//        parentValidationData->cylinder++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        cylinder____cylinder_type__ValidationData* validationData = (cylinder____cylinder_type__ValidationData*)mValidationDataStack.newObject(sizeof(cylinder____cylinder_type__ValidationData));
        memset(validationData, 0, sizeof(cylinder____cylinder_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__cylinder____cylinder_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        cylinder____cylinder_type__ValidationData* validationData = (cylinder____cylinder_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->height < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CYLINDER, (const ParserChar*)0, "child: height" ) )
                return false;
        }
        if ( validationData->radius < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CYLINDER, (const ParserChar*)0, "child: radius" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__height( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 59:
        {
        capsule__ValidationData* parentValidationData = (capsule__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HEIGHT, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HEIGHT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->height >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HEIGHT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->height++;

        break;
        }

        case 470:
        {
        cylinder____cylinder_type__ValidationData* parentValidationData = (cylinder____cylinder_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->radius != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HEIGHT, (const ParserChar*)0, "sibling: radius" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_HEIGHT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->height >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_HEIGHT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->height++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__height()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__capsule( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 597:
        {
        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CAPSULE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CAPSULE;
        }
//        parentValidationData->capsule++;

        break;
        }

        case 598:
        {
        rigid_body__technique_common__shape__ValidationData* parentValidationData = (rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_DENSITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_HOLLOW) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CAPSULE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CAPSULE;
        }
//        parentValidationData->capsule++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        capsule__ValidationData* validationData = (capsule__ValidationData*)mValidationDataStack.newObject(sizeof(capsule__ValidationData));
        memset(validationData, 0, sizeof(capsule__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__capsule()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        capsule__ValidationData* validationData = (capsule__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->height < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CAPSULE, (const ParserChar*)0, "child: height" ) )
                return false;
        }
        if ( validationData->radius < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_CAPSULE, (const ParserChar*)0, "child: radius" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__radius____float3_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        capsule__ValidationData* parentValidationData = (capsule__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->height < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: height" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RADIUS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->radius >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_RADIUS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->radius++;

        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__radius____float3_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rigid_constraint( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_model__ValidationData* parentValidationData = (physics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_physics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RIGID_CONSTRAINT, (const ParserChar*)0, "sibling: instance_physics_model" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RIGID_CONSTRAINT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->rigid_constraint++;

        rigid_constraint__ValidationData* validationData = (rigid_constraint__ValidationData*)mValidationDataStack.newObject(sizeof(rigid_constraint__ValidationData));
        memset(validationData, 0, sizeof(rigid_constraint__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rigid_constraint()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint__ValidationData* validationData = (rigid_constraint__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->ref_attachment < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_RIGID_CONSTRAINT, (const ParserChar*)0, "child: ref_attachment" ) )
                return false;
        }
        if ( validationData->attachment < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_RIGID_CONSTRAINT, (const ParserChar*)0, "child: attachment" ) )
                return false;
        }
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_RIGID_CONSTRAINT, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ref_attachment( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint__ValidationData* parentValidationData = (rigid_constraint__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->attachment != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF_ATTACHMENT, (const ParserChar*)0, "sibling: attachment" ) )
                return false;
        }
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF_ATTACHMENT, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF_ATTACHMENT, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_REF_ATTACHMENT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->ref_attachment >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_REF_ATTACHMENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->ref_attachment++;

        ref_attachment__ValidationData* validationData = (ref_attachment__ValidationData*)mValidationDataStack.newObject(sizeof(ref_attachment__ValidationData));
        memset(validationData, 0, sizeof(ref_attachment__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ref_attachment()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        ref_attachment__ValidationData* validationData = (ref_attachment__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_ROTATE) || (validationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_REF_ATTACHMENT, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__attachment( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint__ValidationData* parentValidationData = (rigid_constraint__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->ref_attachment < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_ATTACHMENT, (const ParserChar*)0, "sibling: ref_attachment" ) )
                return false;
        }
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ATTACHMENT, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ATTACHMENT, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ATTACHMENT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->attachment >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ATTACHMENT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->attachment++;

        attachment__ValidationData* validationData = (attachment__ValidationData*)mValidationDataStack.newObject(sizeof(attachment__ValidationData));
        memset(validationData, 0, sizeof(attachment__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__attachment()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        attachment__ValidationData* validationData = (attachment__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_ROTATE) || (validationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_ATTACHMENT, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rigid_constraint_type____technique_common( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint__ValidationData* parentValidationData = (rigid_constraint__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->attachment < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: attachment" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        rigid_constraint_type____technique_common__ValidationData* validationData = (rigid_constraint_type____technique_common__ValidationData*)mValidationDataStack.newObject(sizeof(rigid_constraint_type____technique_common__ValidationData));
        memset(validationData, 0, sizeof(rigid_constraint_type____technique_common__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rigid_constraint_type____technique_common()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint_type____technique_common__ValidationData* validationData = (rigid_constraint_type____technique_common__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__enabled( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint_type____technique_common__ValidationData* parentValidationData = (rigid_constraint_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->interpenetrate != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ENABLED, (const ParserChar*)0, "sibling: interpenetrate" ) )
                return false;
        }
        if ( parentValidationData->limits != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ENABLED, (const ParserChar*)0, "sibling: limits" ) )
                return false;
        }
        if ( parentValidationData->spring != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ENABLED, (const ParserChar*)0, "sibling: spring" ) )
                return false;
        }
        if ( parentValidationData->enabled >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ENABLED, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->enabled++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__enabled()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__interpenetrate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint_type____technique_common__ValidationData* parentValidationData = (rigid_constraint_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->limits != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INTERPENETRATE, (const ParserChar*)0, "sibling: limits" ) )
                return false;
        }
        if ( parentValidationData->spring != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INTERPENETRATE, (const ParserChar*)0, "sibling: spring" ) )
                return false;
        }
        if ( parentValidationData->interpenetrate >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INTERPENETRATE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->interpenetrate++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__interpenetrate()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rigid_constraint__technique_common__limits( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint_type____technique_common__ValidationData* parentValidationData = (rigid_constraint_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->spring != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LIMITS, (const ParserChar*)0, "sibling: spring" ) )
                return false;
        }
        if ( parentValidationData->limits >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_LIMITS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->limits++;

        rigid_constraint__technique_common__limits__ValidationData* validationData = (rigid_constraint__technique_common__limits__ValidationData*)mValidationDataStack.newObject(sizeof(rigid_constraint__technique_common__limits__ValidationData));
        memset(validationData, 0, sizeof(rigid_constraint__technique_common__limits__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rigid_constraint__technique_common__limits()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint__technique_common__limits__ValidationData* validationData = (rigid_constraint__technique_common__limits__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__swing_cone_and_twist( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint__technique_common__limits__ValidationData* parentValidationData = (rigid_constraint__technique_common__limits__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->linear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SWING_CONE_AND_TWIST, (const ParserChar*)0, "sibling: linear" ) )
                return false;
        }
        if ( parentValidationData->swing_cone_and_twist >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SWING_CONE_AND_TWIST, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->swing_cone_and_twist++;

        swing_cone_and_twist__ValidationData* validationData = (swing_cone_and_twist__ValidationData*)mValidationDataStack.newObject(sizeof(swing_cone_and_twist__ValidationData));
        memset(validationData, 0, sizeof(swing_cone_and_twist__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__swing_cone_and_twist()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        swing_cone_and_twist__ValidationData* validationData = (swing_cone_and_twist__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__min____targetable_float3_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 379:
        {
        rigid_constraint__technique_common__limits__linear__ValidationData* parentValidationData = (rigid_constraint__technique_common__limits__linear__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->max != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIN, (const ParserChar*)0, "sibling: max" ) )
                return false;
        }
        if ( parentValidationData->min >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MIN, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->min++;

        break;
        }

        case 612:
        {
        swing_cone_and_twist__ValidationData* parentValidationData = (swing_cone_and_twist__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->max != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIN, (const ParserChar*)0, "sibling: max" ) )
                return false;
        }
        if ( parentValidationData->min >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MIN, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->min++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        color____targetable_float3_type__ValidationData* validationData = (color____targetable_float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(color____targetable_float3_type__ValidationData));
        memset(validationData, 0, sizeof(color____targetable_float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__min____targetable_float3_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__max____targetable_float3_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 379:
        {
        rigid_constraint__technique_common__limits__linear__ValidationData* parentValidationData = (rigid_constraint__technique_common__limits__linear__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->max >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MAX, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->max++;

        break;
        }

        case 612:
        {
        swing_cone_and_twist__ValidationData* parentValidationData = (swing_cone_and_twist__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->max >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MAX, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->max++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        color____targetable_float3_type__ValidationData* validationData = (color____targetable_float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(color____targetable_float3_type__ValidationData));
        memset(validationData, 0, sizeof(color____targetable_float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__max____targetable_float3_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rigid_constraint__technique_common__limits__linear( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint__technique_common__limits__ValidationData* parentValidationData = (rigid_constraint__technique_common__limits__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->linear >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_LINEAR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->linear++;

        rigid_constraint__technique_common__limits__linear__ValidationData* validationData = (rigid_constraint__technique_common__limits__linear__ValidationData*)mValidationDataStack.newObject(sizeof(rigid_constraint__technique_common__limits__linear__ValidationData));
        memset(validationData, 0, sizeof(rigid_constraint__technique_common__limits__linear__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rigid_constraint__technique_common__limits__linear()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint__technique_common__limits__linear__ValidationData* validationData = (rigid_constraint__technique_common__limits__linear__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__spring( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        rigid_constraint_type____technique_common__ValidationData* parentValidationData = (rigid_constraint_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->spring >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SPRING, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->spring++;

        spring__ValidationData* validationData = (spring__ValidationData*)mValidationDataStack.newObject(sizeof(spring__ValidationData));
        memset(validationData, 0, sizeof(spring__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__spring()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        spring__ValidationData* validationData = (spring__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__angular( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        spring__ValidationData* parentValidationData = (spring__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->linear != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ANGULAR, (const ParserChar*)0, "sibling: linear" ) )
                return false;
        }
        if ( parentValidationData->angular >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ANGULAR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->angular++;

        angular__ValidationData* validationData = (angular__ValidationData*)mValidationDataStack.newObject(sizeof(angular__ValidationData));
        memset(validationData, 0, sizeof(angular__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__angular()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        angular__ValidationData* validationData = (angular__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__stiffness( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 458:
        {
        angular__ValidationData* parentValidationData = (angular__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->damping != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STIFFNESS, (const ParserChar*)0, "sibling: damping" ) )
                return false;
        }
        if ( parentValidationData->target_value != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STIFFNESS, (const ParserChar*)0, "sibling: target_value" ) )
                return false;
        }
        if ( parentValidationData->stiffness >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_STIFFNESS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->stiffness++;

        break;
        }

        case 380:
        {
        spring__linear__ValidationData* parentValidationData = (spring__linear__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->damping != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STIFFNESS, (const ParserChar*)0, "sibling: damping" ) )
                return false;
        }
        if ( parentValidationData->target_value != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_STIFFNESS, (const ParserChar*)0, "sibling: target_value" ) )
                return false;
        }
        if ( parentValidationData->stiffness >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_STIFFNESS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->stiffness++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__stiffness()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__damping( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 458:
        {
        angular__ValidationData* parentValidationData = (angular__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->target_value != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DAMPING, (const ParserChar*)0, "sibling: target_value" ) )
                return false;
        }
        if ( parentValidationData->damping >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DAMPING, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->damping++;

        break;
        }

        case 380:
        {
        spring__linear__ValidationData* parentValidationData = (spring__linear__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->target_value != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DAMPING, (const ParserChar*)0, "sibling: target_value" ) )
                return false;
        }
        if ( parentValidationData->damping >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DAMPING, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->damping++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__damping()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__target_value( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 458:
        {
        angular__ValidationData* parentValidationData = (angular__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->target_value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TARGET_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->target_value++;

        break;
        }

        case 380:
        {
        spring__linear__ValidationData* parentValidationData = (spring__linear__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->target_value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TARGET_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->target_value++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__target_value()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__spring__linear( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        spring__ValidationData* parentValidationData = (spring__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->linear >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_LINEAR, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->linear++;

        spring__linear__ValidationData* validationData = (spring__linear__ValidationData*)mValidationDataStack.newObject(sizeof(spring__linear__ValidationData));
        memset(validationData, 0, sizeof(spring__linear__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__spring__linear()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        spring__linear__ValidationData* validationData = (spring__linear__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_physics_model( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 241:
        {
        physics_model__ValidationData* parentValidationData = (physics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_PHYSICS_MODEL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_physics_model++;

        break;
        }

        case 586:
        {
        physics_scene__ValidationData* parentValidationData = (physics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_PHYSICS_MODEL, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_PHYSICS_MODEL, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_PHYSICS_MODEL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_physics_model++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        instance_physics_model__ValidationData* validationData = (instance_physics_model__ValidationData*)mValidationDataStack.newObject(sizeof(instance_physics_model__ValidationData));
        memset(validationData, 0, sizeof(instance_physics_model__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_physics_model()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_physics_model__ValidationData* validationData = (instance_physics_model__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_force_field( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 571:
        {
        instance_physics_model__ValidationData* parentValidationData = (instance_physics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_rigid_body != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_FORCE_FIELD, (const ParserChar*)0, "sibling: instance_rigid_body" ) )
                return false;
        }
        if ( parentValidationData->instance_rigid_constraint != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_FORCE_FIELD, (const ParserChar*)0, "sibling: instance_rigid_constraint" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_FORCE_FIELD, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_force_field++;

        break;
        }

        case 586:
        {
        physics_scene__ValidationData* parentValidationData = (physics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_physics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_FORCE_FIELD, (const ParserChar*)0, "sibling: instance_physics_model" ) )
                return false;
        }
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_FORCE_FIELD, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_FORCE_FIELD, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_FORCE_FIELD, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_force_field++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        instance_force_field__ValidationData* validationData = (instance_force_field__ValidationData*)mValidationDataStack.newObject(sizeof(instance_force_field__ValidationData));
        memset(validationData, 0, sizeof(instance_force_field__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_force_field()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_force_field__ValidationData* validationData = (instance_force_field__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_rigid_body( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_physics_model__ValidationData* parentValidationData = (instance_physics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_rigid_constraint != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_RIGID_BODY, (const ParserChar*)0, "sibling: instance_rigid_constraint" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_RIGID_BODY, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_rigid_body++;

        instance_rigid_body__ValidationData* validationData = (instance_rigid_body__ValidationData*)mValidationDataStack.newObject(sizeof(instance_rigid_body__ValidationData));
        memset(validationData, 0, sizeof(instance_rigid_body__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_rigid_body()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body__ValidationData* validationData = (instance_rigid_body__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INSTANCE_RIGID_BODY, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_rigid_body_type____technique_common( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body__ValidationData* parentValidationData = (instance_rigid_body__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        instance_rigid_body_type____technique_common__ValidationData* validationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.newObject(sizeof(instance_rigid_body_type____technique_common__ValidationData));
        memset(validationData, 0, sizeof(instance_rigid_body_type____technique_common__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_rigid_body_type____technique_common()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body_type____technique_common__ValidationData* validationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ANGULAR_VELOCITY) || (validationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (validationData->validation_current_state == HASH_ELEMENT_INERTIA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (validationData->validation_current_state == HASH_ELEMENT_MASS) || (validationData->validation_current_state == HASH_ELEMENT_MASS_FRAME) || (validationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (validationData->validation_current_state == HASH_ELEMENT_SHAPE) || (validationData->validation_current_state == HASH_ELEMENT_VELOCITY) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__angular_velocity( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body_type____technique_common__ValidationData* parentValidationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ANGULAR_VELOCITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ANGULAR_VELOCITY;
        }
//        parentValidationData->angular_velocity++;

        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__angular_velocity()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__velocity( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body_type____technique_common__ValidationData* parentValidationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANGULAR_VELOCITY) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VELOCITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VELOCITY;
        }
//        parentValidationData->velocity++;

        origin____float3_type__ValidationData* validationData = (origin____float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(origin____float3_type__ValidationData));
        memset(validationData, 0, sizeof(origin____float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__velocity()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_rigid_body__technique_common__dynamic( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body_type____technique_common__ValidationData* parentValidationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANGULAR_VELOCITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_VELOCITY) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DYNAMIC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DYNAMIC;
        }
//        parentValidationData->dynamic++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_rigid_body__technique_common__dynamic()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_rigid_body__technique_common__mass_frame( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body_type____technique_common__ValidationData* parentValidationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANGULAR_VELOCITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_VELOCITY) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MASS_FRAME, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MASS_FRAME;
        }
//        parentValidationData->mass_frame++;

        instance_rigid_body__technique_common__mass_frame__ValidationData* validationData = (instance_rigid_body__technique_common__mass_frame__ValidationData*)mValidationDataStack.newObject(sizeof(instance_rigid_body__technique_common__mass_frame__ValidationData));
        memset(validationData, 0, sizeof(instance_rigid_body__technique_common__mass_frame__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_rigid_body__technique_common__mass_frame()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body__technique_common__mass_frame__ValidationData* validationData = (instance_rigid_body__technique_common__mass_frame__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->translate != 0 )
            if ( validationData->translate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MASS_FRAME, (const ParserChar*)0, "child: translate" ) )
                    return false;
        if ( validationData->rotate != 0 )
            if ( validationData->rotate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MASS_FRAME, (const ParserChar*)0, "child: rotate" ) )
                    return false;
        if ( validationData->translate == 0 && validationData->rotate == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_MASS_FRAME, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_rigid_body__technique_common__shape( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body_type____technique_common__ValidationData* parentValidationData = (instance_rigid_body_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANGULAR_VELOCITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_DYNAMIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_INERTIA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MASS_FRAME) || (parentValidationData->validation_current_state == HASH_ELEMENT_PHYSICS_MATERIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_SHAPE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VELOCITY) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SHAPE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SHAPE;
        }
//        parentValidationData->shape++;

        instance_rigid_body__technique_common__shape__ValidationData* validationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.newObject(sizeof(instance_rigid_body__technique_common__shape__ValidationData));
        memset(validationData, 0, sizeof(instance_rigid_body__technique_common__shape__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_rigid_body__technique_common__shape()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body__technique_common__shape__ValidationData* validationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_BOX) || (validationData->validation_current_state == HASH_ELEMENT_CAPSULE) || (validationData->validation_current_state == HASH_ELEMENT_CYLINDER) || (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_GEOMETRY) || (validationData->validation_current_state == HASH_ELEMENT_PLANE) || (validationData->validation_current_state == HASH_ELEMENT_ROTATE) || (validationData->validation_current_state == HASH_ELEMENT_SPHERE) || (validationData->validation_current_state == HASH_ELEMENT_TRANSLATE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SHAPE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_rigid_body__technique_common__shape__hollow( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_body__technique_common__shape__ValidationData* parentValidationData = (instance_rigid_body__technique_common__shape__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_HOLLOW, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_HOLLOW;
        }
//        parentValidationData->hollow++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_rigid_body__technique_common__shape__hollow()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_rigid_constraint( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_physics_model__ValidationData* parentValidationData = (instance_physics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_RIGID_CONSTRAINT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_rigid_constraint++;

        instance_rigid_constraint__ValidationData* validationData = (instance_rigid_constraint__ValidationData*)mValidationDataStack.newObject(sizeof(instance_rigid_constraint__ValidationData));
        memset(validationData, 0, sizeof(instance_rigid_constraint__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_rigid_constraint()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_rigid_constraint__ValidationData* validationData = (instance_rigid_constraint__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_physics_scenes( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_PHYSICS_SCENES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_PHYSICS_SCENES;
        }
//        parentValidationData->library_physics_scenes++;

        library_physics_scenes__ValidationData* validationData = (library_physics_scenes__ValidationData*)mValidationDataStack.newObject(sizeof(library_physics_scenes__ValidationData));
        memset(validationData, 0, sizeof(library_physics_scenes__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_physics_scenes()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_physics_scenes__ValidationData* validationData = (library_physics_scenes__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->physics_scene < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_PHYSICS_SCENES, (const ParserChar*)0, "child: physics_scene" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__physics_scene( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_physics_scenes__ValidationData* parentValidationData = (library_physics_scenes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PHYSICS_SCENE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->physics_scene++;

        physics_scene__ValidationData* validationData = (physics_scene__ValidationData*)mValidationDataStack.newObject(sizeof(physics_scene__ValidationData));
        memset(validationData, 0, sizeof(physics_scene__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__physics_scene()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_scene__ValidationData* validationData = (physics_scene__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PHYSICS_SCENE, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__physics_scene_type____technique_common( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_scene__ValidationData* parentValidationData = (physics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        physics_scene_type____technique_common__ValidationData* validationData = (physics_scene_type____technique_common__ValidationData*)mValidationDataStack.newObject(sizeof(physics_scene_type____technique_common__ValidationData));
        memset(validationData, 0, sizeof(physics_scene_type____technique_common__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__physics_scene_type____technique_common()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_scene_type____technique_common__ValidationData* validationData = (physics_scene_type____technique_common__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__gravity( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_scene_type____technique_common__ValidationData* parentValidationData = (physics_scene_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->time_step != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_GRAVITY, (const ParserChar*)0, "sibling: time_step" ) )
                return false;
        }
        if ( parentValidationData->gravity >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_GRAVITY, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->gravity++;

        color____targetable_float3_type__ValidationData* validationData = (color____targetable_float3_type__ValidationData*)mValidationDataStack.newObject(sizeof(color____targetable_float3_type__ValidationData));
        memset(validationData, 0, sizeof(color____targetable_float3_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__gravity()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__time_step( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        physics_scene_type____technique_common__ValidationData* parentValidationData = (physics_scene_type____technique_common__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->time_step >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TIME_STEP, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->time_step++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__time_step()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_visual_scenes( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_VISUAL_SCENES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_VISUAL_SCENES;
        }
//        parentValidationData->library_visual_scenes++;

        library_visual_scenes__ValidationData* validationData = (library_visual_scenes__ValidationData*)mValidationDataStack.newObject(sizeof(library_visual_scenes__ValidationData));
        memset(validationData, 0, sizeof(library_visual_scenes__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_visual_scenes()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_visual_scenes__ValidationData* validationData = (library_visual_scenes__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->visual_scene < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_VISUAL_SCENES, (const ParserChar*)0, "child: visual_scene" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__visual_scene( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_visual_scenes__ValidationData* parentValidationData = (library_visual_scenes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_VISUAL_SCENE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->visual_scene++;

        visual_scene__ValidationData* validationData = (visual_scene__ValidationData*)mValidationDataStack.newObject(sizeof(visual_scene__ValidationData));
        memset(validationData, 0, sizeof(visual_scene__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__visual_scene()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        visual_scene__ValidationData* validationData = (visual_scene__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->node < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_VISUAL_SCENE, (const ParserChar*)0, "child: node" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__evaluate_scene( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        visual_scene__ValidationData* parentValidationData = (visual_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->node < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_EVALUATE_SCENE, (const ParserChar*)0, "sibling: node" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_EVALUATE_SCENE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->evaluate_scene++;

        evaluate_scene__ValidationData* validationData = (evaluate_scene__ValidationData*)mValidationDataStack.newObject(sizeof(evaluate_scene__ValidationData));
        memset(validationData, 0, sizeof(evaluate_scene__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__evaluate_scene()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        evaluate_scene__ValidationData* validationData = (evaluate_scene__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__render( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        evaluate_scene__ValidationData* parentValidationData = (evaluate_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_RENDER, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->render++;

        render__ValidationData* validationData = (render__ValidationData*)mValidationDataStack.newObject(sizeof(render__ValidationData));
        memset(validationData, 0, sizeof(render__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__render()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        render__ValidationData* validationData = (render__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__layer( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        render__ValidationData* parentValidationData = (render__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_material != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LAYER, (const ParserChar*)0, "sibling: instance_material" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_LAYER, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->layer++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__layer()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__render__instance_material( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        render__ValidationData* parentValidationData = (render__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_MATERIAL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->instance_material >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INSTANCE_MATERIAL, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->instance_material++;

        render__instance_material__ValidationData* validationData = (render__instance_material__ValidationData*)mValidationDataStack.newObject(sizeof(render__instance_material__ValidationData));
        memset(validationData, 0, sizeof(render__instance_material__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__render__instance_material()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        render__instance_material__ValidationData* validationData = (render__instance_material__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__technique_override( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        render__instance_material__ValidationData* parentValidationData = (render__instance_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->bind != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_OVERRIDE, (const ParserChar*)0, "sibling: bind" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_OVERRIDE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_override >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_OVERRIDE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_override++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__technique_override()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__render__instance_material__bind( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        render__instance_material__ValidationData* parentValidationData = (render__instance_material__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->bind++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__render__instance_material__bind()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_joints( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_JOINTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_JOINTS;
        }
//        parentValidationData->library_joints++;

        library_joints__ValidationData* validationData = (library_joints__ValidationData*)mValidationDataStack.newObject(sizeof(library_joints__ValidationData));
        memset(validationData, 0, sizeof(library_joints__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_joints()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_joints__ValidationData* validationData = (library_joints__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->joint < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_JOINTS, (const ParserChar*)0, "child: joint" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__joint( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 685:
        {
        technique_common____kinematics_model_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_model_technique_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_JOINT) || (parentValidationData->validation_current_state == HASH_ELEMENT_JOINT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_JOINT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_JOINT;
        }
//        parentValidationData->joint++;

        break;
        }

        case 285:
        {
        library_joints__ValidationData* parentValidationData = (library_joints__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_JOINT, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->joint++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        joint__ValidationData* validationData = (joint__ValidationData*)mValidationDataStack.newObject(sizeof(joint__ValidationData));
        memset(validationData, 0, sizeof(joint__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__joint()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        joint__ValidationData* validationData = (joint__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_PRISMATIC) || (validationData->validation_current_state == HASH_ELEMENT_REVOLUTE) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_JOINT, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__prismatic( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        joint__ValidationData* parentValidationData = (joint__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_PRISMATIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_REVOLUTE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRISMATIC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRISMATIC;
        }
//        parentValidationData->prismatic++;

        prismatic__ValidationData* validationData = (prismatic__ValidationData*)mValidationDataStack.newObject(sizeof(prismatic__ValidationData));
        memset(validationData, 0, sizeof(prismatic__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__prismatic()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        prismatic__ValidationData* validationData = (prismatic__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->axis < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_PRISMATIC, (const ParserChar*)0, "child: axis" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__axis____axis_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        prismatic__ValidationData* parentValidationData = (prismatic__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->limits != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AXIS, (const ParserChar*)0, "sibling: limits" ) )
                return false;
        }
        if ( parentValidationData->axis >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_AXIS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->axis++;

        axis____axis_type__ValidationData* validationData = (axis____axis_type__ValidationData*)mValidationDataStack.newObject(sizeof(axis____axis_type__ValidationData));
        memset(validationData, 0, sizeof(axis____axis_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__axis____axis_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__limits____joint_limits_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        prismatic__ValidationData* parentValidationData = (prismatic__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->axis < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_LIMITS, (const ParserChar*)0, "sibling: axis" ) )
                return false;
        }
        if ( parentValidationData->limits >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_LIMITS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->limits++;

        limits____joint_limits_type__ValidationData* validationData = (limits____joint_limits_type__ValidationData*)mValidationDataStack.newObject(sizeof(limits____joint_limits_type__ValidationData));
        memset(validationData, 0, sizeof(limits____joint_limits_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__limits____joint_limits_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        limits____joint_limits_type__ValidationData* validationData = (limits____joint_limits_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__min____minmax_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        limits____joint_limits_type__ValidationData* parentValidationData = (limits____joint_limits_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->max != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIN, (const ParserChar*)0, "sibling: max" ) )
                return false;
        }
        if ( parentValidationData->min >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MIN, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->min++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__min____minmax_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__max____minmax_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        limits____joint_limits_type__ValidationData* parentValidationData = (limits____joint_limits_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->max >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MAX, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->max++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__max____minmax_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__revolute( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        joint__ValidationData* parentValidationData = (joint__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_PRISMATIC) || (parentValidationData->validation_current_state == HASH_ELEMENT_REVOLUTE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REVOLUTE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REVOLUTE;
        }
//        parentValidationData->revolute++;

        prismatic__ValidationData* validationData = (prismatic__ValidationData*)mValidationDataStack.newObject(sizeof(prismatic__ValidationData));
        memset(validationData, 0, sizeof(prismatic__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__revolute()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        prismatic__ValidationData* validationData = (prismatic__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->axis < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_REVOLUTE, (const ParserChar*)0, "child: axis" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_kinematics_models( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS;
        }
//        parentValidationData->library_kinematics_models++;

        library_kinematics_models__ValidationData* validationData = (library_kinematics_models__ValidationData*)mValidationDataStack.newObject(sizeof(library_kinematics_models__ValidationData));
        memset(validationData, 0, sizeof(library_kinematics_models__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_kinematics_models()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_kinematics_models__ValidationData* validationData = (library_kinematics_models__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->kinematics_model < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS, (const ParserChar*)0, "child: kinematics_model" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__kinematics_model( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_kinematics_models__ValidationData* parentValidationData = (library_kinematics_models__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_KINEMATICS_MODEL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->kinematics_model++;

        kinematics_model__ValidationData* validationData = (kinematics_model__ValidationData*)mValidationDataStack.newObject(sizeof(kinematics_model__ValidationData));
        memset(validationData, 0, sizeof(kinematics_model__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__kinematics_model()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        kinematics_model__ValidationData* validationData = (kinematics_model__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_KINEMATICS_MODEL, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__technique_common____kinematics_model_technique_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        kinematics_model__ValidationData* parentValidationData = (kinematics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        technique_common____kinematics_model_technique_type__ValidationData* validationData = (technique_common____kinematics_model_technique_type__ValidationData*)mValidationDataStack.newObject(sizeof(technique_common____kinematics_model_technique_type__ValidationData));
        memset(validationData, 0, sizeof(technique_common____kinematics_model_technique_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__technique_common____kinematics_model_technique_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____kinematics_model_technique_type__ValidationData* validationData = (technique_common____kinematics_model_technique_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_FORMULA) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_FORMULA) || (validationData->validation_current_state == HASH_ELEMENT_LINK) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__newparam____kinematics_newparam_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 246:
        {
        instance_articulated_system__ValidationData* parentValidationData = (instance_articulated_system__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->newparam++;

        break;
        }

        case 152:
        {
        instance_kinematics_model__ValidationData* parentValidationData = (instance_kinematics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->setparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: setparam" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->newparam++;

        break;
        }

        case 17:
        {
        instance_kinematics_scene__ValidationData* parentValidationData = (instance_kinematics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->setparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: setparam" ) )
                return false;
        }
        if ( parentValidationData->bind_kinematics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: bind_kinematics_model" ) )
                return false;
        }
        if ( parentValidationData->bind_joint_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: bind_joint_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->newparam++;

        break;
        }

        case 726:
        {
        axis_info____kinematics_axis_info_type__ValidationData* parentValidationData = (axis_info____kinematics_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEWPARAM;
        }
//        parentValidationData->newparam++;

        break;
        }

        case 685:
        {
        technique_common____kinematics_model_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_model_technique_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEWPARAM;
        }
//        parentValidationData->newparam++;

        break;
        }

        case 727:
        {
        axis_info____motion_axis_info_type__ValidationData* parentValidationData = (axis_info____motion_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->setparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: setparam" ) )
                return false;
        }
        if ( parentValidationData->speed != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: speed" ) )
                return false;
        }
        if ( parentValidationData->acceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: acceleration" ) )
                return false;
        }
        if ( parentValidationData->deceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: deceleration" ) )
                return false;
        }
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        parentValidationData->newparam++;

        break;
        }

        case 457:
        {
        effector_info__ValidationData* parentValidationData = (effector_info__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->setparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: setparam" ) )
                return false;
        }
        if ( parentValidationData->speed != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: speed" ) )
                return false;
        }
        if ( parentValidationData->acceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: acceleration" ) )
                return false;
        }
        if ( parentValidationData->deceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: deceleration" ) )
                return false;
        }
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        parentValidationData->newparam++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        newparam____kinematics_newparam_type__ValidationData* validationData = (newparam____kinematics_newparam_type__ValidationData*)mValidationDataStack.newObject(sizeof(newparam____kinematics_newparam_type__ValidationData));
        memset(validationData, 0, sizeof(newparam____kinematics_newparam_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__newparam____kinematics_newparam_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____kinematics_newparam_type__ValidationData* validationData = (newparam____kinematics_newparam_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->_int != 0 )
            if ( validationData->_int < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "child: int" ) )
                    return false;
        if ( validationData->SIDREF != 0 )
            if ( validationData->SIDREF < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "child: SIDREF" ) )
                    return false;
        if ( validationData->_bool != 0 )
            if ( validationData->_bool < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "child: bool" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->_int == 0 && validationData->SIDREF == 0 && validationData->_bool == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_joint( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____kinematics_model_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_model_technique_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_JOINT) || (parentValidationData->validation_current_state == HASH_ELEMENT_JOINT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INSTANCE_JOINT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INSTANCE_JOINT;
        }
//        parentValidationData->instance_joint++;

        instance_joint__ValidationData* validationData = (instance_joint__ValidationData*)mValidationDataStack.newObject(sizeof(instance_joint__ValidationData));
        memset(validationData, 0, sizeof(instance_joint__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_joint()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_joint__ValidationData* validationData = (instance_joint__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__link( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 299:
        {
        attachment_full__ValidationData* parentValidationData = (attachment_full__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINK;
        }
//        parentValidationData->link++;

        break;
        }

        case 685:
        {
        technique_common____kinematics_model_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_model_technique_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_JOINT) || (parentValidationData->validation_current_state == HASH_ELEMENT_JOINT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINK) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LINK, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LINK;
        }
//        parentValidationData->link++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        link__ValidationData* validationData = (link__ValidationData*)mValidationDataStack.newObject(sizeof(link__ValidationData));
        memset(validationData, 0, sizeof(link__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__link()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        link__ValidationData* validationData = (link__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_END) || (validationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_FULL) || (validationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_START) || (validationData->validation_current_state == HASH_ELEMENT_ROTATE) || (validationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_LINK, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__attachment_full( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        link__ValidationData* parentValidationData = (link__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_FULL) || (parentValidationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ATTACHMENT_FULL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ATTACHMENT_FULL;
        }
//        parentValidationData->attachment_full++;

        attachment_full__ValidationData* validationData = (attachment_full__ValidationData*)mValidationDataStack.newObject(sizeof(attachment_full__ValidationData));
        memset(validationData, 0, sizeof(attachment_full__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__attachment_full()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        attachment_full__ValidationData* validationData = (attachment_full__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_LINK) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_ATTACHMENT_FULL, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__attachment_start( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        link__ValidationData* parentValidationData = (link__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_FULL) || (parentValidationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ATTACHMENT_START, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ATTACHMENT_START;
        }
//        parentValidationData->attachment_start++;

        attachment_start__ValidationData* validationData = (attachment_start__ValidationData*)mValidationDataStack.newObject(sizeof(attachment_start__ValidationData));
        memset(validationData, 0, sizeof(attachment_start__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__attachment_start()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        attachment_start__ValidationData* validationData = (attachment_start__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->rotate != 0 )
            if ( validationData->rotate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ATTACHMENT_START, (const ParserChar*)0, "child: rotate" ) )
                    return false;
        if ( validationData->translate != 0 )
            if ( validationData->translate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ATTACHMENT_START, (const ParserChar*)0, "child: translate" ) )
                    return false;
        if ( validationData->rotate == 0 && validationData->translate == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_ATTACHMENT_START, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__attachment_end( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        link__ValidationData* parentValidationData = (link__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_END) || (parentValidationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_FULL) || (parentValidationData->validation_current_state == HASH_ELEMENT_ATTACHMENT_START) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROTATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSLATE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ATTACHMENT_END, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ATTACHMENT_END;
        }
//        parentValidationData->attachment_end++;

        attachment_end__ValidationData* validationData = (attachment_end__ValidationData*)mValidationDataStack.newObject(sizeof(attachment_end__ValidationData));
        memset(validationData, 0, sizeof(attachment_end__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__attachment_end()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        attachment_end__ValidationData* validationData = (attachment_end__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->rotate != 0 )
            if ( validationData->rotate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ATTACHMENT_END, (const ParserChar*)0, "child: rotate" ) )
                    return false;
        if ( validationData->translate != 0 )
            if ( validationData->translate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ATTACHMENT_END, (const ParserChar*)0, "child: translate" ) )
                    return false;
        if ( validationData->rotate == 0 && validationData->translate == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_ATTACHMENT_END, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__formula( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 726:
        {
        axis_info____kinematics_axis_info_type__ValidationData* parentValidationData = (axis_info____kinematics_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ACTIVE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORMULA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INDEX) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_FORMULA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMITS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOCKED) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORMULA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORMULA;
        }
//        parentValidationData->formula++;

        break;
        }

        case 685:
        {
        technique_common____kinematics_model_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_model_technique_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_FORMULA) || (parentValidationData->validation_current_state == HASH_ELEMENT_INSTANCE_FORMULA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LINK) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORMULA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORMULA;
        }
//        parentValidationData->formula++;

        break;
        }

        case 101:
        {
        library_formulas__ValidationData* parentValidationData = (library_formulas__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FORMULA, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->formula++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        formula__ValidationData* validationData = (formula__ValidationData*)mValidationDataStack.newObject(sizeof(formula__ValidationData));
        memset(validationData, 0, sizeof(formula__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__formula()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        formula__ValidationData* validationData = (formula__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->target < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FORMULA, (const ParserChar*)0, "child: target" ) )
                return false;
        }
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FORMULA, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__newparam____formula_newparam_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        formula__ValidationData* parentValidationData = (formula__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->target != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: target" ) )
                return false;
        }
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        parentValidationData->newparam++;

        newparam____formula_newparam_type__ValidationData* validationData = (newparam____formula_newparam_type__ValidationData*)mValidationDataStack.newObject(sizeof(newparam____formula_newparam_type__ValidationData));
        memset(validationData, 0, sizeof(newparam____formula_newparam_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__newparam____formula_newparam_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        newparam____formula_newparam_type__ValidationData* validationData = (newparam____formula_newparam_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->_int != 0 )
            if ( validationData->_int < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "child: int" ) )
                    return false;
        if ( validationData->SIDREF != 0 )
            if ( validationData->SIDREF < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "child: SIDREF" ) )
                    return false;
        if ( validationData->_bool != 0 )
            if ( validationData->_bool < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, "child: bool" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->_int == 0 && validationData->SIDREF == 0 && validationData->_bool == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_NEWPARAM, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__target( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        formula__ValidationData* parentValidationData = (formula__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TARGET, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TARGET, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->target >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TARGET, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->target++;

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.newObject(sizeof(target__ValidationData));
        memset(validationData, 0, sizeof(target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__target()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TARGET, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TARGET, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_TARGET, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__param____common_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 282:
        {
        bind_kinematics_model__ValidationData* parentValidationData = (bind_kinematics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 229:
        {
        active__ValidationData* parentValidationData = (active__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 26:
        {
        speed____common_float2_or_param_type__ValidationData* parentValidationData = (speed____common_float2_or_param_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->float2 != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: float2" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 27:
        {
        target__ValidationData* parentValidationData = (target__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 449:
        {
        axis____common_sidref_or_param_type__ValidationData* parentValidationData = (axis____common_sidref_or_param_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        case 594:
        {
        index__ValidationData* parentValidationData = (index__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__param____common_param_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__technique_common____formula_technique_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        formula__ValidationData* parentValidationData = (formula__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->target < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: target" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        technique_common____formula_technique_type__ValidationData* validationData = (technique_common____formula_technique_type__ValidationData*)mValidationDataStack.newObject(sizeof(technique_common____formula_technique_type__ValidationData));
        memset(validationData, 0, sizeof(technique_common____formula_technique_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__technique_common____formula_technique_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____formula_technique_type__ValidationData* validationData = (technique_common____formula_technique_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->math < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: math" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__math( ParserString value )
{
    return validate__math( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__math( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__math__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__math__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NMTOKENS( const ParserString*value, size_t length )
{
    if( length < 1 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NMTOKENS__stream( const ParserString*value, size_t length )
{

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__NMTOKENS__streamEnd( const ParserString*value, size_t length )
{
    if( length < 1 )
        return ParserError::ERROR_VALIDATION_MIN_LENGTH;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__math( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____formula_technique_type__ValidationData* parentValidationData = (technique_common____formula_technique_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->math >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MATH, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->math++;

        math__ValidationData* validationData = (math__ValidationData*)mValidationDataStack.newObject(sizeof(math__ValidationData));
        memset(validationData, 0, sizeof(math__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__math()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        math__ValidationData* validationData = (math__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_MATH, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__cn( const uint64 value )
{
    if( value > 36 )
        return ParserError::ERROR_VALIDATION_MAX_INCLUSIVE;

    if( value < 2 )
        return ParserError::ERROR_VALIDATION_MIN_INCLUSIVE;

    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__cn( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CN;
        }
//        parentValidationData->cn++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CN;
        }
//        parentValidationData->cn++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        cn__ValidationData* validationData = (cn__ValidationData*)mValidationDataStack.newObject(sizeof(cn__ValidationData));
        memset(validationData, 0, sizeof(cn__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__cn()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        cn__ValidationData* validationData = (cn__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SEP) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_CN, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ci( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CI;
        }
//        parentValidationData->ci++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CI;
        }
//        parentValidationData->ci++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CI;
        }
//        parentValidationData->ci++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        ci__ValidationData* validationData = (ci__ValidationData*)mValidationDataStack.newObject(sizeof(ci__ValidationData));
        memset(validationData, 0, sizeof(ci__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ci()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        ci__ValidationData* validationData = (ci__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_CI, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__csymbol( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSYMBOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSYMBOL;
        }
//        parentValidationData->csymbol++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSYMBOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSYMBOL;
        }
//        parentValidationData->csymbol++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSYMBOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSYMBOL;
        }
//        parentValidationData->csymbol++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSYMBOL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSYMBOL;
        }
//        parentValidationData->csymbol++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        csymbol__ValidationData* validationData = (csymbol__ValidationData*)mValidationDataStack.newObject(sizeof(csymbol__ValidationData));
        memset(validationData, 0, sizeof(csymbol__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__csymbol()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        csymbol__ValidationData* validationData = (csymbol__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_CSYMBOL, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__abs( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ABS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ABS;
        }
//        parentValidationData->abs++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__abs()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__conjugate( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONJUGATE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONJUGATE;
        }
//        parentValidationData->conjugate++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__conjugate()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__factorial( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTORIAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTORIAL;
        }
//        parentValidationData->factorial++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__factorial()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arg( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARG;
        }
//        parentValidationData->arg++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arg()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__real( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REAL;
        }
//        parentValidationData->real++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__real()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__imaginary( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARY;
        }
//        parentValidationData->imaginary++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__imaginary()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__floor( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FLOOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FLOOR;
        }
//        parentValidationData->floor++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__floor()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ceiling( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CEILING, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CEILING;
        }
//        parentValidationData->ceiling++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ceiling()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__quotient( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_QUOTIENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_QUOTIENT;
        }
//        parentValidationData->quotient++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__quotient()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__divide( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVIDE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVIDE;
        }
//        parentValidationData->divide++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__divide()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rem( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REM;
        }
//        parentValidationData->rem++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rem()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__minus( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MINUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MINUS;
        }
//        parentValidationData->minus++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__minus()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__plus( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PLUS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PLUS;
        }
//        parentValidationData->plus++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__plus()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__times( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TIMES;
        }
//        parentValidationData->times++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__times()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__power( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_POWER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_POWER;
        }
//        parentValidationData->power++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__power()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__root( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ROOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ROOT;
        }
//        parentValidationData->root++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__root()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__max____Arith_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MAX;
        }
//        parentValidationData->max++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__max____Arith_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__min____Arith_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MIN;
        }
//        parentValidationData->min++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__min____Arith_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__gcd( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GCD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GCD;
        }
//        parentValidationData->gcd++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__gcd()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__lcm( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LCM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LCM;
        }
//        parentValidationData->lcm++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__lcm()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sum( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUM, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUM;
        }
//        parentValidationData->sum++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sum()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__product( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRODUCT;
        }
//        parentValidationData->product++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__product()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__compose( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPOSE;
        }
//        parentValidationData->compose++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__compose()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__domain( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAIN;
        }
//        parentValidationData->domain++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__domain()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__codomain( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CODOMAIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CODOMAIN;
        }
//        parentValidationData->codomain++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__codomain()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__image____Functions_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGE;
        }
//        parentValidationData->image++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__image____Functions_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__domainofapplication( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAINOFAPPLICATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAINOFAPPLICATION;
        }
//        parentValidationData->domainofapplication++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAINOFAPPLICATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAINOFAPPLICATION;
        }
//        parentValidationData->domainofapplication++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAINOFAPPLICATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAINOFAPPLICATION;
        }
//        parentValidationData->domainofapplication++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAINOFAPPLICATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAINOFAPPLICATION;
        }
//        parentValidationData->domainofapplication++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DOMAINOFAPPLICATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DOMAINOFAPPLICATION;
        }
//        parentValidationData->domainofapplication++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        domainofapplication__ValidationData* validationData = (domainofapplication__ValidationData*)mValidationDataStack.newObject(sizeof(domainofapplication__ValidationData));
        memset(validationData, 0, sizeof(domainofapplication__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__domainofapplication()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        domainofapplication__ValidationData* validationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_DOMAINOFAPPLICATION, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ident( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IDENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IDENT;
        }
//        parentValidationData->ident++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ident()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__and( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_AND, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_AND;
        }
//        parentValidationData->_and++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__and()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__or( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OR;
        }
//        parentValidationData->_or++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__or()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__xor( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_XOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_XOR;
        }
//        parentValidationData->_xor++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__xor()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__not( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOT;
        }
//        parentValidationData->_not++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__not()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__exists( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXISTS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXISTS;
        }
//        parentValidationData->exists++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__exists()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__forall( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FORALL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FORALL;
        }
//        parentValidationData->forall++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__forall()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__implies( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMPLIES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMPLIES;
        }
//        parentValidationData->implies++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__implies()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__naturalnumbers( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NATURALNUMBERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NATURALNUMBERS;
        }
//        parentValidationData->naturalnumbers++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__naturalnumbers()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__primes( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRIMES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRIMES;
        }
//        parentValidationData->primes++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__primes()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__integers( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTEGERS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTEGERS;
        }
//        parentValidationData->integers++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__integers()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__rationals( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_RATIONALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_RATIONALS;
        }
//        parentValidationData->rationals++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__rationals()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__reals( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_REALS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_REALS;
        }
//        parentValidationData->reals++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__reals()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__complexes( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COMPLEXES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COMPLEXES;
        }
//        parentValidationData->complexes++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__complexes()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__emptyset( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EMPTYSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EMPTYSET;
        }
//        parentValidationData->emptyset++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__emptyset()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__exponentiale( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXPONENTIALE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXPONENTIALE;
        }
//        parentValidationData->exponentiale++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__exponentiale()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__imaginaryi( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IMAGINARYI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IMAGINARYI;
        }
//        parentValidationData->imaginaryi++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__imaginaryi()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__pi( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PI, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PI;
        }
//        parentValidationData->pi++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__pi()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__eulergamma( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EULERGAMMA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EULERGAMMA;
        }
//        parentValidationData->eulergamma++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__eulergamma()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__true( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRUE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRUE;
        }
//        parentValidationData->_true++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__true()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__false( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FALSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FALSE;
        }
//        parentValidationData->_false++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__false()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__infinity( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INFINITY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INFINITY;
        }
//        parentValidationData->infinity++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__infinity()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__notanumber( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTANUMBER, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTANUMBER;
        }
//        parentValidationData->notanumber++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__notanumber()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__set( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SET;
        }
//        parentValidationData->set++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SET;
        }
//        parentValidationData->set++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SET;
        }
//        parentValidationData->set++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SET;
        }
//        parentValidationData->set++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SET;
        }
//        parentValidationData->set++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SET;
        }
//        parentValidationData->set++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        set__ValidationData* validationData = (set__ValidationData*)mValidationDataStack.newObject(sizeof(set__ValidationData));
        memset(validationData, 0, sizeof(set__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__set()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        set__ValidationData* validationData = (set__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SET, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__list( ParserString value )
{
    return validate__list( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__list( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__list__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__list__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__list( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIST, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIST;
        }
//        parentValidationData->list++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIST, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIST;
        }
//        parentValidationData->list++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIST, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIST;
        }
//        parentValidationData->list++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIST, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIST;
        }
//        parentValidationData->list++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIST, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIST;
        }
//        parentValidationData->list++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIST, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIST;
        }
//        parentValidationData->list++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIST, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIST;
        }
//        parentValidationData->list++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        list__ValidationData* validationData = (list__ValidationData*)mValidationDataStack.newObject(sizeof(list__ValidationData));
        memset(validationData, 0, sizeof(list__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__list()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        list__ValidationData* validationData = (list__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_LIST, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__union( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UNION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UNION;
        }
//        parentValidationData->_union++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__union()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__intersect( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERSECT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERSECT;
        }
//        parentValidationData->intersect++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__intersect()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__in( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_IN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_IN;
        }
//        parentValidationData->_in++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__in()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__notin( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTIN;
        }
//        parentValidationData->notin++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__notin()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__subset( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SUBSET;
        }
//        parentValidationData->subset++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__subset()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__prsubset( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PRSUBSET;
        }
//        parentValidationData->prsubset++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__prsubset()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__notsubset( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTSUBSET;
        }
//        parentValidationData->notsubset++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__notsubset()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__notprsubset( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NOTPRSUBSET, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NOTPRSUBSET;
        }
//        parentValidationData->notprsubset++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__notprsubset()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__setdiff( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SETDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SETDIFF;
        }
//        parentValidationData->setdiff++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__setdiff()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__card( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARD;
        }
//        parentValidationData->card++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__card()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__cartesianproduct( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CARTESIANPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CARTESIANPRODUCT;
        }
//        parentValidationData->cartesianproduct++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__cartesianproduct()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__eq( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQ;
        }
//        parentValidationData->eq++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__eq()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__neq( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_NEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_NEQ;
        }
//        parentValidationData->neq++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__neq()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__leq( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LEQ;
        }
//        parentValidationData->leq++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__leq()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__lt( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LT;
        }
//        parentValidationData->lt++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__lt()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__geq( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GEQ, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GEQ;
        }
//        parentValidationData->geq++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__geq()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__gt( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GT;
        }
//        parentValidationData->gt++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__gt()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__equivalent( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EQUIVALENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EQUIVALENT;
        }
//        parentValidationData->equivalent++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__equivalent()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__approx( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPROX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPROX;
        }
//        parentValidationData->approx++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__approx()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__factorof( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_FACTOROF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_FACTOROF;
        }
//        parentValidationData->factorof++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__factorof()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__exp( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_EXP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_EXP;
        }
//        parentValidationData->exp++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__exp()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__ln( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LN;
        }
//        parentValidationData->ln++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__ln()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__log( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOG, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOG;
        }
//        parentValidationData->log++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__log()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__logbase( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGBASE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGBASE;
        }
//        parentValidationData->logbase++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGBASE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGBASE;
        }
//        parentValidationData->logbase++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGBASE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGBASE;
        }
//        parentValidationData->logbase++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGBASE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGBASE;
        }
//        parentValidationData->logbase++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGBASE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGBASE;
        }
//        parentValidationData->logbase++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGBASE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGBASE;
        }
//        parentValidationData->logbase++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGBASE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGBASE;
        }
//        parentValidationData->logbase++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOGBASE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOGBASE;
        }
//        parentValidationData->logbase++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        logbase__ValidationData* validationData = (logbase__ValidationData*)mValidationDataStack.newObject(sizeof(logbase__ValidationData));
        memset(validationData, 0, sizeof(logbase__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__logbase()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        logbase__ValidationData* validationData = (logbase__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_LOGBASE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sin( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SIN;
        }
//        parentValidationData->sin++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sin()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__cos( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COS;
        }
//        parentValidationData->cos++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__cos()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__tan( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TAN;
        }
//        parentValidationData->tan++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__tan()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sec( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEC;
        }
//        parentValidationData->sec++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sec()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__csc( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSC;
        }
//        parentValidationData->csc++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__csc()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__cot( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COT;
        }
//        parentValidationData->cot++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__cot()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arcsin( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSIN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSIN;
        }
//        parentValidationData->arcsin++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arcsin()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arccos( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOS;
        }
//        parentValidationData->arccos++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arccos()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arctan( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTAN;
        }
//        parentValidationData->arctan++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arctan()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arcsec( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSEC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSEC;
        }
//        parentValidationData->arcsec++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arcsec()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arccsc( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSC, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSC;
        }
//        parentValidationData->arccsc++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arccsc()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arccot( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOT;
        }
//        parentValidationData->arccot++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arccot()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sinh( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SINH;
        }
//        parentValidationData->sinh++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sinh()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__cosh( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COSH;
        }
//        parentValidationData->cosh++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__cosh()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__tanh( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TANH;
        }
//        parentValidationData->tanh++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__tanh()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sech( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SECH;
        }
//        parentValidationData->sech++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sech()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__csch( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CSCH;
        }
//        parentValidationData->csch++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__csch()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__coth( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_COTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_COTH;
        }
//        parentValidationData->coth++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__coth()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arccosh( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOSH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOSH;
        }
//        parentValidationData->arccosh++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arccosh()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arccoth( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCOTH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCOTH;
        }
//        parentValidationData->arccoth++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arccoth()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arccsch( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCCSCH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCCSCH;
        }
//        parentValidationData->arccsch++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arccsch()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arcsech( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSECH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSECH;
        }
//        parentValidationData->arcsech++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arcsech()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arcsinh( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCSINH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCSINH;
        }
//        parentValidationData->arcsinh++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arcsinh()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__arctanh( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ARCTANH, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ARCTANH;
        }
//        parentValidationData->arctanh++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__arctanh()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__int____int_type____mathml( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INT;
        }
//        parentValidationData->_int++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__int____int_type____mathml()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__diff( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIFF;
        }
//        parentValidationData->diff++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__diff()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__partialdiff( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PARTIALDIFF, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PARTIALDIFF;
        }
//        parentValidationData->partialdiff++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__partialdiff()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__limit( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMIT;
        }
//        parentValidationData->limit++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__limit()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__lowlimit( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOWLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOWLIMIT;
        }
//        parentValidationData->lowlimit++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOWLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOWLIMIT;
        }
//        parentValidationData->lowlimit++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOWLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOWLIMIT;
        }
//        parentValidationData->lowlimit++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOWLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOWLIMIT;
        }
//        parentValidationData->lowlimit++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOWLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOWLIMIT;
        }
//        parentValidationData->lowlimit++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOWLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOWLIMIT;
        }
//        parentValidationData->lowlimit++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOWLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOWLIMIT;
        }
//        parentValidationData->lowlimit++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOWLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOWLIMIT;
        }
//        parentValidationData->lowlimit++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOWLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOWLIMIT;
        }
//        parentValidationData->lowlimit++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        lowlimit__ValidationData* validationData = (lowlimit__ValidationData*)mValidationDataStack.newObject(sizeof(lowlimit__ValidationData));
        memset(validationData, 0, sizeof(lowlimit__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__lowlimit()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        lowlimit__ValidationData* validationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_LOWLIMIT, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__uplimit( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UPLIMIT;
        }
//        parentValidationData->uplimit++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UPLIMIT;
        }
//        parentValidationData->uplimit++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UPLIMIT;
        }
//        parentValidationData->uplimit++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UPLIMIT;
        }
//        parentValidationData->uplimit++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UPLIMIT;
        }
//        parentValidationData->uplimit++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UPLIMIT;
        }
//        parentValidationData->uplimit++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UPLIMIT;
        }
//        parentValidationData->uplimit++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UPLIMIT;
        }
//        parentValidationData->uplimit++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UPLIMIT;
        }
//        parentValidationData->uplimit++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_UPLIMIT;
        }
//        parentValidationData->uplimit++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        uplimit__ValidationData* validationData = (uplimit__ValidationData*)mValidationDataStack.newObject(sizeof(uplimit__ValidationData));
        memset(validationData, 0, sizeof(uplimit__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__uplimit()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        uplimit__ValidationData* validationData = (uplimit__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_UPLIMIT, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__tendsto( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TENDSTO, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TENDSTO;
        }
//        parentValidationData->tendsto++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__tendsto()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__vector( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTOR;
        }
//        parentValidationData->vector++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        vector__ValidationData* validationData = (vector__ValidationData*)mValidationDataStack.newObject(sizeof(vector__ValidationData));
        memset(validationData, 0, sizeof(vector__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__vector()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        vector__ValidationData* validationData = (vector__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_VECTOR, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__matrix____matrix_type____mathml( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MATRIX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MATRIX;
        }
//        parentValidationData->matrix++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        matrix____matrix_type____mathml__ValidationData* validationData = (matrix____matrix_type____mathml__ValidationData*)mValidationDataStack.newObject(sizeof(matrix____matrix_type____mathml__ValidationData));
        memset(validationData, 0, sizeof(matrix____matrix_type____mathml__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__matrix____matrix_type____mathml()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        matrix____matrix_type____mathml__ValidationData* validationData = (matrix____matrix_type____mathml__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->matrixrow < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MATRIX, (const ParserChar*)0, "child: matrixrow" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__matrixrow( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        matrix____matrix_type____mathml__ValidationData* parentValidationData = (matrix____matrix_type____mathml__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->matrixrow >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MATRIXROW, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->matrixrow++;

        matrixrow__ValidationData* validationData = (matrixrow__ValidationData*)mValidationDataStack.newObject(sizeof(matrixrow__ValidationData));
        memset(validationData, 0, sizeof(matrixrow__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__matrixrow()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        matrixrow__ValidationData* validationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_MATRIXROW, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__determinant( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DETERMINANT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DETERMINANT;
        }
//        parentValidationData->determinant++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__determinant()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__transpose( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_TRANSPOSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_TRANSPOSE;
        }
//        parentValidationData->transpose++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__transpose()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__selector( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SELECTOR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SELECTOR;
        }
//        parentValidationData->selector++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__selector()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__vectorproduct( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VECTORPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VECTORPRODUCT;
        }
//        parentValidationData->vectorproduct++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__vectorproduct()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__scalarproduct( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCALARPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCALARPRODUCT;
        }
//        parentValidationData->scalarproduct++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__scalarproduct()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__outerproduct( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OUTERPRODUCT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OUTERPRODUCT;
        }
//        parentValidationData->outerproduct++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__outerproduct()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__divergence( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DIVERGENCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DIVERGENCE;
        }
//        parentValidationData->divergence++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__divergence()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__grad( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_GRAD, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_GRAD;
        }
//        parentValidationData->grad++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__grad()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__curl( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CURL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CURL;
        }
//        parentValidationData->curl++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__curl()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__laplacian( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAPLACIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAPLACIAN;
        }
//        parentValidationData->laplacian++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__laplacian()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mean( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEAN;
        }
//        parentValidationData->mean++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mean()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sdev( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SDEV, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SDEV;
        }
//        parentValidationData->sdev++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sdev()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__variance( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_VARIANCE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_VARIANCE;
        }
//        parentValidationData->variance++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__variance()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__median( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MEDIAN, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MEDIAN;
        }
//        parentValidationData->median++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__median()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__mode____mode_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MODE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MODE;
        }
//        parentValidationData->mode++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__mode____mode_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__moment( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENT;
        }
//        parentValidationData->moment++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__moment()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__momentabout( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOMENTABOUT;
        }
//        parentValidationData->momentabout++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        momentabout__ValidationData* validationData = (momentabout__ValidationData*)mValidationDataStack.newObject(sizeof(momentabout__ValidationData));
        memset(validationData, 0, sizeof(momentabout__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__momentabout()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        momentabout__ValidationData* validationData = (momentabout__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_MOMENTABOUT, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__apply( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_APPLY;
        }
//        parentValidationData->apply++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        apply__ValidationData* validationData = (apply__ValidationData*)mValidationDataStack.newObject(sizeof(apply__ValidationData));
        memset(validationData, 0, sizeof(apply__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__apply()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        apply__ValidationData* validationData = (apply__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_APPLY, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__interval( ParserString value )
{
    return validate__interval( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__interval( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__interval__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__interval__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__interval( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INTERVAL;
        }
//        parentValidationData->interval++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        interval__ValidationData* validationData = (interval__ValidationData*)mValidationDataStack.newObject(sizeof(interval__ValidationData));
        memset(validationData, 0, sizeof(interval__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__interval()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        interval__ValidationData* validationData = (interval__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_INTERVAL, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__inverse( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INVERSE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INVERSE;
        }
//        parentValidationData->inverse++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__inverse()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__condition( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_CONDITION;
        }
//        parentValidationData->condition++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        condition__ValidationData* validationData = (condition__ValidationData*)mValidationDataStack.newObject(sizeof(condition__ValidationData));
        memset(validationData, 0, sizeof(condition__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__condition()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        condition__ValidationData* validationData = (condition__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_CONDITION, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__declare( ParserString value )
{
    return validate__declare( value.str, value.length );
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__declare( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__declare__stream( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
ParserError::ErrorType validate__declare__streamEnd( const ParserChar* value, size_t length )
{
    return ParserError::SIMPLE_TYPE_VALIDATION_OK;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__declare( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DECLARE;
        }
//        parentValidationData->declare++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        declare__ValidationData* validationData = (declare__ValidationData*)mValidationDataStack.newObject(sizeof(declare__ValidationData));
        memset(validationData, 0, sizeof(declare__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__declare()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        declare__ValidationData* validationData = (declare__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_DECLARE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__lambda( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LAMBDA;
        }
//        parentValidationData->lambda++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        lambda__ValidationData* validationData = (lambda__ValidationData*)mValidationDataStack.newObject(sizeof(lambda__ValidationData));
        memset(validationData, 0, sizeof(lambda__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__lambda()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        lambda__ValidationData* validationData = (lambda__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_LAMBDA, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__piecewise( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECEWISE;
        }
//        parentValidationData->piecewise++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        piecewise__ValidationData* validationData = (piecewise__ValidationData*)mValidationDataStack.newObject(sizeof(piecewise__ValidationData));
        memset(validationData, 0, sizeof(piecewise__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__piecewise()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        piecewise__ValidationData* validationData = (piecewise__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_OTHERWISE) || (validationData->validation_current_state == HASH_ELEMENT_PIECE) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_PIECEWISE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__piece( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        piecewise__ValidationData* parentValidationData = (piecewise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_OTHERWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_PIECE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_PIECE;
        }
//        parentValidationData->piece++;

        piece__ValidationData* validationData = (piece__ValidationData*)mValidationDataStack.newObject(sizeof(piece__ValidationData));
        memset(validationData, 0, sizeof(piece__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__piece()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        piece__ValidationData* validationData = (piece__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_PIECE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bvar( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_BVAR;
        }
//        parentValidationData->bvar++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bvar__ValidationData* validationData = (bvar__ValidationData*)mValidationDataStack.newObject(sizeof(bvar__ValidationData));
        memset(validationData, 0, sizeof(bvar__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bvar()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bvar__ValidationData* validationData = (bvar__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_BVAR, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__degree( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_DEGREE;
        }
//        parentValidationData->degree++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        degree__ValidationData* validationData = (degree__ValidationData*)mValidationDataStack.newObject(sizeof(degree__ValidationData));
        memset(validationData, 0, sizeof(degree__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__degree()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        degree__ValidationData* validationData = (degree__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_DEGREE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__semantics( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 119:
        {
        apply__ValidationData* parentValidationData = (apply__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 415:
        {
        bvar__ValidationData* parentValidationData = (bvar__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 116:
        {
        ci__ValidationData* parentValidationData = (ci__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 539:
        {
        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 671:
        {
        condition__ValidationData* parentValidationData = (condition__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 159:
        {
        csymbol__ValidationData* parentValidationData = (csymbol__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 381:
        {
        declare__ValidationData* parentValidationData = (declare__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 70:
        {
        degree__ValidationData* parentValidationData = (degree__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 368:
        {
        domainofapplication__ValidationData* parentValidationData = (domainofapplication__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 158:
        {
        interval__ValidationData* parentValidationData = (interval__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 543:
        {
        lambda__ValidationData* parentValidationData = (lambda__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 725:
        {
        list__ValidationData* parentValidationData = (list__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 556:
        {
        logbase__ValidationData* parentValidationData = (logbase__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 723:
        {
        lowlimit__ValidationData* parentValidationData = (lowlimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 563:
        {
        math__ValidationData* parentValidationData = (math__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 558:
        {
        matrixrow__ValidationData* parentValidationData = (matrixrow__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 114:
        {
        momentabout__ValidationData* parentValidationData = (momentabout__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 595:
        {
        otherwise__ValidationData* parentValidationData = (otherwise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 572:
        {
        piece__ValidationData* parentValidationData = (piece__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 245:
        {
        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 633:
        {
        set__ValidationData* parentValidationData = (set__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 5:
        {
        uplimit__ValidationData* parentValidationData = (uplimit__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        case 344:
        {
        vector__ValidationData* parentValidationData = (vector__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEMANTICS;
        }
//        parentValidationData->semantics++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        semantics__ValidationData* validationData = (semantics__ValidationData*)mValidationDataStack.newObject(sizeof(semantics__ValidationData));
        memset(validationData, 0, sizeof(semantics__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__semantics()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        semantics__ValidationData* validationData = (semantics__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (validationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_SEMANTICS, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__annotation( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ANNOTATION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ANNOTATION;
        }
//        parentValidationData->annotation++;

        annotation__ValidationData* validationData = (annotation__ValidationData*)mValidationDataStack.newObject(sizeof(annotation__ValidationData));
        memset(validationData, 0, sizeof(annotation__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__annotation()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        annotation__ValidationData* validationData = (annotation__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__annotation_xml( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        semantics__ValidationData* parentValidationData = (semantics__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_ANNOTATION_XML) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ANNOTATION_XML, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ANNOTATION_XML;
        }
//        parentValidationData->annotation_xml++;

        annotation_xml__ValidationData* validationData = (annotation_xml__ValidationData*)mValidationDataStack.newObject(sizeof(annotation_xml__ValidationData));
        memset(validationData, 0, sizeof(annotation_xml__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__annotation_xml()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        annotation_xml__ValidationData* validationData = (annotation_xml__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_wildcard < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ANNOTATION_XML, (const ParserChar*)0, "child: " ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__otherwise( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        piecewise__ValidationData* parentValidationData = (piecewise__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_PIECE) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_OTHERWISE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_OTHERWISE;
        }
//        parentValidationData->otherwise++;

        otherwise__ValidationData* validationData = (otherwise__ValidationData*)mValidationDataStack.newObject(sizeof(otherwise__ValidationData));
        memset(validationData, 0, sizeof(otherwise__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__otherwise()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        otherwise__ValidationData* validationData = (otherwise__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ABS) || (validationData->validation_current_state == HASH_ELEMENT_AND) || (validationData->validation_current_state == HASH_ELEMENT_APPLY) || (validationData->validation_current_state == HASH_ELEMENT_APPROX) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (validationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (validationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (validationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (validationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (validationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (validationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (validationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (validationData->validation_current_state == HASH_ELEMENT_ARG) || (validationData->validation_current_state == HASH_ELEMENT_BVAR) || (validationData->validation_current_state == HASH_ELEMENT_CARD) || (validationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_CEILING) || (validationData->validation_current_state == HASH_ELEMENT_CI) || (validationData->validation_current_state == HASH_ELEMENT_CN) || (validationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (validationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (validationData->validation_current_state == HASH_ELEMENT_CONDITION) || (validationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (validationData->validation_current_state == HASH_ELEMENT_COS) || (validationData->validation_current_state == HASH_ELEMENT_COSH) || (validationData->validation_current_state == HASH_ELEMENT_COT) || (validationData->validation_current_state == HASH_ELEMENT_COTH) || (validationData->validation_current_state == HASH_ELEMENT_CSC) || (validationData->validation_current_state == HASH_ELEMENT_CSCH) || (validationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (validationData->validation_current_state == HASH_ELEMENT_CURL) || (validationData->validation_current_state == HASH_ELEMENT_DECLARE) || (validationData->validation_current_state == HASH_ELEMENT_DEGREE) || (validationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (validationData->validation_current_state == HASH_ELEMENT_DIFF) || (validationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (validationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (validationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (validationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (validationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (validationData->validation_current_state == HASH_ELEMENT_EQ) || (validationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (validationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (validationData->validation_current_state == HASH_ELEMENT_EXISTS) || (validationData->validation_current_state == HASH_ELEMENT_EXP) || (validationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (validationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (validationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (validationData->validation_current_state == HASH_ELEMENT_FALSE) || (validationData->validation_current_state == HASH_ELEMENT_FLOOR) || (validationData->validation_current_state == HASH_ELEMENT_FORALL) || (validationData->validation_current_state == HASH_ELEMENT_GCD) || (validationData->validation_current_state == HASH_ELEMENT_GEQ) || (validationData->validation_current_state == HASH_ELEMENT_GRAD) || (validationData->validation_current_state == HASH_ELEMENT_GT) || (validationData->validation_current_state == HASH_ELEMENT_IDENT) || (validationData->validation_current_state == HASH_ELEMENT_IMAGE) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (validationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (validationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (validationData->validation_current_state == HASH_ELEMENT_IN) || (validationData->validation_current_state == HASH_ELEMENT_INFINITY) || (validationData->validation_current_state == HASH_ELEMENT_INT) || (validationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (validationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (validationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (validationData->validation_current_state == HASH_ELEMENT_INVERSE) || (validationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (validationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (validationData->validation_current_state == HASH_ELEMENT_LCM) || (validationData->validation_current_state == HASH_ELEMENT_LEQ) || (validationData->validation_current_state == HASH_ELEMENT_LIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LIST) || (validationData->validation_current_state == HASH_ELEMENT_LN) || (validationData->validation_current_state == HASH_ELEMENT_LOG) || (validationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (validationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_LT) || (validationData->validation_current_state == HASH_ELEMENT_MATRIX) || (validationData->validation_current_state == HASH_ELEMENT_MAX) || (validationData->validation_current_state == HASH_ELEMENT_MEAN) || (validationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (validationData->validation_current_state == HASH_ELEMENT_MIN) || (validationData->validation_current_state == HASH_ELEMENT_MINUS) || (validationData->validation_current_state == HASH_ELEMENT_MODE) || (validationData->validation_current_state == HASH_ELEMENT_MOMENT) || (validationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (validationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (validationData->validation_current_state == HASH_ELEMENT_NEQ) || (validationData->validation_current_state == HASH_ELEMENT_NOT) || (validationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (validationData->validation_current_state == HASH_ELEMENT_NOTIN) || (validationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_OR) || (validationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (validationData->validation_current_state == HASH_ELEMENT_PI) || (validationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (validationData->validation_current_state == HASH_ELEMENT_PLUS) || (validationData->validation_current_state == HASH_ELEMENT_POWER) || (validationData->validation_current_state == HASH_ELEMENT_PRIMES) || (validationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (validationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (validationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (validationData->validation_current_state == HASH_ELEMENT_REAL) || (validationData->validation_current_state == HASH_ELEMENT_REALS) || (validationData->validation_current_state == HASH_ELEMENT_REM) || (validationData->validation_current_state == HASH_ELEMENT_ROOT) || (validationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_SDEV) || (validationData->validation_current_state == HASH_ELEMENT_SEC) || (validationData->validation_current_state == HASH_ELEMENT_SECH) || (validationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (validationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (validationData->validation_current_state == HASH_ELEMENT_SET) || (validationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (validationData->validation_current_state == HASH_ELEMENT_SIN) || (validationData->validation_current_state == HASH_ELEMENT_SINH) || (validationData->validation_current_state == HASH_ELEMENT_SUBSET) || (validationData->validation_current_state == HASH_ELEMENT_SUM) || (validationData->validation_current_state == HASH_ELEMENT_TAN) || (validationData->validation_current_state == HASH_ELEMENT_TANH) || (validationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (validationData->validation_current_state == HASH_ELEMENT_TIMES) || (validationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (validationData->validation_current_state == HASH_ELEMENT_TRUE) || (validationData->validation_current_state == HASH_ELEMENT_UNION) || (validationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (validationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (validationData->validation_current_state == HASH_ELEMENT_VECTOR) || (validationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (validationData->validation_current_state == HASH_ELEMENT_XOR) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_OTHERWISE, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__sep( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        cn__ValidationData* parentValidationData = (cn__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ABS) || (parentValidationData->validation_current_state == HASH_ELEMENT_AND) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPLY) || (parentValidationData->validation_current_state == HASH_ELEMENT_APPROX) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOS) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCOTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCCSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCSINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARCTANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_ARG) || (parentValidationData->validation_current_state == HASH_ELEMENT_BVAR) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARD) || (parentValidationData->validation_current_state == HASH_ELEMENT_CARTESIANPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_CEILING) || (parentValidationData->validation_current_state == HASH_ELEMENT_CI) || (parentValidationData->validation_current_state == HASH_ELEMENT_CN) || (parentValidationData->validation_current_state == HASH_ELEMENT_CODOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPLEXES) || (parentValidationData->validation_current_state == HASH_ELEMENT_COMPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONDITION) || (parentValidationData->validation_current_state == HASH_ELEMENT_CONJUGATE) || (parentValidationData->validation_current_state == HASH_ELEMENT_COS) || (parentValidationData->validation_current_state == HASH_ELEMENT_COSH) || (parentValidationData->validation_current_state == HASH_ELEMENT_COT) || (parentValidationData->validation_current_state == HASH_ELEMENT_COTH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSC) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSCH) || (parentValidationData->validation_current_state == HASH_ELEMENT_CSYMBOL) || (parentValidationData->validation_current_state == HASH_ELEMENT_CURL) || (parentValidationData->validation_current_state == HASH_ELEMENT_DECLARE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DEGREE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DETERMINANT) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVERGENCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DIVIDE) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_DOMAINOFAPPLICATION) || (parentValidationData->validation_current_state == HASH_ELEMENT_EMPTYSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_EQUIVALENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_EULERGAMMA) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXISTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXP) || (parentValidationData->validation_current_state == HASH_ELEMENT_EXPONENTIALE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTORIAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_FACTOROF) || (parentValidationData->validation_current_state == HASH_ELEMENT_FALSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_FLOOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_FORALL) || (parentValidationData->validation_current_state == HASH_ELEMENT_GCD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_GRAD) || (parentValidationData->validation_current_state == HASH_ELEMENT_GT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IDENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGE) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARY) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMAGINARYI) || (parentValidationData->validation_current_state == HASH_ELEMENT_IMPLIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_IN) || (parentValidationData->validation_current_state == HASH_ELEMENT_INFINITY) || (parentValidationData->validation_current_state == HASH_ELEMENT_INT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTEGERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERSECT) || (parentValidationData->validation_current_state == HASH_ELEMENT_INTERVAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_INVERSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAMBDA) || (parentValidationData->validation_current_state == HASH_ELEMENT_LAPLACIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LCM) || (parentValidationData->validation_current_state == HASH_ELEMENT_LEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIST) || (parentValidationData->validation_current_state == HASH_ELEMENT_LN) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOG) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOGBASE) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOWLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_LT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MATRIX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MAX) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MEDIAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_MINUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_MODE) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_MOMENTABOUT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NATURALNUMBERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEQ) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTANUMBER) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTPRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_NOTSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_OR) || (parentValidationData->validation_current_state == HASH_ELEMENT_OUTERPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PARTIALDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_PI) || (parentValidationData->validation_current_state == HASH_ELEMENT_PIECEWISE) || (parentValidationData->validation_current_state == HASH_ELEMENT_PLUS) || (parentValidationData->validation_current_state == HASH_ELEMENT_POWER) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_PRSUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_QUOTIENT) || (parentValidationData->validation_current_state == HASH_ELEMENT_RATIONALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REAL) || (parentValidationData->validation_current_state == HASH_ELEMENT_REALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_REM) || (parentValidationData->validation_current_state == HASH_ELEMENT_ROOT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SCALARPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_SDEV) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEC) || (parentValidationData->validation_current_state == HASH_ELEMENT_SECH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SELECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEMANTICS) || (parentValidationData->validation_current_state == HASH_ELEMENT_SEP) || (parentValidationData->validation_current_state == HASH_ELEMENT_SET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SETDIFF) || (parentValidationData->validation_current_state == HASH_ELEMENT_SIN) || (parentValidationData->validation_current_state == HASH_ELEMENT_SINH) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUBSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_SUM) || (parentValidationData->validation_current_state == HASH_ELEMENT_TAN) || (parentValidationData->validation_current_state == HASH_ELEMENT_TANH) || (parentValidationData->validation_current_state == HASH_ELEMENT_TENDSTO) || (parentValidationData->validation_current_state == HASH_ELEMENT_TIMES) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRANSPOSE) || (parentValidationData->validation_current_state == HASH_ELEMENT_TRUE) || (parentValidationData->validation_current_state == HASH_ELEMENT_UNION) || (parentValidationData->validation_current_state == HASH_ELEMENT_UPLIMIT) || (parentValidationData->validation_current_state == HASH_ELEMENT_VARIANCE) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTOR) || (parentValidationData->validation_current_state == HASH_ELEMENT_VECTORPRODUCT) || (parentValidationData->validation_current_state == HASH_ELEMENT_XOR) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SEP, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SEP;
        }
//        parentValidationData->sep++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__sep()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_articulated_systems( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS;
        }
//        parentValidationData->library_articulated_systems++;

        library_articulated_systems__ValidationData* validationData = (library_articulated_systems__ValidationData*)mValidationDataStack.newObject(sizeof(library_articulated_systems__ValidationData));
        memset(validationData, 0, sizeof(library_articulated_systems__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_articulated_systems()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_articulated_systems__ValidationData* validationData = (library_articulated_systems__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->articulated_system < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS, (const ParserChar*)0, "child: articulated_system" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__articulated_system( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_articulated_systems__ValidationData* parentValidationData = (library_articulated_systems__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ARTICULATED_SYSTEM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->articulated_system++;

        articulated_system__ValidationData* validationData = (articulated_system__ValidationData*)mValidationDataStack.newObject(sizeof(articulated_system__ValidationData));
        memset(validationData, 0, sizeof(articulated_system__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__articulated_system()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        articulated_system__ValidationData* validationData = (articulated_system__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_EXTRA) || (validationData->validation_current_state == HASH_ELEMENT_KINEMATICS) || (validationData->validation_current_state == HASH_ELEMENT_MOTION) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_ARTICULATED_SYSTEM, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__kinematics( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        articulated_system__ValidationData* parentValidationData = (articulated_system__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_KINEMATICS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_KINEMATICS;
        }
//        parentValidationData->kinematics++;

        kinematics__ValidationData* validationData = (kinematics__ValidationData*)mValidationDataStack.newObject(sizeof(kinematics__ValidationData));
        memset(validationData, 0, sizeof(kinematics__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__kinematics()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        kinematics__ValidationData* validationData = (kinematics__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->instance_kinematics_model < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_KINEMATICS, (const ParserChar*)0, "child: instance_kinematics_model" ) )
                return false;
        }
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_KINEMATICS, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_kinematics_model( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 300:
        {
        kinematics_scene__ValidationData* parentValidationData = (kinematics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_articulated_system != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_KINEMATICS_MODEL, (const ParserChar*)0, "sibling: instance_articulated_system" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_KINEMATICS_MODEL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_kinematics_model++;

        break;
        }

        case 465:
        {
        kinematics__ValidationData* parentValidationData = (kinematics__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_KINEMATICS_MODEL, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_KINEMATICS_MODEL, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_KINEMATICS_MODEL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_kinematics_model++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        instance_kinematics_model__ValidationData* validationData = (instance_kinematics_model__ValidationData*)mValidationDataStack.newObject(sizeof(instance_kinematics_model__ValidationData));
        memset(validationData, 0, sizeof(instance_kinematics_model__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_kinematics_model()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_kinematics_model__ValidationData* validationData = (instance_kinematics_model__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bind____kinematics_bind_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 246:
        {
        instance_articulated_system__ValidationData* parentValidationData = (instance_articulated_system__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->setparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: setparam" ) )
                return false;
        }
        if ( parentValidationData->newparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: newparam" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->bind++;

        break;
        }

        case 152:
        {
        instance_kinematics_model__ValidationData* parentValidationData = (instance_kinematics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->newparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: newparam" ) )
                return false;
        }
        if ( parentValidationData->setparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: setparam" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->bind++;

        break;
        }

        case 727:
        {
        axis_info____motion_axis_info_type__ValidationData* parentValidationData = (axis_info____motion_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->newparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: newparam" ) )
                return false;
        }
        if ( parentValidationData->setparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: setparam" ) )
                return false;
        }
        if ( parentValidationData->speed != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: speed" ) )
                return false;
        }
        if ( parentValidationData->acceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: acceleration" ) )
                return false;
        }
        if ( parentValidationData->deceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: deceleration" ) )
                return false;
        }
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        parentValidationData->bind++;

        break;
        }

        case 457:
        {
        effector_info__ValidationData* parentValidationData = (effector_info__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->newparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: newparam" ) )
                return false;
        }
        if ( parentValidationData->setparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: setparam" ) )
                return false;
        }
        if ( parentValidationData->speed != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: speed" ) )
                return false;
        }
        if ( parentValidationData->acceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: acceleration" ) )
                return false;
        }
        if ( parentValidationData->deceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: deceleration" ) )
                return false;
        }
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        parentValidationData->bind++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        bind____kinematics_bind_type__ValidationData* validationData = (bind____kinematics_bind_type__ValidationData*)mValidationDataStack.newObject(sizeof(bind____kinematics_bind_type__ValidationData));
        memset(validationData, 0, sizeof(bind____kinematics_bind_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bind____kinematics_bind_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bind____kinematics_bind_type__ValidationData* validationData = (bind____kinematics_bind_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_bool != 0 )
            if ( validationData->_bool < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND, (const ParserChar*)0, "child: bool" ) )
                    return false;
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->_int != 0 )
            if ( validationData->_int < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND, (const ParserChar*)0, "child: int" ) )
                    return false;
        if ( validationData->SIDREF != 0 )
            if ( validationData->SIDREF < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND, (const ParserChar*)0, "child: SIDREF" ) )
                    return false;
        if ( validationData->param == 0 && validationData->_bool == 0 && validationData->_float == 0 && validationData->_int == 0 && validationData->SIDREF == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_BIND, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__param____kinematics_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bind____kinematics_bind_type__ValidationData* parentValidationData = (bind____kinematics_bind_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->_bool != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: bool" ) )
                return false;
        }
        if ( parentValidationData->_float != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: float" ) )
                return false;
        }
        if ( parentValidationData->_int != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: int" ) )
                return false;
        }
        if ( parentValidationData->SIDREF != 0 ) // sibling in choice
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_PARAM, (const ParserChar*)0, "sibling: SIDREF" ) )
                return false;
        }
        if ( parentValidationData->param >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_PARAM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->param++;

    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__param____kinematics_param_type()
{
    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__setparam____kinematics_setparam_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 246:
        {
        instance_articulated_system__ValidationData* parentValidationData = (instance_articulated_system__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->newparam != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: newparam" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->setparam++;

        break;
        }

        case 152:
        {
        instance_kinematics_model__ValidationData* parentValidationData = (instance_kinematics_model__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->setparam++;

        break;
        }

        case 17:
        {
        instance_kinematics_scene__ValidationData* parentValidationData = (instance_kinematics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->bind_kinematics_model != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: bind_kinematics_model" ) )
                return false;
        }
        if ( parentValidationData->bind_joint_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: bind_joint_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->setparam++;

        break;
        }

        case 727:
        {
        axis_info____motion_axis_info_type__ValidationData* parentValidationData = (axis_info____motion_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->speed != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: speed" ) )
                return false;
        }
        if ( parentValidationData->acceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: acceleration" ) )
                return false;
        }
        if ( parentValidationData->deceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: deceleration" ) )
                return false;
        }
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        parentValidationData->setparam++;

        break;
        }

        case 457:
        {
        effector_info__ValidationData* parentValidationData = (effector_info__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->speed != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: speed" ) )
                return false;
        }
        if ( parentValidationData->acceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: acceleration" ) )
                return false;
        }
        if ( parentValidationData->deceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: deceleration" ) )
                return false;
        }
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        parentValidationData->setparam++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        setparam____kinematics_setparam_type__ValidationData* validationData = (setparam____kinematics_setparam_type__ValidationData*)mValidationDataStack.newObject(sizeof(setparam____kinematics_setparam_type__ValidationData));
        memset(validationData, 0, sizeof(setparam____kinematics_setparam_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__setparam____kinematics_setparam_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        setparam____kinematics_setparam_type__ValidationData* validationData = (setparam____kinematics_setparam_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->_int != 0 )
            if ( validationData->_int < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "child: int" ) )
                    return false;
        if ( validationData->SIDREF != 0 )
            if ( validationData->SIDREF < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "child: SIDREF" ) )
                    return false;
        if ( validationData->_bool != 0 )
            if ( validationData->_bool < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "child: bool" ) )
                    return false;
        if ( validationData->connect_param != 0 )
            if ( validationData->connect_param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, "child: connect_param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->_int == 0 && validationData->SIDREF == 0 && validationData->_bool == 0 && validationData->connect_param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_SETPARAM, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__technique_common____kinematics_technique_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        kinematics__ValidationData* parentValidationData = (kinematics__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_kinematics_model < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: instance_kinematics_model" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        technique_common____kinematics_technique_type__ValidationData* validationData = (technique_common____kinematics_technique_type__ValidationData*)mValidationDataStack.newObject(sizeof(technique_common____kinematics_technique_type__ValidationData));
        memset(validationData, 0, sizeof(technique_common____kinematics_technique_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__technique_common____kinematics_technique_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____kinematics_technique_type__ValidationData* validationData = (technique_common____kinematics_technique_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->frame_origin < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: frame_origin" ) )
                return false;
        }
        if ( validationData->frame_tip < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "child: frame_tip" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__axis_info____kinematics_axis_info_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____kinematics_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_technique_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->frame_origin != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AXIS_INFO, (const ParserChar*)0, "sibling: frame_origin" ) )
                return false;
        }
        if ( parentValidationData->frame_tip != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AXIS_INFO, (const ParserChar*)0, "sibling: frame_tip" ) )
                return false;
        }
        if ( parentValidationData->frame_tcp != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AXIS_INFO, (const ParserChar*)0, "sibling: frame_tcp" ) )
                return false;
        }
        if ( parentValidationData->frame_object != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AXIS_INFO, (const ParserChar*)0, "sibling: frame_object" ) )
                return false;
        }
        parentValidationData->axis_info++;

        axis_info____kinematics_axis_info_type__ValidationData* validationData = (axis_info____kinematics_axis_info_type__ValidationData*)mValidationDataStack.newObject(sizeof(axis_info____kinematics_axis_info_type__ValidationData));
        memset(validationData, 0, sizeof(axis_info____kinematics_axis_info_type__ValidationData));
        validationData->validation_current_state = STATE_MACHINE_ROOT;


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__axis_info____kinematics_axis_info_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis_info____kinematics_axis_info_type__ValidationData* validationData = (axis_info____kinematics_axis_info_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( !( (validationData->validation_current_state == HASH_ELEMENT_ACTIVE) || (validationData->validation_current_state == HASH_ELEMENT_FORMULA) || (validationData->validation_current_state == HASH_ELEMENT_INDEX) || (validationData->validation_current_state == HASH_ELEMENT_INSTANCE_FORMULA) || (validationData->validation_current_state == HASH_ELEMENT_LIMITS) || (validationData->validation_current_state == HASH_ELEMENT_LOCKED) || (validationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (validationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_CLOSING_TAG, HASH_ELEMENT_AXIS_INFO, (const ParserChar*)0, 0) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__active( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis_info____kinematics_axis_info_type__ValidationData* parentValidationData = (axis_info____kinematics_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_ACTIVE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_ACTIVE;
        }
//        parentValidationData->active++;

        active__ValidationData* validationData = (active__ValidationData*)mValidationDataStack.newObject(sizeof(active__ValidationData));
        memset(validationData, 0, sizeof(active__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__active()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        active__ValidationData* validationData = (active__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_bool != 0 )
            if ( validationData->_bool < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ACTIVE, (const ParserChar*)0, "child: bool" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ACTIVE, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_bool == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_ACTIVE, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__locked( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis_info____kinematics_axis_info_type__ValidationData* parentValidationData = (axis_info____kinematics_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ACTIVE) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LOCKED, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LOCKED;
        }
//        parentValidationData->locked++;

        active__ValidationData* validationData = (active__ValidationData*)mValidationDataStack.newObject(sizeof(active__ValidationData));
        memset(validationData, 0, sizeof(active__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__locked()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        active__ValidationData* validationData = (active__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_bool != 0 )
            if ( validationData->_bool < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LOCKED, (const ParserChar*)0, "child: bool" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LOCKED, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_bool == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_LOCKED, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__index( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis_info____kinematics_axis_info_type__ValidationData* parentValidationData = (axis_info____kinematics_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ACTIVE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INDEX) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOCKED) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_INDEX, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_INDEX;
        }
//        parentValidationData->index++;

        index__ValidationData* validationData = (index__ValidationData*)mValidationDataStack.newObject(sizeof(index__ValidationData));
        memset(validationData, 0, sizeof(index__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__index()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        index__ValidationData* validationData = (index__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_int != 0 )
            if ( validationData->_int < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INDEX, (const ParserChar*)0, "child: int" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_INDEX, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_int == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_INDEX, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__limits____kinematics_limits_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis_info____kinematics_axis_info_type__ValidationData* parentValidationData = (axis_info____kinematics_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ACTIVE) || (parentValidationData->validation_current_state == HASH_ELEMENT_INDEX) || (parentValidationData->validation_current_state == HASH_ELEMENT_LOCKED) || (parentValidationData->validation_current_state == HASH_ELEMENT_NEWPARAM) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIMITS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIMITS;
        }
//        parentValidationData->limits++;

        limits____kinematics_limits_type__ValidationData* validationData = (limits____kinematics_limits_type__ValidationData*)mValidationDataStack.newObject(sizeof(limits____kinematics_limits_type__ValidationData));
        memset(validationData, 0, sizeof(limits____kinematics_limits_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__limits____kinematics_limits_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        limits____kinematics_limits_type__ValidationData* validationData = (limits____kinematics_limits_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->min < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIMITS, (const ParserChar*)0, "child: min" ) )
                return false;
        }
        if ( validationData->max < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIMITS, (const ParserChar*)0, "child: max" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__min____common_float_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        limits____kinematics_limits_type__ValidationData* parentValidationData = (limits____kinematics_limits_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->max != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_MIN, (const ParserChar*)0, "sibling: max" ) )
                return false;
        }
        if ( parentValidationData->min >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MIN, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->min++;

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.newObject(sizeof(target__ValidationData));
        memset(validationData, 0, sizeof(target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__min____common_float_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MIN, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MIN, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_MIN, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__max____common_float_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        limits____kinematics_limits_type__ValidationData* parentValidationData = (limits____kinematics_limits_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->min < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_MAX, (const ParserChar*)0, "sibling: min" ) )
                return false;
        }
        if ( parentValidationData->max >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_MAX, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->max++;

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.newObject(sizeof(target__ValidationData));
        memset(validationData, 0, sizeof(target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__max____common_float_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MAX, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MAX, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_MAX, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__frame_origin( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____kinematics_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_technique_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->frame_tip != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRAME_ORIGIN, (const ParserChar*)0, "sibling: frame_tip" ) )
                return false;
        }
        if ( parentValidationData->frame_tcp != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRAME_ORIGIN, (const ParserChar*)0, "sibling: frame_tcp" ) )
                return false;
        }
        if ( parentValidationData->frame_object != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRAME_ORIGIN, (const ParserChar*)0, "sibling: frame_object" ) )
                return false;
        }
        if ( parentValidationData->frame_origin >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FRAME_ORIGIN, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->frame_origin++;

        frame_origin__ValidationData* validationData = (frame_origin__ValidationData*)mValidationDataStack.newObject(sizeof(frame_origin__ValidationData));
        memset(validationData, 0, sizeof(frame_origin__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__frame_origin()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        frame_origin__ValidationData* validationData = (frame_origin__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__frame_tip( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____kinematics_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_technique_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->frame_origin < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_FRAME_TIP, (const ParserChar*)0, "sibling: frame_origin" ) )
                return false;
        }
        if ( parentValidationData->frame_tcp != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRAME_TIP, (const ParserChar*)0, "sibling: frame_tcp" ) )
                return false;
        }
        if ( parentValidationData->frame_object != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRAME_TIP, (const ParserChar*)0, "sibling: frame_object" ) )
                return false;
        }
        if ( parentValidationData->frame_tip >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FRAME_TIP, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->frame_tip++;

        frame_origin__ValidationData* validationData = (frame_origin__ValidationData*)mValidationDataStack.newObject(sizeof(frame_origin__ValidationData));
        memset(validationData, 0, sizeof(frame_origin__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__frame_tip()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        frame_origin__ValidationData* validationData = (frame_origin__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->translate != 0 )
            if ( validationData->translate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FRAME_TIP, (const ParserChar*)0, "child: translate" ) )
                    return false;
        if ( validationData->rotate != 0 )
            if ( validationData->rotate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FRAME_TIP, (const ParserChar*)0, "child: rotate" ) )
                    return false;
        if ( validationData->translate == 0 && validationData->rotate == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_FRAME_TIP, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__frame_tcp( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____kinematics_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_technique_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->frame_tip < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_FRAME_TCP, (const ParserChar*)0, "sibling: frame_tip" ) )
                return false;
        }
        if ( parentValidationData->frame_object != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_FRAME_TCP, (const ParserChar*)0, "sibling: frame_object" ) )
                return false;
        }
        if ( parentValidationData->frame_tcp >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FRAME_TCP, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->frame_tcp++;

        frame_origin__ValidationData* validationData = (frame_origin__ValidationData*)mValidationDataStack.newObject(sizeof(frame_origin__ValidationData));
        memset(validationData, 0, sizeof(frame_origin__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__frame_tcp()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        frame_origin__ValidationData* validationData = (frame_origin__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->translate != 0 )
            if ( validationData->translate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FRAME_TCP, (const ParserChar*)0, "child: translate" ) )
                    return false;
        if ( validationData->rotate != 0 )
            if ( validationData->rotate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FRAME_TCP, (const ParserChar*)0, "child: rotate" ) )
                    return false;
        if ( validationData->translate == 0 && validationData->rotate == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_FRAME_TCP, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__frame_object( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____kinematics_technique_type__ValidationData* parentValidationData = (technique_common____kinematics_technique_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->frame_tip < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_FRAME_OBJECT, (const ParserChar*)0, "sibling: frame_tip" ) )
                return false;
        }
        if ( parentValidationData->frame_object >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_FRAME_OBJECT, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->frame_object++;

        frame_origin__ValidationData* validationData = (frame_origin__ValidationData*)mValidationDataStack.newObject(sizeof(frame_origin__ValidationData));
        memset(validationData, 0, sizeof(frame_origin__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__frame_object()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        frame_origin__ValidationData* validationData = (frame_origin__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->translate != 0 )
            if ( validationData->translate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FRAME_OBJECT, (const ParserChar*)0, "child: translate" ) )
                    return false;
        if ( validationData->rotate != 0 )
            if ( validationData->rotate < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_FRAME_OBJECT, (const ParserChar*)0, "child: rotate" ) )
                    return false;
        if ( validationData->translate == 0 && validationData->rotate == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_FRAME_OBJECT, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__motion( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        articulated_system__ValidationData* parentValidationData = (articulated_system__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == STATE_MACHINE_ROOT) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_MOTION, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_MOTION;
        }
//        parentValidationData->motion++;

        motion__ValidationData* validationData = (motion__ValidationData*)mValidationDataStack.newObject(sizeof(motion__ValidationData));
        memset(validationData, 0, sizeof(motion__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__motion()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        motion__ValidationData* validationData = (motion__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->instance_articulated_system < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MOTION, (const ParserChar*)0, "child: instance_articulated_system" ) )
                return false;
        }
        if ( validationData->technique_common < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_MOTION, (const ParserChar*)0, "child: technique_common" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_articulated_system( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        switch ( mElementDataStack.back().typeID )
        {
        case 300:
        {
        kinematics_scene__ValidationData* parentValidationData = (kinematics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_ARTICULATED_SYSTEM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_articulated_system++;

        break;
        }

        case 407:
        {
        motion__ValidationData* parentValidationData = (motion__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->technique_common != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_ARTICULATED_SYSTEM, (const ParserChar*)0, "sibling: technique_common" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_ARTICULATED_SYSTEM, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_ARTICULATED_SYSTEM, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->instance_articulated_system >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INSTANCE_ARTICULATED_SYSTEM, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->instance_articulated_system++;

        break;
        }

        default:
            // TODO we could throw some error here
            break;
        }
        instance_articulated_system__ValidationData* validationData = (instance_articulated_system__ValidationData*)mValidationDataStack.newObject(sizeof(instance_articulated_system__ValidationData));
        memset(validationData, 0, sizeof(instance_articulated_system__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_articulated_system()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_articulated_system__ValidationData* validationData = (instance_articulated_system__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__technique_common____motion_technique_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        motion__ValidationData* parentValidationData = (motion__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_articulated_system < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: instance_articulated_system" ) )
                return false;
        }
        if ( parentValidationData->technique != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: technique" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->technique_common >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_TECHNIQUE_COMMON, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->technique_common++;

        technique_common____motion_technique_type__ValidationData* validationData = (technique_common____motion_technique_type__ValidationData*)mValidationDataStack.newObject(sizeof(technique_common____motion_technique_type__ValidationData));
        memset(validationData, 0, sizeof(technique_common____motion_technique_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__technique_common____motion_technique_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____motion_technique_type__ValidationData* validationData = (technique_common____motion_technique_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__axis_info____motion_axis_info_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____motion_technique_type__ValidationData* parentValidationData = (technique_common____motion_technique_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->effector_info != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AXIS_INFO, (const ParserChar*)0, "sibling: effector_info" ) )
                return false;
        }
        parentValidationData->axis_info++;

        axis_info____motion_axis_info_type__ValidationData* validationData = (axis_info____motion_axis_info_type__ValidationData*)mValidationDataStack.newObject(sizeof(axis_info____motion_axis_info_type__ValidationData));
        memset(validationData, 0, sizeof(axis_info____motion_axis_info_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__axis_info____motion_axis_info_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis_info____motion_axis_info_type__ValidationData* validationData = (axis_info____motion_axis_info_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__speed____common_float_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis_info____motion_axis_info_type__ValidationData* parentValidationData = (axis_info____motion_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->acceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPEED, (const ParserChar*)0, "sibling: acceleration" ) )
                return false;
        }
        if ( parentValidationData->deceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPEED, (const ParserChar*)0, "sibling: deceleration" ) )
                return false;
        }
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPEED, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        if ( parentValidationData->speed >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SPEED, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->speed++;

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.newObject(sizeof(target__ValidationData));
        memset(validationData, 0, sizeof(target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__speed____common_float_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPEED, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPEED, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_SPEED, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__acceleration____common_float_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis_info____motion_axis_info_type__ValidationData* parentValidationData = (axis_info____motion_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->deceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, "sibling: deceleration" ) )
                return false;
        }
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        if ( parentValidationData->acceleration >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->acceleration++;

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.newObject(sizeof(target__ValidationData));
        memset(validationData, 0, sizeof(target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__acceleration____common_float_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__deceleration____common_float_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis_info____motion_axis_info_type__ValidationData* parentValidationData = (axis_info____motion_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DECELERATION, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        if ( parentValidationData->deceleration >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DECELERATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->deceleration++;

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.newObject(sizeof(target__ValidationData));
        memset(validationData, 0, sizeof(target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__deceleration____common_float_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_DECELERATION, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_DECELERATION, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_DECELERATION, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__jerk____common_float_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis_info____motion_axis_info_type__ValidationData* parentValidationData = (axis_info____motion_axis_info_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->jerk >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_JERK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->jerk++;

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.newObject(sizeof(target__ValidationData));
        memset(validationData, 0, sizeof(target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__jerk____common_float_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_JERK, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_JERK, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_JERK, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__effector_info( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        technique_common____motion_technique_type__ValidationData* parentValidationData = (technique_common____motion_technique_type__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->effector_info >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_EFFECTOR_INFO, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->effector_info++;

        effector_info__ValidationData* validationData = (effector_info__ValidationData*)mValidationDataStack.newObject(sizeof(effector_info__ValidationData));
        memset(validationData, 0, sizeof(effector_info__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__effector_info()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effector_info__ValidationData* validationData = (effector_info__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__speed____common_float2_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effector_info__ValidationData* parentValidationData = (effector_info__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->acceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPEED, (const ParserChar*)0, "sibling: acceleration" ) )
                return false;
        }
        if ( parentValidationData->deceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPEED, (const ParserChar*)0, "sibling: deceleration" ) )
                return false;
        }
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_SPEED, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        if ( parentValidationData->speed >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_SPEED, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->speed++;

        speed____common_float2_or_param_type__ValidationData* validationData = (speed____common_float2_or_param_type__ValidationData*)mValidationDataStack.newObject(sizeof(speed____common_float2_or_param_type__ValidationData));
        memset(validationData, 0, sizeof(speed____common_float2_or_param_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__speed____common_float2_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        speed____common_float2_or_param_type__ValidationData* validationData = (speed____common_float2_or_param_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->float2 != 0 )
            if ( validationData->float2 < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPEED, (const ParserChar*)0, "child: float2" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_SPEED, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->float2 == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_SPEED, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__acceleration____common_float2_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effector_info__ValidationData* parentValidationData = (effector_info__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->deceleration != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, "sibling: deceleration" ) )
                return false;
        }
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        if ( parentValidationData->acceleration >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->acceleration++;

        speed____common_float2_or_param_type__ValidationData* validationData = (speed____common_float2_or_param_type__ValidationData*)mValidationDataStack.newObject(sizeof(speed____common_float2_or_param_type__ValidationData));
        memset(validationData, 0, sizeof(speed____common_float2_or_param_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__acceleration____common_float2_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        speed____common_float2_or_param_type__ValidationData* validationData = (speed____common_float2_or_param_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->float2 != 0 )
            if ( validationData->float2 < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, "child: float2" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->float2 == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_ACCELERATION, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__deceleration____common_float2_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effector_info__ValidationData* parentValidationData = (effector_info__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->jerk != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_DECELERATION, (const ParserChar*)0, "sibling: jerk" ) )
                return false;
        }
        if ( parentValidationData->deceleration >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_DECELERATION, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->deceleration++;

        speed____common_float2_or_param_type__ValidationData* validationData = (speed____common_float2_or_param_type__ValidationData*)mValidationDataStack.newObject(sizeof(speed____common_float2_or_param_type__ValidationData));
        memset(validationData, 0, sizeof(speed____common_float2_or_param_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__deceleration____common_float2_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        speed____common_float2_or_param_type__ValidationData* validationData = (speed____common_float2_or_param_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->float2 != 0 )
            if ( validationData->float2 < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_DECELERATION, (const ParserChar*)0, "child: float2" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_DECELERATION, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->float2 == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_DECELERATION, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__jerk____common_float2_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        effector_info__ValidationData* parentValidationData = (effector_info__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->jerk >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_JERK, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->jerk++;

        speed____common_float2_or_param_type__ValidationData* validationData = (speed____common_float2_or_param_type__ValidationData*)mValidationDataStack.newObject(sizeof(speed____common_float2_or_param_type__ValidationData));
        memset(validationData, 0, sizeof(speed____common_float2_or_param_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__jerk____common_float2_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        speed____common_float2_or_param_type__ValidationData* validationData = (speed____common_float2_or_param_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->float2 != 0 )
            if ( validationData->float2 < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_JERK, (const ParserChar*)0, "child: float2" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_JERK, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->float2 == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_JERK, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_kinematics_scenes( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES;
        }
//        parentValidationData->library_kinematics_scenes++;

        library_kinematics_scenes__ValidationData* validationData = (library_kinematics_scenes__ValidationData*)mValidationDataStack.newObject(sizeof(library_kinematics_scenes__ValidationData));
        memset(validationData, 0, sizeof(library_kinematics_scenes__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_kinematics_scenes()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_kinematics_scenes__ValidationData* validationData = (library_kinematics_scenes__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->kinematics_scene < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES, (const ParserChar*)0, "child: kinematics_scene" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__kinematics_scene( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_kinematics_scenes__ValidationData* parentValidationData = (library_kinematics_scenes__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_KINEMATICS_SCENE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->kinematics_scene++;

        kinematics_scene__ValidationData* validationData = (kinematics_scene__ValidationData*)mValidationDataStack.newObject(sizeof(kinematics_scene__ValidationData));
        memset(validationData, 0, sizeof(kinematics_scene__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__kinematics_scene()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        kinematics_scene__ValidationData* validationData = (kinematics_scene__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__library_formulas( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_LIBRARY_FORMULAS, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_LIBRARY_FORMULAS;
        }
//        parentValidationData->library_formulas++;

        library_formulas__ValidationData* validationData = (library_formulas__ValidationData*)mValidationDataStack.newObject(sizeof(library_formulas__ValidationData));
        memset(validationData, 0, sizeof(library_formulas__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__library_formulas()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        library_formulas__ValidationData* validationData = (library_formulas__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->formula < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_LIBRARY_FORMULAS, (const ParserChar*)0, "child: formula" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__scene( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        COLLADA__ValidationData* parentValidationData = (COLLADA__ValidationData*)mValidationDataStack.top();
        if ( !( (parentValidationData->validation_current_state == HASH_ELEMENT_ASSET) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATIONS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CAMERAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_CONTROLLERS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_EFFECTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORCE_FIELDS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_FORMULAS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_GEOMETRIES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_IMAGES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_JOINTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_LIGHTS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_NODES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_MODELS) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_PHYSICS_SCENES) || (parentValidationData->validation_current_state == HASH_ELEMENT_LIBRARY_VISUAL_SCENES) ) )
        {
            if ( handleError(ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_UNEXPECTED_ELEMENT, HASH_ELEMENT_SCENE, (const ParserChar*)0, 0) )
                return false;
        }
        else
        {
            parentValidationData->validation_current_state = HASH_ELEMENT_SCENE;
        }
//        parentValidationData->scene++;

        scene__ValidationData* validationData = (scene__ValidationData*)mValidationDataStack.newObject(sizeof(scene__ValidationData));
        memset(validationData, 0, sizeof(scene__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__scene()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        scene__ValidationData* validationData = (scene__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_physics_scene( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        scene__ValidationData* parentValidationData = (scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_visual_scene != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_PHYSICS_SCENE, (const ParserChar*)0, "sibling: instance_visual_scene" ) )
                return false;
        }
        if ( parentValidationData->instance_kinematics_scene != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_PHYSICS_SCENE, (const ParserChar*)0, "sibling: instance_kinematics_scene" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_PHYSICS_SCENE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_physics_scene++;

        instance_animation__ValidationData* validationData = (instance_animation__ValidationData*)mValidationDataStack.newObject(sizeof(instance_animation__ValidationData));
        memset(validationData, 0, sizeof(instance_animation__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_physics_scene()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_animation__ValidationData* validationData = (instance_animation__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_visual_scene( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        scene__ValidationData* parentValidationData = (scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->instance_kinematics_scene != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_VISUAL_SCENE, (const ParserChar*)0, "sibling: instance_kinematics_scene" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_VISUAL_SCENE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        if ( parentValidationData->instance_visual_scene >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_INSTANCE_VISUAL_SCENE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->instance_visual_scene++;

        instance_animation__ValidationData* validationData = (instance_animation__ValidationData*)mValidationDataStack.newObject(sizeof(instance_animation__ValidationData));
        memset(validationData, 0, sizeof(instance_animation__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_visual_scene()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_animation__ValidationData* validationData = (instance_animation__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__instance_kinematics_scene( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        scene__ValidationData* parentValidationData = (scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_INSTANCE_KINEMATICS_SCENE, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->instance_kinematics_scene++;

        instance_kinematics_scene__ValidationData* validationData = (instance_kinematics_scene__ValidationData*)mValidationDataStack.newObject(sizeof(instance_kinematics_scene__ValidationData));
        memset(validationData, 0, sizeof(instance_kinematics_scene__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__instance_kinematics_scene()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_kinematics_scene__ValidationData* validationData = (instance_kinematics_scene__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bind_kinematics_model( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_kinematics_scene__ValidationData* parentValidationData = (instance_kinematics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->bind_joint_axis != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_KINEMATICS_MODEL, (const ParserChar*)0, "sibling: bind_joint_axis" ) )
                return false;
        }
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_KINEMATICS_MODEL, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->bind_kinematics_model++;

        bind_kinematics_model__ValidationData* validationData = (bind_kinematics_model__ValidationData*)mValidationDataStack.newObject(sizeof(bind_kinematics_model__ValidationData));
        memset(validationData, 0, sizeof(bind_kinematics_model__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bind_kinematics_model()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bind_kinematics_model__ValidationData* validationData = (bind_kinematics_model__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->SIDREF != 0 )
            if ( validationData->SIDREF < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND_KINEMATICS_MODEL, (const ParserChar*)0, "child: SIDREF" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND_KINEMATICS_MODEL, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->SIDREF == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_BIND_KINEMATICS_MODEL, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__bind_joint_axis( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        instance_kinematics_scene__ValidationData* parentValidationData = (instance_kinematics_scene__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->extra != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_BIND_JOINT_AXIS, (const ParserChar*)0, "sibling: extra" ) )
                return false;
        }
        parentValidationData->bind_joint_axis++;

        bind_joint_axis__ValidationData* validationData = (bind_joint_axis__ValidationData*)mValidationDataStack.newObject(sizeof(bind_joint_axis__ValidationData));
        memset(validationData, 0, sizeof(bind_joint_axis__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__bind_joint_axis()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bind_joint_axis__ValidationData* validationData = (bind_joint_axis__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->axis < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND_JOINT_AXIS, (const ParserChar*)0, "child: axis" ) )
                return false;
        }
        if ( validationData->value < 1 ) // children minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_BIND_JOINT_AXIS, (const ParserChar*)0, "child: value" ) )
                return false;
        }

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__axis____common_sidref_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bind_joint_axis__ValidationData* parentValidationData = (bind_joint_axis__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->value != 0 ) // following sibling
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_NEXT_SIBLING_ALREADY_PRESENT, HASH_ELEMENT_AXIS, (const ParserChar*)0, "sibling: value" ) )
                return false;
        }
        if ( parentValidationData->axis >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_AXIS, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->axis++;

        axis____common_sidref_or_param_type__ValidationData* validationData = (axis____common_sidref_or_param_type__ValidationData*)mValidationDataStack.newObject(sizeof(axis____common_sidref_or_param_type__ValidationData));
        memset(validationData, 0, sizeof(axis____common_sidref_or_param_type__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__axis____common_sidref_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        axis____common_sidref_or_param_type__ValidationData* validationData = (axis____common_sidref_or_param_type__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->SIDREF != 0 )
            if ( validationData->SIDREF < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_AXIS, (const ParserChar*)0, "child: SIDREF" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_AXIS, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->SIDREF == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_AXIS, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateBegin__value____common_float_or_param_type( const ParserAttributes& attributes, void ** attributeDataPtr, void ** validationDataPtr )
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        bind_joint_axis__ValidationData* parentValidationData = (bind_joint_axis__ValidationData*)mValidationDataStack.top();
        if ( parentValidationData->axis < 1 ) // previous sibling minOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_SEQUENCE_PREVIOUS_SIBLING_NOT_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, "sibling: axis" ) )
                return false;
        }
        if ( parentValidationData->value >= 1 ) // maxOccurs
        {
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MAX_OCCURS_EXCEEDED, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;
        }
        parentValidationData->value++;

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.newObject(sizeof(target__ValidationData));
        memset(validationData, 0, sizeof(target__ValidationData));


    } // validation
#endif

    return true;
}

//---------------------------------------------------------------------
bool ColladaParserAutoGen15Private::_validateEnd__value____common_float_or_param_type()
{
#ifdef GENERATEDSAXPARSER_VALIDATION
    if ( mValidate )
    {

        target__ValidationData* validationData = (target__ValidationData*)mValidationDataStack.top();
        DISABLE_WARNING_UNUSED(validationData)
        if ( validationData->_float != 0 )
            if ( validationData->_float < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_VALUE, (const ParserChar*)0, "child: float" ) )
                    return false;
        if ( validationData->param != 0 )
            if ( validationData->param < 1 ) // children minOccurs
                if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_MIN_OCCURS_UNMATCHED, HASH_ELEMENT_VALUE, (const ParserChar*)0, "child: param" ) )
                    return false;
        if ( validationData->_float == 0 && validationData->param == 0 ) // no child present
            if ( handleError ( ParserError::SEVERITY_ERROR_NONCRITICAL, ParserError::ERROR_VALIDATION_CHOICE_NO_CHILD_PRESENT, HASH_ELEMENT_VALUE, (const ParserChar*)0, 0 ) )
                return false;

        mValidationDataStack.deleteObject();


    } // validation
#endif

    return true;
}

#endif
} // namespace
